﻿Imports System.IO
Imports System.ComponentModel
Imports System.Runtime.InteropServices
Imports System.Threading
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Media
Imports UserLib

Public Class Form_Main

#Region "// Class (Type)"

    Private Class MenuField

        Public Property id As Integer
        Public Property text As String

        Public Sub New(ByVal id As Integer, ByVal text As String)
            Me.id = id
            Me.text = text
        End Sub

        Public Overrides Function ToString() As String
            Return _text
        End Function

    End Class

#End Region

#Region "// constant, variable declaration"

#Region "-- Const"

    Private Const _MaxNumberCount = 50

    Private Const _Msg_Cap = "確認"
    Private Const _Msg_Fnf = "ファイルが見つかりません。" & vbCr & vbCr
    Private Const _Msg_Pnf = "移動元、または、移動先のパスが見つかりません。" & vbCr & vbCr
    Private Const _Msg_Dup = "既に存在するファイルを作成することはできません。" & vbCr & vbCr
    Private Const _Msg_Uae = "ファイルへのアクセスが拒否されました。" & vbCr & vbCr
    Private Const _Msg_Io1 = "別のプロセスで使用されているため、"
    Private Const _Msg_Io2 = "プロセスはファイルにアクセスできません。" & vbCr & vbCr
    Private Const _Msg_Nxt = vbCr & vbCr & "処理を継続しますか？。"
    Private Const _Msg_to_ = vbCr & "  ↓" & vbCr

    Private Const _Msg_FailedClipCopy = "クリップボードへのコピーに失敗しました。"
    Private Const _Msg_FailedClipMove = "クリップボードへの切り取りに失敗しました。"

    Private Const _FileDialogFilter =
                  "すべてのファイル|*.*|" &
                  "テキストファイル (.txt)|*.txt|" &
                  "書庫ファイル (.7z .zip .rar)|*.7z;*.zip;*.rar|" &
                  "画像ファイル (.bmp .jpg .jpeg .png .gif .psd)|*.bmp;*.jpg;*.jpeg;*.png;*.gif;*.psd|" &
                  "bmpファイル|*.bmp|" &
                  "jpgファイル|*.jpg;*.jpeg|" &
                  "pngファイル|*.png|" &
                  "gifファイル|*.gif|" &
                  "psdファイル|*.psd|" &
                  "mp3ファイル|*.mp3|" &
                  "mp4ファイル|*.mp4"

    Private Const _LblItemsFormat = "{0:###,##0} items"
    Private Const _DateTimeFormat = "yyyy/MM/dd HH:mm:ss"

    Private _OS_Type As Boolean

    Private _ToolTip_Text As String(,) = {
            {"ファイル", "フォルダ", "フォルダ"},
            {"Single mode", "SubFolder mode", "Tree mode"},
            {"File mode", "Folder mode", "All mode"},
            {"Ignore mode", "Check mode", ""}}

    Private _Cmb_File_ShortCutKeys As Keys() = {
            Keys.X, Keys.L, Keys.S, Keys.W, Keys.F, Keys.M, Keys.U,
            Keys.R, Keys.D, Keys.T, Keys.B, Keys.H, Keys.E}

#End Region

#Region "-- File Data"

    Private _FilePath As String = My.Application.Info.DirectoryPath
    Private _App_Name As String = My.Application.Info.AssemblyName
    Private _ini_File As String = Path.Combine(_FilePath, _App_Name & ".ini")   'Win32APIを使用する為フルパスに変換
    Private _Log_File As String = Path.Combine(_FilePath, _App_Name & ".log")
    Private _TempFile As String = Path.Combine(_FilePath, _App_Name & ".tmp")
    Private _WorkFile As String = Path.Combine(_FilePath, _App_Name & ".$$$")
    Private _UndoFile As String = Path.Combine(_FilePath, _App_Name & ".Undo")
    Private _Err_File As String = Path.Combine(_FilePath, _App_Name & "_Error.log")

    Private _TempFolderName As String = TempFolderName(12)
    Private _InvalidFileNameChars As Char() = Path.GetInvalidFileNameChars()
    Private _Remove_FileNamePattern As String = String.Format("[{0}]", Regex.Escape(_InvalidFileNameChars))

#End Region

#Region "-- Setting"

    Private _DropFileList As String()   'DragDrop Table List

    Private _onSelectItem As Boolean    'item 選択状態
    Private _onStyleChange As Boolean   'True:  VisualStyle
    Private _onDelaySetting As Boolean  'False: SystemColorChange / True: Resize
    Private _onToolTipActive As Boolean
    Private _onDialogForm As Boolean

    Private _WindowMinimized As Boolean '最小化判定ﾌﾗｸﾞ
    Private _WindowMaximized As Boolean '最大化判定ﾌﾗｸﾞ

    Private _FileExt0Format As String = " [{0}]"
    Private _FileExt1Format As String = ".{0,3:000}"

    Private _AttrType(2) As Integer
    Private _OpenDirectory As String
    Private _OfdAutoUpgrade As Boolean
    Private _MessageBoxSound As Boolean
    Private _IniSave As Boolean
    Private _LogFileWriting As Boolean
    Private _LogFileMaxSize As Integer = 16384
    Private _LogFileLeaveCount As Integer = 3
    Private _ClipboardMonitor As Boolean
    Private _UseGridLines As Boolean
    Private _UseCtrlUpDown As Boolean           '[CTRL]+[↑↓] True:有効
    Private _UseHideSelection As Boolean
    Private _UseEmphasisColor As Integer = 2

    Private _UseCultureInfo As String = "ja-JP"
    Private _DateNameFormat As String = "yyyyMMdd-HHmmss"

 'ListView から Explorer 等へのDrop 時の処理遅延
    Private _DropDelay As Integer = 250

    Private _FontSizeMpf As Single = Me.CreateGraphics.DpiX / 96.0F

 '処理Flag
    Private _ChangeCancelFlag As Boolean        'イベントキャンセルフラグ (Cmb_Text_TextChangedで使用)
    Private _OnMouseWheelFlag As Boolean        'マウスホィール動作フラグ (Cmb_Text 関連で使用)

 'フォーム(クライアント)のサイズ
    Private _MinClientSize As Size

 '起動時のウィンドウサイズ
    Private _StartWindow_Width As Integer
    Private _StartWindowHeight As Integer

 '基準となるスクロール バーのサイズ
    Private _BeforeScrollBarWidth As Integer

 'ComboBox (Cmb_Num) の既定幅
    Private _Cmb_Num_Font As Font
    Private _Cmb_Num_BaseWidth As Integer

 'List View 処理（制御）用
    Private _DoubleBuffering As Boolean = True

    Private _List_RegistLimit As Integer = 10000    'リスト登録制限値

    Private _List_AttrType As Integer           '読込対象属性 (RHS)
    Private _List_TreeMode As Integer           '0:List   1:Tree
    Private _List_FileMode As Integer           '0:File   1:Folder
    Private _List_ChckMode As Integer = 1       '0:Ignore 1:Check

#End Region

#Region "-- List View (Form Label)"

    Private _Lbl_FontName() As String = {"MS UI Gothic", "Meiryo UI", "MeiryoKe_UIGothic"}

    Private _Lsv_FontName As String = _Lbl_FontName(0)
    Private _Lsv_FontSize() As Single = {9.75F, 10.5F, 11.25F, 12.0F, 12.75F, 14.25F, 15.0F}
    Private _Lsv_FontSize_Id As Integer = 1

    Private _Lsv_Columns As Integer = 2
    Private _Lsv_Row_Height As Integer
    Private _Lsv_Col0_Width As Integer          'Classic:16 / Visual:19
    Private _Lsv_Col2_Width As Integer
    Private _Lsv_ColA_Width As Integer

    Private _Lsv_DupCheckFlg As Boolean

    Private _Lsv_DisplayRows As Integer         'リスト表示行数
    Private _Lsv_DisplayMode As Integer = 1     '0:Free 1:Smart 2:File
    Private _Lsv_LabelHeight As Integer         'MeiryoKe_UIGothic, 9.75pt/ Meiryo UI, 9pt (MS UI Gothic, 9.75pt:19)

    Private _Lsv_FrameSize_W As Integer         '枠のサイズ
    Private _Lsv_FrameSize_H As Integer
    Private _Lsv_MinUnused_W As Integer         '未使用部分(枠,ラベル)のサイズ
    Private _Lsv_MinUnused_H As Integer
    Private _Lsv_MaxUnused_W As Integer         'ScrollBarを含めた未使用部分のサイズ
    Private _Lsv_MaxUnused_H As Integer

    Private _Lsv_InnerDrop As Boolean           'ListView 内でDrop
    Private _Lsv_DragState As Boolean           'ドラッグ状態
    Private _Lsv_DragCancel As Boolean          'ドロップキャンセル

    Private _Lsv_PrevIndex As Integer           '１つ前のドラッグ位置

#End Region

#Region "-- Color"

    Private _AttrColorName As String() = {"ControlDark", "Magenta", "LightBlue", "WindowText", "Red"}
    Private _EmphasisValue As Single() = {0.0F, 0.9F, 0.85F, 0.8F}

    Private _ColorSuccess As Color = Color.RoyalBlue
    Private _ColorFailure As Color = Color.DeepPink
    Private _ColorBreak As Color = Color.Red
    Private _ColorUndo As Color = Color.ForestGreen
    Private _ColorMain As Color = SystemColors.Highlight
    Private _ColorSub As Color = Color.LimeGreen

    Private _BackColorSuccess As Color
    Private _BackColorFailure As Color
    Private _BackColorBreak As Color
    Private _BackColorUndo As Color

#End Region

#End Region

#Region "// Property"

''' <summary>
''' サブフォームにホバー色を渡す為のプロパティです。
''' </summary>

    Public ReadOnly Property HoverColor() As Color
        Get
          HoverColor = _ColorMain
        End Get
    End Property

''' <summary>
''' サブフォームに警告色を渡す為のプロパティです。
''' </summary>

    Public ReadOnly Property WarningColor() As Color
        Get
          WarningColor = _ColorBreak
        End Get
    End Property

#End Region

#Region "// Sub-Routine & Function"

#Region "-- Main"

    Private Sub SetClipboardText(ByVal text As String, ByVal msg As String)
        Try
          Clipboard.Clear()
          Thread.Sleep(100)     'エラー回避試行

          Clipboard.SetText(text)

          If msg.Length > 0 Then
             ShowInfoMessage(msg & "をクリップボードにコピーしました。", _App_Name)
          End If

        Catch ex As Exception
          ShowWarningMessage(_Msg_FailedClipCopy, "", _App_Name)
        End Try
    End Sub

    <DebuggerStepThrough()>
    Private Function PathExists(ByVal path As String) As Boolean
        If File.Exists(path) OrElse Directory.Exists(path) Then Return True

        Return False
    End Function

    Private Sub RootSelectDialog(ByVal RootPath As String)

        Dim msg As String = String.Format("ルートディレクトリ ""{0}"" は指定できません。", RootPath)
        ShowWarningMessage(msg, "", _Msg_Cap)
    End Sub

    Private Function GetAttributeColor(ByVal Attr As Integer) As Color

        Dim i As Integer

        If (Attr And 4) <> 0 OrElse
           (Attr And FileAttributes.ReparsePoint) <> 0 Then
           i = 4
        ElseIf (Attr And 2) <> 0 Then
           i = 2
        ElseIf (Attr And 1) <> 0 Then
           i = 1
        Else
           i = 3
        End If

        Return Color.FromName(_AttrColorName(i))
    End Function

    Private Sub SetEmphasisBackColor()
        If _UseEmphasisColor <> 0 Then
           Dim mpf As Single = _EmphasisValue(_UseEmphasisColor)

           _BackColorSuccess = ColorEx.IncreaseBrightness(_ColorSuccess, mpf)
           _BackColorFailure = ColorEx.IncreaseBrightness(_ColorFailure, mpf)
           _BackColorBreak = ColorEx.IncreaseBrightness(_ColorBreak, mpf)
           _BackColorUndo = ColorEx.IncreaseBrightness(_ColorUndo, mpf)
        Else
           _BackColorSuccess = SystemColors.Window
           _BackColorFailure = SystemColors.Window
           _BackColorBreak = SystemColors.Window
           _BackColorUndo = SystemColors.Window
        End If
    End Sub

    Private Function NameUpdate(ByVal d1 As String, ByRef d2 As String, state As Boolean) As Integer

        Dim i As Integer
        Dim s As Integer
        Dim d As String

        Dim Fattr As FileAttributes

      Try
        Fattr = File.GetAttributes(d1)
        If (Fattr And FileAttributes.Directory) <> 0 Then
           If d1.ToLower = d2.ToLower Then

'           フォルダ名が大文字小文字を区別していない為("aa" = "AA")、一時的に架空のフォルダ名に変える
              d = Path.Combine(Path.GetDirectoryName(d2), _TempFolderName)

              Directory.Move(d1, d)
              Directory.Move(d, d2)
           Else
              Directory.Move(d1, d2)
           End If
        Else
           File.Move(d1, d2)
        End If

        Return 0
'
      Catch ex As FileNotFoundException
        If _List_ChckMode <> 0 Then

           ListView.EndUpdate()
             s = ShowFnfMessage(d1, state)
           ListView.BeginUpdate()

           Return s - 5                                         '1:ok / 2:Cancel
        End If

      Catch ex As IOException

       '-2147024864 (ファイルが別のプロセスで使用されているため、プロセスはファイルにアクセスできません。)
       '-2147024891 (パス '' へのアクセスが拒否されました。)
       '-2147024893 (パスの一部が見つかりません。)
       '-2147024713 (既に存在するファイルを作成することはできません。)

        If _List_ChckMode <> 0 Then
           ListView.EndUpdate()

             Dim MsgBox As New MessageBoxEx

             Select Case Marshal.GetHRForException(ex)
               Case -2147024864                                 'ロック
                 s = Encoding.GetEncoding(932).GetByteCount(d1) 'パス(shift-jis)のﾊﾞｲﾄ数を取得
                 If s <= 40 Then                                '_Msg_Ioe2 のﾊﾞｲﾄ数*2 =40
                    d = _Msg_Io1 & vbCr & _Msg_Io2 & d1
                 Else
                    d = _Msg_Io1 & _Msg_Io2 & d1
                 End If

                 MsgBox.SetButtonText(2, "スキップ")
                 MsgBox.SetButtonText(3, "中止")
                 s = MsgBox.Dialog(Me, d, _Msg_Cap, _
                                   MessageBoxEx.Buttons.RetryIgnoreCancel, _
                                   MessageBoxIcon.Warning, _
                                   MessageBoxDefaultButton.Button3, , _MessageBoxSound)

               Case -2147024893                                 'パスが無効
                 s = MsgBox.Dialog(Me, _Msg_Pnf & d1 & vbCr & d2 & _Msg_Nxt, _Msg_Cap, _
                                   MessageBoxEx.Buttons.CenterYesNo, _
                                   MessageBoxIcon.Warning, _
                                   MessageBoxDefaultButton.Button2, , _MessageBoxSound)

                 s = s - 5          '1:Yes(Skip) / 2:No(Cancel)

               Case -2147024713                                 '既に存在

                 MsgBox.SetButtonText(1, "名前修正")
                 MsgBox.SetButtonText(2, "スキップ")
                 MsgBox.SetButtonText(3, "中止")
                 s = MsgBox.Dialog(Me, _Msg_Dup & d1 & _Msg_to_ & d2, _Msg_Cap, _
                                   MessageBoxEx.Buttons.RetryIgnoreCancel, _
                                   MessageBoxIcon.Warning, _
                                   MessageBoxDefaultButton.Button3, , _MessageBoxSound)

                 If s = DialogResult.Retry Then

                    Dim Sform As String
                    Dim Dname As String = Path.GetDirectoryName(d2)
                    Dim Fname As String = Path.GetFileNameWithoutExtension(d2)
                    Dim Ename As String = Path.GetExtension(d2)

                    If String.IsNullOrEmpty(Ename) Then
                       Sform = _FileExt0Format
                    Else
                       Sform = _FileExt1Format
                    End If

                    If (Fattr And FileAttributes.Directory) <> 0 Then
                       For i = 2 To 999
                         d2 = Path.Combine(Dname, Fname & String.Format(Sform, i) & Ename)

                         If Not Directory.Exists(d2) Then

                            Directory.Move(d1, d2)
                            Exit For
                         End If
                       Next
                    Else
                       For i = 2 To 999
                         d2 = Path.Combine(Dname, Fname & String.Format(Sform, i) & Ename)

                         If Not File.Exists(d2) Then

                            File.Move(d1, d2)
                            Exit For
                         End If
                       Next
                    End If

                    s = Convert.ToInt32(i > 999) * DialogResult.Ignore
                 End If

               Case Else
                 MsgBox.Show(Me, ex.Message, _App_Name, _
                             MessageBoxEx.Buttons.CenterOK, _
                             MessageBoxIcon.Error, , , _MessageBoxSound)

                 s = DialogResult.Cancel

                 Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
                   sw.WriteLine(Now & vbTab & "NameUpdate" & vbTab & ex.Message)
                 End Using

             End Select

             MsgBox.Dispose()

             Me.Update()
             WinMessage.Clear(Me)

           ListView.BeginUpdate()

           Return s                                             '4:Retry / 2:Cancel / 5:Ignore
        End If

      Catch ex As UnauthorizedAccessException                   'アクセス拒否: HResult= -2147024891
        If _List_ChckMode <> 0 Then
           ListView.EndUpdate()
             Dim MsgBox As New MessageBoxEx
             s = MsgBox.Dialog(Me, _Msg_Uae & d1 & _Msg_Nxt, _Msg_Cap, _
                               MessageBoxEx.Buttons.CenterYesNo, _
                               MessageBoxIcon.Warning, , , _MessageBoxSound)
             MsgBox.Dispose()

             Me.Update()
             WinMessage.Clear(Me)

           ListView.BeginUpdate()

           Return s - 5                                         '1:ok / 2:Cancel
        End If

      Catch ex As Exception
        ListView.EndUpdate()
          Dim MsgBox As New MessageBoxEx
          MsgBox.Show(Me, ex.Message, _App_Name, _
                      MessageBoxEx.Buttons.CenterOK, _
                      MessageBoxIcon.Error, , _
                      MessageBoxEx.Align.CenterRed, _MessageBoxSound)
          MsgBox.Dispose()

          Me.Update()
          WinMessage.Clear(Me)

        ListView.BeginUpdate()

        Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
          sw.WriteLine(Now & vbTab & "NameUpdate" & vbTab & ex.Message)
        End Using

        Return DialogResult.Abort
      End Try
'
        Return DialogResult.OK                                  '1:ok (Ignore)
    End Function

    Private Function ShowFnfMessage(ByVal path As String, ByVal state As Boolean) As Integer

        Dim s As Integer
        Dim MsgBox As New MessageBoxEx

        If state Then
           s = MsgBox.Dialog(Me, _Msg_Fnf & path & _Msg_Nxt, _Msg_Cap, _
                             MessageBoxEx.Buttons.CenterYesNo, _
                             MessageBoxIcon.Warning, _
                             MessageBoxDefaultButton.Button2, , _MessageBoxSound)
        Else
           s = DialogResult.No

           MsgBox.Show(Me, _Msg_Fnf & path, _Msg_Cap, _
                       MessageBoxEx.Buttons.CenterOK, _
                       MessageBoxIcon.Warning, , , _MessageBoxSound)
        End If

        MsgBox.Dispose()

        Me.Update()
        WinMessage.Clear(Me)

        Return s
    End Function

    Private Sub ShowInfoMessage(ByVal msg As String, ByVal cap As String)

        Dim MsgBox As New MessageBoxEx

        MsgBox.Show(Me, msg, cap, _
                    MessageBoxEx.Buttons.CenterOK, _
                    MessageBoxIcon.Information, , , _MessageBoxSound)
        MsgBox.Dispose()

        Me.Update()
        WinMessage.Clear(Me)
    End Sub

    Private Sub ShowWarningMessage(ByVal msg1 As String, ByVal msg2 As String, ByVal cap As String)

        Dim MsgBox As New MessageBoxEx

        If Not String.IsNullOrEmpty(msg2) Then
           msg1 &= vbCrLf & vbCrLf
        End If

        MsgBox.Show(Me, msg1 & msg2, cap, _
                    MessageBoxEx.Buttons.CenterOK, _
                    MessageBoxIcon.Warning, , , _MessageBoxSound)
        MsgBox.Dispose()

        Me.Update()
        WinMessage.Clear(Me)
    End Sub

    Private Sub ShowSettings(ByVal Alt As Boolean)

        Dim cForm As New Form_F1

        cForm.Chk_DialogBox.Checked = StripMenu_DialogBox.Checked
        cForm.Chk_ToolTip.CheckState = StripMenu_ToolTip.CheckState
        cForm.Chk_MsgSound.Checked = StripMenu_MsgSound.Checked
        cForm.Chk_LogWrite.Checked = StripMenu_LogWrite.Checked
        cForm.Chk_IniSave.Checked = StripMenu_IniSave.Checked

        cForm.Chk_GridLines.Checked = StripMenu_GridLines.Checked
        cForm.Chk_CtrlUpDown.Checked = StripMenu_CtrlUpDown.Checked

        cForm.Trk_Emphasis.Value = _UseEmphasisColor

     'コンボボックスの値を渡す（選択値もセット）
        cForm.Cmb_Culture.Items.AddRange(StripMenu_Cmb_Culture.Items.Cast(Of Object)().ToArray())
        cForm.Cmb_DateTimeFormat.Items.AddRange(StripMenu_Cmb_DateTimeFormat.Items.Cast(Of Object)().ToArray())

        cForm.Cmb_Culture.SelectedIndex = StripMenu_Cmb_Culture.SelectedIndex
        cForm.Cmb_DateTimeFormat.SelectedIndex = StripMenu_Cmb_DateTimeFormat.SelectedIndex

        cForm.Log_CheckSize = _LogFileMaxSize
        cForm.Log_LeaveCount = _LogFileLeaveCount

        cForm.ColorName(0) = _AttrColorName(1)
        cForm.ColorName(1) = _AttrColorName(2)
        cForm.ColorName(2) = _AttrColorName(4)
        cForm.ColorName(3) = Lbl_Items.ForeColor.Name
        cForm.ColorName(4) = _ColorSuccess.Name
        cForm.ColorName(5) = _ColorFailure.Name
        cForm.ColorName(6) = _ColorBreak.Name
        cForm.ColorName(7) = _ColorUndo.Name
        cForm.ColorName(8) = _ColorMain.Name
        cForm.ColorName(9) = _ColorSub.Name

        cForm.Lsv_FontName = _Lsv_FontName
        cForm.Lsv_FontSize_Id = _Lsv_FontSize_Id
        cForm.Lsv_FontSize = _Lsv_FontSize

        cForm.ToolTip.Active = Me.ToolTip.Active
        cForm.Show(Me, Alt)

        If cForm.DialogResult = DialogResult.OK Then

           StripMenu_DialogBox.Checked = cForm.Chk_DialogBox.Checked
           StripMenu_ToolTip.CheckState = cForm.Chk_ToolTip.CheckState
           StripMenu_MsgSound.Checked = cForm.Chk_MsgSound.Checked
           StripMenu_LogWrite.Checked = cForm.Chk_LogWrite.Checked
           StripMenu_IniSave.Checked = cForm.Chk_IniSave.Checked

           StripMenu_GridLines.Checked = cForm.Chk_GridLines.Checked()
           StripMenu_CtrlUpDown.Checked = cForm.Chk_CtrlUpDown.Checked

           _UseEmphasisColor = cForm.Trk_Emphasis.Value

        'StripMenu Emphasis チェック表示更新
           StripMenu_E_Control()

           StripMenu_Cmb_Culture.SelectedIndex = cForm.Cmb_Culture.SelectedIndex
           StripMenu_Cmb_DateTimeFormat.SelectedIndex = cForm.Cmb_DateTimeFormat.SelectedIndex

           _LogFileMaxSize = cForm.Log_CheckSize
           _LogFileLeaveCount = cForm.Log_LeaveCount

           _AttrColorName(1) = cForm.ColorName(0)
           _AttrColorName(2) = cForm.ColorName(1)
           _AttrColorName(4) = cForm.ColorName(2)
           _ColorSuccess = Color.FromName(cForm.ColorName(4))
           _ColorFailure = Color.FromName(cForm.ColorName(5))
           _ColorBreak = Color.FromName(cForm.ColorName(6))
           _ColorUndo = Color.FromName(cForm.ColorName(7))
           _ColorMain = Color.FromName(cForm.ColorName(8))
           _ColorSub = Color.FromName(cForm.ColorName(9))

        '強調背景色セット
           SetEmphasisBackColor()

           Lbl_Items.ForeColor = Color.FromName(cForm.ColorName(3))

           _Lsv_FontName = cForm.Lsv_FontName
           _Lsv_FontSize_Id = cForm.Lsv_FontSize_Id

           HoverColorSet()
           Lsv_SetInfo()
           Lsv_DisplayMode_Control(True)
        End If

        cForm.Dispose()

        Me.Update()
        WinMessage.Clear(Me, 200)
    End Sub

    Private Sub FileNameChange()
        With ListView

          Dim index As Integer
          Dim IsFolder As Boolean   'ファイル属性 (True:ディレクトリ)
          Dim TitleText As String

          If (.SelectedItems.Count <> 1) Then
             Beep()
             Return
          Else
             Dim fn As String = .SelectedItems(0).Text
             Dim fi As New FileInfo(fn)

             index = .SelectedIndices(0)
             .Items(index).Selected = False     'ちらつきを減らす為、選択状態を一時解除

             If fi.Attributes = -1 Then
                ShowFnfMessage(fn, False)

                .Items(index).Selected = True   '選択状態を戻す
                Return
             End If

             IsFolder = ((fi.Attributes And FileAttributes.Directory) <> 0)

             If IsFolder Then
                TitleText = "フォルダ名の変更"

             'リスト内に同じフォルダのアイテムがないかをチェック
                If Not IsNothing(.FindItemWithText(fn & "\", False, 0, True)) Then

                   ShowWarningMessage("リスト内に同じフォルダのアイテムがある為、" & vbCrLf &
                                       TitleText & "はできません。", fn, _Msg_Cap)

                   .Items(index).Selected = True   '選択状態を戻す
                   Return
                End If
             Else
                TitleText = "ファイル名の変更"
             End If
          End If

          Dim nForm As New Form_F2
          Dim bPath As String = .Items(index).Text
          Dim aPath As String = ""
          Dim bName As String = Path.GetFileName(bPath)
          Dim ext_f As Boolean

          nForm.Text = TitleText
          nForm.ToolTip.Active = Me.ToolTip.Active

          nForm.Show(Me, bName, ext_f, _ClipboardMonitor)
          Me.Update()

          If nForm.DialogResult = DialogResult.OK Then
             If ext_f Then
                aPath = Path.Combine(Path.GetDirectoryName(bPath), bName)
             Else
                aPath = Path.Combine(Path.GetDirectoryName(bPath), bName & Path.GetExtension(bPath))
             End If

             Dim s As Integer
             Do
               Try
                 If IsFolder Then
                    Directory.Move(bPath, aPath)
                 Else
                    File.Move(bPath, aPath)
                 End If

                 .BeginUpdate()
                 .Items(index).SubItems(0).Text = aPath
                 .Items(index).SubItems(1).Text = Path.GetFileName(aPath)
                 .Items(index).SubItems(2).Text = Path.GetExtension(aPath)
                 .EndUpdate()

                 Exit Do

               Catch ex As Exception

                 Dim MsgBox As New MessageBoxEx

                 MsgBox.SetButtonText(2, "中止")
                 s = MsgBox.Dialog(Me, TitleText & "に失敗しました。" & vbCr & vbCr &
                                   String.Format("  ( {0} ) ", ex.Message.Trim) & vbCr & vbCr &
                                   Path.GetFileName(bPath) & vbCr & "  ↓" & vbCr &
                                   Path.GetFileName(aPath), _App_Name,
                                   MessageBoxEx.Buttons.RetryCancel,
                                   MessageBoxIcon.Warning,
                                   MessageBoxDefaultButton.Button2, , _MessageBoxSound)
                 MsgBox.Dispose()

                 Me.Update()
                 WinMessage.Clear(Me)
               End Try
             Loop Until s = DialogResult.Cancel

          'Log Write
             If (s = 0) AndAlso _LogFileWriting Then
                If File.Exists(_Log_File) Then
                   If File.Exists(_TempFile) Then File.Delete(_TempFile)
                   File.Move(_Log_File, _TempFile)
                Else
                   Using sw As New StreamWriter(_TempFile, False, Encoding.UTF8) : End Using
                End If

                Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
                  sw.WriteLine("-- {0} : Rename --", Date.Now.ToString(_DateTimeFormat))
                  sw.WriteLine(bPath)
                  sw.WriteLine("  to")
                  sw.WriteLine(aPath)
                  sw.WriteLine()

                  Dim fi As New FileInfo(_TempFile)
                  If fi.Length > 0 Then
                     Using sr As New StreamReader(_TempFile, Encoding.UTF8)
                       sw.Write(sr.ReadToEnd())
                     End Using
                  End If
                End Using

                File.Delete(_TempFile)
             End If

          'Undoを無効にする
             If s = 0 Then Btn_Undo.Enabled = False
          End If

          nForm.Dispose()

          .Items(index).Selected = True '選択状態を戻す
        End With
    End Sub

    Private Sub FileAttributesChange()

        If ListView.SelectedItems.Count = 0 Then
           Beep()
           Return
        End If

        Dim SelectIndex As New List(Of Integer)
        Dim Select_Path As New List(Of String)

        Dim state As Boolean = (ListView.SelectedItems.Count <> 1)

        For Each LsvItem As ListViewItem In ListView.SelectedItems

          If Not PathExists(LsvItem.Text) Then

             Lsv_BackColor_Set(LsvItem, _BackColorFailure)
             If ShowFnfMessage(LsvItem.Text, state) <> DialogResult.Yes Then Return
          Else

             SelectIndex.Add(LsvItem.Index)
             Select_Path.Add(LsvItem.Text)

             ListView.Items(LsvItem.Index).Selected = False 'ちらつきを減らす為、選択状態を一時解除
          End If
        Next

        If Select_Path.Count = 0 Then Return

        Dim Fname As String
        Dim Fattr As Integer

        If Select_Path.Count = 1 Then
           Fattr = File.GetAttributes(Select_Path.Item(0)) And 23
           Fname = Path.GetFileName(Select_Path.Item(0)).TrimStart
        Else
           Fattr = -1
           Fname = String.Format("{0:###,##0} 個の項目が選択されています。", Select_Path.Count)
        End If

        Dim nForm As New Form_F3

        nForm.Show(Me, Fname, Fattr)
        Me.Update()

        If nForm.DialogResult = DialogResult.OK Then

           Dim i As Integer
           Dim c As Color = GetAttributeColor(Fattr)

           For Each p As String In Select_Path

             Try
               Dim fa As FileAttributes = DirectCast((File.GetAttributes(p) And Not 7) Or Fattr, FileAttributes)

               File.SetAttributes(p, fa)

               ListView.Items(SelectIndex(i)).SubItems(1).ForeColor = c
               i += 1

             Catch ex As Exception

               Dim msg As String = String.Format("  ( {0} ) ", ex.Message.Trim) & vbCr & vbCr & p
               ShowWarningMessage("属性の変更に失敗しました。", msg, _App_Name)
             End Try
           Next
        End If

        nForm.Dispose()

   '選択状態を戻す
      ListView.BeginUpdate()
        For Each i As Integer In SelectIndex
           ListView.Items(i).Selected = True
        Next
      ListView.EndUpdate()
    End Sub

#End Region

#Region "-- FormLoad, FormClosed"

    Private Sub iniLoad()

        If Not File.Exists(_ini_File) Then
           _OpenDirectory = My.Application.Info.DirectoryPath

           Dim MsgBox As New MessageBoxEx

           If MsgBox.Dialog(Me, "設定" & _Msg_Fnf & "設定ファイルを使用しますか？", _Msg_Cap, _
                            MessageBoxEx.Buttons.CenterYesNo, _
                            MessageBoxIcon.Warning, _
                            MessageBoxDefaultButton.Button1, , _MessageBoxSound) _
                          = DialogResult.Yes Then

              StripMenu_ToolTip.Checked = True
              StripMenu_LogWrite.Checked = True
              StripMenu_IniSave.Checked = True
           End If

           MsgBox.Dispose()
           WinMessage.Clear(Me)

        '既定値をセット (_UseEmphasisColor = 2)
           StripMenu_GridLines.Checked = True
        Else
           Dim i, n As Integer
           Dim s As String
           Dim f As Integer
           Dim sBuffer As New StringBuilder

           sBuffer.Capacity = 512  'バッファのサイズを指定

           Me.Top = GetPrivateProfileInt("Form", "Top", 0, _ini_File)
           Me.Left = GetPrivateProfileInt("Form", "Left", 0, _ini_File)

           _StartWindow_Width = GetPrivateProfileInt("Form", "Width", _StartWindow_Width, _ini_File)
           _StartWindowHeight = GetPrivateProfileInt("Form", "Height", _StartWindowHeight, _ini_File)

'   Switch
           f = GetPrivateProfileString("Form", "OfdAutoUpgrade", _OfdAutoUpgrade.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_DialogBox.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("Form", "ToolTipActive", "True", sBuffer, sBuffer.Capacity, _ini_File)

             If StringToBoolean(sBuffer.ToString) Then
                StripMenu_ToolTip.Checked = True
                ToolTip.Active = True
             End If

           f = GetPrivateProfileString("Form", "ItemToolTips", "True", sBuffer, sBuffer.Capacity, _ini_File)

             If StringToBoolean(sBuffer.ToString) Then
                ListView.ShowItemToolTips = True

                If Not ToolTip.Active Then
                   StripMenu_ToolTip.CheckState = CheckState.Indeterminate
                End If
             End If

           f = GetPrivateProfileString("Form", "MessageSound", _MessageBoxSound.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_MsgSound.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("Form", "LogWriting", _LogFileWriting.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_LogWrite.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("Form", "IniWriting", _IniSave.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_IniSave.Checked = StringToBoolean(sBuffer.ToString)

'   Format (Read Only)
           If 0 <> GetPrivateProfileString("Format", "Culture", _UseCultureInfo, sBuffer, sBuffer.Capacity, _ini_File) Then
              _UseCultureInfo = sBuffer.ToString
           End If
           If 0 <> GetPrivateProfileString("Format", "DateName", _DateNameFormat, sBuffer, sBuffer.Capacity, _ini_File) Then
              _DateNameFormat = sBuffer.ToString
           End If
           If 0 <> GetPrivateProfileString("Format", "NoExtensionFile", _FileExt0Format, sBuffer, sBuffer.Capacity, _ini_File) Then
              _FileExt0Format = sBuffer.ToString
           End If
           If 0 <> GetPrivateProfileString("Format", "AnExtensionFile", _FileExt1Format, sBuffer, sBuffer.Capacity, _ini_File) Then
              _FileExt1Format = sBuffer.ToString
           End If

'   ListView
           f = GetPrivateProfileInt("ListView", "RegistLimit", _List_RegistLimit, _ini_File)
             If (f > 0) AndAlso (f <= 100000) Then _List_RegistLimit = f

           If 0 <> GetPrivateProfileString("ListView", "FontName", _Lsv_FontName, sBuffer, sBuffer.Capacity, _ini_File) Then
              _Lsv_FontName = sBuffer.ToString
           End If

           f = GetPrivateProfileInt("ListView", "FontSize", _Lsv_FontSize_Id, _ini_File)
             If (f >= 0) AndAlso (f < _Lsv_FontSize.Length) Then
                _Lsv_FontSize_Id = f
             End If

           f = GetPrivateProfileString("ListView", "GridLines", _UseGridLines.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_GridLines.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("ListView", "CtrlUpDown", _UseCtrlUpDown.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_CtrlUpDown.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileInt("ListView", "EmphasisColor", _UseEmphasisColor, _ini_File)
             If (f >= 0) AndAlso (f <= 3) Then _UseEmphasisColor = f

           f = GetPrivateProfileInt("ListView", "DropDelay", _DropDelay, _ini_File)
             If f < 100 Then _DropDelay = 100
'   Mode
           f = GetPrivateProfileInt("Mode", "TreeMode", _List_TreeMode, _ini_File)
             If (f >= 0) AndAlso (f <= 2) Then _List_TreeMode = f

           f = GetPrivateProfileInt("Mode", "FileMode", _List_FileMode, _ini_File)
             If (f >= 0) AndAlso (f <= 2) Then _List_FileMode = f

           f = GetPrivateProfileInt("Mode", "ChckMode", _List_ChckMode, _ini_File)
             If (f = 0) OrElse (f = 1) Then _List_ChckMode = f

'   Attributes
           f = GetPrivateProfileString("Attributes", "ReadOnly", "False", sBuffer, sBuffer.Capacity, _ini_File)
             If StringToBoolean(sBuffer.ToString) Then _AttrType(0) = 1
           f = GetPrivateProfileString("Attributes", "Hidden", "False", sBuffer, sBuffer.Capacity, _ini_File)
             If StringToBoolean(sBuffer.ToString) Then _AttrType(1) = 2
           f = GetPrivateProfileString("Attributes", "System", "False", sBuffer, sBuffer.Capacity, _ini_File)
             If StringToBoolean(sBuffer.ToString) Then _AttrType(2) = 4

           _List_AttrType = _AttrType(0) + _AttrType(1) + _AttrType(2)

'   Log File
           f = GetPrivateProfileInt("Log", "LogFileMaxSize", _LogFileMaxSize, _ini_File)
             If (f >= 4096) AndAlso (f <= 65536) Then _LogFileMaxSize = f

           f = GetPrivateProfileInt("Log", "LogFileLeaveCount", _LogFileLeaveCount, _ini_File)
             If (f >= 1) AndAlso (f <= 5) Then _LogFileLeaveCount = f

'   Path
           f = GetPrivateProfileString("Path", "OpenDirectory", "", sBuffer, sBuffer.Capacity, _ini_File)
             _OpenDirectory = sBuffer.ToString

'   Text
           n = GetPrivateProfileInt("Text", "TextA_Count", 0, _ini_File)
           For i = 1 To n
             s = String.Format("TextA_{0}", i)

             If 0 <> GetPrivateProfileString("Text", s, "", sBuffer, sBuffer.Capacity, _ini_File) Then
                Cmb_Text_1.Items.Add(sBuffer.ToString)
             End If
           Next

           n = GetPrivateProfileInt("Text", "TextB_Count", 0, _ini_File)
           For i = 1 To n
             s = String.Format("TextB_{0}", i)

             If 0 <> GetPrivateProfileString("Text", s, "", sBuffer, sBuffer.Capacity, _ini_File) Then
                Cmb_Text_2.Items.Add(sBuffer.ToString)
             End If
           Next

'   Color
           f = GetPrivateProfileString("Color", "AttributeR", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _AttrColorName(1) = sBuffer.ToString
           f = GetPrivateProfileString("Color", "AttributeH", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _AttrColorName(2) = sBuffer.ToString
           f = GetPrivateProfileString("Color", "AttributeS", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _AttrColorName(4) = sBuffer.ToString

           f = GetPrivateProfileString("Color", "itemsCount", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then Lbl_Items.ForeColor = Color.FromName(sBuffer.ToString)

           f = GetPrivateProfileString("Color", "Success", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorSuccess = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "Failure", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorFailure = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "Break", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorBreak = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "Undo", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorUndo = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "HoverMain", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorMain = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "HoverSub", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorSub = Color.FromName(sBuffer.ToString)

           sBuffer.Clear()
        End If
    End Sub

    Private Sub iniSave()

        Dim f As Integer

        If Not _IniSave Then
        '設定ファイル書込みを禁止、かつ、設定ファイルがない場合、設定を破棄
           If (Not File.Exists(_ini_File)) Then
              Return
           Else
              Dim sBuffer As New StringBuilder

              sBuffer.Capacity = 512
              f = GetPrivateProfileString("Form", "IniWriting", "True", sBuffer, sBuffer.Capacity, _ini_File)

           '設定ファイル書込みを禁止している場合でも、書き込み設定を更新
              If StringToBoolean(sBuffer.ToString) Then
                 f = WritePrivateProfileString("Form", "IniWriting", "False", _ini_File)
              End If
              sBuffer.Clear()

              Return
           End If
        End If

        f = WritePrivateProfileString("info", "Name", My.Application.Info.ProductName, _ini_File)
        f = WritePrivateProfileString("info", "Ver", My.Application.Info.Version.ToString, _ini_File)

        If Me.WindowState = FormWindowState.Normal Then
           f = WritePrivateProfileString("Form", "Top", Me.Top.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Left", Me.Left.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Width", Me.Width.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Height", Me.Height.ToString, _ini_File)
        Else
'        フォームが最大化、あるいは最小化状態
           f = WritePrivateProfileString("Form", "Top", Me.RestoreBounds.Top.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Left", Me.RestoreBounds.Left.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Width", Me.RestoreBounds.Width.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Height", Me.RestoreBounds.Height.ToString, _ini_File)
        End If

        f = WritePrivateProfileString("Form", "OfdAutoUpgrade", _OfdAutoUpgrade.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "ToolTipActive", ToolTip.Active.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "ItemToolTips", ListView.ShowItemToolTips.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "MessageSound", _MessageBoxSound.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "LogWriting", _LogFileWriting.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "IniWriting", _IniSave.ToString, _ini_File)

        f = WritePrivateProfileString("Format", "Culture", _UseCultureInfo, _ini_File)
        f = WritePrivateProfileString("Format", "DateName", _DateNameFormat, _ini_File)

        f = WritePrivateProfileString("ListView", "FontName", _Lsv_FontName, _ini_File)
        f = WritePrivateProfileString("ListView", "FontSize", _Lsv_FontSize_Id.ToString, _ini_File)
        f = WritePrivateProfileString("ListView", "GridLines", _UseGridLines.ToString, _ini_File)
        f = WritePrivateProfileString("ListView", "CtrlUpDown", _UseCtrlUpDown.ToString, _ini_File)
        f = WritePrivateProfileString("ListView", "EmphasisColor", _UseEmphasisColor.ToString, _ini_File)

        f = WritePrivateProfileString("Mode", "TreeMode", _List_TreeMode.ToString, _ini_File)
        f = WritePrivateProfileString("Mode", "FileMode", _List_FileMode.ToString, _ini_File)
        f = WritePrivateProfileString("Mode", "ChckMode", _List_ChckMode.ToString, _ini_File)

        f = WritePrivateProfileString("Attributes", "ReadOnly", Convert.ToBoolean(_AttrType(0) = 1).ToString, _ini_File)
        f = WritePrivateProfileString("Attributes", "Hidden", Convert.ToBoolean(_AttrType(1) = 2).ToString, _ini_File)
        f = WritePrivateProfileString("Attributes", "System", Convert.ToBoolean(_AttrType(2) = 4).ToString, _ini_File)

        f = WritePrivateProfileString("Log", "LogFileMaxSize", _LogFileMaxSize.ToString, _ini_File)
        f = WritePrivateProfileString("Log", "LogFileLeaveCount", _LogFileLeaveCount.ToString, _ini_File)

        f = WritePrivateProfileString("Path", "OpenDirectory", _OpenDirectory, _ini_File)

        f = WritePrivateProfileString("Color", "AttributeR", _AttrColorName(1), _ini_File)
        f = WritePrivateProfileString("Color", "AttributeH", _AttrColorName(2), _ini_File)
        f = WritePrivateProfileString("Color", "AttributeS", _AttrColorName(4), _ini_File)
        f = WritePrivateProfileString("Color", "itemsCount", Lbl_Items.ForeColor.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Success", _ColorSuccess.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Failure", _ColorFailure.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Break", _ColorBreak.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Undo", _ColorUndo.Name, _ini_File)
        f = WritePrivateProfileString("Color", "HoverMain", _ColorMain.Name, _ini_File)
        f = WritePrivateProfileString("Color", "HoverSub", _ColorSub.Name, _ini_File)
    End Sub

    Private Sub CmdSendToLoad()

        If My.Application.CommandLineArgs.Count = 0 Then Return

'     設定ファイルの内容より、スイッチを優先適用。
'     同系統のスイッチを指定した場合は、後で指定したものを適用。

        For Each cmd As String In My.Application.CommandLineArgs
          If cmd.StartsWith("/") Then
             Select Case cmd.TrimStart("/"c).ToLower
               Case "x" : If _IniSave Then
                             StripMenu_IniSave.Checked = False
                          End If

               Case "l" : If _LogFileWriting Then
                             StripMenu_LogWrite.Checked = False
                          End If

               Case "c0" : _List_ChckMode = 0
               Case "c1" : _List_ChckMode = 1
               Case "t0" : _List_TreeMode = 0
               Case "t1" : _List_TreeMode = 1
               Case "t2" : _List_TreeMode = 2
               Case "f0" : _List_FileMode = 0
               Case "f1" : _List_FileMode = 1
               Case "f2" : _List_FileMode = 2

               Case "b" : _DoubleBuffering = False
               Case "m" : _ClipboardMonitor = True
               Case "h" : _UseHideSelection = True
                          ListView.HideSelection = True


               Case "st" : StripMenu_ToolTip.CheckState = CheckState.Indeterminate
                           ToolTip.Active = False
                           ListView.ShowItemToolTips = True

               Case "ht" : StripMenu_TaskBar.Checked = False
                           Me.ShowInTaskbar = False

               Case Else
                 Dim msg As String = String.Format("無効なスイッチ ""{0}"" が指定されています。", cmd)
                 ShowWarningMessage(msg, "", _Msg_Cap)
             End Select
          End If
        Next

'     パス情報を取得
        Dim l As New List(Of String)
        For Each cmd As String In My.Application.CommandLineArgs
          If Not cmd.StartsWith("/") Then

          '先頭の１文字を大文字、末尾の "\" を取り除く
             cmd = cmd.Substring(0, 1).ToUpper & cmd.Substring(1).TrimEnd("\"c)

             If File.Exists(cmd) Then
                l.Add(cmd)
             Else
                If Directory.Exists(cmd) Then
                'ルートチェック
                   If IsNothing(Path.GetDirectoryName(cmd)) Then
                      RootSelectDialog(cmd)
                   Else
                      l.Add(cmd)
                   End If
                Else
                   ShowWarningMessage("無効なパスが指定されています。", cmd, _Msg_Cap)
                End If
             End If
             Exit For
          End If
        Next

'     取得したパス情報をDropテーブルへ移す
        If l.Count <> 0 Then
           _DropFileList = l.ToArray

'        ListView登録 (Lsv_AddRange 内で ListView.ColumnWidthChanged イベント登録)
           Lsv_AddRange()
        End If
    End Sub

''' <summary>
''' フォント (ListView はフォント情報のみ) をセットします。
''' </summary>

    Private Sub FontSetting()

        Dim FontSize As Single = Me.Font.Size
        Dim FontNameId As Integer = 0

'   フォントチェック
        For Each fnt As FontFamily In FontFamily.Families
          Select Case fnt.Name
            Case _Lbl_FontName(1) : FontNameId = 1
            Case _Lbl_FontName(2) : FontNameId = 2 : Exit For
          End Select
        Next

'   Dpi check
        If _FontSizeMpf <> 1.0F Then

           FontSize = GetNearFontSize(FontSize / _FontSizeMpf)

           Dim s As Single = GetNearFontSize(Lbl_Attr_R.Font.Size / _FontSizeMpf)
           Dim f As New Font("Tahoma", s)

           Lbl_Attr_R.Font = f
           Lbl_Attr_H.Font = f
           Lbl_Attr_S.Font = f

           s = GetNearFontSize(Lbl_Items.Font.Size / _FontSizeMpf)
           Lbl_Items.Font = New Font("Tahoma", s)

           s = GetNearFontSize(Cmb_Num.Font.Size / _FontSizeMpf)
           Cmb_Num.Font = New Font("Lucida Console", s)

           Btn_Dummy.Font = New Font("Verdana", FontSize)

           Dim w As Integer = Convert.ToInt32(StripMenu_Cmb_FontSize.Width * _FontSizeMpf)
           Dim h As Integer = StripMenu_Cmb_FontSize.Height

           StripMenu_Cmb_FontSize.Size = New Size(w, h)

           w = Convert.ToInt32(StripMenu_Cmb_Culture.Width * _FontSizeMpf)
           StripMenu_Cmb_Culture.Size = New Size(w, h)

           w = Convert.ToInt32(StripMenu_Cmb_DateTimeFormat.Width * _FontSizeMpf)
           StripMenu_Cmb_DateTimeFormat.Size = New Size(w, h)
        End If

'   フォント変更
        Me.Font = New Font(_Lbl_FontName(FontNameId), FontSize)
    End Sub

    Private Sub HoverColorSet()
        For Each ctl As Control In Me.Controls
          If ctl.GetType.Name = "LabelSL" Then
             DirectCast(ctl, LabelSL).HoverColor = _ColorMain
          End If
        Next

        GroupBox1.HoverColor = _ColorSub

        Btn_Clear_1.FlatAppearance.MouseOverBackColor = _ColorMain
        Btn_Clear_2.FlatAppearance.MouseOverBackColor = _ColorMain

        Btn_Exec.HoverColor = _ColorMain
    End Sub

#End Region

#Region "-- Form"

    Private Sub StyleSetting()

        Dim state As Boolean = Application.RenderWithVisualStyles

     'スタイル設定
        If state Then
           If _OS_Type Then
              ListView.HeaderVisual = ListViewEx.Header.Gradation   '8, 10
           Else

           'Visual Style Setting
              ListView.HeaderVisual = ListViewEx.Header.Normal      'XP, 7
           End If
        Else

        'Classic Style Setting
           ListView.HeaderVisual = ListViewEx.Header.Gradation
        End If

        _onStyleChange = state
    End Sub

    Private Sub Lbl_Items_Update()
        If ListView.Items.Count Mod 13 = 0 Then
           Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)
           Lbl_Items.Update()
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub ModeLamp_Control()
        Lbl_Mode_Lamp_1.Enabled = (_Lsv_DisplayMode = 1)
        Lbl_Mode_Lamp_2.Enabled = (_Lsv_DisplayMode = 2)
    End Sub

    Private Overloads Sub Btn_Exec_Control()

        Dim f As Boolean
        Dim id As Integer

        If Cmb_Ext.SelectedIndex = 3 Then
           id = 20
        Else
           id = DirectCast(Cmb_File.SelectedValue, Integer)
        End If

        If ListView.Items.Count <> 0 Then
           Select Case id
             Case 20, 30
               f = ((Cmb_Text_1.Text.Length <> 0) AndAlso (Cmb_Text_2.Text.Length <> 0)) And _
                    (Cmb_Text_1.Text <> Cmb_Text_2.Text)

             Case 10, 11, 21, 31
               f = (Cmb_Text_1.Text.Length <> 0)
             Case Else : f = True
           End Select
        Else
           f = False
        End If

        Btn_Exec_Control(f)
    End Sub

    Private Overloads Sub Btn_Exec_Control(ByVal state As Boolean)
        If state <> Btn_Exec.Enabled Then
           If state Then
              Btn_Exec.Cursor = Cursors.Hand
           Else
              Btn_Exec.Cursor = Cursors.Default
           End If

           Btn_Exec.Enabled = state
        End If
    End Sub

#End Region

#Region "-- Button"

    Private Sub Open_File()

        If Lsv_LimitCheck(False, True) Then Return

        If Not Directory.Exists(_OpenDirectory) Then
           _OpenDirectory = My.Application.Info.DirectoryPath
        End If

        Dim ofd As New OpenFileDialog()

        ofd.AutoUpgradeEnabled = _OfdAutoUpgrade
        ofd.InitialDirectory = _OpenDirectory
        ofd.RestoreDirectory = True
        ofd.Title = "リストに登録するファイルを選択してください"
        ofd.Filter = _FileDialogFilter
        ofd.Multiselect = True

        If ofd.ShowDialog() = DialogResult.OK Then
           _OpenDirectory = Path.GetDirectoryName(ofd.FileName)

           Dim FocusedIndex As Integer = ListView.Items.Count

           ListView.BeginUpdate()

             If _Lsv_DupCheckFlg Then
                For Each Fpath As String In ofd.FileNames
                  If IsNothing(ListView.FindItemWithText(Fpath, False, 0, False)) Then
                     Lsv_Add(Fpath, 0)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If
                Next
             Else
                For Each Fpath As String In ofd.FileNames
                  Lsv_Add(Fpath, 0)
                  Lbl_Items_Update()

                  If Lsv_LimitCheck(True, False) Then Exit For
                Next
             End If

          'マーカークリア
             If ListView.Items.Count <> FocusedIndex AndAlso
                ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)

             Lsv_Display_Control(FocusedIndex)

           ListView.EndUpdate()
        End If

        ofd.Dispose()
    End Sub

    Private Sub Open_Folder()

        If Lsv_LimitCheck(False, True) Then Return

        If Not Directory.Exists(_OpenDirectory) Then
           _OpenDirectory = My.Application.Info.DirectoryPath
        End If

        Dim fbd As New FolderBrowserDialog

        fbd.RootFolder = Environment.SpecialFolder.Desktop
        fbd.SelectedPath = _OpenDirectory
        fbd.Description = "リストに登録するフォルダを選択してください。"
        fbd.ShowNewFolderButton = False

        If fbd.ShowDialog() = DialogResult.OK Then
           _OpenDirectory = fbd.SelectedPath

           Dim c As Integer = ListView.Items.Count
           Dim p As String = fbd.SelectedPath

           If IsNothing(Path.GetDirectoryName(p)) Then  'ルートチェック
              RootSelectDialog(p)
           Else
              _DropFileList = New String() {p}

              'ListView登録
                 Lsv_AddRange()

              'マーカークリア
                 If ListView.Items.Count <> c AndAlso
                    ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)
              End If
        End If

        fbd.Dispose()
    End Sub

#End Region

#Region "-- ToolTip"

    Private Function OnOffText(ByVal value As Integer) As String
        If value = 0 Then
           Return " OFF "
        Else
           Return " ON "
        End If
    End Function

    Private Sub Attr_ToolTip_Change()

        Const AttrText = vbCr & "  読み込むファイルの属性を設定します。"

        Dim fmt As String() = {"Read Only", "Hide", "System"}
        Dim tip As String

        For i As Integer = 0 To 2
          If _onToolTipActive Then
             tip = String.Format("{0} ({1})", fmt(i), OnOffText(_AttrType(i)))
          Else
             tip = String.Format("[ {0} ] -{1}-", fmt(i), OnOffText(_AttrType(i))) & AttrText
          End If

          ToolTip.SetToolTip(Me.Controls("Lbl_Attr_" & "RHS".Substring(i, 1)), tip)
        Next
    End Sub

    Private Sub ListMode_ToolTip_Change()

        Const openText = "を開きます。"

        Dim tip As String = _ToolTip_Text(0, _List_FileMode) & openText & vbCrLf & _
                            _ToolTip_Text(0, Convert.ToInt32(_List_FileMode = 0)) & openText

        ToolTip.SetToolTip(Btn_Open, tip)
        ToolTip.SetToolTip(Btn_TreeMode, _ToolTip_Text(1, _List_TreeMode))
        ToolTip.SetToolTip(Btn_FileMode, _ToolTip_Text(2, _List_FileMode))
        ToolTip.SetToolTip(Btn_ChckMode, _ToolTip_Text(3, _List_ChckMode))
    End Sub

#End Region

#Region "-- Context Menu"

''' <summary>
''' ファイルの拡張子をチェックします。(実行可能, DLL, インストーラー)
''' </summary>

    Private Function CheckExtension(ByVal TargetPath As String) As Boolean
        Const Extension = _
          ".exe.com.obj.bat.cmd.vbs.vbe.js.jse.wsf.wsh.msc.jar.hta.scr.cpl.lnk.url" & _
          ".acm.ocx.ax.cpa.cpi.cpx.dump.ebd.etl.iec.ime.lex.lrc.man.mod.mof.mui.nls.nt" & _
          ".olb.pro.resources.resx.rom.rs.rsp.sdi.sep.tbl.tlb.tsk.tsp.uce.vp" & _
          ".dll.sys.drv.rat" & _
          ".inf.msi."

        If Not File.Exists(TargetPath) Then
           Return False     'ディレクトリ、または、ファイルが存在しない
        End If

        Return (Extension.IndexOf(Path.GetExtension(TargetPath) & ".".ToLower) = -1)
    End Function

''' <summary>
''' サブフォームが開かれているかをチェックし、適切なフォームをアクティブにします。
''' </summary>

    Private Function CheckOpenForms() As Boolean

        Dim ofm As FormCollection = Application.OpenForms
        Dim state As Boolean = ((ofm.Count = 0) OrElse
                               ((ofm.Count = 1) AndAlso (ofm(0).Name = Me.Name)))

        If state Then
           Me.Activate()
        Else
           ofm(ofm.Count - 1).Activate()
        End If

        state = (_onDialogForm OrElse Not state)
        If state Then Beep()

        Return state
    End Function

    Private Sub StripMenu_E_Control()
        For i = 0 To StripMenu_Emphasis.DropDownItems.Count - 1
          DirectCast(StripMenu_Emphasis.DropDownItems(i), ToolStripMenuItem).CheckState = _
          DirectCast(2 * Convert.ToInt32(i = _UseEmphasisColor), CheckState)
        Next
    End Sub

#End Region

#Region "-- ListView 関連"

''' <summary>
''' ListView の表示に関係する値を設定します。
''' </summary>

    Private Sub Lsv_SetInfo()

        Dim sz As Single = _Lsv_FontSize(_Lsv_FontSize_Id) / _FontSizeMpf
        Dim lsv As New ListView

        lsv.View = View.Details
        lsv.Font = New Font(_Lsv_FontName, sz, FontStyle.Regular)
        lsv.Scrollable = False
        lsv.SmallImageList = Me.ImageList2

        lsv.Columns.Add("")
        lsv.Columns.Add("")
        lsv.Items.Add(New ListViewItem({"", "XXXXXXXX"}))
        lsv.Items(0).ImageIndex = 0

        lsv.Columns(0).Width = -1
        lsv.Columns(1).Width = -1

'   高さ (ラベル, 行)、幅 (パス, 日付)
        _Lsv_LabelHeight = lsv.Items(0).Bounds.Y
        _Lsv_Row_Height = lsv.Items(0).Bounds.Height                '最小値＝アイコンの高さ

        _Lsv_Col0_Width = lsv.SmallImageList.ImageSize.Width + 5    'アイコン表示最小幅
        _Lsv_Col2_Width = lsv.Columns(1).Width
        _Lsv_ColA_Width = _Lsv_Col0_Width + _Lsv_Col2_Width

'   枠のサイズ(3D境界線のサイズ:固定値)
        _Lsv_FrameSize_W = lsv.Width - lsv.ClientSize.Width
        _Lsv_FrameSize_H = lsv.Height - lsv.ClientSize.Height

'   未使用部分(枠,ラベル)のサイズ
        _Lsv_MinUnused_W = _Lsv_FrameSize_W
        _Lsv_MinUnused_H = _Lsv_FrameSize_H + _Lsv_LabelHeight

'   ScrollBarを含めた未使用部分のサイズ
'     ※スクロールバーの幅が変更された場合は、SystemColorsChanged で再設定
        _Lsv_MaxUnused_W = _Lsv_FrameSize_W + SystemInformation.VerticalScrollBarWidth
        _Lsv_MaxUnused_H = _Lsv_FrameSize_H + SystemInformation.HorizontalScrollBarHeight + _Lsv_LabelHeight

        lsv.Dispose()

     '※先頭行が表示されていない時に
     '  フォントサイズ, SmallImageListを変更すると表示が壊れる為の措置。 
        Dim Topidx As Integer

        If Not ListView.TopItem Is Nothing Then Topidx = ListView.TopItem.Index
        If ListView.Items.Count <> 0 Then ListView.TopItem = ListView.Items(0)
     '
        ListView.Font = New Font(_Lsv_FontName, sz, FontStyle.Regular)

     '※表示位置を元に戻す
        If Topidx <> 0 Then ListView.TopItem = ListView.Items(Topidx)
    End Sub

    Private Sub Lsv_Display_Control(ByVal cnt As Integer)
        If ListView.Items.Count = cnt Then
           Beep()
        Else
           _Lsv_DupCheckFlg = True

           With ListView
             .SelectedItems.Clear()
             .EnsureVisible(cnt)
             .Items(cnt).Focused = True
             .Items(cnt).Selected = True
             .Select()

             Lbl_Items.Text = String.Format(_LblItemsFormat, .Items.Count)
           End With

           Btn_Erase.Enabled = True
           Btn_Exec_Control()

           Lsv_DisplayMode_Control(False)
        End If
    End Sub

    Private Sub Lsv_DisplayMode_Control(ByVal update As Boolean)

        If _Lsv_DisplayMode = 0 Then Return

        ModeLamp_Control()

        RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

        If update Then ListView.BeginUpdate()

        With ListView
            If .Columns(2).Width <> _Lsv_Col2_Width Then
               .Columns(2).Width = _Lsv_Col2_Width
            End If

          If _Lsv_DisplayMode = 1 Then

         '水平ScrollBarが表示されないように調整する
             _Lsv_DisplayRows = (.Height - _Lsv_MinUnused_H) \ _Lsv_Row_Height    'ListView内で表示できる行数(水平Scroll Barなし)

             If _Lsv_DisplayRows >= .Items.Count Then                             '垂直ScrollBarなし
                .Columns(1).Width = .Width - _Lsv_MinUnused_W - _Lsv_ColA_Width
             Else
                .Columns(1).Width = .Width - _Lsv_MaxUnused_W - _Lsv_ColA_Width
             End If
             If .Columns(0).Width <> _Lsv_Col0_Width Then
                .Columns(0).Text = ""
                .Columns(0).Width = _Lsv_Col0_Width
             End If
          Else

          '日付部分を非表示(画面外)にする⇒水平ScrollBar表示
             _Lsv_DisplayRows = (.Height - _Lsv_MaxUnused_H) \ _Lsv_Row_Height  'ListView内で表示できる行数(水平Scroll Barあり)

             If _Lsv_DisplayRows >= .Items.Count Then                           '垂直ScrollBarなし
                .Columns(0).Width = .Width - _Lsv_MinUnused_W
             Else
                .Columns(0).Width = .Width - _Lsv_MaxUnused_W
             End If

                .Columns(0).Text = "パス"
             If .Columns(1).Width <> 0 Then
                .Columns(1).Width = 0
             End If
          End If
        End With

        If update Then ListView.EndUpdate()

        AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
    End Sub

    Private Function Lsv_LimitCheck(ByVal update As Boolean, ByVal state As Boolean) As Boolean

        If ListView.Items.Count >= _List_RegistLimit Then

           If update Then
              Lsv_DisplayMode_Control(False)
              ListView.EndUpdate()
           End If

           Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

           Dim msg1 As String
           Dim msg2 As String

           If state Then
              msg1 = "リスト登録数が最大値に達しています。"
              msg2 = "これ以上リストに登録できません。"
           Else
              msg1 = "リスト登録数が最大値に達しました。"
              msg2 = "リスト登録を中止します。"
           End If
           ShowWarningMessage(msg1, msg2, _Msg_Cap)

           If update Then ListView.BeginUpdate()

           Return True
        End If

        Return False
    End Function

    Private Sub Lsv_Add(ByVal Fpath As String, ByVal id As Integer)

        If Lsv_LimitCheck(False, True) Then Return

        Dim Fattr As Integer

        Try
          Dim fa As Integer = File.GetAttributes(Fpath)

          Fattr = fa And 7  'RHS以外をﾏｽｸ
          If (Fattr And Not _List_AttrType) <> 0 Then Return

          Fattr = fa

        Catch ex As Exception
          Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
            sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
          End Using

          MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
          Return
        End Try

        Dim LsvItem As New ListViewItem

        With LsvItem
          .Text = Fpath
          .ImageIndex = id
          .SubItems.Add(Path.GetFileName(Fpath))
          .SubItems.Add(Path.GetExtension(Fpath))

          .SubItems(1).ForeColor = GetAttributeColor(Fattr)

      'ジャンクション，シンボリックリンクを強調
         If (Fattr And FileAttributes.ReparsePoint) <> 0 Then
            .SubItems(0).ForeColor = GetAttributeColor(Fattr)
         End If
        End With
        ListView.Items.Add(LsvItem)
    End Sub

    Private Sub Lsv_AddRange()

        If Lsv_LimitCheck(False, True) Then Return

        Dim state As Boolean = (_DropFileList.Length <> 1)
        Dim break As Boolean
        Dim Fpath As String
        Dim Dpath As String

        Dim FocusedIndex As Integer = ListView.Items.Count
        Dim DirectorySearchOption As SearchOption = DirectCast(Convert.ToInt32(_List_TreeMode = 2), SearchOption)

        Me.Cursor = Cursors.WaitCursor

        ListView.BeginUpdate()

          If _Lsv_DupCheckFlg Then

             For Each Fpath In _DropFileList
               If File.Exists(Fpath) Then
                  If (_List_FileMode <> 1) AndAlso IsNothing(ListView.FindItemWithText(Fpath, False, 0, False)) Then
                     Lsv_Add(Fpath, 0)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If
               ElseIf Not Directory.Exists(Fpath) Then

                  If ShowFnfMessage(Fpath, state) <> DialogResult.Yes Then Exit For
               Else
                  If (_List_FileMode <> 0) AndAlso IsNothing(ListView.FindItemWithText(Fpath, False, 0, False)) Then
                     Lsv_Add(Fpath, 1)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If

                  If _List_TreeMode <> 0 Then
                     Try
                       Dim sFolder As String() = Directory.GetFileSystemEntries(Fpath, "*", DirectorySearchOption)
                       For Each Dpath In sFolder
                         If File.Exists(Dpath) Then
                            If (_List_FileMode <> 1) AndAlso IsNothing(ListView.FindItemWithText(Dpath, False, 0, False)) Then
                               Lsv_Add(Dpath, 0)
                               Lbl_Items_Update()
                            End If
                         Else
                            If (_List_FileMode <> 0) AndAlso IsNothing(ListView.FindItemWithText(Dpath, False, 0, False)) Then
                               Lsv_Add(Dpath, 1)
                               Lbl_Items_Update()
                            End If
                         End If

                      '登録制限値，[ESC] Key Check
                         If Lsv_LimitCheck(True, False) OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then
                            break = True
                            Exit For
                         End If
                       Next

                  '不正なジャンクション，シンボリックリンクは無視
                     Catch ex As DirectoryNotFoundException

                     Catch ex As Exception
                       Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
                         sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
                       End Using

                       MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
                     End Try
                  End If
               End If

            '登録制限値，[ESC] Key Check
               If break OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then Exit For

             Next
          Else

             For Each Fpath In _DropFileList
               If File.Exists(Fpath) Then
                  If _List_FileMode <> 1 Then
                     Lsv_Add(Fpath, 0)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If
               ElseIf Not Directory.Exists(Fpath) Then

                  If ShowFnfMessage(Fpath, state) <> DialogResult.Yes Then Exit For
               Else
                  If _List_FileMode <> 0 Then
                     Lsv_Add(Fpath, 1)           'Folder,All mode (Folder add)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If

                  If _List_TreeMode <> 0 Then
                     Try
                       Dim sFolder As String() = Directory.GetFileSystemEntries(Fpath, "*", DirectorySearchOption)
                       For Each Dpath In sFolder
                         If File.Exists(Dpath) Then
                            If _List_FileMode <> 1 Then
                               Lsv_Add(Dpath, 0)
                               Lbl_Items_Update()
                            End If
                         Else
                            If _List_FileMode <> 0 Then
                               Lsv_Add(Dpath, 1)
                               Lbl_Items_Update()
                            End If
                         End If

                      '登録制限値，[ESC] Key Check
                         If Lsv_LimitCheck(True, False) OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then
                            break = True
                            Exit For
                         End If
                       Next

                  '不正なジャンクション，シンボリックリンクは無視
                     Catch ex As DirectoryNotFoundException

                     Catch ex As Exception
                       Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
                         sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
                       End Using

                       MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
                     End Try
                  End If
               End If

            '登録制限値，[ESC] Key Check
               If break OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then Exit For

             Next
          End If
'
          Lsv_Display_Control(FocusedIndex)

        ListView.EndUpdate()
'
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub Lsv_Set(ByVal state As Integer, ByRef Lsv As ListViewItem, ByVal itm() As String)

        Dim i As Integer = 1 + Convert.ToInt32(state <> 0)

        If IsNothing(Lsv) Then
           Lsv = New ListViewItem
           With Lsv
             .Text = itm(i)
             .ImageIndex = Convert.ToInt32(itm(0))
             .SubItems.Add(Path.GetFileName(itm(i)))
             .SubItems.Add(Path.GetExtension(itm(i)))
           End With
           ListView.Items.Add(Lsv)
        Else
           Lsv.SubItems(0).Text = itm(i)
           Lsv.SubItems(1).Text = Path.GetFileName(itm(i))
           Lsv.SubItems(2).Text = Path.GetExtension(itm(i))
        End If
    End Sub

    Private Sub Lsv_Refresh()

        ListView.BeginUpdate()
        ListView.SelectedItems.Clear()

          For Each LsvItem As ListViewItem In ListView.Items

            Dim Fpath As String = LsvItem.SubItems(0).Text

            If Not File.Exists(Fpath) AndAlso Not Directory.Exists(Fpath) Then
               LsvItem.Remove()
            Else
               Try
                 Dim fa As Integer = File.GetAttributes(Fpath)

                 For i As Integer = 0 To ListView.Columns.Count - 1
                   LsvItem.SubItems(i).BackColor = SystemColors.Window
                 Next

                 LsvItem.SubItems(1).ForeColor = GetAttributeColor(fa)

               Catch ex As Exception
                 Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
                   sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
                 End Using

                 MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
               End Try
            End If
          Next

        Lsv_DisplayMode_Control(False)

        Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        If ListView.Items.Count <> 0 Then
           ListView.Items(0).Focused = True
           ListView.Items(0).Selected = True
        Else
           _Lsv_DupCheckFlg = False

           Btn_Erase.Enabled = False
        End If

        Btn_Exec_Control()

        ListView.EndUpdate()
    End Sub

    Private Sub Lsv_Delete(ByVal state As Boolean)

        Dim n As Integer = ListView.Items.Count
        Dim idx As Integer
        Dim LsvItem As ListViewItem

        With ListView
          Try
            idx = .FocusedItem.Index                        '最後に選択したフォーカス(カーソル)の位置
          Catch ex As Exception
            idx = -1
          End Try

          .BeginUpdate()

          If state Then
             For Each LsvItem In .Items                     '処理済で成功したアイテムを削除
               Select Case LsvItem.SubItems(1).BackColor
                 Case _BackColorSuccess, _BackColorUndo
                   LsvItem.Remove()
               End Select
             Next

             If n = .Items.Count Then                       '成功したアイテムがない
                .EndUpdate()

                Beep()
                Return
             End If
          Else
             For Each LsvItem In .SelectedItems
               LsvItem.Remove()
             Next
          End If

          If .Items.Count <> 0 Then
             If idx <> -1 Then
                .Items(.FocusedItem.Index).Selected = True  'フォーカス(カーソル)位置を設定
             End If
          Else
             _Lsv_DupCheckFlg = False
          End If

          Lsv_DisplayMode_Control(False)

          Lbl_Items.Text = String.Format(_LblItemsFormat, .Items.Count)
'
          Btn_Exec_Control()

          .EndUpdate()
        End With
    End Sub

    Private Sub Lsv_Clear()

        ListView.Items.Clear()

        Lsv_DisplayMode_Control(True)

        Lbl_Items.Text = String.Format(_LblItemsFormat, 0)

        Btn_Erase.Enabled = False
        Btn_Exec_Control()

        _Lsv_DupCheckFlg = False
    End Sub

    Private Sub Lsv_BackColor_Clear()
        For Each Lsv As ListViewItem In ListView.Items
          For i As Integer = 0 To Lsv.SubItems.Count - 1
            Lsv.SubItems(i).BackColor = Nothing
          Next
        Next
    End Sub

    Private Sub Lsv_BackColor_Set(ByRef Lsv As ListViewItem, ByVal Bcolor As Color)

        If _UseEmphasisColor = 0 Then Return

        For i As Integer = 0 To Lsv.SubItems.Count - 1
          Lsv.SubItems(i).BackColor = Bcolor
        Next
    End Sub

    Private Sub Lsv_DragColor_Control()
        If _Lsv_DragState Then
           _Lsv_DragState = False

           Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
        End If
    End Sub

    Private Sub Lsv_Dragcolor_Change(ByVal index As Integer, ByVal Bcolor As Color)

        If _UseEmphasisColor = 0 Then Return
        If Bcolor.IsEmpty Then Bcolor = SystemColors.Window

        Dim lsv As ListViewItem = ListView.Items(index)

        If lsv.SubItems(0).BackColor = Bcolor Then Return

        For i As Integer = 0 To lsv.SubItems.Count - 1
          lsv.SubItems(i).BackColor = Bcolor
        Next
    End Sub

    Private Sub ShowOnlyResults()
        Dim n As Integer

        ListView.BeginUpdate()
          For Each Lsv As ListViewItem In ListView.Items
            If Lsv.SubItems(1).BackColor = SystemColors.Window Then
               Lsv.Remove()
               n += 1
            End If
          Next

          If n = 0 Then
             Beep()
          Else
             Lsv_DisplayMode_Control(False)
          End If
        ListView.EndUpdate()

        Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        Dim f As Boolean = (ListView.Items.Count <> 0)

        Btn_Erase.Enabled = f
        Btn_Exec_Control()

        _Lsv_DupCheckFlg = f
    End Sub

#End Region

#Region "-- ComboBox 関連"

    Private Sub Cmb_ButtonControl()

     'DropDownボタン変更

     'ボタンの幅をスクロールバーの幅に対応させる為の処理
        Cmb_File.Font = Nothing
        Cmb_File.Font = Me.Font
        Cmb_Ext.Font = Nothing
        Cmb_Ext.Font = Me.Font
        Cmb_Text_1.Font = Nothing
        Cmb_Text_1.Font = Me.Font
        Cmb_Text_2.Font = Nothing
        Cmb_Text_2.Font = Me.Font

        Cmb_Num.Font = Nothing
        Cmb_Num.Font = _Cmb_Num_Font
        Cmb_Num.Width = _Cmb_Num_BaseWidth + SystemInformation.VerticalScrollBarWidth
    End Sub

    Private Sub Cmb_File_Initialize()

        Dim m(16) As MenuField

        m(0) = New MenuField(0, "変換なし")
        m(1) = New MenuField(1, "全て大文字")
        m(2) = New MenuField(2, "全て小文字")
        m(3) = New MenuField(4, "先頭のみ大文字")
        m(4) = New MenuField(3, "各単語の先頭を大文字")
        m(5) = New MenuField(10, "先頭に文字を挿入")
        m(6) = New MenuField(11, "末尾に文字を追加")
        m(7) = New MenuField(20, "指定文字の置換")
        m(8) = New MenuField(21, "指定文字の削除")
        m(9) = New MenuField(30, "指定文字(*)の置換")
        m(10) = New MenuField(31, "指定文字(*)の削除")
        m(11) = New MenuField(40, "先頭の指定文字数を残す")
        m(12) = New MenuField(41, "先頭の指定文字数を削除")
        m(13) = New MenuField(50, "末尾の指定文字数を残す")
        m(14) = New MenuField(51, "末尾の指定文字数を削除")
        m(15) = New MenuField(5, "作成日時に変更")
        m(16) = New MenuField(6, "更新日時に変更")

        Cmb_File.DisplayMember = "text"
        Cmb_File.ValueMember = "id"
        Cmb_File.DataSource = m

        Cmb_File.SelectedValue = 2
    End Sub

    Private Sub Cmb_File_ForeColorSet()

        Dim id As Integer = DirectCast(Cmb_File.SelectedValue, Integer)

        Select Case id
          Case Is < 5 : Cmb_File.ForeColor = SystemColors.WindowText
          Case 5 : Cmb_File.ForeColor = Color.ForestGreen
          Case 6 : Cmb_File.ForeColor = Color.MediumVioletRed
          Case Else
            If (id Mod 10) <> 0 Then
               Cmb_File.ForeColor = Color.Red
            Else
               Cmb_File.ForeColor = Color.Blue
            End If
        End Select
    End Sub

    Private Sub Cmb_Text_Control()

        Const TipTxt1 = "検索挿入追加置換削除残す"
        Const TipTxt2 = "文字列文字数"
        Const TipTxt3 = "する文字列を入力します。"
        Const TipTxt4 = "する文字数を選択します。"

'-------------------------------------------------------------------------------
'     Label & ToolTip Change
'-------------------------------------------------------------------------------
        Dim id, ix As Integer

        If Cmb_Ext.SelectedIndex = 3 Then
           id = 20                                          '置換ﾒﾆｭｰ互換
        Else
           id = DirectCast(Cmb_File.SelectedValue, Integer)
        End If

        Cmb_Text_1.Visible = ((id >= 10) AndAlso (id < 40))
        Cmb_Text_2.Visible = ((id = 20) OrElse (id = 30))   '置換ﾒﾆｭｰ
        Cmb_Num.Visible = (id >= 40)                        '指定文字数削除ﾒﾆｭｰ
        Label3.Visible = (id >= 10)
        Label4.Visible = ((id = 20) OrElse (id = 30))
'-------------------------------------------------------------------------------
       If id = 0 Then
         '変換なし選択時に値を初期化
           _ChangeCancelFlag = True

             Cmb_Text_1.Text = ""
             Cmb_Text_2.Text = ""
             Cmb_Num.SelectedIndex = 0

           _ChangeCancelFlag = False
        Else
           If id >= 10 Then
              Select Case id
                Case 10     '挿入
                  ix = 2
                Case 11     '追加
                  ix = 4
                Case 20, 30 '置換 (検索)
                  ix = 0
                Case 40, 50 '残す
                  ix = 10
                Case Else   '削除
                  ix = 8
              End Select

              Dim t1 As String = TipTxt1.Substring(ix, 2)
              Dim t2 As String = TipTxt1.Substring(6, 2)

              If id < 40 Then
                 ix = 0

                 ToolTip.SetToolTip(Cmb_Text_1, t1 & TipTxt3)
                 ToolTip.SetToolTip(Cmb_Text_2, t2 & TipTxt3)
              Else
                 ix = 3

                 If (id Mod 10) = 1 Then
                    ToolTip.SetToolTip(Cmb_Num, t1 & TipTxt4)
                 Else
                    ToolTip.SetToolTip(Cmb_Num, t1 & TipTxt4.Substring(2, TipTxt4.Length - 2))
                 End If
              End If

              Label3.Text = t1 & TipTxt2.Substring(ix, 3)
           End If
        End If
    End Sub

    Private Sub Cmb_Text_SelectionReset()
'---------------------------------------------------------------------------------------------
'     ComboBoxに値が入力されている時にWidth値を変更(ReSize、最大化、元のサイズに戻す）すると、
'     値が無条件に選択される為(Bug?)、選択を強制解除
'---------------------------------------------------------------------------------------------
        If Not Cmb_Text_1.Focused Then Cmb_Text_1.SelectionLength = 0
        If Not Cmb_Text_2.Focused Then Cmb_Text_2.SelectionLength = 0
    End Sub

#End Region

#End Region

#Region "// ---- Form & Timer -------------------------------------------------"

    Private Sub DelaySettingTimer_Tick(sender As Object, e As EventArgs) Handles DelaySettingTimer.Tick

        DelaySettingTimer.Stop()

        Lsv_DisplayMode_Control(True)

        If _onDelaySetting Then
           _onDelaySetting = False

           For n As Double = 0.1R To 1.0R Step 0.2R
             Me.Opacity = n
             Thread.Sleep(10)
           Next
           Me.Opacity = 1.0R
        End If
    End Sub

    Private Sub Me_Load(sender As Object, e As EventArgs) Handles Me.Load

      Me.SuspendLayout()

      Me.Text = _App_Name
      Notify_Icon.Text = _App_Name

'   OS が ８以降かを調べる
        _OS_Type = GetOsType()

'   開始ウィンドウサイズをセット
        _StartWindow_Width = Me.Width
        _StartWindowHeight = Me.Height

'   基準となるスクロール バーのサイズをセット
        _BeforeScrollBarWidth = SystemInformation.VerticalScrollBarWidth
'-------------------------------------------------------------------------------
'   設定読み込み、
        iniLoad()
'-------------------------------------------------------------------------------
'   フォントチェック(変更)
        FontSetting()
'-------------------------------------------------------------------------------
'   Style Setting
        StyleSetting()
'-------------------------------------------------------------------------------
'   ComboBox (Cmb_Num) のフォント，基準となるサイズをセット
        _Cmb_Num_Font = Cmb_Num.Font
        _Cmb_Num_BaseWidth = Cmb_Num.Width - 16 '16: デザイン時のスクロールバーの幅

'   DropDownボタン変更
        Cmb_ButtonControl()
'-------------------------------------------------------------------------------
'   画面制御値設定
        _MinClientSize = Me.ClientSize

        Me.MinimumSize = Me.Size

'   フォームのサイズ変更
        If _StartWindow_Width < Me.MinimumSize.Width Then
           Me.Width = Me.MinimumSize.Width
        Else
           Me.Width = _StartWindow_Width
        End If

        If _StartWindowHeight < Me.MinimumSize.Height Then
           Me.Height = Me.MinimumSize.Height
        Else
           Me.Height = _StartWindowHeight
        End If
'-------------------------------------------------------------------------------
'   Cmb_File・初期化
        Cmb_File_Initialize()
'-------------------------------------------------------------------------------
'   Cmb_Num・初期化 & 処理区分・初期値設定
        For i As Integer = 1 To _MaxNumberCount
          Cmb_Num.Items.Add(String.Format("{0,4:#}", i))
        Next

        Cmb_Ext.SelectedIndex = 2
        Cmb_Num.SelectedIndex = 0
'-------------------------------------------------------------------------------
'   ListView 表示制御値を設定
        Lsv_SetInfo()

'   Label 表示補正を停止
        Lbl_Items.LockedCorrection = True
        Lbl_Items.Text = String.Format(_LblItemsFormat, 0)

'   ToolTip
        Me.ToolTip.SetToolTip(GroupBox1.Label, "右クリックで" & vbCr & "動作確認 (設定) メニューが開きます。")
'-------------------------------------------------------------------------------
      Me.ResumeLayout()
'-------------------------------------------------------------------------------
'   コマンドライン、送る チェック
'    (リストが登録された場合は、ListView.ColumnWidthChanged イベントが登録される)
        CmdSendToLoad()
'-------------------------------------------------------------------------------
      Me.SuspendLayout()
'-------------------------------------------------------------------------------
'   ダブルバッファ on
        If _DoubleBuffering Then ListView.DoubleBuffering = True
'-------------------------------------------------------------------------------
'   ボタン関連
        Btn_TreeMode.ImageIndex = _List_TreeMode + 4
        Btn_FileMode.ImageIndex = _List_FileMode + 1
        Btn_ChckMode.ImageIndex = _List_ChckMode + 9

        HoverColorSet()

        ListMode_ToolTip_Change()

'   ファイル属性表示
        Lbl_Attr_R.ForeColor = Color.FromName(_AttrColorName(_AttrType(0)))
        Lbl_Attr_H.ForeColor = Color.FromName(_AttrColorName(_AttrType(1)))
        Lbl_Attr_S.ForeColor = Color.FromName(_AttrColorName(_AttrType(2)))

        Attr_ToolTip_Change()

'   強調背景色関連
        SetEmphasisBackColor()
        StripMenu_E_Control()

'   フォントサイズ (ContextMenu_Setting)

        Const DisplayFormat = "     0.00 pt"

        Dim t As String
        Dim l As Integer = DisplayFormat.Length

        For i As Integer = 0 To _Lsv_FontSize.Count - 1
          If _Lsv_FontSize(i) >= 10.0F Then
             l = DisplayFormat.Length - 1
          End If

          t = _Lsv_FontSize(i).ToString(DisplayFormat)
          t = t.Substring(t.Length - l)

          StripMenu_Cmb_FontSize.Items.Add(t)
        Next
        StripMenu_Cmb_FontSize.SelectedIndex = _Lsv_FontSize_Id

'   カルチャ (ContextMenu_Setting)
      Try
        Dim ci As New Globalization.CultureInfo(_UseCultureInfo)

      Catch ex As Exception
        StripMenu_Cmb_Culture.SelectedIndex = 0
      End Try

        StripMenu_Cmb_Culture.SelectedItem = _UseCultureInfo
          If StripMenu_Cmb_Culture.SelectedItem Is Nothing Then
             StripMenu_Cmb_Culture.Items.Insert(0, _UseCultureInfo)
             StripMenu_Cmb_Culture.SelectedIndex = 0
          End If

'   日付ファイル名の書式 (ContextMenu_Setting)
        StripMenu_Cmb_DateTimeFormat.SelectedItem = _DateNameFormat
          If StripMenu_Cmb_DateTimeFormat.SelectedItem Is Nothing Then
             StripMenu_Cmb_DateTimeFormat.Items.Insert(0, _DateNameFormat)
             StripMenu_Cmb_DateTimeFormat.SelectedIndex = 0
          End If

'   画面制御・設定
        If ListView.Items.Count <> 0 Then
           _Lsv_DupCheckFlg = True

           ListView.Items(0).Selected = True
           ListView.Select()
           Btn_Erase.Enabled = True
           Btn_Exec_Control()
        End If

     'ListView.ColumnWidthChanged イベント登録
        Lsv_DisplayMode_Control(True)

      Me.ResumeLayout()

        AddHandler Me.Resize, AddressOf Me_Resize

        AddHandler Cmb_File.SelectedIndexChanged, AddressOf Cmb_File_SelectedIndexChanged
        AddHandler Cmb_Ext.SelectedIndexChanged, AddressOf Cmb_Ext_SelectedIndexChanged

        AddHandler StripMenu_ToolTip.CheckedChanged, AddressOf StripMenu_ToolTip_CheckedChanged
        AddHandler StripMenu_TaskBar.CheckedChanged, AddressOf StripMenu_TaskBar_CheckedChanged
    End Sub

    Private Sub Me_Shown(sender As Object, e As EventArgs) Handles Me.Shown

        Const Msg_Log = "ログファイルが既定サイズを超えています。" & vbCrLf & vbCrLf &
                        "ログファイルを初期化します。"

        Me.Update()
        For n As Single = 0.1 To 1 Step 0.2
          Me.Opacity = n
          Thread.Sleep(10)
        Next
        Me.Opacity = 1.0R

     'Log File Clear
        Dim fi As New FileInfo(_Log_File)

        If fi.Exists Then
           If fi.Length > _LogFileMaxSize Then

              Dim MsgBox As New MessageBoxEx
              MsgBox.SetButtonText(1, "初期化")
              MsgBox.SetButtonText(2, "整 理")

              Dim f As Integer = MsgBox.Dialog(Me, Msg_Log, _Msg_Cap,
                                               MessageBoxEx.Buttons.CenterYesNoCancel,
                                               MessageBoxIcon.Warning, , , _MessageBoxSound)
              MsgBox.Dispose()

              If f = DialogResult.Yes Then
                 Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
                 End Using

              ElseIf f = DialogResult.No Then

                 Dim cnt As Integer
                 Dim buf As String

                 If File.Exists(_TempFile) Then File.Delete(_TempFile)
                 File.Move(_Log_File, _TempFile)

                 Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
                 Using sr As New StreamReader(_TempFile, Encoding.UTF8)

                   Do
                     buf = sr.ReadLine()
                     If String.IsNullOrEmpty(buf) Then cnt += 1
                     sw.WriteLine(buf)
                   Loop Until (cnt = _LogFileLeaveCount) OrElse (sr.Peek = -1)

                 End Using
                 End Using

                 File.Delete(_TempFile)
              End If

              Me.Update()
              WinMessage.Clear(Me)
           End If
        End If
    End Sub

    Private Sub Me_FormClosed(sender As Object, e As FormClosedEventArgs) Handles Me.FormClosed
        iniSave()

'     Opacity 変更時、StyleChanged が発生する為 ハンドルを削除
'       RemoveHandler Me.StyleChanged, AddressOf Me_StyleChanged

        For n As Single = 1 To 0 Step -0.2
          Me.Opacity = n
          Thread.Sleep(10)
        Next
        Me.Opacity = 0.0R
    End Sub

    Private Sub Me_SystemColorsChanged(sender As Object, e As EventArgs) Handles Me.SystemColorsChanged

        Dim ScrollBarWidth As Integer = SystemInformation.VerticalScrollBarWidth

     'Style Change
        If _onStyleChange <> Application.RenderWithVisualStyles Then

           StyleSetting()
           Lsv_SetInfo()
        Else

        'ListView の ScrollBarを含めた未使用部分のサイズを再設定
           _Lsv_MaxUnused_W = _Lsv_FrameSize_W + ScrollBarWidth
           _Lsv_MaxUnused_H = _Lsv_FrameSize_H + ScrollBarWidth + _Lsv_LabelHeight
        End If

     'DropDownボタン変更
        If _BeforeScrollBarWidth <> ScrollBarWidth Then
           Cmb_ButtonControl()
        End If

     'スクロールバー（垂直）が表示されている場合に、スクロールバーの幅を小さく変更すると
     'スクロールバー（水平）が残ったままになる不具合に対する措置

        If _Lsv_DisplayMode <> 0 Then
           If (_BeforeScrollBarWidth > ScrollBarWidth) AndAlso _
              (ListView.ClientSize.Width < ListView.Width - _Lsv_FrameSize_W) Then

              DelaySettingTimer.Interval = 500
              DelaySettingTimer.Start()
           Else
              Lsv_DisplayMode_Control(True)
           End If
        End If

     '基準となるスクロール バーのサイズをセット
        _BeforeScrollBarWidth = ScrollBarWidth

     '強調背景色セット
        SetEmphasisBackColor()
    End Sub

    Private Sub Me_ResizeBegin(sender As Object, e As EventArgs) Handles Me.ResizeBegin

     'リサイズ中はリスト表示は更新せず、リサイズ確定後にリスト表示を更新する為、Resize イベントを停止
        RemoveHandler Me.Resize, AddressOf Me_Resize
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs) 'Handles Me.Resize

'     最小化
        If Me.WindowState = FormWindowState.Minimized Then
           _WindowMinimized = True

           Me.Opacity = 0.0R
           Return
        End If

'     最小化⇒元に戻された
        If _WindowMinimized Then
           _WindowMinimized = False

           _onDelaySetting = True

           DelaySettingTimer.Interval = 100
           DelaySettingTimer.Start()
           Return
        End If

'     最大化
        If Me.WindowState = FormWindowState.Maximized Then
           _WindowMaximized = True

           Lsv_DisplayMode_Control(True)
           Return
        End If

'     最大化⇒元に戻された
        If _WindowMaximized Then
           _WindowMaximized = False

           Lsv_DisplayMode_Control(True)
           Return
        End If
'---------------------------------------------------------------------------------------------
'     画面のプロパティ(アクティブタイトルバー、ウィンドウの境界・間隔)が変更された時の処理
'     （最小ウィンドウサイズの変更のみ実施）
'---------------------------------------------------------------------------------------------
        Dim fw As Integer = SystemInformation.FrameBorderSize.Width * 2
        Dim fh As Integer = SystemInformation.CaptionHeight + fw
        Dim mw As Integer = _MinClientSize.Width + fw
        Dim mh As Integer = _MinClientSize.Height + fh

        If Me.Size = Me.MinimumSize Then

           If mw < Me.MinimumSize.Width OrElse mh < Me.MinimumSize.Height Then

              RemoveHandler Me.Resize, AddressOf Me_Resize

                Me.MinimumSize = New Size(mw, mh)
                Me.Size = Me.MinimumSize

              AddHandler Me.Resize, AddressOf Me_Resize
           End If
        Else
           If mw <> Me.MinimumSize.Width OrElse mh <> Me.MinimumSize.Height Then

              Me.MinimumSize = New Size(mw, mh)
           End If
        End If

     'リスト表示を更新
        Lsv_DisplayMode_Control(True)
    End Sub

    Private Sub Me_ResizeEnd(sender As Object, e As EventArgs) Handles Me.ResizeEnd

     'リサイズ確定後にリスト表示を更新
        Lsv_DisplayMode_Control(True)

     'ComboBox バグ対策 (Width が変った時に選択状態になるのを回避)
        If Not Cmb_Text_1.Focused Then
           If Cmb_Text_1.SelectionLength <> 0 Then
              Cmb_Text_1.SelectionLength = 0
           End If
        End If

        If Not Cmb_Text_2.Focused Then
           If Cmb_Text_2.SelectionLength <> 0 Then
              Cmb_Text_2.SelectionLength = 0
           End If
        End If

     'Resize イベントを再開
        AddHandler Me.Resize, AddressOf Me_Resize
    End Sub

    Private Sub Me_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown

        If e.KeyCode = Keys.F1 Then
           e.Handled = True

           ShowSettings(e.Alt)
           Return
        End If

        If e.KeyData = Keys.Enter Then
           e.Handled = True

           If Btn_Exec.Enabled Then
              Btn_Exec.Select()
              Btn_Exec.PerformClick()
           Else
              Beep()
           End If
           Return
        End If

        If e.Control Then
           If e.KeyCode = Keys.H Then   '[ctrl]+H
              e.Handled = True
              e.SuppressKeyPress = True 'KeyPressイベントをキャンセル

              _UseHideSelection = _UseHideSelection Xor True
              ListView.HideSelection = _UseHideSelection
           End If
           Return
        End If

        Dim ShortCut_ID As Integer = Array.IndexOf(_Cmb_File_ShortCutKeys, e.KeyCode)

        If ShortCut_ID <> -1 Then

           If Me.ActiveControl Is Nothing Then
              Beep()
              Return

           ElseIf Me.ActiveControl.Name = "ListView" Then
              e.SuppressKeyPress = True

             'Cmb_File メニューの色を変更
             '(SelectedIndexChanged で、色を変更出来る様に _OnMouseWheelFlag をセットする。)

              _OnMouseWheelFlag = True

           ElseIf Me.ActiveControl.Name = "Cmb_File" Then
              If Cmb_File.DroppedDown Then
                 Cmb_File.DroppedDown = False
              End If
           Else
              Return
           End If

           If ShortCut_ID <= 6 Then
           '変換なし(0) ～ 先頭のみ大文字(4), 作成日時に変更(5), 更新日時に変更(6)

              Cmb_File.SelectedValue = ShortCut_ID
              Return
           End If

           Select Case e.KeyCode
             Case Keys.R : Cmb_File.SelectedValue = 20 '指定文字の置換
             Case Keys.D : Cmb_File.SelectedValue = 21 '指定文字の削除
             Case Keys.T : Cmb_File.SelectedValue = 10 '先頭に文字を挿入
             Case Keys.B : Cmb_File.SelectedValue = 11 '末尾に文字を追加
             Case Keys.H : Cmb_File.SelectedValue = 41 '先頭の指定文字数を削除
             Case Keys.E : Cmb_File.SelectedValue = 51 '末尾の指定文字数を削除
           End Select

           If ShortCut_ID <= 10 Then
              Cmb_Text_1.Select()
           Else
              Cmb_Num.Select()
           End If
        End If
    End Sub

    Private Sub Me_DragEnter(sender As Object, e As DragEventArgs) Handles Me.DragEnter
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
           e.Effect = DragDropEffects.Copy
        Else
           e.Effect = DragDropEffects.None
        End If
    End Sub

    Private Sub Me_DragDrop(sender As Object, e As DragEventArgs) Handles Me.DragDrop

        Me.Activate()
        Me.Update()

        Dim l As New List(Of String)

        _DropFileList = DirectCast(e.Data.GetData(DataFormats.FileDrop, False), String())

'     Dropテーブル ルートチェック
        For Each p As String In _DropFileList
          If IsNothing(Path.GetDirectoryName(p)) Then   'ルートチェック
             RootSelectDialog(p)
          Else
             l.Add(p)
          End If
        Next

        If l.Count <> 0 Then
'        Dropテーブル再構成
           If _DropFileList.Length <> l.Count Then
              _DropFileList = l.ToArray
           End If

        'ListView登録
           Lsv_AddRange()

        'マーカークリア
           If ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)
        End If
    End Sub

#End Region

#Region "// ListView"

    <DebuggerStepThrough()>
    Private Sub ListView_ColumnWidthChanged(sender As Object, e As ColumnWidthChangedEventArgs)

         If _Lsv_DisplayMode <> 0 Then
           _Lsv_DisplayMode = 0 : ModeLamp_Control()
         End If

         If e.ColumnIndex = 0 Then
            RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

              If ListView.Columns(0).Width <= _Lsv_Col0_Width Then
                 ListView.Columns(0).Width = _Lsv_Col0_Width
                 ListView.Columns(0).Text = ""
              Else
                 ListView.Columns(0).Text = "パス"
              End If

            AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
         End If
    End Sub

    Private Sub ListView_MouseDown(sender As Object, e As MouseEventArgs) Handles ListView.MouseDown
        With ListView
          If .Items.Count <> 0 Then
             If e.Button = MouseButtons.Middle Then
                If .SelectedItems.Count <> 0 Then
                   If .SelectedItems.Count = .Items.Count Then
                      Lsv_Clear()
                   Else
                      Lsv_Delete(False)
                   End If
                End If
             End If
         End If
        End With
    End Sub

    Private Sub ListView_MouseUp(sender As Object, e As MouseEventArgs) Handles ListView.MouseUp

     'ContextMenu_ListView_Opening で表示内容を変更すると、表示座標がずれる為の措置
        If e.Button = MouseButtons.Right Then

           Dim id As Integer = DirectCast(Cmb_File.SelectedValue, Integer)
           Dim c0 As Boolean = (ListView.Items.Count > 0)
           Dim s0, s1, fc, bc, ec, nc As Boolean

           If c0 Then
              Dim sc As Integer = ListView.SelectedItems.Count

              s0 = (sc > 0)

              If s0 Then
                 s1 = (sc = 1)

                 Dim fl As String = ListView.SelectedItems.Item(0).Text

                 fc = (sc > 1) OrElse (s1 AndAlso PathExists(fl))
                 nc = s1 AndAlso ((Cmb_Ext.SelectedIndex = 3) OrElse ((id >= 10) AndAlso (id <= 31)))
                 ec = (s1 AndAlso File.Exists(fl)) AndAlso CheckExtension(fl)
                 bc = (s0 AndAlso PathExists(fl)) AndAlso (Btn_Exec.Enabled)
              End If
           End If

           StripMenu_SelectConversion.Visible = bc
           StripMenu_NameAcquisition.Visible = nc
           StripMenu_Open.Visible = ec
           StripMenu_SelectAllItem.Visible = c0
           StripMenu_ToggleSelection.Visible = c0
           StripMenu_DeleteItem.Visible = s0
           StripMenu_AttributeChange.Visible = fc

           StripSeparator_L1.Visible = bc
           StripSeparator_L2.Visible = nc
           StripSeparator_L3.Visible = ec
           StripSeparator_L4.Visible = c0
           StripSeparator_L5.Visible = fc
        End If
    End Sub

    Private Sub ListView_KeyDown(sender As Object, e As KeyEventArgs) Handles ListView.KeyDown
        With ListView
          If .Items.Count = 0 OrElse
             e.KeyCode = Keys.Menu Then         'Altキーを無効
             e.Handled = True
             e.SuppressKeyPress = True
             Return
          End If

          Select Case e.KeyData
            Case Keys.Delete
              If .SelectedItems.Count <> 0 Then
                 If .SelectedItems.Count = .Items.Count Then
                    Lsv_Clear()
                 Else
                    Lsv_Delete(False)
                 End If
              End If

            Case DirectCast(131137, Keys)       '[ctrl]+A
             .BeginUpdate()

              For Each LsvItem As ListViewItem In .Items
                LsvItem.Selected = True
              Next

             .EndUpdate()

            Case DirectCast(131110, Keys), DirectCast(131112, Keys)     '[ctrl]+↑,↓
              If Not _UseCtrlUpDown Then
                 e.SuppressKeyPress = True                              'KeyPressイベントをキャンセル
                 Return
              End If

            Case Keys.F2                        '[F2]
              FileNameChange()

            Case Keys.F3                        '[F3]
              FileAttributesChange()

            Case Keys.F5                        '[F5]
              Lsv_Refresh()

            Case Keys.F7                        '[F7]

              Dim sb As New StringBuilder
              For Each LsvItem As ListViewItem In ListView.Items
                sb.Append(LsvItem.SubItems(0).Text & vbCrLf)
              Next

              SetClipboardText(sb.ToString, "ファイルパスのリスト")

              sb.Clear()

            Case Keys.F8                        '[F8]

              Dim sb As New StringBuilder
              For Each LsvItem As ListViewItem In ListView.Items
                sb.Append(LsvItem.SubItems(1).Text & vbCrLf)
              Next

              SetClipboardText(sb.ToString, "ファイル名のリスト")

              sb.Clear()

            Case DirectCast(131191, Keys)       '[Ctrl]+[F8]

              Dim sb As New StringBuilder
              For Each LsvItem As ListViewItem In ListView.Items
                sb.Append(Path.GetFileNameWithoutExtension(LsvItem.SubItems(0).Text) & vbCrLf)
              Next

              SetClipboardText(sb.ToString, "ファイル名のリスト")

              sb.Clear()

            Case DirectCast(65655, Keys)        '[Shift]+[F8]

              Const NotExistText = "** Not Found **"

              Dim sb As New StringBuilder
              Dim DateText As String
              Dim SizeText As String
              Dim Dir_Text As String

              For Each LsvItem As ListViewItem In ListView.Items
                With LsvItem
                  If .ImageIndex = 0 Then
                     Dim fi As New FileInfo(.SubItems(0).Text)

                     If fi.Exists Then

                        DateText = fi.LastWriteTime.ToString(_DateTimeFormat)
                        SizeText = String.Format("{0,15:#,0}", fi.Length)

                        If (fi.Attributes And FileAttributes.ReparsePoint) = 0 Then
                           Dir_Text = StrDup(2, vbTab)
                        Else
                           Dir_Text = "   <SYMLINK>" & vbTab
                        End If
                     Else
                        DateText = StrDup(19, " ")
                        SizeText = NotExistText
                        Dir_Text = StrDup(2, vbTab)
                     End If
                  Else
                     Dim di As New DirectoryInfo(.SubItems(0).Text)

                     If di.Exists Then

                        DateText = di.LastWriteTime.ToString(_DateTimeFormat)
                        SizeText = String.Format("{0,15:#,0}", GetDirectorySize(di))

                        If (di.Attributes And FileAttributes.ReparsePoint) = 0 Then
                           Dir_Text = "       <DIR>" & vbTab
                        Else
                           Dir_Text = "  <SYMLINKD>" & vbTab
                        End If
                     Else
                        DateText = StrDup(19, " ")
                        SizeText = NotExistText
                        Dir_Text = StrDup(2, vbTab)
                     End If
                  End If

                  sb.Append(DateText & Dir_Text & SizeText & vbTab & .SubItems(1).Text & vbCrLf)
                End With
              Next

              SetClipboardText(sb.ToString, "ファイル名のリスト（詳細）")

              sb.Clear()

            Case DirectCast(131154, Keys)       '[Ctrl]+R
              ShowOnlyResults()

            Case DirectCast(131149, Keys)       '[Ctrl]+M
              _ClipboardMonitor = _ClipboardMonitor Xor True
          End Select
        End With
    End Sub

    Private Sub ListView_DoubleClick(sender As Object, e As EventArgs) Handles ListView.DoubleClick
        FileNameChange()
    End Sub

    Private Sub ListView_ItemDrag(sender As Object, e As ItemDragEventArgs) Handles ListView.ItemDrag

        Lsv_BackColor_Clear()

        Dim DropList As New List(Of String)

        For Each item As ListViewItem In ListView.SelectedItems

          If File.Exists(item.Text) OrElse Directory.Exists(item.Text) Then
             DropList.Add(item.Text)
          Else
          '選択されたファイルがない場合、選択状態を解除
             item.Selected = False

             Lsv_BackColor_Set(item, _BackColorFailure)
          End If
        Next

        If DropList.Count = 0 Then
           ShowWarningMessage("Drop 可能なアイテムがありません。", "", _Msg_Cap)
           Return
        End If

     'リスト表示行数セット
        _Lsv_DisplayRows = (ListView.ClientRectangle.Height - _Lsv_LabelHeight) \ _Lsv_Row_Height
        _Lsv_InnerDrop = False

        Me.AllowDrop = False
        ListView.AllowDrop = True

        Dim DataObj As New DataObject(DataFormats.FileDrop, DropList.ToArray)
        Dim Effects As DragDropEffects

        Effects = ListView.DoDragDrop(DataObj, DragDropEffects.Copy Or DragDropEffects.Move)

    'Effects が正しい値を返さない
    ' None:      Drag Cancel, File Move (Exploler)
    ' Copy:      File Copy (Exploler)
    ' Move:      Move (Vix, XnView), Listview Item Move
    ' Copy+Move: File Copy (Exploler++), File Move (Exploler++)

        If Not (_Lsv_DragCancel Or _Lsv_InnerDrop) Then
           If (Effects = DragDropEffects.None) OrElse
              ((Effects And DragDropEffects.Move) = DragDropEffects.Move) Then

          'ファイル操作の遅延を考慮し、一時処理停止
             Thread.Sleep(_DropDelay)

             With ListView
              .BeginUpdate()

              For Each item As ListViewItem In .SelectedItems
                If Not File.Exists(item.Text) Then
                   item.Remove()
                End If
              Next
              If .Items.Count = 0 Then _Lsv_DupCheckFlg = False

              Lsv_DisplayMode_Control(False)
              Lbl_Items.Text = String.Format(_LblItemsFormat, .Items.Count)
              Btn_Exec_Control()

              .EndUpdate()
             End With
           End If
        End If

If _Lsv_DragState Then
   MsgBox("Error (DragState: True)")

   Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
     sw.WriteLine(Now & vbTab & "ItemDrag (_Lsv_DragState: True)")
   End Using
End If

        ListView.AllowDrop = False
        Me.AllowDrop = True
    End Sub

    Private Sub ListView_QueryContinueDrag(sender As Object, e As QueryContinueDragEventArgs) Handles ListView.QueryContinueDrag

'     e.KeyState の値
'       0:何も押されていない＝マウスのボタンが離された (e.Action = DragAction.Drop)
'       1:マウスの左ボタン
'       2:マウスの右ボタン  
'       4:Shift キー, 8:CTRL キー, 16:マウスの中央ボタン, 32:Alt キー

'     マウスのボタンが離された時、
'       Move:     リストビューの範囲外の場合、ドラッグをキャンセル
'       FileDrop: リストビューの範囲内  〃

        If e.Action = DragAction.Drop Then

           Dim p As Point = Me.PointToClient(Cursor.Position)

        'マウスの右ボタンが Form 内 かつ ListView 外で離された場合、ドラッグをキャンセル
           If Me.ClientRectangle.Contains(p) Then
              p = ListView.PointToClient(Cursor.Position)
              If Not ListView.ClientRectangle.Contains(p) Then
                 e.Action = DragAction.Cancel
              Else
                 _Lsv_InnerDrop = True      'ListView 内でDrop

                 Lsv_DragColor_Control()    'ドラッグ位置の表示を消去
              End If
           End If

'     マウスの右ボタンが押された場合、ドラッグをキャンセル
        Else
           If (e.KeyState And 2) = 2 Then
              e.Action = DragAction.Cancel
           End If
        End If

     'ドラッグキャンセル情報をセット
        _Lsv_DragCancel = (e.Action = DragAction.Cancel)
    End Sub

    Private Sub ListView_DragEnter(sender As Object, e As DragEventArgs) Handles ListView.DragEnter
        e.Effect = DragDropEffects.Move
    End Sub

    Private Sub ListView_DragLeave(sender As Object, e As EventArgs) Handles ListView.DragLeave
        Lsv_DragColor_Control()
    End Sub

    Private Sub ListView_DragOver(sender As Object, e As DragEventArgs) Handles ListView.DragOver

        Const ScrollWaitTime = 200    'スクロール速度制限値 (ms)

        Dim CurPos As Point = ListView.PointToClient(New Point(e.X, e.Y))
        Dim CurIdx As Integer
        Dim ScWait As Boolean

        Dim scroll As Boolean

'     ListViewの表示行数(完全に表示されている)より登録数が多い場合のみ処理
        If _Lsv_DisplayRows < ListView.Items.Count Then

        '↑Scroll

           If CurPos.Y < _Lsv_LabelHeight Then

              CurIdx = ListView.TopItem.Index

           'マウスを速く移動した際に強調表示行が残ったままになるのを回避
              If CurIdx <> _Lsv_PrevIndex Then
                 Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
              End If

              If CurIdx > 0 Then
                 scroll = True
                 ScWait = (CurPos.Y > (_Lsv_LabelHeight \ 2))

                 _Lsv_PrevIndex = CurIdx - 1
              End If

        '↓Scroll

           ElseIf CurPos.Y > (ListView.ClientRectangle.Height - _Lsv_Row_Height) Then

              CurIdx = ListView.TopItem.Index + (_Lsv_DisplayRows - 1)

           'マウスを速く移動した際に強調表示行が残ったままになるのを回避
              If CurIdx <> _Lsv_PrevIndex Then
                 Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
              End If

              If CurIdx < ListView.Items.Count - 1 Then
                 scroll = True
                 ScWait = (CurPos.Y < ListView.ClientRectangle.Height - (_Lsv_Row_Height \ 2))

                 _Lsv_PrevIndex = CurIdx + 1
              End If
           End If

        'ドラッグ位置の表示 (スクロール)

           If scroll Then
              ListView.BeginUpdate()
                _Lsv_DragState = True

                Lsv_Dragcolor_Change(CurIdx, Nothing)
                ListView.EnsureVisible(_Lsv_PrevIndex)
                Lsv_Dragcolor_Change(_Lsv_PrevIndex, _BackColorSuccess)
              ListView.EndUpdate()
              ListView.Update()

              If ScWait Then Thread.Sleep(ScrollWaitTime)

              Return
           End If
        End If

     'ドラッグ位置の表示

        Dim itm As ListViewItem = ListView.GetItemAt(CurPos.X, CurPos.Y)

        If Not itm Is Nothing Then
           If Not _Lsv_DragState Then
              _Lsv_DragState = True
           Else
              If _Lsv_PrevIndex <> itm.Index Then                '同一処理を回避
                 Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
              End If
           End If

           _Lsv_PrevIndex = itm.Index

           Lsv_Dragcolor_Change(_Lsv_PrevIndex, _BackColorSuccess)
        Else

        'リストの未登録行にマウスカーソルがある場合は強調表示を解除
           If ListView.ClientRectangle.Contains(CurPos) Then
              Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
           End If
        End If
    End Sub

    Private Sub ListView_DragDrop(sender As Object, e As DragEventArgs) Handles ListView.DragDrop

        If ListView.SelectedItems.Count = ListView.Items.Count Then Return

        Dim p As Point = ListView.PointToClient(New Point(e.X, e.Y))
        Dim DistItem As ListViewItem = ListView.GetItemAt(p.X, p.Y)
        Dim DistIndex As Integer
        Dim PushIndex As Integer

        If Not (DistItem Is Nothing) Then

        '移動先の位置を記録
           If ListView.SelectedIndices.Contains(DistItem.Index) Then

              Dim sc As Integer = ListView.SelectedItems.Count - 1
              Dim i0 As Integer = ListView.SelectedItems(0).Index
              Dim i1 As Integer = ListView.SelectedItems(sc).Index

              If sc = i1 - i0 Then Return '移動先が同じ

              For i As Integer = 0 To sc
                i0 = ListView.SelectedItems(i).Index

                If i0 = DistItem.Index Then
                   i1 = i                           '移動位置までの選択行数
                   Exit For
                End If
              Next

              PushIndex = DistItem.Index - i1
           Else
              PushIndex = DistItem.Index - 1
           End If
        End If

      ListView.BeginUpdate()

        Lsv_DragColor_Control()

        Dim ItemsCount As Integer = ListView.SelectedItems.Count
        Dim PushItem As New List(Of ListViewItem)

     '移動元のアイテムを退避して削除
        For Each SrcItem As ListViewItem In ListView.SelectedItems
          PushItem.Add(SrcItem)
          ListView.Items.Remove(SrcItem)
        Next

     '移動位置を確定
        If DistItem Is Nothing Then
           If p.Y < _Lsv_LabelHeight Then
              DistIndex = 0                         'ラベルを選択した場合はリストの先頭に
           Else
              DistIndex = ListView.Items.Count      '空白行を選択した場合はリストの最後に
           End If
        Else
           If DistItem.Index = -1 Then
              DistIndex = PushIndex
           Else
              DistIndex = DistItem.Index
           End If
        End If

     '退避したアイテムを移動位置に復元
        For i As Integer = 0 To ItemsCount - 1
          ListView.Items.Insert(DistIndex + i, PushItem(i))
        Next

     '移動位置へフォーカスを移す
        ListView.Items(DistIndex).Focused = True
        ListView.Items(DistIndex).Selected = True

     'マーカークリア
        If ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)

      ListView.EndUpdate()
    End Sub

#End Region

#Region "// Context Menu"

#Region "-- Setting"

    Private Sub ContextMenu_Setting_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Setting.Opening
        StripMenu_IniOpen.Enabled = File.Exists(_ini_File)
        StripMenu_UndoOpen.Enabled = File.Exists(_UndoFile)
        StripMenu_LogOpen.Enabled = File.Exists(_Log_File)
        StripMenu_Cmb_FontSize.SelectedIndex = _Lsv_FontSize_Id
    End Sub

    Private Sub ContextMenu_Setting_Closed(sender As Object, e As ToolStripDropDownClosedEventArgs) Handles ContextMenu_Setting.Closed

     'ListView 表示変更
        If _Lsv_FontSize_Id <> StripMenu_Cmb_FontSize.SelectedIndex Then
           _Lsv_FontSize_Id = StripMenu_Cmb_FontSize.SelectedIndex

           Lsv_SetInfo()

           Lsv_DisplayMode_Control(True)
        End If
    End Sub

    Private Sub StripMenu_DialogBox_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_DialogBox.CheckedChanged
        _OfdAutoUpgrade = StripMenu_DialogBox.Checked
    End Sub

    Private Sub StripMenu_ToolTip_Click(sender As Object, e As EventArgs) Handles StripMenu_ToolTip.Click

        Dim v As Integer = StripMenu_ToolTip.CheckState

        v += 1
        If v > 2 Then v = 0

        StripMenu_ToolTip.CheckState = DirectCast(v, CheckState)
    End Sub

    Private Sub StripMenu_ToolTip_CheckedChanged(sender As Object, e As EventArgs)

        ToolTip.Active = (StripMenu_ToolTip.CheckState = CheckState.Checked)

     'ListView.ShowItemToolTips が変更されると ListView.ColumnWidthChanged が発生する

      RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
        ListView.ShowItemToolTips = (StripMenu_ToolTip.CheckState <> CheckState.Unchecked)
      AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
    End Sub

    Private Sub StripMenu_MsgSound_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_MsgSound.CheckedChanged
        _MessageBoxSound = StripMenu_MsgSound.Checked
    End Sub

    Private Sub StripMenu_LogWrite_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_LogWrite.CheckedChanged
        _LogFileWriting = StripMenu_LogWrite.Checked
    End Sub

    Private Sub StripMenu_IniSave_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_IniSave.CheckedChanged
        _IniSave = StripMenu_IniSave.Checked
    End Sub

    Private Sub StripMenu_IniOpen_Click(sender As Object, e As EventArgs) Handles StripMenu_IniOpen.Click

'     設定ファイルをメモ帳で開く
        Diagnostics.Process.Start("Notepad", _ini_File)
    End Sub

    Private Sub StripMenu_UndoOpen_Click(sender As Object, e As EventArgs) Handles StripMenu_UndoOpen.Click

'     アンドゥファイルをメモ帳で開く
        Diagnostics.Process.Start("Notepad", _UndoFile)
    End Sub

    Private Sub StripMenu_LogOpen_Click(sender As Object, e As EventArgs) Handles StripMenu_LogOpen.Click

'     ログファイルをメモ帳で開く
        Diagnostics.Process.Start("Notepad", _Log_File)
    End Sub

    Private Sub StripMenu_GridLines_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_GridLines.CheckedChanged
        _UseGridLines = StripMenu_GridLines.Checked

        ListView.GridLines = StripMenu_GridLines.Checked
    End Sub

    Private Sub StripMenu_CtrlUpDown_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_CtrlUpDown.CheckedChanged
        _UseCtrlUpDown = StripMenu_CtrlUpDown.Checked
    End Sub

    Private Sub StripMenu_E_Click(sender As Object, e As EventArgs) _
                Handles StripMenu_E0.Click, StripMenu_E1.Click, StripMenu_E2.Click, StripMenu_E3.Click

        Dim n As String = DirectCast(sender, ToolStripItem).Name

        _UseEmphasisColor = Convert.ToInt32(n.Substring(n.Length - 1))

        SetEmphasisBackColor()
        StripMenu_E_Control()
    End Sub

    Private Sub StripMenu_Cmb_Culture_SelectedIndexChanged(sender As Object, e As EventArgs) Handles StripMenu_Cmb_Culture.SelectedIndexChanged
        _UseCultureInfo = StripMenu_Cmb_Culture.Text
    End Sub

    Private Sub StripMenu_Cmb_DateTimeFormat_SelectedIndexChanged(sender As Object, e As EventArgs) Handles StripMenu_Cmb_DateTimeFormat.SelectedIndexChanged
        _DateNameFormat = StripMenu_Cmb_DateTimeFormat.Text
    End Sub

#End Region

#Region "-- ListView"

   'Private Sub ContextMenu_ListView_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_ListView.Opening
      'ここで表示内容を変更すると、表示座標がずれる為 ListView_MouseUp で処理
   'End Sub

    Private Sub StripMenu_SelectConversion_Click(sender As Object, e As EventArgs) Handles StripMenu_SelectConversion.Click
        _onSelectItem = True

        Btn_Exec.Select()
        Btn_Exec.PerformClick()

        _onSelectItem = False
    End Sub

    Private Sub StripMenu_NameAcquisition_Click(sender As Object, e As EventArgs) Handles StripMenu_NameAcquisition.Click
        Dim s As String = ListView.Items(ListView.SelectedItems(0).Index).Text

        _ChangeCancelFlag = True
          If Cmb_Ext.SelectedIndex = 3 Then
             Cmb_Text_1.Text = Replace(Path.GetExtension(s), ".", "", 1, 1, CompareMethod.Binary)
          Else
             Cmb_Text_1.Text = Path.GetFileNameWithoutExtension(s)
          End If
          Clipboard.SetText(Cmb_Text_1.Text)
        _ChangeCancelFlag = False
    End Sub

    Private Sub StripMenu_Open_Click(sender As Object, e As EventArgs) Handles StripMenu_Open.Click

'     ファイルを関連付けされたアプリケーションで開く
        Try
          Dim p As Diagnostics.Process = Diagnostics.Process.Start(ListView.SelectedItems.Item(0).Text)

          If Not p Is Nothing Then
             p.WaitForInputIdle()
             p.Close()
             p.Dispose()
          End If

        Catch ex As Exception
          MsgBox(ex.Message)
        End Try
    End Sub

    Private Sub StripMenu_SelectAllItem_Click(sender As Object, e As EventArgs) Handles StripMenu_SelectAllItem.Click
        For Each LsvItem As ListViewItem In ListView.Items
          LsvItem.Selected = True
        Next
    End Sub

    Private Sub StripMenu_ToggleSelection_Click(sender As Object, e As EventArgs) Handles StripMenu_ToggleSelection.Click
        For Each LsvItem As ListViewItem In ListView.Items
          LsvItem.Selected = LsvItem.Selected Xor True
        Next
    End Sub

    Private Sub StripMenu_DeleteItem_Click(sender As Object, e As EventArgs) Handles StripMenu_DeleteItem.Click
        Lsv_Delete(False)
    End Sub

    Private Sub StripMenu_AttributeChange_Click(sender As Object, e As EventArgs) Handles StripMenu_AttributeChange.Click
        FileAttributesChange()
    End Sub

    Private Sub StripMenu_VerInfo_Click(sender As Object, e As EventArgs) Handles StripMenu_VerInfo.Click, StripMenu_About.Click
        Dim VerInfo As New AboutBoxEx

        VerInfo.Showdialog(Me)
        VerInfo.Dispose()

        Me.Update()
        WinMessage.Clear(Me, 200)
    End Sub

#End Region

#Region "-- Notify"

   'Private Sub ContextMenu_Notify_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Notify.Opening
      'ここで表示内容を変更すると、表示座標がずれる為 Notify_Icon_MouseDown で処理
   'End Sub

    Private Sub Notify_Icon_MouseDown(sender As Object, e As MouseEventArgs) Handles Notify_Icon.MouseDown

     'ContextMenu_Notify_Opening で表示内容を変更すると、表示座標がずれる為の措置

     'Notify_Icon_MouseDown→ContextMenu_Notify_Opening→
     'Notify_Icon_MouseUp→Notify_Icon_MouseClick

        If e.Button = MouseButtons.Right Then

           Dim f As Boolean = Not _WindowMinimized

           StripMenu_Setting.Visible = f
           StripSeparator_N1.Visible = f
           StripMenu_About.Visible = f
        End If
    End Sub

    Private Sub Notify_Icon_MouseClick(sender As Object, e As MouseEventArgs) Handles Notify_Icon.MouseClick

        If Me.ShowInTaskbar Then Return

        If e.Button = MouseButtons.Left Then
           If _WindowMinimized Then

           'ContextMenu が表示されている時に、最小化を解除すると
           'ContextMenu が残ったままになる場合がある為の措置

              ContextMenu_Notify.Close()

              If _WindowMaximized Then
                 Me.WindowState = FormWindowState.Maximized
              Else
                 Me.WindowState = FormWindowState.Normal
              End If

           'Normal 復帰時にフォーム表示が壊れる事がある為の措置
              Me.Refresh()
           Else
              If (Not CheckOpenForms()) Then
                 Me.WindowState = FormWindowState.Minimized
              End If
           End If
        End If
    End Sub

    Private Sub ContextMenu_Notify_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Notify.Opening

     'DialogForm (FileDialog, FolderBrowserDialog)、または、
     '他のフォームが表示されている場合はキャンセル

        If CheckOpenForms() Then

           e.Cancel = True
        End If
    End Sub

    Private Sub StripMenu_Setting_Click(sender As Object, e As EventArgs) Handles StripMenu_Setting.Click
        ShowSettings(False)
    End Sub

    Private Sub StripMenu_TaskBar_CheckedChanged(sender As Object, e As EventArgs)

        Dim ChangeState As Boolean
        Dim MaximizedState As Boolean = _WindowMaximized

     'ShowInTaskbar が変更されると フォーム表示にちらつきが発生する為の措置
        If Not _WindowMinimized Then
           ChangeState = True
           Me.WindowState = FormWindowState.Minimized
        End If

     'ShowInTaskbar が変更されると
     'ComboBoxEx: Resize ⇒ Resize ⇒ OnCreateControl が発生する

     'ShowInTaskbar が変更されると ListView.ColumnWidthChanged が発生する為の措置

      RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
        Me.ShowInTaskbar = StripMenu_TaskBar.Checked
      AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

        If ChangeState Then
           If MaximizedState Then
              Me.WindowState = FormWindowState.Maximized
           Else
              Me.WindowState = FormWindowState.Normal
           End If
        End If
    End Sub

    Private Sub StripMenu_Quit_Click(sender As Object, e As EventArgs) Handles StripMenu_Quit.Click
        Me.Close()
    End Sub

#End Region

#End Region

#Region "// GroupBox, Label"

#Region "-- Menu (Top, GroupBox 1)"

    Private Sub GroupBox1_MouseDown(sender As Object, e As MouseEventArgs) Handles GroupBox1.MouseDown
        If e.Button = MouseButtons.Right Then
           If GroupBox1.LabelRectangle.Contains(e.Location) Then
              ContextMenu_Setting.Show(Me, New Point(0, 0))
           End If
        End If
    End Sub

    Private Sub Lbl_Attr_MouseLeave(sender As Object, e As EventArgs) Handles Lbl_Attr_R.MouseLeave, Lbl_Attr_H.MouseLeave, Lbl_Attr_S.MouseLeave
        If _onToolTipActive Then
           _onToolTipActive = False

           Me.ToolTip.Active = False
           Me.ToolTip.IsBalloon = False
           Me.ToolTip.InitialDelay = 500

           Attr_ToolTip_Change()
        End If
    End Sub

    Private Sub Lbl_Attr_MouseClick(sender As Object, e As MouseEventArgs) Handles Lbl_Attr_R.MouseClick, Lbl_Attr_H.MouseClick, Lbl_Attr_S.MouseClick
        Dim i As Integer = "RHS".IndexOf(DirectCast(sender, LabelSL).Text, StringComparison.Ordinal)

        If e.Button <> MouseButtons.Left Then Return

        _AttrType(i) = _AttrType(i) Xor Convert.ToInt32(2 ^ i)
        _List_AttrType = _AttrType(0) + _AttrType(1) + _AttrType(2)

        DirectCast(sender, LabelSL).ForeColor = Color.FromName(_AttrColorName(_AttrType(i)))

        If Not Me.ToolTip.Active Then
           _onToolTipActive = True

           Me.ToolTip.InitialDelay = 50
           Me.ToolTip.IsBalloon = True
           Me.ToolTip.Active = True
        End If

        Attr_ToolTip_Change()
    End Sub

    Private Sub Lbl_Blank2_DoubleClick(sender As Object, e As EventArgs) Handles Lbl_Blank2.DoubleClick
        Dim fi As New FileInfo(_UndoFile)

        If (Not Btn_Undo.Enabled) AndAlso (fi.Exists) AndAlso (fi.Length <> 0) Then

           Dim MsgBox As New MessageBoxEx
           If MsgBox.Dialog(Me, "Undo 情報を再読込しますか？。", _Msg_Cap, _
                            MessageBoxEx.Buttons.CenterYesNo, _
                            MessageBoxIcon.Question, _
                            MessageBoxDefaultButton.Button2, , _MessageBoxSound) =
           DialogResult.Yes Then
              Btn_Undo.Enabled = True
           End If
           MsgBox.Dispose()

           Me.Update()
        End If
    End Sub

    Private Sub Lbl_Items_DoubleClick(sender As Object, e As EventArgs) Handles Lbl_Items.DoubleClick
        Dim Reg_SZ As String = DirectCast(Microsoft.Win32.Registry.GetValue( _
                               "HKEY_CURRENT_USER\AppEvents\Schemes\Apps\Explorer\EmptyRecycleBin\.Current", "", Nothing), String)

        If Reg_SZ Is Nothing Then
           Beep()
        Else
           Dim windir As String = Environment.GetFolderPath(Environment.SpecialFolder.Windows)
           Dim F_Name As String = Path.GetFileName(Reg_SZ)
           Dim F_Path As String

           If Path.GetDirectoryName(Reg_SZ) = "" Then
              F_Path = Path.Combine(windir, "Media", F_Name)    'XP対策
           Else
              F_Path = Replace(Reg_SZ, "%SystemRoot%", windir)
           End If

           If File.Exists(F_Path) Then
              Dim player As New SoundPlayer(F_Path)
              player.PlaySync()
              player.Dispose()
              player = Nothing
           End If
        End If

        GC.Collect()
        GC.WaitForPendingFinalizers()
        GC.Collect()
    End Sub

#End Region

#Region "-- Menu (Bottom)"

    Private Sub Lbl_Mode_Lamp_MouseClick(sender As Object, e As MouseEventArgs) Handles Lbl_Mode_Lamp_1.MouseClick, Lbl_Mode_Lamp_2.MouseClick
        If e.Button = MouseButtons.Left Then
           _Lsv_DisplayMode = 0
           ModeLamp_Control()
        End If
    End Sub

    Private Sub Lbl_Check_Lamp_MouseClick(sender As Object, e As MouseEventArgs) Handles Lbl_CheckLamp.MouseClick
        Lbl_CheckLamp.Visible = False
    End Sub

#End Region

#End Region

#Region "// ComboBox"

    Private Sub Cmb_File_DropDown(sender As Object, e As EventArgs) Handles Cmb_File.DropDown
        Cmb_File.ForeColor = SystemColors.WindowText
    End Sub

    Private Sub Cmb_File_MouseWheel(sender As Object, e As MouseEventArgs) Handles Cmb_File.MouseWheel

'     このイベントの後で値が変わる為、
'     SelectedIndexChanged で、色を変更出来る様に _OnMouseWheelFlag をセットする。
'
'      [ Wheel Redirector 等、マウスツール使用時の対策 ]

        _OnMouseWheelFlag = Not Cmb_File.Focused
    End Sub

    Private Sub Cmb_File_SelectedIndexChanged(sender As Object, e As EventArgs)

        Cmb_Text_Control()

'     マウスのホイールでメニューを変更した場合、Cmb_File_Leaveが発生しない為、ここで色を変更
'
'      [ Wheel Redirector 等、マウスツール使用時の対策 ]

        If _OnMouseWheelFlag Then
           _OnMouseWheelFlag = False

           Cmb_File_ForeColorSet()
        End If

        Btn_Exec_Control()
    End Sub

    Private Sub Cmb_File_Leave(sender As Object, e As EventArgs) Handles Cmb_File.Leave
        Cmb_File_ForeColorSet()
    End Sub

    Private Sub Cmb_Ext_DropDown(sender As Object, e As EventArgs) Handles Cmb_Ext.DropDown
        Cmb_Ext.ForeColor = SystemColors.WindowText
    End Sub

    Private Sub Cmb_Ext_MouseWheel(sender As Object, e As MouseEventArgs) Handles Cmb_Ext.MouseWheel

'     このイベントの後で値が変わる為、
'     SelectedIndexChanged で、色を変更出来る様に _OnMouseWheelFlag をセットする。
'
'      [ Wheel Redirector 等、マウスツール使用時の対策 ]

        _OnMouseWheelFlag = Not Cmb_Ext.Focused
    End Sub

    Private Sub Cmb_Ext_SelectedIndexChanged(sender As Object, e As EventArgs)

        Dim f As Boolean = (Cmb_Ext.SelectedIndex <> 3)

        Label1.Visible = f
        Cmb_File.Visible = f

'     マウスのホイールでメニューを変更した場合、Cmb_File_Leaveが発生しない為、ここで色を変更
'
'      [ Wheel Redirector 等、マウスツール使用時の対策 ]

        If _OnMouseWheelFlag Then
           _OnMouseWheelFlag = False

            If Cmb_Ext.SelectedIndex = 3 Then
               Cmb_Ext.ForeColor = Color.Red
            Else
               Cmb_Ext.ForeColor = SystemColors.WindowText
            End If
        End If

'     メニュー変更時、[拡張子変更]選択時以外は不正な文字になる"\0"をクリア

        If Cmb_Ext.SelectedIndex <> 3 Then
           _ChangeCancelFlag = True
             If Cmb_Text_1.Text = "\0" Then Cmb_Text_1.Text = ""
             If Cmb_Text_2.Text = "\0" Then Cmb_Text_2.Text = ""
           _ChangeCancelFlag = False
        End If

        Cmb_Text_Control()
        Btn_Exec_Control()
    End Sub

    Private Sub Cmb_Ext_Leave(sender As Object, e As EventArgs) Handles Cmb_Ext.Leave
        If Cmb_Ext.SelectedIndex = 3 Then
           Cmb_Ext.ForeColor = Color.Red
        Else
           Cmb_Ext.ForeColor = SystemColors.WindowText
        End If
    End Sub

    Private Sub Cmb_Text_VisibleChanged(sender As Object, e As EventArgs) Handles Cmb_Text_1.VisibleChanged, Cmb_Text_2.VisibleChanged

        Dim ctl As ComboBoxEx = DirectCast(sender, ComboBoxEx)
        Dim s As Boolean = (ctl.Visible AndAlso (ctl.Items.Count <> 0))

        If ctl.Name.EndsWith("1") Then
           Btn_Clear_1.Visible = s
        Else
           Btn_Clear_2.Visible = s
        End If
    End Sub

    Private Sub Cmb_Text_KeyDown(sender As Object, e As KeyEventArgs) Handles Cmb_Text_1.KeyDown, Cmb_Text_2.KeyDown

        Dim ctl As ComboBoxEx = DirectCast(sender, ComboBoxEx)

        If ctl.DroppedDown Then
           If e.KeyCode = Keys.Delete Then

              Dim id As Integer = ctl.SelectedIndex

              If id <> -1 Then
                 ctl.DroppedDown = False
                 ctl.Items.RemoveAt(id)

                 If ctl.Items.Count = 0 Then
                    If ctl.Name.EndsWith("1") Then
                       Btn_Clear_1.Visible = False
                    Else
                       Btn_Clear_2.Visible = False
                    End If
                 End If
              Else
                 Beep()
              End If
           End If
        End If
    End Sub

    Private Sub Cmb_Text_KeyPress(sender As Object, e As KeyPressEventArgs) Handles Cmb_Text_1.KeyPress, Cmb_Text_2.KeyPress
        Static Esc_Check As Boolean

        Dim ctl As ComboBoxEx = DirectCast(sender, ComboBoxEx)

        If Cmb_Ext.SelectedIndex = 3 Then
           If e.KeyChar = "\"c Then
              Esc_Check = True
           Else
              If Esc_Check AndAlso (e.KeyChar = "0"c) Then
                 Esc_Check = False
                 _ChangeCancelFlag = True
                   ctl.Text = "\0"
                   ctl.SelectAll()
                 _ChangeCancelFlag = False
                 e.Handled = True
                 Return
              End If
              Esc_Check = False
           End If
        Else
           Esc_Check = False
        End If

        Select Case AscW(e.KeyChar)
          Case Keys.Enter : e.Handled = True

          Case 1                                        '[Ctrl]+A

'       通常[Ctrl]+C 等と同じく何もしなくても全選択状態になる筈なのに
'       全選択状態にならない、ハンドルを解除しないとBeep音が出力される等
'       不可解な動作をする為、強制的にハンドルを解除し全選択状態にする

            ctl.SelectAll()
            e.Handled = True

          Case 3, 22, 24, Keys.Back                     '[Ctrl]+{C(3),V(22),X(24)},[BS]

          Case Else
            Dim c As String = e.KeyChar

            If c.IndexOfAny(_InvalidFileNameChars) = 0 Then
               e.Handled = True
               Beep()
            End If
        End Select
    End Sub

    Private Sub Cmb_Text_TextChanged(sender As Object, e As EventArgs) Handles Cmb_Text_1.TextChanged, Cmb_Text_2.TextChanged
        If _ChangeCancelFlag Then Return

        Dim ctl As ComboBoxEx = DirectCast(sender, ComboBoxEx)

        If ctl.Text.Length <> 0 Then
           Dim old_Text As String = ctl.Text
           Dim new_Text As String = Regex.Replace(old_Text, _Remove_FileNamePattern, " ")

           If old_Text <> new_Text Then
              Dim CursorPosition As Integer = ctl.SelectionStart

              _ChangeCancelFlag = True
                ctl.Text = new_Text
                ctl.SelectionStart = CursorPosition + (new_Text.Length - old_Text.Length)
              _ChangeCancelFlag = False
           End If
        End If

        Btn_Exec_Control()
    End Sub

    Private Sub Cmb_Text_Leave(sender As Object, e As EventArgs) Handles Cmb_Text_1.Leave, Cmb_Text_2.Leave

        Dim i As Integer
        Dim ctl As ComboBoxEx = DirectCast(sender, ComboBoxEx)

        If (ctl.Text.Length <> 0) AndAlso (ctl.Text <> "\0") Then
           For i = 0 To ctl.Items.Count - 1
             If ctl.Items(i).ToString = ctl.Text Then Return
           Next
           ctl.Items.Add(ctl.Text)

           If ctl.Name.EndsWith("1") Then
              Btn_Clear_1.Visible = True
           Else
              Btn_Clear_2.Visible = True
           End If
        End If
    End Sub

#End Region

#Region "// Button"

#Region "-- Menu (Top)"

    Private Sub Btn_Open_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Open.MouseDown
        If e.Button = MouseButtons.Middle Then
           Btn_Dummy.Enabled = True
           Btn_Dummy.Select()
           Btn_Open.Enabled = False

           _onDialogForm = True

           If _List_FileMode <> 0 Then
              Open_File()
           Else
              Open_Folder()
           End If

           Me.Update()
           WinMessage.Clear(Me)

           ListView.Select()

           Btn_Open.Enabled = True
           Btn_Dummy.Enabled = False
        End If

        _onDialogForm = False
    End Sub

    Private Sub Btn_Open_Click(sender As Object, e As EventArgs) Handles Btn_Open.Click
        Btn_Dummy.Enabled = True
        Btn_Dummy.Select()
        Btn_Open.Enabled = False

        _onDialogForm = True

        If _List_FileMode <> 0 Then
           Open_Folder()
        Else
           Open_File()
        End If

        Me.Update()
        WinMessage.Clear(Me)

        ListView.Select()

        Btn_Open.Enabled = True
        Btn_Dummy.Enabled = False

        _onDialogForm = False
    End Sub

    Private Sub Btn_Erase_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Erase.MouseDown
        If e.Button = MouseButtons.Middle Then
           ListView.HideSelection = True

           Lsv_Delete(True)
           ListView.Select()

           ListView.HideSelection = _UseHideSelection
        End If
    End Sub

    Private Sub Btn_Erase_Click(sender As Object, e As EventArgs) Handles Btn_Erase.Click
        ListView.HideSelection = True

        Lsv_Clear()
        ListView.Select()

        ListView.HideSelection = _UseHideSelection
    End Sub

#End Region

#Region "-- Menu (Bottom)"

    Private Sub Btn_Clear_Click(sender As Object, e As EventArgs) Handles Btn_Clear_1.Click, Btn_Clear_2.Click

        Dim ctl As ButtonEx = DirectCast(sender, ButtonEx)

        ctl.Visible = False

        If ctl.Name.EndsWith("1") Then
           Cmb_Text_1.Items.Clear()
        Else
           Cmb_Text_2.Items.Clear()
        End If
    End Sub

    Private Sub Btn_Col0Reset_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Col0Reset.MouseDown

        If _Lsv_DisplayMode <> 1 Then Return

        If e.Button = MouseButtons.Right Then

           _Lsv_DisplayMode = 0
           ModeLamp_Control()

           RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

           With ListView
             .BeginUpdate()

             .Columns(1).Width += _Lsv_Col2_Width
             .Columns(2).Width = 0

             .EndUpdate()
           End With

           AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
        End If
    End Sub

    Private Sub Btn_Col0Reset_Click(sender As Object, e As EventArgs) Handles Btn_Col0Reset.Click
        _Lsv_DisplayMode = 1

        Lsv_DisplayMode_Control(True)
        ListView.Select()
    End Sub

    Private Sub Btn_Col2Reset_Click(sender As Object, e As EventArgs) Handles Btn_Col2Reset.Click
        _Lsv_DisplayMode = 2

        Lsv_DisplayMode_Control(True)
        ListView.Select()
    End Sub

    Private Sub Btn_ChckMode_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_ChckMode.MouseDown
        If e.Button = MouseButtons.Middle Then
           _List_ChckMode = _List_ChckMode Xor 1
           Btn_ChckMode.ImageIndex = _List_ChckMode + 9
           ListMode_ToolTip_Change()

           Lbl_CheckLamp.Visible = False
        End If
    End Sub

    Private Sub Btn_ChckMode_Click(sender As Object, e As EventArgs) Handles Btn_ChckMode.Click
        _List_ChckMode = _List_ChckMode Xor 1
        Btn_ChckMode.ImageIndex = _List_ChckMode + 9
        ListMode_ToolTip_Change()

        Lbl_CheckLamp.Visible = True
    End Sub

    Private Sub Btn_TreeMode_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_TreeMode.MouseDown
        If e.Button = MouseButtons.Middle Then
           _List_TreeMode = (_List_TreeMode Xor 2) Mod 3
           Btn_TreeMode.ImageIndex = _List_TreeMode + 4
           ListMode_ToolTip_Change()
        End If
    End Sub

    Private Sub Btn_TreeMode_Click(sender As Object, e As EventArgs) Handles Btn_TreeMode.Click
        _List_TreeMode = (_List_TreeMode + 1) Mod 3
        Btn_TreeMode.ImageIndex = _List_TreeMode + 4
        ListMode_ToolTip_Change()
    End Sub

    Private Sub Btn_FileMode_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_FileMode.MouseDown
        If e.Button = MouseButtons.Middle Then
           _List_FileMode = (_List_FileMode Xor 2) Mod 3
           Btn_FileMode.ImageIndex = _List_FileMode + 1
           ListMode_ToolTip_Change()
        End If
    End Sub

    Private Sub Btn_FileMode_Click(sender As Object, e As EventArgs) Handles Btn_FileMode.Click
        _List_FileMode = (_List_FileMode + 1) Mod 3
        Btn_FileMode.ImageIndex = _List_FileMode + 1
        ListMode_ToolTip_Change()
    End Sub

#End Region

#End Region

#Region "// ---- Main Button --------------------------------------------------"

    Private Sub Btn_Exec_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Exec.MouseDown
        If e.Button = MouseButtons.Middle Then

           Dim cMode As Integer = _List_ChckMode
           Dim mMode As Boolean = Lbl_CheckLamp.Visible

           If mMode Then Lbl_CheckLamp.Enabled = False

           _List_ChckMode = 0   '0:Ignore

           Btn_Exec.Select()
           Btn_Exec.PerformClick()

           If mMode Then Lbl_CheckLamp.Enabled = True

           _List_ChckMode = cMode
        End If
    End Sub

    Private Sub Btn_Exec_Click(sender As Object, e As EventArgs) Handles Btn_Exec.Click

        Dim n As Integer                       '処理件数
        Dim i, j As Integer
        Dim id As Integer

        Dim FName As String
        Dim Ename As String
        Dim DirName As String
        Dim OldName As String
        Dim NewName As String

        Dim msg As String
        Dim Lsv As ListViewItem

        Me.Cursor = Cursors.WaitCursor

        Btn_Dummy.Enabled = True
        Btn_Exec_Control(False)                 'Btn_Dummy (NextControl) へ Focusを移す (フォーカスを消す)

        Btn_Undo.Enabled = False
'-------------------------------------------------------------------------------
'     Show ProgressBar (ListView.BeginUpdate の前に表示しないと再描画により
'                       ListView 内の表示がクリアされる為)
'-------------------------------------------------------------------------------
        ProgressBar.Value = 0
        ProgressBar.Visible = True
'-------------------------------------------------------------------------------
'     前処理(パスを昇順ソート)
'-------------------------------------------------------------------------------
        ListView.BeginUpdate()
        ListView.HideSelection = True

        ListView.MarkerSet(0, 0)        'ソート
        ListView.MarkerClear()          '▲▼クリア
'-------------------------------------------------------------------------------
'     作業用パス一括読込
'-------------------------------------------------------------------------------
        Dim wPath As New List(Of String)        'パス(作業用)
        For Each Lsv In ListView.Items
          wPath.Add(Lsv.Text)
        Next
'-------------------------------------------------------------------------------
'     表示色クリア
'-------------------------------------------------------------------------------
        Lsv_BackColor_Clear()
'-------------------------------------------------------------------------------
'     Job Start
'-------------------------------------------------------------------------------
        Using sw As New StreamWriter(_UndoFile, False, Encoding.UTF8)
'
        If Cmb_Ext.SelectedIndex = 3 Then
           id = -1
        Else
           id = DirectCast(Cmb_File.SelectedValue, Integer)
        End If

        sw.WriteLine(id)                        '処理モード書き込み
'
        n = 0

     'Target Item Set
        If Not _onSelectItem Then
           For Each Lsv In ListView.Items
             Lsv.Selected = True
           Next
        End If

        Dim state As Boolean = (ListView.SelectedItems.Count <> 1)  'MsgBox 内での処理継続判定用
        Dim Result As Integer

     'ProgressBar Control value setting
        Dim pVal As Single = 0.0F
        Dim pAdd As Single = Convert.ToSingle(100 / ListView.SelectedItems.Count)

        For Each Lsv In ListView.SelectedItems
          OldName = wPath(Lsv.Index)
          DirName = Path.GetDirectoryName(OldName)
          FName = Path.GetFileNameWithoutExtension(OldName)
          Ename = Path.GetExtension(OldName)
'-------------------------------------------------------------------------------
'       ファイル名修正
'-------------------------------------------------------------------------------
          If IsNothing(DirName) Then      'ルートは対象外
             FName = ""
          Else
             Select Case id
'･･･････････････････････････････････････････････････････････････････････････････
'       拡張子の変更
               Case -1
'･･･････････････････････････････････････････････････････････････････････････････
'       処理モード (0 - 4) 大文字、小文字変換
'･･･････････････････････････････････････････････････････････････････････････････
               Case 0

               'DDMMYYYY ⇒ YYYYMMDD
                 'Dim s1 As String = FName.Substring(0, 2)
                 'Dim s2 As String = FName.Substring(2, 2)
                 'Dim s3 As String = FName.Substring(4, 4)
                 'Mid(FName, 1, 8) = s3 & s2 & s1
'･･･････････････････････････････････････････････････････････････････････････････
               Case 1 To 3
                 FName = StrConv(FName, DirectCast(id, VbStrConv))

               Case 4
                 FName = StrConv(FName, vbLowerCase)
                 Mid(FName, 1, 1) = StrConv(FName.Substring(0, 1), vbUpperCase)
'･･･････････････････････････････････････････････････････････････････････････････
'       処理モード (5 - 6) 作成日時、更新日時
'･･･････････････････････････････････････････････････････････････････････････････
               Case 5, 6
                 Dim ci As New Globalization.CultureInfo(_UseCultureInfo)

                 If id = 5 Then
                    FName = File.GetCreationTime(OldName).ToString(_DateNameFormat, ci)
                 Else
                    FName = File.GetLastWriteTime(OldName).ToString(_DateNameFormat, ci)
                 End If

                 FName = New Regex("[/:]").Replace(FName, ".")
'･･･････････････････････････････････････････････････････････････････････････････
'       処理モード (10 - 51) 挿入・置換・削除
'･･･････････････････････････････････････････････････････････････････････････････
               Case 10
                '先頭に挿入
                 FName = Cmb_Text_1.Text & FName

               Case 11
                '末尾に挿入
                 FName = FName & Cmb_Text_1.Text

               Case 20
                '指定文字の置換
                 FName = Replace(FName, Cmb_Text_1.Text, Cmb_Text_2.Text, 1, 1, CompareMethod.Binary)

               Case 21
                '指定文字の削除
                 FName = Replace(FName, Cmb_Text_1.Text, "", 1, 1, CompareMethod.Binary)

               Case 30
                '指定文字(*)の置換
                 FName = Replace(FName, Cmb_Text_1.Text, Cmb_Text_2.Text, 1, -1, CompareMethod.Binary)

               Case 31
                '指定文字(*)の削除
                 FName = Replace(FName, Cmb_Text_1.Text, "", 1, -1, CompareMethod.Binary)

               Case 40, 50
                '指定文字数を残す
                 i = Convert.ToInt32(Cmb_Num.Text)
                 If i >= FName.Length Then   'ファイル名の文字数より指定文字数が大きい
                    FName = ""
                 Else
                    If id = 40 Then
                       FName = FName.Substring(0, i)
                    Else
                       FName = FName.Substring(FName.Length - i)
                    End If
                 End If

               Case 41, 51
                '指定文字数を削除
                 i = Convert.ToInt32(Cmb_Num.Text)
                 If i >= FName.Length Then   'ファイル名の文字数より指定文字数が大きい
                    FName = ""
                 Else
                    j = FName.Length - i
                    If id = 41 Then
                       FName = FName.Substring(FName.Length - j)
                    Else
                       FName = FName.Substring(0, j)
                    End If
                 End If
             End Select
          End If
'-------------------------------------------------------------------------------
'       拡張子名変更
'-------------------------------------------------------------------------------
          Select Case Cmb_Ext.SelectedIndex
            Case 0
            Case 1, 2
              Ename = StrConv(Ename, DirectCast(Cmb_Ext.SelectedIndex, VbStrConv))

            Case 3
              If Cmb_Text_1.Text = "\0" Then        '拡張子の追加
                 If Ename.Length = 0 Then
                    Ename = "." & Cmb_Text_2.Text.TrimEnd
                 End If

              ElseIf Cmb_Text_2.Text = "\0" Then    '拡張子の削除(指定文字列)
                 If Ename.Length <> 0 Then
                    Ename = Replace(Ename, Cmb_Text_1.Text, "", 1, 1, CompareMethod.Binary)
                      If Ename.Length = 1 Then Ename = ""
                 End If

              Else                                  '拡張子の変更
                 If Ename.Length <> 0 Then
                    Ename = Replace(Ename, Cmb_Text_1.Text, Cmb_Text_2.Text, 1, 1, CompareMethod.Binary)
                 End If
              End If
          End Select
'-------------------------------------------------------------------------------
'       ファイル名変更、リスト更新
'-------------------------------------------------------------------------------
          If IsNothing(FName) Then FName = ""

'       フォルダ・ファイル名の先頭が空白(半角・全角)にならないように修正
          FName = FName.TrimStart

          If (FName.Length <> 0) OrElse (Ename.Length <> 0) Then
             NewName = Path.Combine(DirName, FName & Ename)

             If OldName = NewName Then

'             ファイル名の変更がなくても親フォルダが変更されている場合、リスト(パス)を更新
'             Undo用に、パス情報保存

                If OldName <> Lsv.Text Then
                   sw.WriteLine("*" & vbTab & Lsv.Text & vbTab & OldName)
                   Lsv.SubItems(0).Text = OldName
                End If

             Else
                Do
                  Result = NameUpdate(OldName, NewName, state)
                Loop While Result = DialogResult.Retry

                Select Case Result
                  Case DialogResult.None
'                  undo data write (ファイル種別，変更前，変更後のパス)
                    sw.WriteLine(Lsv.ImageIndex & vbTab & Lsv.Text & vbTab & NewName)

                    If Lsv.ImageIndex <> 0 Then

'                    作業用パスのフォルダ名変更
                       For i = Lsv.Index + 1 To wPath.Count - 1
                         If wPath(i).IndexOf(OldName & "\") = 0 Then
                            wPath(i) = Replace(wPath(i), OldName, NewName, 1, 1, CompareMethod.Binary)
                         Else
                            Exit For
                         End If
                       Next
                    End If

                    If _List_ChckMode <> 0 Then
                       Lsv.SubItems(0).Text = NewName
                       Lsv.SubItems(1).Text = Path.GetFileName(NewName)
                       Lsv.SubItems(2).Text = Path.GetExtension(NewName)

                       Lsv_BackColor_Set(Lsv, _BackColorSuccess)
                    Else
                       wPath.RemoveAt(Lsv.Index)
                       Lsv.Remove()
                    End If
                    n += 1

                  Case DialogResult.OK, DialogResult.Ignore
                    Lsv_BackColor_Set(Lsv, _BackColorFailure)

                  Case DialogResult.Cancel
                    Lsv_BackColor_Set(Lsv, _BackColorBreak)
                    Exit For

                  Case DialogResult.Abort
                    Exit For
                End Select
             End If
          End If
          pVal += pAdd
          ProgressBar.Value = Convert.ToInt32(pVal)
        Next
        Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        End Using

        If Not _onSelectItem Then
           ListView.SelectedItems.Clear()
        End If

        Lsv_DisplayMode_Control(False)

        ListView.EndUpdate()
'-------------------------------------------------------------------------------
'     Log Write
'-------------------------------------------------------------------------------
        If (n <> 0) AndAlso _LogFileWriting Then

           Dim buf, itm() As String

           If File.Exists(_Log_File) Then
              If File.Exists(_TempFile) Then File.Delete(_TempFile)
              File.Move(_Log_File, _TempFile)
           Else
              Using sw As New StreamWriter(_TempFile, False, Encoding.UTF8) : End Using
           End If

           Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
           Using sr As New StreamReader(_UndoFile, Encoding.UTF8)

             If n > 1 Then
                msg = String.Format(_LblItemsFormat, n) & " --"
             Else
                msg = "--"
             End If
             sw.WriteLine("-- {0} : Change {1}", Date.Now.ToString(_DateTimeFormat), msg)

             buf = sr.ReadLine
             Do While Not sr.EndOfStream
               buf = sr.ReadLine
               itm = buf.Split(CChar(vbTab))
               sw.WriteLine("[{0}] {1}{2}--> {3}", itm(0), itm(1), vbTab, itm(2))
             Loop
             sw.WriteLine()
           End Using

           Dim fi As New FileInfo(_TempFile)
           If fi.Length > 0 Then
              Using sr As New StreamReader(_TempFile, Encoding.UTF8)
                sw.Write(sr.ReadToEnd())
              End Using
           End If
           End Using

           File.Delete(_TempFile)
        End If
'-------------------------------------------------------------------------------
'     Message Disp. 
'-------------------------------------------------------------------------------
        Me.Update()
        Me.Cursor = Cursors.Default

        If n <> 0 Then
           ProgressBar.Value = 100

           If (Not Lbl_CheckLamp.Visible) OrElse (Not Lbl_CheckLamp.Enabled) Then
              WinMessage.Clear(Me, 200)
           Else
              msg = String.Format("{0:#,##0} 個のファイルを変更しました。", n)
              ShowInfoMessage(msg, _App_Name)
           End If

           Btn_Undo.Enabled = True
        Else
           Beep()
        End If

        ProgressBar.Visible = False
        Btn_Exec_Control()

        Btn_Dummy.Enabled = False
        Btn_Erase.Enabled = (ListView.Items.Count <> 0)

        ListView.HideSelection = _UseHideSelection
        ListView.Select()

        _Lsv_DupCheckFlg = (ListView.Items.Count <> 0)
    End Sub

    Private Sub Btn_Undo_Click(sender As Object, e As EventArgs) Handles Btn_Undo.Click

        Btn_Dummy.Enabled = True
        Btn_Dummy.Select()          'Btn_Dummy へ Focusを移す (フォーカスを消す)
        Btn_Exec.Enabled = False
        Btn_Undo.Enabled = False

        Dim fi As New FileInfo(_UndoFile)

        If Not (fi.Exists) OrElse (fi.Length = 0) Then

           ShowWarningMessage("アンドゥファイルがありません。", "", _Msg_Cap)
        Else
           Btn_Undo_Execute()
        End If

        Btn_Exec_Control()

        Btn_Dummy.Enabled = False
        Btn_Erase.Enabled = (ListView.Items.Count <> 0)

        ListView.Select()

        _Lsv_DupCheckFlg = (ListView.Items.Count <> 0)
    End Sub

    Private Sub Btn_Undo_Execute()

        Dim n As Integer
        Dim c As Integer = ListView.Items.Count
        Dim id As Integer
        Dim FlType As Integer           'File Type (0:File , 1:Folder)
        Dim Result As Integer           '処理結果
        Dim PrevFolderResult As Integer '親フォルダの処理結果 (0:Success 1:Failure)
        Dim msg As String
        Dim pAdd, pVal As Single
        Dim Lsv As ListViewItem
        Dim buf, itm() As String

        Me.Cursor = Cursors.WaitCursor
'-------------------------------------------------------------------------------
'     作業用パス一括読込
'-------------------------------------------------------------------------------
        Dim ip As Integer = 0
        Dim wPath As New List(Of String)    'パス(作業用)

        Using sr As New StreamReader(_UndoFile, Encoding.UTF8)
          buf = sr.ReadLine
          id = Convert.ToInt32(buf)         '処理モード (Cmb_File [id]) 読込
          Do While Not sr.EndOfStream
            buf = sr.ReadLine
            itm = buf.Split(CChar(vbTab))
            wPath.Add(itm(2))
          Loop
        End Using
'-------------------------------------------------------------------------------
'     ProgressBar init.
'-------------------------------------------------------------------------------
        ProgressBar.Value = 0
        ProgressBar.Visible = True

        pVal = 0.0F
        pAdd = Convert.ToSingle(100 / wPath.Count)
'-------------------------------------------------------------------------------
'     表示色クリア
'-------------------------------------------------------------------------------
        ListView.BeginUpdate()
        ListView.HideSelection = True

        ListView.MarkerClear()          '▲▼クリア
        Lsv_BackColor_Clear()
'-------------------------------------------------------------------------------
'     Undo Start
'-------------------------------------------------------------------------------
        Using sr As New StreamReader(_UndoFile, Encoding.UTF8)
        Using sw As New StreamWriter(_WorkFile, False, Encoding.UTF8)

          buf = sr.ReadLine
          Do While Not sr.EndOfStream
            buf = sr.ReadLine
            itm = buf.Split(CChar(vbTab))

            If _Lsv_DupCheckFlg Then
               Try
                 Lsv = ListView.FindItemWithText(itm(2), False, 0, False)
               Catch ex As ArgumentOutOfRangeException
                 Lsv = Nothing
               End Try
            Else
               Lsv = Nothing
            End If

            If itm(0) = "*" Then

            'リストを変更 (ChckMode 関係なし)
               If Not IsNothing(Lsv) Then
                  If PrevFolderResult <> 0 Then

                  'フォルダのUnDo に失敗している為、不整合が生じている
                     Lsv_BackColor_Set(Lsv, _BackColorFailure)
                  Else
                     Lsv_Set(0, Lsv, itm)   '更新（変更前）
                  End If
               End If
            Else
               Do
                 Result = NameUpdate(wPath(ip), itm(1), True)
               Loop While Result = DialogResult.Retry

               If Result = DialogResult.Abort Then Exit Do

               If _List_ChckMode <> 0 Then

               '結果を問わずすべて表示
                  Lsv_Set(Result, Lsv, itm)

                  Select Case Result
                    Case DialogResult.None
                      n += 1
                      Lsv_BackColor_Set(Lsv, _BackColorUndo)

                   'Logファイル書込み用の為に、Undoに成功したデータを一時保存
                      If _LogFileWriting Then sw.WriteLine(buf)

                    Case DialogResult.OK, DialogResult.Ignore
                      Lsv_BackColor_Set(Lsv, _BackColorFailure)

                    Case DialogResult.Cancel
                      Lsv_BackColor_Set(Lsv, _BackColorBreak)
                      Exit Do
                  End Select
               Else

               'Undoに失敗した場合のみ表示
                  If Result <> DialogResult.None Then
                     Lsv_Set(Result, Lsv, itm)

                     If Result = DialogResult.Cancel Then
                        Lsv_BackColor_Set(Lsv, _BackColorBreak)
                        Exit Do
                     Else
                        Lsv_BackColor_Set(Lsv, _BackColorFailure)
                     End If
                  Else
                     n += 1
                     If Not IsNothing(Lsv) Then Lsv.Remove()

                  'Logファイル書込み用の為に、Undoに成功したデータを一時保存
                     If _LogFileWriting Then sw.WriteLine(buf)

                  End If
               End If

            'フォルダ関連の処理
               FlType = Convert.ToInt32(itm(0))
               If Result = DialogResult.None Then
                  If FlType <> 0 Then

                  '作業用パスのフォルダ名変更
                     For i = ip + 1 To wPath.Count - 1
                       If wPath(i).IndexOf(wPath(ip) & "\") = 0 Then
                          wPath(i) = Replace(wPath(i), wPath(ip), itm(1), 1, 1, CompareMethod.Binary)
                       Else
                          Exit For
                       End If
                     Next

                     PrevFolderResult = 0    'フォルダのUnDo に成功
                  End If
               Else
                  If FlType <> 0 Then
                     PrevFolderResult = 1    'フォルダのUnDo に失敗
                  End If
               End If
            End If
            ip += 1
            pVal += pAdd
            ProgressBar.Value = Convert.ToInt32(pVal)
          Loop
          Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        End Using
        End Using

     'マーカークリア
        If ListView.Items.Count <> c AndAlso
           ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)

        ListView.SelectedItems.Clear()

        Lsv_DisplayMode_Control(False)

        ListView.EndUpdate()
'-------------------------------------------------------------------------------
'     Log Write
'-------------------------------------------------------------------------------
        If (n <> 0) AndAlso _LogFileWriting Then

           If File.Exists(_Log_File) Then
              If File.Exists(_TempFile) Then File.Delete(_TempFile)
              File.Move(_Log_File, _TempFile)
           Else
              Using sw As New StreamWriter(_TempFile, False, Encoding.UTF8) : End Using
           End If

           Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
           Using sr As New StreamReader(_WorkFile, Encoding.UTF8)

             If n > 1 Then
                msg = String.Format(_LblItemsFormat, n) & " --"
             Else
                msg = "--"
             End If
             sw.WriteLine("-- {0} : Undo {1}", Date.Now.ToString(_DateTimeFormat), msg)

             Do While Not sr.EndOfStream
               buf = sr.ReadLine
               itm = buf.Split(CChar(vbTab))
               sw.WriteLine("[{0}] {1}{2}--> {3}", itm(0), itm(2), vbTab, itm(1))
             Loop
             sw.WriteLine()
           End Using

           Using sr As New StreamReader(_TempFile, Encoding.UTF8)
             sw.Write(sr.ReadToEnd())
           End Using
           End Using

           File.Delete(_TempFile)
        End If

        File.Delete(_WorkFile)
'-------------------------------------------------------------------------------
'     Message Disp. 
'-------------------------------------------------------------------------------
        Me.Update()
        Me.Cursor = Cursors.Default

        If n <> 0 Then
           ProgressBar.Value = 100

           msg = String.Format("{0:#,##0} 個のファイルを復元しました。", n)
           ShowInfoMessage(msg, _App_Name)
        Else
           ShowWarningMessage("ファイルの復元に失敗しました。", "", _App_Name)
        End If

        ProgressBar.Visible = False

        ListView.HideSelection = _UseHideSelection
        ListView.Select()
    End Sub

#End Region

End Class
