﻿'
'   TabControlEx 1.0.3.1 Sample (Calling DrawItem) Ver.
'
'     Multiline = True 制限仕様
'
'       Required Class: 
'         ColorEx.vb
'
'       Property:
'         _SelectedTabColor
'
'       Method
'         Bounds  [Add, Insert, RemoveAt]
'
Imports System.ComponentModel
Imports System.Drawing.Drawing2D

Namespace UserLib

<DebuggerStepThrough()>
Public Class TabControlEx
    Inherits TabControl

    Private _onCreate As Boolean

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property Appearance() As TabAppearance = TabAppearance.Normal
    <Browsable(False)> _
    Public Shadows Property HotTrack() As Boolean = False

#End Region

#Region "** DoubleBuffer 有効"

    Private WS_EX_COMPOSITED As Integer = &H2000000

    Protected Overrides ReadOnly Property CreateParams() As CreateParams
      Get
        If Me.DesignMode Then Return MyBase.CreateParams

        Dim c As CreateParams

        c = MyBase.CreateParams
        c.ExStyle = c.ExStyle Or Me.WS_EX_COMPOSITED
        Return c
      End Get
    End Property

#End Region

#Region "** 表示"

    Private _SelectedTabColor As Color = SystemColors.Highlight

''' <summary>
''' 選択されたタブの色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("選択されたタブの色を指定します。")> _
    Public Property SelectedTabColor() As Color
      Get
        Return _SelectedTabColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _SelectedTabColor = SystemColors.Highlight
        Else
           _SelectedTabColor = value
        End If

        SetTabColor(_SelectedTabColor)

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

#End Region

#End Region

#Region "// Method"

''' <summary>
''' 領域情報をリストの末尾に追加します。
''' </summary>

    Public Sub BoundsAdd()

        _TabBounds.Add(New Rectangle(0, 0, 0, 0))
    End Sub

''' <summary>
''' 領域情報を指定した位置に追加します。
''' </summary>

    Public Sub BoundsInsert(ByVal index As Integer)

        If index < 0 OrElse index > Me.TabCount Then
           Throw New IndexOutOfRangeException
        End If

        _TabBounds.Insert(index, New Rectangle(0, 0, 0, 0))
    End Sub

''' <summary>
''' 領域情報を指定した位置から削除します。
''' </summary>

    Public Sub BoundsRemoveAt(ByVal index As Integer)

        If index < 0 OrElse index > Me.TabCount - 1 Then
           Throw New IndexOutOfRangeException
        End If

        _TabBounds.RemoveAt(index)
    End Sub

#End Region

#Region "// constant, variable declaration"

    Private _Tab_BrightColor As Color       'タブの背景〈階調）光色
    Private _Tab_HoverColor As Color        '      〃          光色
    Private _Tab_LightColor As Color        '      〃          明色
    Private _Tab_DarkColor As Color         '      〃          暗色

    Private _Tab_onMouse As Boolean
    Private _Tab_onMouseIndex As Integer = -1
    Private _Tab_onDeSelected As Boolean

    Private _TabBounds As New List(Of Rectangle)

#End Region

#Region "// Sub-Routine & Function"

''' <summary>
''' 選択したタブの色を設定します。
''' </summary>

    Private Sub SetTabColor(ByVal c As Color)
        _Tab_BrightColor = ColorEx.IncreaseBrightness(c, 0.95F)
        _Tab_HoverColor = ColorEx.IncreaseBrightness(c, 0.925F)
        _Tab_LightColor = ColorEx.IncreaseBrightness(c, 0.9F)
        _Tab_DarkColor = ColorEx.IncreaseBrightness(c, 0.75F)
    End Sub

#End Region

    <DebuggerStepThrough()>
    Public Sub New()

       'Me.Appearance = TabAppearance.Normal
       'Me.HotTrack = False
        Me.DrawMode = TabDrawMode.OwnerDrawFixed
        Me.Multiline = True

        SetTabColor(_SelectedTabColor)
    End Sub

    Protected Overrides Sub OnCreateControl()

        If Me.DesignMode Then Return

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        For i As Integer = 1 To Me.TabCount
          _TabBounds.Add(New Rectangle(0, 0, 0, 0))
        Next

        MyBase.OnCreateControl()
    End Sub

#Region "// Me"

    Private Sub Me_MouseMove(sender As Object, e As MouseEventArgs) Handles Me.MouseMove

        If Me.DesignMode Then Return

        Dim p As Point = e.Location

        For i As Integer = 0 To Me.TabCount - 1
          If _TabBounds(i).Contains(p) Then

             Dim state As DrawItemState
             Dim onMouseIndex As Integer = _Tab_onMouseIndex

             If _Tab_onMouse Then
                If _Tab_onMouseIndex = i Then Return '同じタブ上にマウスがある
                If _Tab_onMouseIndex <> -1 Then      '他のタブからマウスが移動した

                '以前マウスがあったタブの表示をクリア
                   state = DirectCast(Convert.ToInt32(_Tab_onMouseIndex = Me.SelectedIndex), DrawItemState)
                   _Tab_onMouseIndex = -1

                   Me_DrawItem(Me, New DrawItemEventArgs(Me.CreateGraphics, Me.Font, _TabBounds(onMouseIndex), onMouseIndex, state))
                End If
             Else
                _Tab_onMouse = True
             End If

          'マウスがあるタブの表示を更新
             state = DirectCast(Convert.ToInt32(i = Me.SelectedIndex), DrawItemState)
             _Tab_onMouseIndex = i

             Me_DrawItem(Me, New DrawItemEventArgs(Me.CreateGraphics, Me.Font, _TabBounds(i), i, state))
             Return
          End If
        Next
    End Sub

    Private Sub Me_MouseLeave(sender As Object, e As EventArgs) Handles Me.MouseLeave

        If _Tab_onMouseIndex <> -1 Then

        '以前マウスがあったタブの表示をクリア
           Dim state As DrawItemState = DirectCast(Convert.ToInt32(_Tab_onMouseIndex = Me.SelectedIndex), DrawItemState)
           Dim onMouseIndex As Integer = _Tab_onMouseIndex

           _Tab_onMouseIndex = -1

           Me_DrawItem(Me, New DrawItemEventArgs(Me.CreateGraphics, Me.Font, _TabBounds(onMouseIndex), onMouseIndex, state))
        End If

        _Tab_onMouse = False
    End Sub

    Private Sub Me_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        If e.Alt Then
           e.Handled = True
           e.SuppressKeyPress = True
        End If
    End Sub

    Private Sub Me_SelectedIndexChanged(sender As Object, e As EventArgs) Handles Me.SelectedIndexChanged

        If _Tab_onDeSelected Then
           _Tab_onDeSelected = False

        'レイアウト ロジックを再開
           Me.ResumeLayout(False)
        End If
    End Sub

    Private Sub Me_Deselected(sender As Object, e As TabControlEventArgs) Handles Me.Deselected

        If Not _Tab_onMouse Then Return

        _Tab_onDeSelected = True

     'ちらつきを減らす為、レイアウト ロジックを一時中断
        Me.SuspendLayout()
    End Sub

    Private Sub Me_DrawItem(ByVal sender As Object, ByVal e As DrawItemEventArgs) Handles Me.DrawItem

        Dim ctl As TabControl = DirectCast(sender, TabControl)

        Dim ReDrawState As Boolean
        Dim ReDrawColor As Color

        Dim B_rect As Rectangle '背景
        Dim L_rect As Rectangle 'Focus 枠
        Dim T_rect As Rectangle 'Text

        Dim gb As LinearGradientBrush
        Dim bl As New ColorBlend()

        Dim x As Integer = e.Bounds.X
        Dim y As Integer = e.Bounds.Y
        Dim w As Integer = e.Bounds.Width
        Dim h As Integer = e.Bounds.Height

     'Bounds BackUp
        If Not Me.DesignMode Then
           _TabBounds(e.Index) = e.Bounds
        End If

     'Back Ground
        If (e.State And DrawItemState.Selected) = DrawItemState.Selected Then

        '選択されているタブ

        '背景（ごみ）クリア
           Select Case Me.Alignment
             Case TabAlignment.Top
               B_rect = New Rectangle(x + 1, y + h - 2, w - 4, 1)
             Case TabAlignment.Bottom
               B_rect = New Rectangle(x + 1, y, w - 4, 1)
             Case TabAlignment.Left
               B_rect = New Rectangle(x + w - 2, y + 2, 1, h - 5)
             Case TabAlignment.Right
               B_rect = New Rectangle(x, y + 1, 1, h - 4)
           End Select

           e.Graphics.DrawRectangle(New Pen(SystemColors.Control), B_rect)

        '左（上）端チェック
           Dim state As Boolean = (e.Index = 0)

           If Not state Then
              If Me.RowCount > 1 Then
                 If Me.Alignment < 2 Then   'Top, Bottom
                    state = (x = 0)
                 Else                       'Left, Right
                    state = (y = 0)
                 End If
              End If
           End If

           If state Then

           '左端
              Select Case Me.Alignment
                Case TabAlignment.Top
                  B_rect = New Rectangle(x + 4, y + 2, w - 7, h - 3)
                  L_rect = New Rectangle(x + 5, y + 3, w - 9, h - 4)
                  T_rect = New Rectangle(x + 1, y + 3, w, h)

                Case TabAlignment.Bottom
                  B_rect = New Rectangle(x + 4, y, w - 7, h - 3)
                  L_rect = New Rectangle(x + 5, y, w - 9, h - 4)
                  T_rect = New Rectangle(x + 1, y, w, h)

                Case TabAlignment.Left
                  B_rect = New Rectangle(x + 2, y + 4, w - 3, h - 7)
                  L_rect = New Rectangle(x + 3, y + 5, w - 4, h - 9)
                  T_rect = New Rectangle(x - 1, y, w, h)

                Case TabAlignment.Right
                  B_rect = New Rectangle(x, y + 4, w - 3, h - 7)
                  L_rect = New Rectangle(x, y + 5, w - 4, h - 9)
                  T_rect = New Rectangle(x - 4, y, w, h)

              End Select
           Else

           '左端以外
              Select Case Me.Alignment
                Case TabAlignment.Top
                  B_rect = New Rectangle(x + 2, y + 2, w - 5, h - 3)
                  L_rect = New Rectangle(x + 3, y + 3, w - 7, h - 4)
                  T_rect = New Rectangle(x, y + 3, w, h)

                Case TabAlignment.Bottom
                  B_rect = New Rectangle(x + 2, y, w - 5, h - 3)
                  L_rect = New Rectangle(x + 3, y, w - 7, h - 4)
                  T_rect = New Rectangle(x, y, w, h)

                Case TabAlignment.Left
                  B_rect = New Rectangle(x + 2, y + 2, w - 3, h - 5)
                  L_rect = New Rectangle(x + 3, y + 3, w - 4, h - 7)
                  T_rect = New Rectangle(x - 1, y, w, h)

                Case TabAlignment.Right
                  B_rect = New Rectangle(x, y + 2, w - 3, h - 5)
                  L_rect = New Rectangle(x, y + 3, w - 4, h - 7)
                  T_rect = New Rectangle(x - 4, y, w, h)

              End Select
           End If

           If Me.SizeMode <> TabSizeMode.Fixed Then

           '右（下）端のタブ表示補正
              If Me.Alignment < 2 Then          'Top, Bottom
                 If Me.SizeMode = TabSizeMode.Normal Then
                    If x + w >= Me.Width - 1 Then
                       B_rect.Width -= 1
                       L_rect.Width -= 1
                    End If
                 Else                           'FillToRight
                    If x + w >= Me.Width Then
                       B_rect.Width -= 2
                       L_rect.Width -= 2
                       T_rect.X -= 1
                    End If
                 End If
              Else                              'Left, Right
                 If Me.SizeMode = TabSizeMode.Normal Then
                    If y + h >= Me.Height - 1 Then
                       B_rect.Height -= 1
                       L_rect.Height -= 1
                    End If
                 Else                           'FillToRight
                    If y + h >= Me.Height Then
                       B_rect.Height -= 2
                       L_rect.Height -= 2
                       T_rect.Y -= 1
                    End If
                 End If
              End If
           End If

           e.Graphics.DrawRectangle(New Pen(ctl.TabPages(e.Index).BackColor), B_rect)

           bl.Positions = New Single() {0.0F, 0.4F, 1.0F}
           bl.Colors = New Color() {_Tab_BrightColor, _Tab_LightColor, _Tab_DarkColor}

        Else

        '選択されていないタブ

           Select Case Me.Alignment
             Case TabAlignment.Top
               L_rect = New Rectangle(x, y + 2, w - 1, h - 1)
               T_rect = New Rectangle(x, y + 3, w, h)

             Case TabAlignment.Bottom
               L_rect = New Rectangle(x, y - 1, w - 1, h - 1)
               T_rect = New Rectangle(x, y, w, h)

             Case TabAlignment.Left
               L_rect = New Rectangle(x + 2, y, w - 1, h - 1)
               T_rect = New Rectangle(x - 1, y - 1, w, h)

             Case TabAlignment.Right
               L_rect = New Rectangle(x - 1, y, w - 2, h - 1)
               T_rect = New Rectangle(x - 4, y - 1, w, h)

           End Select

           ReDrawState = True
           bl.Positions = New Single() {0.0F, 1.0F}

           If _Tab_onMouse AndAlso (e.Index = _Tab_onMouseIndex) Then
              ReDrawColor = _Tab_HoverColor
              bl.Colors = New Color() {_Tab_HoverColor, _Tab_LightColor}
           Else
              ReDrawColor = SystemColors.ControlLightLight
              bl.Colors = New Color() {SystemColors.ControlLightLight, SystemColors.Control}
           End If
        End If

     'タブ背景
        Dim lgmode As LinearGradientMode

        Select Case Me.Alignment
          Case TabAlignment.Left
            lgmode = LinearGradientMode.ForwardDiagonal
          Case TabAlignment.Right
            lgmode = LinearGradientMode.BackwardDiagonal
          Case Else
            lgmode = LinearGradientMode.Vertical
        End Select
        gb = New LinearGradientBrush(L_rect, Color.Black, Color.White, lgmode)

        gb.InterpolationColors = bl
        e.Graphics.FillRectangle(gb, L_rect)

     'Focus 枠
        If (e.State And DrawItemState.Selected) = DrawItemState.Selected Then
           e.Graphics.DrawRectangle(New Pen(_SelectedTabColor), L_rect)
        Else

        '120 Dpi 時にグラデーション描画の上端１ラインが正しく描画されない事への措置
           If ReDrawState Then
              e.Graphics.DrawLine(New Pen(ReDrawColor), L_rect.X, L_rect.Y, L_rect.X + L_rect.Width - 1, L_rect.Y)
           End If
        End If

     'Text
        Dim sb As Brush
        Dim sf As New StringFormat

        If Me.Alignment < 2 Then
        'Top, Bottom
           sf.FormatFlags = StringFormatFlags.NoWrap
        Else
        'Left, Right
           sf.FormatFlags = StringFormatFlags.NoWrap Or StringFormatFlags.DirectionVertical
        End If

        sf.LineAlignment = StringAlignment.Center
        sf.Alignment = StringAlignment.Center

        If _Tab_onMouse AndAlso (e.Index = _Tab_onMouseIndex) Then
           sb = New SolidBrush(_SelectedTabColor)
        Else
           sb = New SolidBrush(SystemColors.ControlText)
        End If

        e.Graphics.DrawString(ctl.TabPages(e.Index).Text.Trim, e.Font, sb, RectangleF.op_Implicit(T_rect), sf)

        sb.Dispose()
        sf.Dispose()
        gb.Dispose()

        sb = Nothing
        sf = Nothing
        gb = Nothing
    End Sub

#End Region

End Class

End Namespace
