﻿'
'   Gradation Button 1.6.0.1 (Classic & Visual Style Consolidated edition)
'
'       Required Class: 
'         ColorEx.vb
'         WinMessage.vb
'
'       Property:
'         AllowDoubleClick
'         DisabledColorState
'         HotkeyPrefix
'
'         ImagePadding
'         TextPadding
'         TextLineAdjust
'
'         GradationEffect
'         DisabledColor
'         HoverColor
'         HoverEffect
'
'       Method
'         None
'
'   ※  TextAlign に Middle (垂直方向の中央揃え)を指定している場合、文字の内容に応じて自動調整されます。
'
'    (注意事項) 
'       フォームイメージ確認用の為に、FlatStyle の変更は可能ですが、
'       Standard 以外では機能が制限されます。
'
'       コントロール移行時に Padding の値を TextPadding, ImagePadding へ 値を移しますので
'       適宜修正して下さい。

'       下記のプロパティは FlatStyle が Standard の時は無効です。
'       また、デザイナでの設定もできません。
'
'         AutoEllipsis
'         BackgroundImage
'         BackgroundImageLayout
'         Padding
'         RightToLeft
'         TextimageRelation
'         UseCompatibleTextRendering
'         UseVisualStyleBackColor
'
'   ※  AutoSize を True にすると異常動作(仕様？）します。
'
Imports System.ComponentModel
Imports System.Drawing.Drawing2D
Imports System.Drawing.Text
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices

Namespace UserLib

<DebuggerStepThrough()>
Public Class ButtonEx
    Inherits Button

Public Enum Gradation As Integer
    None = -3
    Simple = -2
    Weak = -1
    Normal = 0
    Strength = 1
End Enum

Public Enum Hover As Integer
    DarkDark = -2
    Dark = -1
    Normal = 0
    Light = 1
    LightLight = 2
End Enum

#Region "// constant, variable declaration"

    Private Const _InternaldPadding = 4
    Private Const _InternalOffset = 2

    Private _TextHeight As Integer
    Private _AutoTextAdjust As Integer

    Private _BackImage(1) As Drawing.Image

    Private _onCreate As Boolean
    Private _onClick As Boolean
    Private _onMouse As Boolean

    Private _pt_OutTop As Point()
    Private _pt_OutBottom As Point()
    Private _pt_InTop1 As Point()
    Private _pt_InTop2 As Point()
    Private _pt_InBottom As Point()

    Private _cl_FaceLight As Color  'Control           (Light)
    Private _cl_Highlight0 As Color 'ControlLightLight
    Private _cl_Highlight1 As Color 'ControlLightLight (Light)
    Private _cl_Highlight2 As Color 'ControlLightLight (LightLight)
    Private _cl_Shadow0 As Color    'ControlDark
    Private _cl_Shadow1 As Color    'ControlDark       (Light)
    Private _cl_Shadow2 As Color    'ControlDark       (LightLight)
                                    'ControlDarkDark   (RGB:64,64,64) System固定

    Private _MatrixScale() As Single = {0.9F, 0.95F, 1.02F, 1.05F, 1.08F}

#End Region

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AutoEllipsis() As Boolean
    <Browsable(False)> _
    Public Shadows Property UseCompatibleTextRendering() As Boolean

    <Browsable(False)> _
    Public Shadows Property Padding() As Padding

    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout
    <Browsable(False)> _
    Public Shadows Property RightToLeft() As RightToLeft
    <Browsable(False)> _
    Public Shadows Property TextImageRelation() As TextImageRelation
    <Browsable(False), DefaultValue(False)> _
    Public Shadows Property UseVisualStyleBackColor() As Boolean

#End Region

    Private _AllowDoubleClick As Boolean = True
    Private _DisabledColorState As Boolean = True
    Private _HotkeyPrefix As HotkeyPrefix
    Private _ImagePadding As Padding
    Private _TextPadding As Padding
    Private _TextAlign As ContentAlignment = ContentAlignment.MiddleCenter
    Private _TextLineAdjust As Integer
    Private _GradationEffect As Gradation
    Private _HoverEffect As Hover
    Private _DisabledColor As Color = SystemColors.GrayText
    Private _HoverColor As Color = SystemColors.Highlight

#Region "** 動作"

''' <summary>
''' ボタンの２度押しを許可するかを決定します。
''' </summary>

    <DefaultValue(True), Category("動作"),
     Description("ボタンの２度押しを許可するかを決定します。")> _
    Public Property AllowDoubleClick() As Boolean
      Get
        Return _AllowDoubleClick
      End Get

      Set(ByVal value As Boolean)
        _AllowDoubleClick = value
      End Set
    End Property

#End Region

#Region "** 配置"

''' <summary>
''' コントロールの内部スペース（対象：イメージ）を指定します。
''' </summary>

    <DefaultValue(GetType(Padding), "0,0,0,0"), Category("配置"),
     Description("コントロールの内部スペース（対象：イメージ）を指定します。")> _
    Public Property ImagePadding() As Padding
      Get
        Return _ImagePadding
      End Get

      Set(ByVal value As Padding)
        If value.Left < 0 Then value.Left = 0
        If value.Top < 0 Then value.Top = 0
        If value.Right < 0 Then value.Right = 0
        If value.Bottom < 0 Then value.Bottom = 0

        _ImagePadding = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' 行の開始位置補正値を指定します。
''' </summary>

    <DefaultValue(0), Category("配置"),
     Description("行の開始位置補正値（対象：テキスト）を指定します。")> _
    Public Property TextLineAdjust() As Integer
      Get
        Return _TextLineAdjust
      End Get

      Set(ByVal value As Integer)
        If value < -2 Then
           value = -2
        ElseIf value > 2 Then
           value = 2
        End If

        _TextLineAdjust = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' コントロールの内部スペース（対象：テキスト）を指定します。
''' </summary>

    <DefaultValue(GetType(Padding), "0,0,0,0"), Category("配置"),
     Description("コントロールの内部スペース（対象：テキスト）を指定します。")> _
    Public Property TextPadding() As Padding
      Get
        Return _TextPadding
      End Get

      Set(ByVal value As Padding)
        If value.Left < 0 Then value.Left = 0
        If value.Top < 0 Then value.Top = 0
        If value.Right < 0 Then value.Right = 0
        If value.Bottom < 0 Then value.Bottom = 0

        _TextPadding = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

#End Region

#Region "** 表示"

''' <summary>
''' Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）
''' </summary>

    <DefaultValue(GetType(Color), "GrayText"), Category("表示"),
     Description("Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）")> _
    Public Property DisabledColor() As Color
      Get
        Return _DisabledColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           _DisabledColor = SystemColors.GrayText
        Else
           _DisabledColor = value
        End If
      End Set
    End Property

''' <summary>
''' Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。
''' </summary>

    <DefaultValue(True), Category("表示"),
     Description("Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。")> _
    Public Property DisabledColorState() As Boolean
      Get
        Return _DisabledColorState
      End Get

      Set(ByVal value As Boolean)
        _DisabledColorState = value

        If _onCreate AndAlso (Not Me.Enabled) Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' ボタンの背景色のグラデーションの強さを指定します。
''' </summary>

    <DefaultValue(Gradation.Normal), Category("表示"),
     Description("ボタンの背景色のグラデーションの強さを指定します。")> _
    Public Property GradationEffect() As Gradation
      Get
        Return _GradationEffect
      End Get

      Set(ByVal value As Gradation)
        If Not [Enum].IsDefined(GetType(Gradation), value) Then
           value = Gradation.Normal
        End If
        _GradationEffect = value

        SetGradationImage()

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' ホット キー プリフィックスの表示の種類を指定します。
''' </summary>

    <DefaultValue(HotkeyPrefix.None), Category("表示"),
     Description("ホット キー プリフィックスの表示の種類を指定します。")> _
    Public Property HotkeyPrefix() As HotkeyPrefix
      Get
        Return _HotkeyPrefix
      End Get

      Set(ByVal value As HotkeyPrefix)
        If Not [Enum].IsDefined(GetType(HotkeyPrefix), value) Then
           value = HotkeyPrefix.None
        End If

        _HotkeyPrefix = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' ボタン上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("ボタン上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor() As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If

        If _onCreate AndAlso _onMouse Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' ボタン上にマウスカーソルがある時のホバーの明るさを指定します。
''' </summary>

    <DefaultValue(Hover.Normal), Category("表示"),
     Description("ボタン上にマウスカーソルがある時のホバーの明るさを指定します。")> _
    Public Property HoverEffect() As Hover
      Get
        Return _HoverEffect
      End Get

      Set(value As Hover)
        If Not [Enum].IsDefined(GetType(Hover), value) Then
           value = Hover.Normal
        End If

        _HoverEffect = value

        SetGradationImage()

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' ボタン コントロールのテキストの配置を取得または設定します。
''' </summary>

    <DefaultValue(ContentAlignment.MiddleCenter), Category("表示"),
     Description("コントロールに表示される文字列の配置です。")> _
    Public Overloads Property TextAlign As ContentAlignment
      Get
        Return _TextAlign
      End Get

      Set(value As ContentAlignment)
        _TextAlign = value

        If _onCreate Then
           SetAutoTextAdjust()

           Me.Invalidate()
        End If
      End Set
    End Property

#End Region

#End Region

#Region "// Function, Sub"

''' <summary>
''' コントロールの Region を設定します。
''' </summary>

    Private Sub SetRegion()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

''' <summary>
''' BackColorから、各コントロール色を生成します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub SetButtonColor()
        Dim H As Single = Me.BackColor.GetHue
        Dim S As Single = Me.BackColor.GetSaturation
        Dim B As Single = Me.BackColor.GetBrightness

        Dim c0 As Single = 1.0F - B
        Dim c1 As Single = B * 0.665F
        Dim c2 As Single = B - c1

        If c0 > 0.4F Then
           c0 = 0.4F
        End If

        _cl_FaceLight = ColorEx.ColorHSVtoRGB(H, S, B + (c0 * 0.15F))
        _cl_Highlight0 = ColorEx.ColorHSVtoRGB(H, S, B + (c0 * 0.3F))
        _cl_Highlight1 = ColorEx.ColorHSVtoRGB(H, S, B + (c0 * 0.45F))
        _cl_Highlight2 = ColorEx.ColorHSVtoRGB(H, S, B + (c0 * 0.75F))
        _cl_Shadow0 = ColorEx.ColorHSVtoRGB(H, S, c1)
        _cl_Shadow1 = ColorEx.ColorHSVtoRGB(H, S, c1 + (c2 * 0.4F))
        _cl_Shadow2 = ColorEx.ColorHSVtoRGB(H, S, c1 + (c2 * 0.8F))
    End Sub

''' <summary>
''' ボタンのグラデーションイメージを設定します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub SetGradationImage()

        If (Me.Width <= _InternaldPadding) OrElse (Me.Height <= _InternaldPadding) Then
           _BackImage(0) = Nothing
           _BackImage(1) = Nothing

           Return
        End If

        Dim w As Integer = Me.Width - _InternaldPadding
        Dim h As Integer = Me.Height - _InternaldPadding

        Dim r As New Rectangle(0, 0, w, h)
        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        Dim n As Integer
        Dim p1, p2 As Single
        Dim c0, c1, c2 As Color

        Dim v(,) As Single = {{0.4, 0.6}, {0.3, 0.7}, {0.4, 0.5}, {0.3, 0.6}, {0.375, 0.5}, {0.275, 0.6}, {0.35, 0.5}, {0.25, 0.6}}

        If _GradationEffect = Gradation.None Then
           g.FillRectangle(New SolidBrush(Me.BackColor), r)

           _BackImage(0) = DirectCast(b.Clone, Drawing.Image)
        Else
           If _GradationEffect = Gradation.Simple Then
              p1 = 0.4F
              p2 = 0.4F
           Else
              If h >= 22 Then
                 n = 6
              ElseIf h > 18 Then
                 n = 4
              ElseIf h > 14 Then
                 n = 2
              End If

              p1 = v(n, 0)
              p2 = v(n, 1)
           End If

           c0 = _cl_FaceLight

           If (_GradationEffect = Gradation.Simple) OrElse (h <= 16) Then
              c1 = _cl_FaceLight
           ElseIf _GradationEffect = Gradation.Strength Then
              c1 = _cl_Highlight1
           Else
              c1 = _cl_Highlight0
           End If

           If _GradationEffect = Gradation.Strength Then
              c2 = _cl_Shadow0
           ElseIf _GradationEffect = Gradation.Weak Then
              c2 = Me.BackColor
           Else
              c2 = _cl_Shadow2
           End If

           Dim Brush As New LinearGradientBrush(r, Color.Black, Color.White, LinearGradientMode.Vertical)
           Dim Blend As New ColorBlend()

           Blend.Positions = New Single() {0, p1, p2, 1}
           Blend.Colors = New Color() {c0, c1, c1, c2}

           Brush.InterpolationColors = Blend
           g.FillRectangle(Brush, r)

           _BackImage(0) = DirectCast(b.Clone, Drawing.Image)

           Brush.Dispose()
           Brush = Nothing
        End If

        Dim cm As New ColorMatrix()
        Dim ia As New ImageAttributes()

        p1 = _MatrixScale(_HoverEffect + 2)

        cm.Matrix00 = p1
        cm.Matrix11 = p1
        cm.Matrix22 = p1
        cm.Matrix33 = 1.0F
        cm.Matrix44 = 1.0F

        ia.SetColorMatrix(cm)

        g.DrawImage(b, r, 0, 0, b.Width, b.Height, GraphicsUnit.Pixel, ia)

        _BackImage(1) = DirectCast(b.Clone, Drawing.Image)

        ia.Dispose()
        g.Dispose()
        b.Dispose()

        ia = Nothing
        g = Nothing
        b = Nothing
    End Sub

''' <summary>
''' ボタンの枠を描画するための座標をセットします。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub SetDrawPoint()

        Dim w As Integer = Me.Width - 1
        Dim h As Integer = Me.Height - 1

        _pt_OutTop = {New Point(1, h - 1), New Point(0, h - 1), New Point(0, 1), New Point(1, 1),
                      New Point(1, 0), New Point(w - 1, 0), New Point(w - 1, 1)}

        _pt_OutBottom = {New Point(w, 1), New Point(w, h - 1), New Point(w - 1, h - 1),
                        New Point(w - 1, h), New Point(1, h)}

        _pt_InTop1 = {New Point(2, h - 2), New Point(1, h - 2), New Point(1, 2), New Point(2, 2),
                      New Point(2, 1), New Point(w - 2, 1), New Point(w - 2, 2)}

        _pt_InTop2 = {New Point(1, h - 2), New Point(1, 2), New Point(2, 2),
                      New Point(2, 1), New Point(w - 2, 1)}

        _pt_InBottom = {New Point(w - 1, 2), New Point(w - 1, h - 2), New Point(w - 2, h - 2),
                       New Point(w - 2, h - 1), New Point(2, h - 1)}
    End Sub

''' <summary>
''' TextAlign が Middle 時のテキストの位置（垂直方向）の調整値を取得します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub SetAutoTextAdjust()

        Dim w As Integer = Me.Width - Me.Padding.Horizontal - _InternaldPadding
        Dim h As Integer = Me.Height - Me.Padding.Vertical - _InternaldPadding

     'TextAlignがMiddleの時、位置を調整
        If (_TextAlign And 112) <> 0 AndAlso (h >= Me.Font.Height) AndAlso (w > 1) Then

           Dim b As New Bitmap(w, h)
           Dim g As Graphics = Graphics.FromImage(b)
           Dim c As Color = b.GetPixel(0, 0)

           Dim sf As New StringFormat

           If (Me.TextAlign And 273) <> 0 Then          'Left
              sf.Alignment = StringAlignment.Near
           ElseIf (Me.TextAlign And 546) <> 0 Then      'Center
              sf.Alignment = StringAlignment.Center
           Else                                         'Right
              sf.Alignment = StringAlignment.Far
           End If

           sf.LineAlignment = StringAlignment.Center
           sf.FormatFlags = StringFormatFlags.LineLimit
           sf.HotkeyPrefix = _HotkeyPrefix

           g.DrawString(Me.Text, Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

           Dim BackColorArgb As Integer = c.ToArgb()

           Dim y0 As Integer = -1
           Dim y1 As Integer = -1

           For yp As Integer = 0 To h - 1
             For xp As Integer = 0 To w - 1
               If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                  y0 = yp
                  Exit For
               End If
             Next
             If y0 >= 0 Then Exit For
           Next

           If y0 = -1 Then
             _AutoTextAdjust = 0
           Else
              For yp As Integer = h - 1 To y0 Step -1
                For xp As Integer = 0 To w - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     y1 = yp
                     Exit For
                  End If
                Next
                If y1 >= 0 Then Exit For
              Next

              Dim TextHeight As Integer = (y1 - y0) + 1
              Dim TopMargin As Integer = Convert.ToInt32(Math.Ceiling((h - TextHeight) / 2))

              _AutoTextAdjust = TopMargin - y0
           End If

           sf.Dispose()
           g.Dispose()
           b.Dispose()

           sf = Nothing
           g = Nothing
           b = Nothing
        Else
           _AutoTextAdjust = 0
        End If
'Console.WriteLine("{0}/ Me {1} fh {2} th {3}", _AutoTextAdjust, Me.Height, Me.Font.Height, _TextHeight)
    End Sub

#End Region

    <DebuggerStepThrough()>
    Public Sub New()

     '使用しないプロパテｨをリセット
        MyBase.UseVisualStyleBackColor = False
    End Sub

    <DebuggerStepThrough()>
    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

   '(デザイナーの設定修正)

     'コントロール移行時に元データを移す
        If Not Me.Padding.Size.IsEmpty Then
           Me.TextPadding = Me.Padding
           Me.ImagePadding = Me.Padding
           Me.Padding = Nothing
        End If

        SetRegion()
        SetDrawPoint()
        SetButtonColor()
        SetGradationImage()
        SetAutoTextAdjust()

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler Me.EnabledChanged, AddressOf Me_EnabledChanged

        AddHandler Me.SystemColorsChanged, AddressOf Me_System_BackColorChanged
        AddHandler Me.BackColorChanged, AddressOf Me_System_BackColorChanged
        AddHandler Me.FontChanged, AddressOf Me_Font_TextChanged
        AddHandler Me.TextChanged, AddressOf Me_Font_TextChanged

        MyBase.OnCreateControl()
    End Sub

    <DebuggerStepThrough()>
    Protected Overrides Sub OnClick(ByVal e As EventArgs)

        If Not _AllowDoubleClick Then
           WinMessage.Clear()
        End If

        MyBase.OnClick(e)
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        If Me.FlatStyle <> FlatStyle.Standard Then
           MyBase.OnPaint(e)
           Return
        End If

        Dim sb As SolidBrush
        Dim p1 As Pen
        Dim p2 As Pen
        Dim p3 As Pen
        Dim p4 As Pen

        Dim w, h As Integer

     'ボタン表面描画
        If Me.Enabled Then

           Dim isButtonControl As Boolean

           If Not Me.DesignMode Then

           'ボタンがアクセプトボタンか調べる

              If Not (Me.FindForm.AcceptButton Is Nothing) Then
                 If Me.Name = DirectCast(Me.FindForm.AcceptButton, Button).Name Then

                    Dim ActiveControlType As Type = Me.FindForm.ActiveControl.GetType

                    isButtonControl = Not ((ActiveControlType.Name = "Button") OrElse
                                           (ActiveControlType.BaseType.Name = "Button"))
                 End If
              End If
           End If

           If _onClick OrElse Me.Focused OrElse isButtonControl Then
              If _onMouse Then
                 p1 = New Pen(_HoverColor)
              Else
                 p1 = New Pen(Me.ForeColor)
              End If
              p2 = p1
           Else
              p1 = New Pen(_cl_Shadow0)
              If (_GradationEffect = Gradation.Weak) OrElse
                 (_GradationEffect = Gradation.Simple) Then
                 p2 = New Pen(_cl_Shadow0)
              Else
                 p2 = New Pen(SystemColors.ControlDarkDark)
              End If
           End If

           If _onClick Then
              p3 = New Pen(_cl_Shadow1)
           Else
              p3 = New Pen(_cl_Highlight0)
           End If

           If (_GradationEffect = Gradation.Weak) OrElse
              (_GradationEffect = Gradation.Simple) Then
              p4 = New Pen(_cl_Shadow2)
           ElseIf _GradationEffect = Gradation.Normal Then
              p4 = New Pen(_cl_Shadow1)
           Else
              p4 = New Pen(_cl_Shadow0)
           End If

           If _onMouse Then
              sb = New SolidBrush(_HoverColor)
           Else
              sb = New SolidBrush(Me.ForeColor)
           End If

           Dim i As Integer = Convert.ToInt32(_onMouse)

           If _BackImage(i) IsNot Nothing Then
              e.Graphics.DrawImage(_BackImage(i), New Point(_InternalOffset, _InternalOffset))
           End If
        Else
           w = Me.Width - _InternaldPadding
           h = Me.Height - _InternaldPadding

           e.Graphics.FillRectangle(New SolidBrush(_cl_Highlight0),
                                    New Rectangle(_InternalOffset, _InternalOffset, w, h))

           sb = New SolidBrush(_cl_Highlight0)
           p1 = New Pen(_cl_Shadow0)
           p2 = p1
           p3 = New Pen(_cl_Highlight2)
           p4 = p3
        End If

     'Image描画
        Dim x, y As Integer
        Dim ClickAdjust As Integer = Convert.ToInt32(_onClick)  'クリック時 のラベル、Image 座標補正値

        If Me.Image IsNot Nothing Then

           If (Me.ImageAlign And 273) <> 0 Then         'Left
              x = _ImagePadding.Left + ClickAdjust
           ElseIf (Me.ImageAlign And 546) <> 0 Then     'Center
              x = _ImagePadding.Left + (Me.Width - _ImagePadding.Horizontal - Me.Image.Width) \ 2 + ClickAdjust
           Else                                         'Right
              x = (Me.Width - _ImagePadding.Right - Me.Image.Width) + ClickAdjust
           End If

           If (Me.ImageAlign And 7) <> 0 Then           'Top
              y = _ImagePadding.Top + ClickAdjust
           ElseIf (Me.ImageAlign And 112) <> 0 Then     'Center
              y = _ImagePadding.Top + (Me.Height - _ImagePadding.Vertical - Me.Image.Height) \ 2 + ClickAdjust
           Else                                         'Bottom
              y = (Me.Height - _ImagePadding.Bottom - Me.Image.Height) + ClickAdjust
           End If

           If Me.Enabled Then
              e.Graphics.DrawImage(Me.Image, New Rectangle(x, y, Me.Image.Width, Me.Image.Height))
           Else
              ControlPaint.DrawImageDisabled(e.Graphics, Me.Image, x, y, _cl_Highlight0)
           End If
        End If

     'Text描画
        If (Me.Text.Trim.Length > 0) AndAlso (Me.Width > 8) AndAlso (Me.Height > 8) Then

           Dim sf As New StringFormat

           If (Me.TextAlign And 273) <> 0 Then          'Left
              sf.Alignment = StringAlignment.Near
           ElseIf (Me.TextAlign And 546) <> 0 Then      'Center
              sf.Alignment = StringAlignment.Center
           Else                                         'Right
              sf.Alignment = StringAlignment.Far
           End If

           If (Me.TextAlign And 7) <> 0 Then            'Top
              sf.LineAlignment = StringAlignment.Near
           ElseIf (Me.TextAlign And 112) <> 0 Then      'Center
              sf.LineAlignment = StringAlignment.Center
           Else                                         'Bottom
              sf.LineAlignment = StringAlignment.Far
           End If

           x = _TextPadding.Left + ClickAdjust + _InternalOffset
           y = _TextPadding.Top + ClickAdjust + _InternalOffset

           w = Me.Width - _TextPadding.Horizontal - _InternaldPadding
           h = Me.Height - _TextPadding.Vertical - _InternaldPadding

           sf.FormatFlags = StringFormatFlags.LineLimit
           sf.HotkeyPrefix = _HotkeyPrefix

           y += (_AutoTextAdjust + _TextLineAdjust)

           If Me.Enabled Then
              e.Graphics.DrawString(Me.Text, Me.Font, sb, New RectangleF(x, y, w, h), sf)
           Else
              If _DisabledColorState Then
                 ControlPaint.DrawStringDisabled(e.Graphics, Me.Text, Me.Font,
                   Me.BackColor,
                   New RectangleF(x, y, w, h), sf)
              Else
                 sb.Color = _DisabledColor
                 e.Graphics.DrawString(Me.Text, Me.Font, sb, New RectangleF(x, y, w, h), sf)
              End If
           End If

           sf.Dispose()
           sf = Nothing
        End If

     'ボタン枠描画
        e.Graphics.DrawLines(p1, _pt_OutTop)
        e.Graphics.DrawLines(p2, _pt_OutBottom)
        e.Graphics.DrawLines(p4, _pt_InBottom)

        If _onClick OrElse Not Me.Enabled Then
           e.Graphics.DrawLines(p3, _pt_InTop1)
        Else
           e.Graphics.DrawLines(p3, _pt_InTop2)
        End If

        sb.Dispose()
        p1.Dispose()
        p2.Dispose()
        p3.Dispose()
        p4.Dispose()

        sb = Nothing
        p1 = Nothing
        p2 = Nothing
        p3 = Nothing
        p4 = Nothing
    End Sub

#Region "// Me"

    <DebuggerStepThrough()>
    Private Sub Me_System_BackColorChanged(ByVal sender As Object, ByVal e As EventArgs)

        SetButtonColor()
        SetGradationImage()
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_Font_TextChanged(ByVal sender As Object, ByVal e As EventArgs)
        SetAutoTextAdjust()
    End Sub

    Private Sub Me_EnabledChanged(ByVal sender As Object, ByVal e As EventArgs)

        If Me.Enabled Then
           If Not _onMouse Then
              If Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position)) Then
                 _onMouse = True

                 Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))

              'MouseEnter を発生
                 Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
              End If
           End If
        Else
           _onMouse = False
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_Resize(ByVal sender As Object, ByVal e As EventArgs)

        SetRegion()
        SetDrawPoint()
        SetGradationImage()
        SetAutoTextAdjust()
    End Sub

    <DebuggerStepThrough()>
    Private Sub ButtonEx_LostFocus(sender As Object, e As EventArgs) Handles Me.LostFocus

        _onClick = False
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseEnter(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseEnter

        _onMouse = True
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseLeave

        _onMouse = False
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseDown(ByVal sender As Object, ByVal e As MouseEventArgs) Handles Me.MouseDown

        If e.Button = MouseButtons.Left Then
           _onClick = True
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseUp(ByVal sender As Object, ByVal e As MouseEventArgs) Handles Me.MouseUp

        _onClick = False

        Me.Invalidate()
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs) Handles Me.KeyDown

        If e.KeyData = Keys.Space Then _onClick = True
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_KeyUp(ByVal sender As Object, ByVal e As KeyEventArgs) Handles Me.KeyUp

        _onClick = False

        Me.Invalidate()
    End Sub

#End Region

End Class

End Namespace
