﻿Namespace UserLib

Public Class ColorEx

''' <summary>
''' 色値をHSV(HSB)からRGBへ変換します。
''' </summary>

    <DebuggerStepThrough()> _
    Public Shared Function ColorHSVtoRGB(ByVal H As Single, ByVal S As Single, ByVal V As Single) As Color
        If H < 0.0F Then
           H = 0.0F
        ElseIf H >= 360.0F Then
           H = 359.9999F
        End If

        If S < 0.0F Then
           S = 0.0F
        ElseIf S > 1.0F Then
           S = 1.0F
        End If

        If V < 0.0F Then
           V = 0.0F
        ElseIf V > 1.0F Then
           V = 1.0F
        End If

        Dim R As Single, G As Single, B As Single
        If S = 0 Then
            R = V
            G = V
            B = V
        Else
            Dim d As Single = H / 60.0F
            Dim i As Integer = Convert.ToInt32(Math.Floor(d))   '=0-5
            Dim f As Single = d - i                             '=0-0.9999
            Dim c As Single

            If V < 0.5F Then
                c = 2.0F * S * V
            Else
                c = 2.0F * S * (1.0F - V)
            End If

            Dim m As Single = V - c / 2.0F
            Dim p As Single = c + m
            Dim q As Single

            If i Mod 2 = 0 Then
                q = V + c * (f - 0.5F)
            Else
                q = V - c * (f - 0.5F)
            End If

            Select Case i
                Case 0 : R = p : G = q : B = m
                Case 1 : R = q : G = p : B = m
                Case 2 : R = m : G = p : B = q
                Case 3 : R = m : G = q : B = p
                Case 4 : R = q : G = m : B = p
                Case 5 : R = p : G = m : B = q
            End Select
        End If

        Return Color.FromArgb(Convert.ToInt32(Math.Round(R * 255.0F)), _
                              Convert.ToInt32(Math.Round(G * 255.0F)), _
                              Convert.ToInt32(Math.Round(B * 255.0F)))
    End Function

''' <summary>
''' 指定した乗率で輝度を変更（明るく）します。
''' </summary>

    <DebuggerStepThrough()> _
    Public Shared Function IncreaseBrightness(ByVal color As Color, mpf As Single) As Color

        Dim H As Single = color.GetHue
        Dim S As Single = color.GetSaturation
        Dim V As Single = color.GetBrightness

        V = V + ((1.0F - V) * mpf)

        Return ColorHSVtoRGB(H, S, V)
    End Function

End Class

End Namespace
