﻿Imports System.IO
Imports System.Reflection
Imports System.Runtime.InteropServices
Imports System.Text
Imports System.Text.RegularExpressions

Module Generic_Module

'*******************************************************************************
'     Common Sub-Routine & Function
'*******************************************************************************

#Region "// Ini File Read/Write"

    <DllImport("KERNEL32.DLL", CharSet:=CharSet.Auto)> _
    Public Function WritePrivateProfileString( _
        ByVal lpAppName As String, _
        ByVal lpKeyName As String, _
        ByVal lpString As String, _
        ByVal lpFileName As String
        ) As Integer
    End Function

    <DllImport("KERNEL32.DLL", CharSet:=CharSet.Auto)> _
    Public Function GetPrivateProfileString( _
        ByVal lpAppName As String, _
        ByVal lpKeyName As String, _
        ByVal lpDefault As String, _
        ByVal lpReturnedString As StringBuilder, _
        ByVal nSize As Integer, _
        ByVal lpFileName As String _
        ) As Integer
    End Function

    <DllImport("KERNEL32.DLL", CharSet:=CharSet.Auto)> _
    Public Function GetPrivateProfileInt( _
        ByVal lpAppName As String, ByVal lpKeyName As String, _
        ByVal nDefault As Integer, ByVal lpFileName As String _
        ) As Integer
    End Function

#End Region

'-- Generic function -----------------------------------------------------------

''' <summary>
''' OS のタイプを調べます。（Windows 8 以降のOSの場合 True を返します。)
''' </summary> 

    <DebuggerStepThrough()> _
    Public Function GetOsType() As Boolean
        Dim OS As OperatingSystem = System.Environment.OSVersion
        Dim OS_Ver As Single = Convert.ToSingle(OS.Version.Major + (OS.Version.Minor / 10))

        Return (OS_Ver > 6.1F)
    End Function

''' <summary>
''' 120Dpi 時の適正表示文字サイズを求めます。
''' </summary>

    <DebuggerStepThrough()> _
    Public Function GetNearFontSize(ByVal value As Single) As Single

     '120Dpi 時のラベルとテキストボックスの表示文字サイズが同じになる近似値を求める。
        Dim fs As New List(Of Single)

        For p = 6 To 15 Step 3
          fs.Add(p)
          fs.Add(p + 0.55F)
          fs.Add(p + 1.15F)
          fs.Add(p + 1.75F)
          fs.Add(p + 2.35F)
        Next

        For i As Integer = 0 To fs.Count - 1
          If value < fs(i) Then

             Dim fl As Single = value - fs(i - 1)
             Dim fh As Single = fs(i) - value

             If fl < fh Then
                Return fs(i - 1)
             Else
                Return fs(i)
             End If
          End If
        Next

        Return -1
    End Function

    <DebuggerStepThrough()> _
    Public Function GetDirectorySize(ByVal DirPath As String) As Long
        Dim di As DirectoryInfo = New DirectoryInfo(DirPath)

        Return GetDirectorySize(di)
    End Function

    <DebuggerStepThrough()> _
    Public Function GetDirectorySize(ByVal DirInfo As DirectoryInfo) As Long
        Dim size As Long = 0

        Try
          For Each fi As FileInfo In DirInfo.GetFiles
            size += fi.Length
          Next

        Catch ex As Exception
        End Try

        Try
          For Each di As DirectoryInfo In DirInfo.GetDirectories
            size += GetDirectorySize(di)
          Next

        Catch ex As Exception
        End Try

        Return size
    End Function

    <DebuggerStepThrough()> _
    Public Function StringToInteger(ByVal value As String,
                            Optional ByVal CommaExclusion As Boolean = False) As Integer
'
'     Val() の代用関数: Cfix(Val(string))と同等 / ex: " 2,000.52" ⇒ 2000
'
'       CommaExclusion: True (カンマ除外)
'
        If CommaExclusion Then value = value.Replace(",", "")
        If String.IsNullOrEmpty(value) Then Return 0

        Dim i As Integer
        Dim r As New Regex("^(\s*)?[\+-]?\d*", RegexOptions.None)
        Dim m As Match = r.Match(value)

        If m.Success AndAlso Integer.TryParse(m.Value, i) Then
           Return i
        End If

        Return 0
    End Function

    <DebuggerStepThrough()> _
    Public Function StringToDouble(ByVal value As String,
                            Optional ByVal CommaExclusion As Boolean = False) As Double
'
'     Val() の代用関数: Cdbl(Val(string))と同等 / ex: "2,000.52" ⇒ 2000.52
'
'       CommaExclusion: True (カンマ除外)
'
        If CommaExclusion Then value = value.Replace(",", "")
        If String.IsNullOrEmpty(value) Then Return 0

        Dim d As Double
        Dim r As New Regex("^(\s*)?[\+-]?\d*(\.\d*)?", RegexOptions.None)
        Dim m As Match = r.Match(value)

        If m.Success AndAlso Double.TryParse(m.Value, d) Then
           Return d
        End If

        Return 0
    End Function

    <DebuggerStepThrough()> _
    Public Function StringToBoolean(ByVal value As String) As Boolean
'
'     CBool() の代用関数
'
        Dim f As Boolean

        If Boolean.TryParse(value, f) Then
           Return f
        Else
           Return False
        End If
    End Function

End Module
