﻿'
'   MessageBoxEx 1.6.2.0 (Classic & Visual Style Consolidated edition)
'
'       Required Class: 
'         WinMessage.vb
'         ButtonEx.vb
'
'       Return:     
'         DialogResult
'
'       Property:
'         MaximumWidth
'         MaximumHeight
'         ButtonSizeMode (GrowAndFix / GrowAndShrink / Equalize)
'
'       Method
'         SetButtonText
'
'         HoverColor
'           デザイナーでボタンの HoverColor プロパティを指定しても無視されます。
'           呼出し元のフォームに HoverColor プロパティがある場合で、MessageBoxEx 呼出し時に
'           HoverColor プロパティの値が指定されていない場合、呼出し元の値が選択されます。
'
'       ※ メッセージフォントはシステム設定（ウィンドウの色とデザイン）を使用します。
'
'   < Vertical >
'       SystemInformation.FixedFrameBorderSize.Height
'       SystemInformation.CaptionHeight
'       _UpperBothHeight
'       _iconSize
'       _UppeBothHeight
'       _LowerHeight1
'       _ButtonHeight
'       _LowerHeight2
'       SystemInformation.FixedFrameBorderSize.Height
'
'   < Horizontal >
'       SystemInformation.FixedFrameBorderSize.Width
'       _UpperBothWidth ┐
'       _iconSize       ├ Lbl_icon.Width ┐
'       _ControlMargin  ┘                ├ Lbl_text.Width
'       ( Message )                       │
'       _UpperBothWidth ─────────┘
'       SystemInformation.FixedFrameBorderSize.Width
'
Imports System.Media
Imports System.Reflection
Imports System.Drawing.Text
Imports System.ComponentModel

Namespace UserLib

<DebuggerStepThrough()>
Public Class MessageBoxEx

Public Enum Buttons As Integer
     OK = 0
     OKCancel = 1
     AbortRetryIgnore = 2
     YesNoCancel = 3
     YesNo = 4
     RetryCancel = 5
     RetryIgnoreCancel = 6
     IgnoreCancel = 7
     CenterOK = 8
     CenterOKCancel = 9
     CenterAbortRetryIgnore = 10
     CenterYesNoCancel = 11
     CenterYesNo = 12
     CenterRetryCancel = 13
     CenterRetryIgnoreCancel = 14
     CenterIgnoreCancel = 15
End Enum

Public Enum Align As Integer
    Left = 0
    Center = 1
    Right = 2
    LeftRed = 4
    CenterRed = 5
    RightRed = 6
    LeftGreen = 8
    CenterGreen = 9
    RightGreen = 10
    LeftBlue = 12
    CenterBlue = 13
    RightBlue = 14
End Enum

Public Enum SizeMode As Integer
    GrowAndFix = 0
    GrowAndShrink = 1
    Equalize = 2
End Enum

#Region "// Property, Method"

    Private _MaximumWidth As Integer = 800      'メッセージボックスの最大の幅・高さ
    Private _MaximumHeight As Integer = 600

    Private _ButtonText(3) As String            'Custom Buttons Text

''' <summary>
''' メッセージボックスの最大の幅を指定します。
''' </summary>

    Public Property MaximumWidth() As Integer
        Get
          Return _MaximumWidth
        End Get
        Set(ByVal value As Integer)
          If value < _BaseWindowWidth Then
             value = _BaseWindowWidth
          ElseIf value > Screen.GetBounds(Me).Width Then
             value = Screen.GetBounds(Me).Width
          End If
          _MaximumWidth = value
        End Set
    End Property

''' <summary>
''' メッセージボックスの最大の高さを指定します。
''' </summary>

    Public Property MaximumHeight() As Integer
        Get
          Return _MaximumHeight
        End Get
        Set(ByVal value As Integer)
          If value < Me.MinimumSize.Height Then
             value = Me.MinimumSize.Height
          ElseIf value > Screen.GetBounds(Me).Height Then
             value = Screen.GetBounds(Me).Height
          End If
          _MaximumHeight = value
        End Set
    End Property

''' <summary>
''' ボタン上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    Public Property HoverColor() As Color

''' <summary>
''' ボタン(1-3)に表示するテキストを設定します。
''' </summary>

    Public Overloads Sub SetButtonText(ByVal ButtonNumber As Integer, ByVal value As String,
                              Optional ByVal HkPrefix As HotkeyPrefix = HotkeyPrefix.None)

        If ButtonNumber < 1 OrElse ButtonNumber >= _ButtonText.Length Then
           Return
        Else
           _ButtonText(ButtonNumber) = value
        End If

        DirectCast(Me.Controls("Button_" & ButtonNumber), ButtonEx).HotkeyPrefix = HkPrefix
    End Sub

''' <summary>
''' ボタンの幅を変更するモードを指定します。
''' </summary>

    <DefaultValue(SizeMode.GrowAndFix),
    Description("ボタンの幅を変更するモードを指定します。")> _
    Public Property ButtonSizeMode() As SizeMode

#End Region

#Region "// constant, variable declaration"

    Private Const _iconSize = 32
    Private Const _ControlMargin = 8
    Private Const _UpperBothWidth = 16
    Private Const _UpperBothHeight = 16
    Private Const _LowerBothWidth = 8 - 1       '推奨値: 設定値-1 = _LowerHeight2
    Private Const _LowerHeight1 = 8
    Private Const _LowerHeight2 = 8 - 1
    Private Const _TextPadding = 8              'ボタンのテキストの上下左右の余白

    Private Const _MinButton_Width = 24         '    〃　     (最小値）
    Private Const _MinButtonHeight = 24         'ボタンの高さ (　〃　）

    Private Const _MinUpperHeight = (_UpperBothHeight * 2) + _iconSize
    Private Const _MinLowerHeight = _LowerHeight1 + _MinButtonHeight + _LowerHeight2

    Private _msg As String
    Private _cap As String
    Private _btn As Integer
    Private _icn As Integer
    Private _def As Integer
    Private _alg As Integer
    Private _snd As Boolean

    Private _Closing_Flag As Boolean            '閉じる,[ESC] (True: 無効, False:有効 [初期値])

    Private _ButtonCount As Integer             'ボタンの数
'
'   _WindowFrameSize        FixedDialog時の枠のサイズ(固定値)=6
'   _BaseWindowWidth        最小幅のボタンを全て表示した際のウィンドウの幅（参考値）
'   _MinimumWindow_Width    ウィンドウの最小な幅（システム値）
'   _MinimumWindowHeight    ウィンドウの最小な高さ
'
    Private _WindowFrameSize As Integer = SystemInformation.FixedFrameBorderSize.Width * 2
    Private _BaseWindowWidth As Integer = (_LowerBothWidth * 2) + (_ControlMargin * 2) + (_MinButton_Width * 3) + _WindowFrameSize

    Private _MinimumWindow_Width As Integer = SystemInformation.MinimumWindowSize.Width
    Private _MinimumWindowHeight As Integer = _WindowFrameSize + SystemInformation.CaptionHeight + _
                                              _MinUpperHeight + _MinLowerHeight

#End Region

    Public Sub New()
        InitializeComponent()
    End Sub

    <Obsolete("Show(ByVal Owner As IWin32Window, …) を使用してください。", True)> _
    Public Overloads Sub Show()
    End Sub

    <Obsolete("Show(ByVal Owner As IWin32Window, …) を使用してください。", True)> _
    Public Overloads Sub Show(ByVal Owner As IWin32Window)
    End Sub

    Public Overloads Sub Show( _
        ByVal Owner As IWin32Window, _
        ByVal msg As String, _
          Optional ByVal cap As String = "", _
          Optional ByVal btn As Buttons = Buttons.OK, _
          Optional ByVal icn As MessageBoxIcon = MessageBoxIcon.None, _
          Optional ByVal def As MessageBoxDefaultButton = MessageBoxDefaultButton.Button1, _
          Optional ByVal alg As Align = Align.Left, _
          Optional ByVal snd As Boolean = True)

        _msg = msg : _cap = cap : _btn = btn : _icn = icn : _def = def : _alg = alg : _snd = snd

        Form_Build(Owner)

        Me.ShowDialog(Owner)
    End Sub

    Public Function Dialog( _
        ByVal Owner As IWin32Window, _
        ByVal msg As String, _
          Optional ByVal cap As String = "", _
          Optional ByVal btn As Buttons = Buttons.OK, _
          Optional ByVal icn As MessageBoxIcon = MessageBoxIcon.None, _
          Optional ByVal def As MessageBoxDefaultButton = MessageBoxDefaultButton.Button1, _
          Optional ByVal alg As Align = Align.Left, _
          Optional ByVal snd As Boolean = True) As Integer

        _msg = msg : _cap = cap : _btn = btn : _icn = icn : _def = def : _alg = alg : _snd = snd

        Form_Build(Owner)

        Me.ShowDialog(Owner)

        Return Me.DialogResult
    End Function

    Private Sub Form_Build(ByVal Owner As IWin32Window)

      Me.SuspendLayout()

        Dim x As Integer
        Dim w, h As Integer

        Dim btn As ButtonEx

        Dim Min_WinWidth As Integer     'ｼｽﾃﾑで決められた最小ｳｨﾝﾄﾞｳ幅から枠ｻｲｽﾞ(6)を引いたもの
        Dim Center_Mode As Boolean      'for Buttons

        Dim UpperHeight As Integer      '上部(Message)の高さ
        Dim LowerHeight As Integer      '下部(Button) の高さ
        Dim Lower_Width As Integer      '     〃        幅

        Dim MinButton_Width As Integer  'Buttonの最小幅
        Dim MaxButton_Width As Integer  'Buttonの最大幅
        Dim MaxButtonHeight As Integer  '     〃     高さ

        Dim MaxText_Width As Integer    'Messageの最大幅
        Dim MaxTextHeight As Integer    '     〃      高さ

        Dim ButtonsWidth As Integer     'Buttonの総幅(Button間の余白を含む)

        Dim Window_Width As Integer     'Window(枠)内の幅
        Dim WindowHeight As Integer     '     〃       高さ

        If Not [Enum].IsDefined(GetType(Buttons), _btn) Then _btn = Buttons.OK
        If Not [Enum].IsDefined(GetType(MessageBoxIcon), _icn) Then _icn = MessageBoxIcon.None
        If Not [Enum].IsDefined(GetType(MessageBoxDefaultButton), _def) Then _def = MessageBoxDefaultButton.Button1
        If Not [Enum].IsDefined(GetType(Align), _alg) Then _alg = Align.Left

        Me.Visible = False  '確定するまでフォームを表示しない。
        Me.Text = _cap      'Visible =True だと、ここでフォームが表示されてしまう。

' Font
        Me.Font = SystemFonts.MessageBoxFont

' Window Size (Min/Max)
        Me.MinimumSize = New Size(_MinimumWindow_Width, _MinimumWindowHeight)
        Me.MaximumSize = New Size(_MaximumWidth, _MaximumHeight)

' icon
        Select Case _icn
          Case MessageBoxIcon.None
            Lbl_icon.ImageIndex = -1
            If _snd Then SystemSounds.Beep.Play()

          Case MessageBoxIcon.Hand, MessageBoxIcon.Stop, MessageBoxIcon.Error
            Lbl_icon.ImageIndex = 0
            If _snd Then SystemSounds.Hand.Play()

          Case MessageBoxIcon.Question
            Lbl_icon.ImageIndex = 1
            If _snd Then SystemSounds.Question.Play()

          Case MessageBoxIcon.Exclamation, MessageBoxIcon.Warning
            Lbl_icon.ImageIndex = 2
            If _snd Then SystemSounds.Exclamation.Play()

          Case MessageBoxIcon.Asterisk, MessageBoxIcon.Information
            Lbl_icon.ImageIndex = 3
            If _snd Then SystemSounds.Asterisk.Play()

        End Select

' Buttons
        Center_Mode = (_btn >= Buttons.CenterOK)

        Select Case _btn Mod Buttons.CenterOK
          Case Buttons.OK
            _ButtonCount = 1
            Button_1.DialogResult = DialogResult.OK
            Button_1.Text = "OK"

          Case Buttons.OKCancel
            _ButtonCount = 2
            Button_1.DialogResult = DialogResult.OK
            Button_2.DialogResult = DialogResult.Cancel
            Button_1.Text = "OK"
            Button_2.Text = "Cancel"

          Case Buttons.AbortRetryIgnore
            _Closing_Flag = True
            _ButtonCount = 3
            Button_1.DialogResult = DialogResult.Abort
            Button_2.DialogResult = DialogResult.Retry
            Button_3.DialogResult = DialogResult.Ignore
            Button_1.Text = "中止 (&A)"
            Button_2.Text = "再試行 (&R)"
            Button_3.Text = "無視 (&I)"

          Case Buttons.YesNoCancel
            _ButtonCount = 3
            Button_1.DialogResult = DialogResult.Yes
            Button_2.DialogResult = DialogResult.No
            Button_3.DialogResult = DialogResult.Cancel
            Button_1.Text = "はい (&Y)"
            Button_2.Text = "いいえ (&N)"
            Button_3.Text = "キャンセル"

          Case Buttons.YesNo
            _Closing_Flag = True
            _ButtonCount = 2
            Button_1.DialogResult = DialogResult.Yes
            Button_2.DialogResult = DialogResult.No
            Button_1.Text = "はい (&Y)"
            Button_2.Text = "いいえ (&N)"

          Case Buttons.RetryCancel
            _ButtonCount = 2
            Button_1.DialogResult = DialogResult.Retry
            Button_2.DialogResult = DialogResult.Cancel
            Button_1.Text = "再試行 (&R)"
            Button_2.Text = "キャンセル"

          Case Buttons.RetryIgnoreCancel
            _ButtonCount = 3
            Button_1.DialogResult = DialogResult.Retry
            Button_2.DialogResult = DialogResult.Ignore
            Button_3.DialogResult = DialogResult.Cancel
            Button_1.Text = "再試行 (&R)"
            Button_2.Text = "無視 (&I)"
            Button_3.Text = "キャンセル"

          Case Buttons.IgnoreCancel
            _ButtonCount = 2
            Button_1.DialogResult = DialogResult.Ignore
            Button_2.DialogResult = DialogResult.Cancel
            Button_1.Text = "無視 (&I)"
            Button_2.Text = "キャンセル"
        End Select

' Buttons Width acquisition
        Dim b As New Bitmap(Me.MaximumSize.Width, Me.Font.Height)
        Dim g As Graphics = Graphics.FromImage(b)

        If Me.ButtonSizeMode = SizeMode.GrowAndFix Then
           Dim FontSizeMpf As Single = 8.0F * (Me.CreateGraphics.DpiX / 96.0F)

           MinButton_Width = Convert.ToInt32(Me.Font.SizeInPoints * FontSizeMpf)
        Else
           MinButton_Width = _MinButton_Width
        End If

        MaxButtonHeight = _MinButtonHeight

        For i As Integer = 1 To _ButtonCount
          btn = DirectCast(Controls("Button_" & i), ButtonEx)

          If Not String.IsNullOrEmpty(_ButtonText(i)) Then
             btn.Text = _ButtonText(i)
          End If

          Dim mt As Size = TextRenderer.MeasureText(g, btn.Text, btn.Font)

          If mt.Width < (MinButton_Width - _TextPadding) Then
             mt.Width = (MinButton_Width - _TextPadding)
          End If

          If mt.Height < (_MinButtonHeight - _TextPadding) Then
             mt.Height = (_MinButtonHeight - _TextPadding)
          End If

          If Me.ButtonSizeMode = SizeMode.Equalize Then
             If MaxButton_Width < (mt.Width + _TextPadding) Then
                MaxButton_Width = (mt.Width + _TextPadding)
             End If
          Else
             btn.Width = mt.Width + _TextPadding

             ButtonsWidth += btn.Width
          End If

          If MaxButtonHeight < (mt.Height + _TextPadding) Then
             MaxButtonHeight = (mt.Height + _TextPadding)
          End If
        Next

        If Me.ButtonSizeMode = SizeMode.Equalize Then
           ButtonsWidth = _ButtonCount * MaxButton_Width
        End If
        ButtonsWidth += ((_ButtonCount - 1) * _ControlMargin)

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing

        UpperHeight = _MinUpperHeight
        LowerHeight = _LowerHeight1 + MaxButtonHeight + _LowerHeight2

' Message setting
        Min_WinWidth = SystemInformation.MinimumWindowSize.Width - _WindowFrameSize

        MaxText_Width = Me.MaximumSize.Width - _WindowFrameSize
        MaxTextHeight = Me.MaximumSize.Height - _WindowFrameSize _
                     - SystemInformation.CaptionHeight - LowerHeight
'
        If _icn <> 0 Then
           Lbl_icon.Width = _UpperBothWidth + _iconSize + _ControlMargin
           Lbl_icon.Height = UpperHeight
           Lbl_icon.Padding = New Padding(_UpperBothWidth - 2, _UpperBothHeight - 2, 0, 0)
           Lbl_text.Padding = New Padding(_UpperBothWidth + _iconSize + _ControlMargin, _
                                          _UpperBothHeight, _
                                          _UpperBothWidth, _
                                          _UpperBothHeight)
           Lbl_icon.Visible = True
'
           If Center_Mode Then
              Lower_Width = ButtonsWidth + (_LowerBothWidth * 2)
           Else
              Lower_Width = ButtonsWidth + _LowerBothWidth + Lbl_text.Padding.Left
           End If
        Else
           Lbl_icon.Width = 0
           Lbl_text.Padding = New Padding(_UpperBothWidth, _UpperBothHeight, _UpperBothWidth, _UpperBothHeight)
           Lower_Width = ButtonsWidth + (_LowerBothWidth * 2)
        End If

        If Lower_Width < Min_WinWidth Then  'ｼｽﾃﾑの最小幅以下？
           Lower_Width = Min_WinWidth
        End If
'
        Lbl_text.MaximumSize = CType(New Point(MaxText_Width, MaxTextHeight), Drawing.Size)
        Lbl_text.AutoSize = True
        Lbl_text.Text = _msg

        If Lower_Width < Lbl_text.Width Then
           Window_Width = Lbl_text.Width
        Else
           Window_Width = Lower_Width
        End If

        If UpperHeight < Lbl_text.Height Then
           UpperHeight = Lbl_text.Height
           WindowHeight = Lbl_text.Height + LowerHeight
           Lbl_icon.Height = Lbl_text.Height
        Else
           WindowHeight = UpperHeight + LowerHeight
        End If

        Lbl_text.AutoSize = False
        Lbl_text.Width = Window_Width
        Lbl_text.Height = UpperHeight

' Align Setting
        Lbl_text.TextAlign = CType(16 * (2 ^ (_alg Mod 4)), ContentAlignment)
        Select Case _alg \ 4
           Case 0 : Lbl_text.ForeColor = SystemColors.ControlText
           Case 1 : Lbl_text.ForeColor = Color.Red
           Case 2 : Lbl_text.ForeColor = Color.Green
           Case 3 : Lbl_text.ForeColor = Color.Blue
        End Select
        Lbl_text.Visible = True

' Buttons setting
        If _btn < Buttons.CenterOK Then
           x = Window_Width - ButtonsWidth - _LowerBothWidth
        Else
           x = (Window_Width - ButtonsWidth) \ 2 '+ 1
        End If
'
        For i As Integer = 1 To _ButtonCount
          btn = DirectCast(Controls("Button_" & i), ButtonEx)

          If Me.ButtonSizeMode = SizeMode.Equalize Then
             btn.Width = MaxButton_Width
          End If

          btn.Height = MaxButtonHeight
          btn.Top = UpperHeight + _LowerHeight1
          btn.Left = x
          btn.Visible = True

          x += (btn.Width + _ControlMargin)
        Next

' Default Button set
        Dim n As Integer = (_def \ 256) + 1
        If n > _ButtonCount Then n = _ButtonCount

        DirectCast(Controls("Button_" & n), ButtonEx).Select()

' HoverColor Set
        Dim pi As PropertyInfo = Owner.GetType.GetProperty("HoverColor")

        If pi Is Nothing Then
           If _HoverColor = Nothing Then _HoverColor = SystemColors.Highlight
        Else
           If _HoverColor = Nothing Then
              _HoverColor = DirectCast(pi.GetValue(Owner, Nothing), Color)
              If _HoverColor = Nothing Then _HoverColor = SystemColors.Highlight
           End If
        End If

        For i As Integer = 1 To _ButtonCount
          DirectCast(Me.Controls("Button_" & i), ButtonEx).HoverColor = _HoverColor
        Next

' Window Size
        w = Window_Width + _WindowFrameSize
        h = WindowHeight + _WindowFrameSize + SystemInformation.CaptionHeight

        Me.Width = w
        Me.Height = h

      Me.ResumeLayout()
    End Sub

#Region "// Me"

    Private Sub Me_Load(ByVal sender As System.Object, ByVal e As EventArgs) Handles MyBase.Load

        If Owner.OwnedForms.Length = 1 Then
           Owner.Update()
        Else
           Owner.OwnedForms(Owner.OwnedForms.Length - 2).Update()   '1つ上(呼出し元)のフォーム
        End If
    End Sub

    Private Sub Me_Shown(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Shown
        Me.Update()
        WinMessage.Clear(Me, 200)    'ダブルクリック対策
    End Sub

    Private Sub Me_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles Me.FormClosing
        If _Closing_Flag And Me.DialogResult = DialogResult.Cancel Then Beep() : e.Cancel = True
    End Sub

    Private Sub Me_FormClosed(ByVal sender As Object, ByVal e As FormClosedEventArgs) Handles Me.FormClosed
        Me.Visible = False
'     次回ロード用に初期化
        For Each ctl As Control In Me.Controls
          ctl.Visible = False
        Next
        Me.Width = Me.MinimumSize.Width
        Me.Height = Me.MinimumSize.Height

        _Closing_Flag = False
    End Sub

    Private Sub Me_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs) Handles Me.KeyDown
        If e.KeyData = Keys.Escape Then
           If _Closing_Flag Then
              Beep()
           Else
              Me.DialogResult = DialogResult.Cancel : Me.Close()
           End If
        ElseIf e.KeyData = Keys.Control + Keys.C Then
           Clipboard.SetText(String.Format("{0}{1}{2}{1}", _cap, vbCrLf, _msg))
        End If
    End Sub

#End Region

End Class

End Namespace
