﻿'
'   DateTimePickerEx 1.0.1.1F for Fix Classic Style
'
'       Property:
'         Value
'         CalendarFont
'         Format       / CustomFormat
'         ShowUpDown
'
'       Method
'         ReSetShowUpDown
'
'       Event Notification
'         MouseEnter  / MouseLeave
'         MouseDown   / MouseWheel
'         KeyDown     / KeyPress
'         TextChanged / ValueChanged

Imports System.ComponentModel
Imports System.Text.RegularExpressions
Imports System.Runtime.InteropServices

Namespace UserLib

<DebuggerStepThrough()>
Public Class DateTimePickerEx

    Public Shadows Event TextChanged(sender As Object, e As EventArgs)
    Public Event ValueChanged(sender As Object, e As EventArgs)

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AutoValidate() As AutoValidate

    <Browsable(False)> _
    Public Shadows Property AutoScroll() As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin() As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize() As Size
    <Browsable(False)> _
    Public Shadows Property AutoSize() As Boolean
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode() As AutoSizeMode
    <Browsable(False)> _
    Public Shadows Property Padding() As Padding

    <Browsable(False)> _
    Public Shadows Property BackColor As Color
    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout
    <Browsable(False), DefaultValue(BorderStyle.Fixed3D)> _
    Public Shadows Property BorderStyle() As BorderStyle = BorderStyle.Fixed3D
    <Browsable(False)> _
    Public Shadows Property ForeColor As Color

#End Region

    Private _Set_Value As Boolean
    Private _Set_CalendarFont As Boolean

    Private _CalendarFont As Font

#Region "** 動作"

''' <summary>
''' 日付/時刻のカスタム書式指定文字列を取得または設定します。
''' </summary>

    <DefaultValue(""), Category("動作"),
     Description("コントロールに表示される日付または時間の書式を設定するときに使用するカスタム書式設定文字列です。")> _
    Public Property CustomFormat() As String
      Get
        Return DateTimePicker.CustomFormat
      End Get

      Set(value As String)
        DateTimePicker.CustomFormat = value
      End Set
    End Property

''' <summary>
''' コントロールに代入された日付/時刻値を取得または設定します。
''' </summary>

    <Category("動作"),
     Description("このコントロールの現在の日付/時間の値です。")> _
    Public Property Value() As Date
      Get
        Return DateTimePicker.Value
      End Get

      Set(value As DateTime)
        _Set_Value = True

        DateTimePicker.Value = value
      End Set
    End Property

    Private Function ShouldSerializeValue() As Boolean
        Return _Set_Value
    End Function

    Private Sub ResetValue()
        _Set_Value = False

        DateTimePicker.Value = Now
    End Sub

#End Region

#Region "** 表示"

''' <summary>
''' 予定表に適用されるフォント スタイルを取得または設定します。
''' </summary>

    <Category("表示"),
     Description("カレンダー表示に使用されるフォントです。")> _
    Public Property CalendarFont() As Font
      Get
        Return _CalendarFont
      End Get

      Set(value As Font)
        _Set_CalendarFont = Not IsNothing(value)

        DateTimePicker.CalendarFont = value

        If value Is Nothing Then value = Me.Font

        _CalendarFont = value
      End Set
    End Property

    Private Function ShouldSerializeCalendarFont() As Boolean
        Return _Set_CalendarFont
    End Function

    Private Sub ResetCalendarFont()
        _Set_CalendarFont = False
        _CalendarFont = Me.Font
    End Sub

''' <summary>
''' コントロールに表示される日時の書式を取得または設定します。
''' </summary>

    <DefaultValue(DateTimePickerFormat.Long), Category("表示"),
     Description("標準形式またはカスタム形式のどちらで日付と時間を表示するかを決定します。")> _
    Public Property Format() As DateTimePickerFormat
      Get
        Return DateTimePicker.Format
      End Get

      Set(value As DateTimePickerFormat)
        DateTimePicker.Format = value
      End Set
    End Property

''' <summary>
''' スピン ボタン コントロール (アップダウン コントロール) を使用して、
''' 日付/時刻値を調整するかどうかを示す値を取得または設定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("コントロール値の変更のために、ドロップダウン カレンダーではなく、スピン ボタンを表示するかを示します。")> _
    Public Property ShowUpDown() As Boolean
      Get
        Return DateTimePicker.ShowUpDown
      End Get

      Set(value As Boolean)
        DateTimePicker.ShowUpDown = value
      End Set
    End Property

#End Region

#End Region

#Region "// Method"

''' <summary>
''' ボタンとスクロールバーの幅を同じにします。
''' </summary>

    Public Sub ReSetShowUpDown(ByVal state As Boolean)
        DateTimePicker.ShowUpDown = Not state
        DateTimePicker.ShowUpDown = state
    End Sub

#End Region

    Private _onCreate As Boolean

    Private _FramePoint As Point()

#Region "// Sub-Routine & Function"

    <DllImport("USER32.DLL", CharSet:=CharSet.Auto)> _
    Private Shared Function HideCaret( _
        ByVal hwnd As IntPtr) As Integer
    End Function

    <DllImport("USER32.DLL", CharSet:=CharSet.Auto)> _
    Private Shared Function ShowCaret( _
        ByVal hwnd As IntPtr) As Integer
    End Function

    <DebuggerStepThrough()> _
    Private Function StringToInt32(ByVal value As String) As Integer

        If String.IsNullOrEmpty(value) Then Return 0

        value = value.Replace(",", "")

        Dim i As Integer
        Dim r As New Regex("^(\s*)?[\+-]?\d*", RegexOptions.None)
        Dim m As Match = r.Match(value)

        If m.Success AndAlso Integer.TryParse(m.Value, i) Then
           Return i
        End If

        Return 0
    End Function

''' <summary>
''' 外枠の描画座標をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetDrawPoint()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        _FramePoint = {
          New Point(1, 0), New Point(w - 2, 0), New Point(w - 2, 1), New Point(w - 1, 1),
          New Point(w - 1, h - 2), New Point(w - 2, h - 2), New Point(w - 2, h - 1), New Point(1, h - 1),
          New Point(1, h - 2), New Point(0, h - 2), New Point(0, 1), New Point(1, 1)}
    End Sub

''' <summary>
''' Region (Me, Frame) の設定、および、外枠を描画します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub DrawFrame()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        If (w < 1) OrElse (h < 1) Then Return

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        g.DrawRectangle(New Pen(SystemColors.ControlLightLight), 1, 1, w - 3, h - 3)
        g.DrawLines(New Pen(SystemColors.ControlDark), _FramePoint)

        Me.Frame.Image = DirectCast(b.Clone, Drawing.Image)

        path = New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(0, 0, w, h))
        path.AddRectangle(New Rectangle(2, 2, w - 4, h - 4))

        Me.Frame.Region = New Region(path)

        path.Dispose()
        path = Nothing

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing
    End Sub

#End Region

    Public Sub New()

        InitializeComponent()

        Me.Frame.BringToFront()
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        Me.Frame.Size = Me.Size

        SetDrawPoint()
        DrawFrame()

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler Me.FontChanged, AddressOf Me_Resize
        AddHandler Me.SystemColorsChanged, AddressOf Me_SystemColorsChanged

        MyBase.OnCreateControl()
    End Sub

#Region "// Me"

    Private Sub Me_SystemColorsChanged(sender As Object, e As EventArgs)

        If Not (Me.Frame.Size = Me.Size) Then
           Me.Frame.Size = Me.Size

           SetDrawPoint()
        End If

        DrawFrame()
    End Sub

    Private Sub Me_FontChanged(sender As Object, e As EventArgs) Handles Me.FontChanged

     'CalendarFont を変更する為の処理

        If Not _Set_CalendarFont Then
           _CalendarFont = Me.Font
        End If
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

      RemoveHandler Me.Resize, AddressOf Me_Resize

        Me.Height = Me.Font.Height + (SystemInformation.FixedFrameBorderSize.Height * 2) + 1

        Me.Frame.Size = Me.Size

        SetDrawPoint()
        DrawFrame()

      AddHandler Me.Resize, AddressOf Me_Resize
    End Sub

    Private Sub Me_Enter(sender As Object, e As EventArgs) Handles Me.Enter
        DateTimePicker.Select()
    End Sub

#End Region

#Region "// DateTimePicker"

' コントロール配置元にイベント通知

    Private Sub DateTimePicker_MouseEnter(sender As Object, e As EventArgs) Handles DateTimePicker.MouseEnter
        Me.OnMouseEnter(e)
    End Sub

    Private Sub DateTimePicker_MouseLeave(sender As Object, e As EventArgs) Handles DateTimePicker.MouseLeave
        Me.OnMouseLeave(e)
    End Sub

    Private Sub DateTimePicker_MouseDown(sender As Object, e As MouseEventArgs) Handles DateTimePicker.MouseDown
        Me.OnMouseDown(e)

        If e.Button = MouseButtons.Middle Then
           DateTimePicker.ShowUpDown = Not DateTimePicker.ShowUpDown
        End If
    End Sub

    Private Sub DateTimePicker_MouseWheel(sender As Object, e As MouseEventArgs) Handles DateTimePicker.MouseWheel
        Me.OnMouseWheel(e)

        If DateTimePicker.Focused Then

           If e.Delta > 0 Then
              SendKeys.SendWait("{UP}")
           Else
              SendKeys.SendWait("{DOWN}")
           End If
        End If
    End Sub

    Private Sub DateTimePicker_KeyDown(sender As Object, e As KeyEventArgs) Handles DateTimePicker.KeyDown
        Me.OnKeyDown(e)
    End Sub

    Private Sub DateTimePicker_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DateTimePicker.KeyPress
        Me.OnKeyPress(e)
    End Sub

    Private Sub DateTimePicker_TextChanged(sender As Object, e As EventArgs) Handles DateTimePicker.TextChanged
        Me.Text = DateTimePicker.Text

        RaiseEvent TextChanged(Me, e)
    End Sub

    Private Sub DateTimePicker_ValueChanged(sender As Object, e As EventArgs) Handles DateTimePicker.ValueChanged
        Me.Text = DateTimePicker.Text

        RaiseEvent ValueChanged(Me, e)
    End Sub

#End Region

End Class

End Namespace
