﻿'
'   AltTm NC (GroupBoxEx: GDI+ Label Ver)
'
Imports System.IO
Imports System.ComponentModel
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices
Imports System.Threading
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Media
Imports UserLib

Public Class Form_Main

#Region "// constant, variable declaration"

#Region "-- Const"

    Private Const _Msg_Cap = "確認"
    Private Const _Msg_Fnf = "ファイルが見つかりません。" & vbCr & vbCr
    Private Const _Msg_Uae = "ファイルへのアクセスが拒否されました。" & vbCr & vbCr
    Private Const _Msg_Ioe = "別のプロセスで使用されているため、プロセスはファイルにアクセスできません。" & vbCr & vbCr
    Private Const _Msg_Nxt = vbCr & vbCr & "処理を継続しますか？。"

    Private Const _Msg_FailedClipCopy = "クリップボードへのコピーに失敗しました。"
    Private Const _Msg_FailedClipMove = "クリップボードへの切り取りに失敗しました。"

    Private Const _FileDialogFilter =
                  "すべてのファイル|*.*|" &
                  "テキストファイル (.txt)|*.txt|" &
                  "書庫ファイル (.7z .zip .rar)|*.7z;*.zip;*.rar|" &
                  "画像ファイル (.bmp .jpg .jpeg .png .gif .psd)|*.bmp;*.jpg;*.jpeg;*.png;*.gif;*.psd|" &
                  "bmpファイル|*.bmp|" &
                  "jpgファイル|*.jpg;*.jpeg|" &
                  "pngファイル|*.png|" &
                  "gifファイル|*.gif|" &
                  "psdファイル|*.psd|" &
                  "mp3ファイル|*.mp3|" &
                  "mp4ファイル|*.mp4"

    Private Const _LblItemsFormat = "{0:###,##0} items"
    Private Const _DateTimeFormat = "yyyy/MM/dd HH:mm:ss"
    Private Const _CmbDtLogFormat = "  yyyy/MM/dd HH:mm:ss"

    Private _OS_Type As Boolean

    Private _ToolTip_Text As String(,) = {
            {"ファイル", "フォルダ", "フォルダ"},
            {"Single mode", "SubFolder mode", "Tree mode"},
            {"File mode", "Folder mode", "All mode"},
            {"Ignore mode", "Check mode", ""}}

    Private _DateTimeMinDate As Date = New DateTime(1753, 1, 1, 0, 0, 0, 0)
    Private _DateTimeMaxDate As Date = New DateTime(9998, 12, 31, 0, 0, 0, 0)

#End Region

#Region "-- File Data"

    Private _FilePath As String = My.Application.Info.DirectoryPath
    Private _App_Name As String = My.Application.Info.AssemblyName
    Private _ini_File As String = Path.Combine(_FilePath, _App_Name & ".ini")   'Win32APIを使用する為フルパスに変換
    Private _Log_File As String = Path.Combine(_FilePath, _App_Name & ".log")
    Private _TempFile As String = Path.Combine(_FilePath, _App_Name & ".tmp")
    Private _WorkFile As String = Path.Combine(_FilePath, _App_Name & ".$$$")
    Private _UndoFile As String = Path.Combine(_FilePath, _App_Name & ".Undo")
    Private _Err_File As String = Path.Combine(_FilePath, _App_Name & "_Error.log")

#End Region

#Region "-- Setting"

    Private _JobMode As Integer         '0:Normal 1:Make 2:Repl 3:DateTime Set
    Private _DropFileList As String()   'DragDrop Table List

    Private _onAltKeyDown As Boolean    'Altキー押下状態
    Private _onShiftEnter As Boolean    '[Shift]+(Enter] 判定用
    Private _onSelectItem As Boolean    'item 選択状態
    Private _onDeactivate As Boolean    'Deactivate イベント発生状態
    Private _onStyleChange As Boolean   'True:VisualStyle
    Private _onDelaySetting As Boolean  'False: SystemColorChange / True: Resize
    Private _onToolTipActive As Boolean
    Private _onDialogForm As Boolean

    Private _WindowMinimized As Boolean '最小化判定ﾌﾗｸﾞ
    Private _WindowMaximized As Boolean '最大化判定ﾌﾗｸﾞ

    Private _AttrType(2) As Integer
    Private _OpenDirectory As String
    Private _OfdAutoUpgrade As Boolean
    Private _MessageBoxSound As Boolean
    Private _IniSave As Boolean
    Private _LogFileWriting As Boolean
    Private _LogFileMaxSize As Integer = 16384
    Private _LogFileLeaveCount As Integer = 3
    Private _ClipboardMonitor As Boolean
    Private _UseGridLines As Boolean
    Private _UseCtrlUpDown As Boolean           '[CTRL]+[↑↓] True:有効
    Private _UseHideSelection As Boolean
    Private _UseEmphasisColor As Integer = 2

 'ListView から Explorer 等へのDrop 時の処理遅延
    Private _DropDelay As Integer = 250

    Private _FontSizeMpf As Single = Me.CreateGraphics.DpiX / 96.0F

 'フォーム(クライアント)のサイズ
    Private _MinClientSize As Size

 '起動時のウィンドウサイズ
    Private _StartWindow_Width As Integer
    Private _StartWindowHeight As Integer

 '起動時の GroupBox3, DateTimeLog の幅
    Private _GroupBox3Width As Integer
    Private _DateTimeLogWidth As Integer

 '基準となるスクロール バーのサイズ
    Private Const _ScrollBarWidth As Integer = 16   '設計基準値
    Private _BeforeScrollBarWidth As Integer

 'List View 処理（制御）用
    Private _DoubleBuffering As Boolean = True

    Private _List_RegistLimit As Integer = 10000    'リスト登録制限値

    Private _List_AttrType As Integer           '読込対象属性 (RHS)
    Private _List_TreeMode As Integer           '0:List   1:SubFolder 2:Tree
    Private _List_FileMode As Integer           '0:File   1:Folder    2:All
    Private _List_ChckMode As Integer = 1       '0:Ignore 1:Check

#End Region

#Region "-- 日時変更ログ・日付/時刻調整・前回設定値(規定値)"

    Private _DateTimeChangeLog As New DataTable()
    Private _DateTimeChangeLogMaxCount As Integer = 10
    Private _DateTimeValueChangeFlag As Boolean

    Private _M_Adjust_Time As String = "+9:00"  'UTC+9

    Private _M_Adjust_D_Mode As Integer
    Private _M_Adjust_T_Mode As Integer = 1
    Private _M_Adjust_DValue As Integer = 7
    Private _M_Adjust_TValue As Integer = 30
    Private _M_Adjust_RCheck As Boolean
    Private _M_Adjust_LCheck As Boolean = True
    Private _M_Adjust_MValue As Single = 1.5    '※乗率設定 _Adjust_MpfValue(0)÷100

' 日付/時刻調整値
'   ※乗率設定 _Adjust_MpfValue(0)は、_Adjust_MpfValue([1-5]) で設定した値のうちの１つを指定

    Private _Adjust_Time As Integer = TimeConvert(_M_Adjust_Time, _M_Adjust_Time)
    Private _Adjust_Name As String(,) = {{"日", "月"}, _
                                         {"秒", "分"}}

    Private _Adjust_MpfValue() As Integer = {150, 100, 125, 150, 175, 200}  '乗率設定 (100～999)

    Private _Adjust_D_Mode As Integer = _M_Adjust_D_Mode    '日付調整モード (0:日 1:月)
    Private _Adjust_T_Mode As Integer = _M_Adjust_T_Mode    '時刻調整モード (0:秒 1:分)
    Private _Adjust_LCheck As Boolean = _M_Adjust_LCheck    '下限値フラグ   (False:0 True:設定値)  
    Private _Adjust_MValue As Single = _M_Adjust_MValue     '乗率

#End Region

#Region "-- List View (Form Label)"

    Private _Lbl_FontName() As String = {"MS UI Gothic", "Meiryo UI", "MeiryoKe_UIGothic"}

    Private _Lsv_FontName As String = _Lbl_FontName(0)
    Private _Lsv_FontSize() As Single = {9.0F, 9.75F, 10.5F, 11.25F, 12.0F, 12.75F, 14.25F}
    Private _Lsv_FontSize_Id As Integer = 1

    Private _Lsv_Columns As Integer = 3
    Private _Lsv_Row_Height As Integer
    Private _Lsv_Col0_Width As Integer          'Classic:16 / Visual:18
    Private _Lsv_Col2_Width As Integer
    Private _Lsv_ColA_Width As Integer

    Private _Lsv_DupCheckFlg As Boolean

    Private _Lsv_DisplayRows As Integer         'リスト表示行数
    Private _Lsv_DisplayMode As Integer = 1     '0:Free 1:Smart 2:File
    Private _Lsv_LabelHeight As Integer         'MeiryoKe_UIGothic 9.75pt, Meiryo UI 9pt:21 (MS UI Gothic 9.75pt:19)

    Private _Lsv_FrameSize_W As Integer         '枠のサイズ
    Private _Lsv_FrameSize_H As Integer
    Private _Lsv_MinUnused_W As Integer         '未使用部分(枠,ラベル)のサイズ
    Private _Lsv_MinUnused_H As Integer
    Private _Lsv_MaxUnused_W As Integer         'ScrollBarを含めた未使用部分のサイズ (DisplayMode_Controlでのみ使用)
    Private _Lsv_MaxUnused_H As Integer         '                                    (〃)

    Private _Lsv_InnerDrop As Boolean           'ListView 内でDrop
    Private _Lsv_DragState As Boolean           'ドラッグ状態
    Private _Lsv_DragCancel As Boolean          'ドロップキャンセル

    Private _Lsv_PrevIndex As Integer           '１つ前のドラッグ位置

#End Region

#Region "-- ContextMenu"

    Private _CreationTimeText As String
    Private _DateModifiedText As String

#End Region

#Region "-- Color"

    Private _AttrColorName As String() = {"ControlDark", "Magenta", "LightBlue", "WindowText", "Red"}
    Private _EmphasisValue As Single() = {0.0F, 0.9F, 0.85F, 0.8F}

    Private _ColorSuccess As Color = Color.RoyalBlue
    Private _ColorFailure As Color = Color.DeepPink
    Private _ColorBreak As Color = Color.Red
    Private _ColorUndo As Color = Color.ForestGreen
    Private _ColorMain As Color = SystemColors.Highlight
    Private _ColorSub As Color = Color.LimeGreen

    Private _BackColorSuccess As Color
    Private _BackColorFailure As Color
    Private _BackColorBreak As Color
    Private _BackColorUndo As Color

#End Region

#End Region

#Region "// Property"

''' <summary>
''' サブフォームにホバー色を渡す為のプロパティです。
''' </summary>

    Public ReadOnly Property HoverColor() As Color
        Get
          HoverColor = _ColorMain
        End Get
    End Property

''' <summary>
''' サブフォームに警告色を渡す為のプロパティです。
''' </summary>

    Public ReadOnly Property WarningColor() As Color
        Get
          WarningColor = _ColorBreak
        End Get
    End Property

#End Region

#Region "// Sub-Routine & Function"

#Region "-- Main"

    Private Sub SetClipboardText(ByVal text As String, ByVal msg As String)
        Try
          Clipboard.Clear()
          Thread.Sleep(100)     'エラー回避試行

          Clipboard.SetText(text)

          If msg.Length > 0 Then
             ShowInfoMessage(msg & "をクリップボードにコピーしました。", _App_Name)
          End If

        Catch ex As Exception
          ShowWarningMessage(_Msg_FailedClipCopy, "", _App_Name)
        End Try
    End Sub

    <DebuggerStepThrough()>
    Private Function PathExists(ByVal path As String) As Boolean
        If File.Exists(path) OrElse Directory.Exists(path) Then Return True

        Return False
    End Function

    <DebuggerStepThrough()>
    Private Function DriveCheck(ByVal Path As String) As Boolean

        If Path.Substring(0, 2) = "\\" Then 'ネットワークフォルダチェック
           Return True
        End If

        If Regex.Match(Path.ToUpper, "^[A-Z]:").Success Then

           Dim di As New DriveInfo(Path)

           If di.IsReady Then
              Return True
           Else
              Dim state As Integer
              Dim MsgBox As New MessageBoxEx
              Dim msg As String = String.Format("ドライブ ({0}) が準備されていません。", Path.TrimEnd("\"c))
              Do
                state = MsgBox.Dialog(Me, msg, _Msg_Cap,
                                      MessageBoxEx.Buttons.CenterRetryCancel,
                                      MessageBoxIcon.Warning, , , _MessageBoxSound)

                If state = DialogResult.Retry Then
                   di = New DriveInfo(Path)
                   If di.IsReady Then
                      state = 0
                      Exit Do
                   End If
                End If
              Loop Until state = DialogResult.Cancel
              MsgBox.Dispose()
              Me.Update()

              WinMessage.Clear(Me)

              Return (state = 0)
           End If
        End If

        Return False
    End Function

    <DebuggerStepThrough()>
    Private Function RootSelectDialog(ByVal RootPath As String) As Boolean

        Dim s As Integer
        Dim MsgBox As New MessageBoxEx
        Dim msg As String = String.Format("ルートディレクトリ ""{0}"" が指定されています。", RootPath)

        MsgBox.SetButtonText(1, "キャンセル")
        MsgBox.SetButtonText(2, "続行")

        s = MsgBox.Dialog(Me, msg, _Msg_Cap,
                          MessageBoxEx.Buttons.CenterOKCancel,
                          MessageBoxIcon.Warning, , , _MessageBoxSound)
        MsgBox.Dispose()
        Me.Update()

        WinMessage.Clear(Me)

        Return (s = DialogResult.Cancel)
    End Function

    Private Function GetRootEntries(ByVal RootPath As String) As String()

        If RootPath.Substring(0, 2) <> "\\" Then    'ネットワークフォルダチェック

           Dim di As New DriveInfo(RootPath)
           Select Case di.DriveType
             Case DriveType.CDRom, DriveType.Removable
               Return New String() {RootPath}
           End Select
        End If

        Dim e As New List(Of String)
        Dim d As String() = Directory.GetDirectories(RootPath)
        Dim f As String() = Directory.GetFiles(RootPath)

        For Each p As String In d

          Dim Finfo As New FileInfo(p)

          If Not (Finfo.Attributes = 22) Then   'Hidden Or System Or Directory {22}
             e.Add(p)
          End If
        Next

        If f.Length <> 0 Then
           e.AddRange(f)
        End If

        Return e.ToArray
    End Function

    Private Function GetAttributeColor(ByVal Attr As Integer) As Color

        Dim i As Integer

        If (Attr And 4) <> 0 OrElse
           (Attr And FileAttributes.ReparsePoint) <> 0 Then
           i = 4
        ElseIf (Attr And 2) <> 0 Then
           i = 2
        ElseIf (Attr And 1) <> 0 Then
           i = 1
        Else
           i = 3
        End If

        Return Color.FromName(_AttrColorName(i))
    End Function

    Private Sub SetEmphasisBackColor()
        If _UseEmphasisColor <> 0 Then
           Dim mpf As Single = _EmphasisValue(_UseEmphasisColor)

           _BackColorSuccess = ColorEx.IncreaseBrightness(_ColorSuccess, mpf)
           _BackColorFailure = ColorEx.IncreaseBrightness(_ColorFailure, mpf)
           _BackColorBreak = ColorEx.IncreaseBrightness(_ColorBreak, mpf)
           _BackColorUndo = ColorEx.IncreaseBrightness(_ColorUndo, mpf)
        Else
           _BackColorSuccess = SystemColors.Window
           _BackColorFailure = SystemColors.Window
           _BackColorBreak = SystemColors.Window
           _BackColorUndo = SystemColors.Window
        End If
    End Sub

    Private Function DateUpdate(ByVal Fpath As String, ByVal d1 As Date, ByVal d2 As Date, state As Boolean) As Integer

        Dim s As Integer
        Dim Finfo As New FileInfo(Fpath)

     'ファイルが見つかりません (HResult=-2147024894)
        If Finfo.Attributes = -1 Then
           If _List_ChckMode <> 0 Then

              ListView.EndUpdate()
                s = ShowFnfMessage(Fpath, state)
              ListView.BeginUpdate()

              Return s - 5              '1:ok / 2:Cancel
           Else
              Return DialogResult.OK    '1:ok (Ignore)
           End If
        End If

     'Change check
        Dim f As Boolean = (Chk_Make.Checked AndAlso (Finfo.CreationTime <> d1)) OrElse
                           (Chk_Repl.Checked AndAlso (Finfo.LastWriteTime <> d2))

        If Not f Then Return -1

      Try
        If (Finfo.Attributes And FileAttributes.Directory) <> 0 Then
           Dim Dinfo As New DirectoryInfo(Fpath)

           If Chk_Make.Checked Then Dinfo.CreationTime = d1
           If Chk_Repl.Checked Then Dinfo.LastWriteTime = d2
        Else
           If Finfo.IsReadOnly Then
              Finfo.IsReadOnly = False
           Else
              f = False
           End If

           If Chk_Make.Checked Then Finfo.CreationTime = d1
           If Chk_Repl.Checked Then Finfo.LastWriteTime = d2

           If f Then Finfo.IsReadOnly = True
        End If

        Return 0

      Catch ex As Exception

        Dim HResult As Integer = Marshal.GetHRForException(ex)

        Select Case HResult

       'ファイルが別のプロセスで使用...
          Case -2147024864

            If _List_ChckMode <> 0 Then
               ListView.EndUpdate()
                 Dim MsgBox As New MessageBoxEx

                 MsgBox.SetButtonText(2, "スキップ")
                 MsgBox.SetButtonText(3, "中止")
                 s = MsgBox.Dialog(Me, _Msg_Ioe & Fpath, _Msg_Cap, _
                                   MessageBoxEx.Buttons.RetryIgnoreCancel, _
                                   MessageBoxIcon.Warning, _
                                   MessageBoxDefaultButton.Button3, , _MessageBoxSound)
                 MsgBox.Dispose()

                 Me.Update()
                 WinMessage.Clear(Me)

               ListView.BeginUpdate()

               Return s                 '4:Retry / 2:Cancel / 5:Ignore
            End If

       'パス(ファイル)へのアクセスが拒否
          Case -2147024809, -2147024891

            If _List_ChckMode <> 0 Then
               ListView.EndUpdate()
                 Dim MsgBox As New MessageBoxEx
                 s = MsgBox.Dialog(Me, _Msg_Uae & Fpath & _Msg_Nxt, _Msg_Cap, _
                                   MessageBoxEx.Buttons.CenterYesNo, _
                                   MessageBoxIcon.Warning, _
                                   MessageBoxDefaultButton.Button2, , _MessageBoxSound)
                 MsgBox.Dispose()

                 Me.Update()
                 WinMessage.Clear(Me)

               ListView.BeginUpdate()

               Return s - 5             '1:ok / 2:Cancel
            End If

          Case Else

            ListView.EndUpdate()
              Dim MsgBox As New MessageBoxEx
              MsgBox.Show(Me, ex.Message, _App_Name, _
                          MessageBoxEx.Buttons.CenterOK, _
                          MessageBoxIcon.Error, , _
                          MessageBoxEx.Align.CenterRed, _MessageBoxSound)
              MsgBox.Dispose()

              Me.Update()
              WinMessage.Clear(Me)

            ListView.BeginUpdate()

            Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
              sw.WriteLine(Now & vbTab & "DateUpdate" & vbTab & ex.Message)
            End Using

            Return DialogResult.Abort
        End Select
      End Try
'
        Return DialogResult.OK          '1:ok (Ignore)
    End Function

    Private Function DateTryParse(ByVal text As String, ByVal i As Integer, ByRef result As Date) As Boolean

        Dim d As String

        d = text.Substring(0, i + 2) & "/" & text.Substring(i + 2, 2) & "/" & text.Substring(i + 4, 2)

        If DateTime.TryParse(d, result) Then
           If (result >= _DateTimeMinDate) AndAlso _
              (result <= _DateTimeMaxDate) Then Return True
        End If

        Return False
    End Function

    Private Function DateTimeTryParse(ByVal text As String, ByVal i As Integer, ByRef result As Date) As Boolean

        Dim d As String
        Dim t As String

        d = text.Substring(0, i + 2) & "/" & text.Substring(i + 2, 2) & "/" & text.Substring(i + 4, 2)
        t = text.Substring(i + 6, 2) & ":" & text.Substring(i + 8, 2) & ":" & text.Substring(i + 10, 2)

        If DateTime.TryParse(d & " " & t, result) Then
           If (result >= _DateTimeMinDate) AndAlso _
              (result <= _DateTimeMaxDate) Then Return True
        End If

        Return False
    End Function

    Private Function DateTimeTextParse(ByVal text As String, ByRef result As Date) As Boolean

    '1930/01/01 08:59:59 - 2029/01/01 08:59:58  32bit Win7
    '1980/01/01 08:59:59 - 2108/01/01 08:59:58  64bit Win10 Explorer.exe

        Dim NumText As String = text.Where(Function(C) C Like "[0-9]").ToArray()

        If NumText.Length < 6 Then Return False

        If _onAltKeyDown Then
           If NumText.Length >= 12 Then
           'yyMMdd hhmmss
              If DateTimeTryParse(NumText, 0, result) Then Return True
           End If

        'yyMMdd
           Return DateTryParse(NumText, 0, result)
        End If

        If NumText.Length >= 14 Then
        'yyyyMMdd hhmmss
           If DateTimeTryParse(NumText, 2, result) Then Return True
        End If

        If NumText.Length >= 12 Then
        'yyMMdd hhmmss
           If DateTimeTryParse(NumText, 0, result) Then Return True
        End If

        If NumText.Length >= 8 Then
        'yyyyMMdd
           If DateTryParse(NumText, 2, result) Then Return True
        End If

     'yyMMdd
        Return DateTryParse(NumText, 0, result)
    End Function

    Private Function DateTimeCultureParse(ByVal text As String, ByRef result As Date) As Boolean

     'カルチャ (en-US) を指定して簡易チェック
        Dim ci As New Globalization.CultureInfo("en-US")

        Return Date.TryParse(text, ci, Globalization.DateTimeStyles.None, result)
    End Function

    Private Function DateTimePathCheck(ByVal FilePath As String, ByRef result As Date) As Boolean

        If Not File.Exists(FilePath) Then Return False

        Return DateTimeTextParse(Path.GetFileName(FilePath), result)
    End Function

    Private Function DateTimeTextCheck(ByVal text As String, ByRef result As Date) As Boolean

     '和表記の文字列は処理の妨げになる為、置換

        text = New Regex("[年月]").Replace(text, "/")
        text = New Regex("[時分]").Replace(text, ":")

     '全角から半角へ変換
        text = StrConv(text, VbStrConv.Narrow)

        Dim DateState As Boolean = True
        Dim TimeState As Boolean
        Dim DateValue As Date
        Dim TimeValue As Date
        Dim TimeText As String = ""
        Dim TimeRepresentation As String = ""

        Dim m As Match

     '[yy]yy/mm/dd
        m = Regex.Match(text, "(\d{4}|\d{1,2})(\/|-|\.)\d{1,2}(\/|-|\.)\d{1,2}")
        If m.Success Then
           DateState = Date.TryParse(m.Value, DateValue)
        Else

        'mm/dd
           m = Regex.Match(text, "\d{1,2}(\/|-|\.)\d{1,2}")
           If m.Success Then
              DateState = Date.TryParse(String.Format("{0}/{1}", Now.Year, m.Value), DateValue)
           End If
        End If

     '日付の変換に失敗
        If Not DateState Then Return False

     'hh:mm:ss
        m = Regex.Match(text, "(\d{1,2})(:\d{1,2}:\d{1,2})")
        If m.Success Then
           TimeText = m.Value
        Else

        'hh:mm
           m = Regex.Match(text, "(\d{1,2})(:\d{1,2})")
           If m.Success Then
              TimeText = m.Value
           End If
        End If

        If m.Success Then

           Dim v As Integer = Convert.ToInt32(m.Groups(1).Value)

        '24時以上は翌日に修正
           If v > 23 Then
              v -= 24

           '日付が指定されていない場合は現在の日付を使用する
              If (DateValue = Nothing) Then DateValue = Now.Date

              DateValue = DateValue.AddDays(1)
           Else

           'AM, PM 表記を復元
              If Regex.IsMatch(text.ToUpper, "(AM|A.M.|午前)") Then
                 If v < 13 Then
                    TimeRepresentation = "AM"
                 End If
              ElseIf Regex.IsMatch(text.ToUpper, "(PM|P.M.|午後)") Then
                 TimeRepresentation = "PM"
              End If
           End If

           TimeText = String.Format("{0}{1} {2}", v, m.Groups(2).Value, TimeRepresentation)
           TimeState = Date.TryParse(TimeText, TimeValue)

        '時間の変換に失敗
           If Not TimeState Then Return False
        End If

     '日付、時間とも指定されていない場合は False を返す
        If (DateValue = Nothing) Then
           If (TimeValue = Nothing) Then Return False

           DateValue = Now.Date
        End If

        result = DateValue.Add(TimeValue.TimeOfDay)

        Return True
    End Function

    Private Function ShowFnfMessage(ByVal path As String, ByVal state As Boolean) As Integer

        Dim s As Integer
        Dim MsgBox As New MessageBoxEx

        If state Then
           s = MsgBox.Dialog(Me, _Msg_Fnf & path & _Msg_Nxt, _Msg_Cap, _
                             MessageBoxEx.Buttons.CenterYesNo, _
                             MessageBoxIcon.Warning, _
                             MessageBoxDefaultButton.Button2, , _MessageBoxSound)
        Else
           s = DialogResult.No

           MsgBox.Show(Me, _Msg_Fnf & path, _Msg_Cap, _
                       MessageBoxEx.Buttons.CenterOK, _
                       MessageBoxIcon.Warning, , , _MessageBoxSound)
        End If

        MsgBox.Dispose()

        Me.Update()
        WinMessage.Clear(Me)

        Return s
    End Function

    Private Sub ShowInfoMessage(ByVal msg As String, ByVal cap As String)

        Dim MsgBox As New MessageBoxEx

        MsgBox.Show(Me, msg, cap, _
                    MessageBoxEx.Buttons.CenterOK, _
                    MessageBoxIcon.Information, , , _MessageBoxSound)
        MsgBox.Dispose()

        Me.Update()
        WinMessage.Clear(Me)
   End Sub

    Private Sub ShowWarningMessage(ByVal msg1 As String, ByVal msg2 As String, ByVal cap As String)

        Dim MsgBox As New MessageBoxEx

        If Not String.IsNullOrEmpty(msg2) Then
           msg1 &= vbCrLf & vbCrLf
        End If

        MsgBox.Show(Me, msg1 & msg2, cap, _
                    MessageBoxEx.Buttons.CenterOK, _
                    MessageBoxIcon.Warning, , , _MessageBoxSound)
        MsgBox.Dispose()

        Me.Update()
        WinMessage.Clear(Me)
    End Sub

    Private Sub ShowSettings(ByVal Alt As Boolean)

        Dim cForm As New Form_F1

        cForm.Chk_DialogBox.Checked = StripMenu_DialogBox.Checked
        cForm.Chk_ToolTip.CheckState = StripMenu_ToolTip.CheckState
        cForm.Chk_MsgSound.Checked = StripMenu_MsgSound.Checked
        cForm.Chk_LogWrite.Checked = StripMenu_LogWrite.Checked
        cForm.Chk_IniSave.Checked = StripMenu_IniSave.Checked

        cForm.Chk_GridLines.Checked = StripMenu_GridLines.Checked
        cForm.Chk_CtrlUpDown.Checked = StripMenu_CtrlUpDown.Checked

        cForm.Trk_Emphasis.Value = _UseEmphasisColor

        cForm.Log_CheckSize = _LogFileMaxSize
        cForm.Log_LeaveCount = _LogFileLeaveCount
        cForm.Log_MaxCount = _DateTimeChangeLogMaxCount

        cForm.ColorName(0) = _AttrColorName(1)
        cForm.ColorName(1) = _AttrColorName(2)
        cForm.ColorName(2) = _AttrColorName(4)
        cForm.ColorName(3) = Lbl_Items.ForeColor.Name
        cForm.ColorName(4) = _ColorSuccess.Name
        cForm.ColorName(5) = _ColorFailure.Name
        cForm.ColorName(6) = _ColorBreak.Name
        cForm.ColorName(7) = _ColorUndo.Name
        cForm.ColorName(8) = _ColorMain.Name
        cForm.ColorName(9) = _ColorSub.Name

        cForm.Lsv_FontName = _Lsv_FontName
        cForm.Lsv_FontSize_Id = _Lsv_FontSize_Id
        cForm.Lsv_FontSize = _Lsv_FontSize

        cForm.ToolTip.Active = Me.ToolTip.Active
        cForm.Show(Me, Alt)

        If cForm.DialogResult = DialogResult.OK Then

           StripMenu_DialogBox.Checked = cForm.Chk_DialogBox.Checked
           StripMenu_ToolTip.CheckState = cForm.Chk_ToolTip.CheckState
           StripMenu_MsgSound.Checked = cForm.Chk_MsgSound.Checked
           StripMenu_LogWrite.Checked = cForm.Chk_LogWrite.Checked
           StripMenu_IniSave.Checked = cForm.Chk_IniSave.Checked

           StripMenu_GridLines.Checked = cForm.Chk_GridLines.Checked()
           StripMenu_CtrlUpDown.Checked = cForm.Chk_CtrlUpDown.Checked

           _UseEmphasisColor = cForm.Trk_Emphasis.Value

        'StripMenu Emphasis チェック表示更新
           StripMenu_E_Control()

           _LogFileMaxSize = cForm.Log_CheckSize
           _LogFileLeaveCount = cForm.Log_LeaveCount
           _DateTimeChangeLogMaxCount = cForm.Log_MaxCount

           _AttrColorName(1) = cForm.ColorName(0)
           _AttrColorName(2) = cForm.ColorName(1)
           _AttrColorName(4) = cForm.ColorName(2)
           _ColorSuccess = Color.FromName(cForm.ColorName(4))
           _ColorFailure = Color.FromName(cForm.ColorName(5))
           _ColorBreak = Color.FromName(cForm.ColorName(6))
           _ColorUndo = Color.FromName(cForm.ColorName(7))
           _ColorMain = Color.FromName(cForm.ColorName(8))
           _ColorSub = Color.FromName(cForm.ColorName(9))

        '強調背景色セット
           SetEmphasisBackColor()

           Lbl_Items.ForeColor = Color.FromName(cForm.ColorName(3))

           _Lsv_FontName = cForm.Lsv_FontName
           _Lsv_FontSize_Id = cForm.Lsv_FontSize_Id

           HoverColorSet()
           Lsv_SetInfo()

           Lsv_DisplayMode_Control(True)
        End If

        cForm.Dispose()

        Me.Update()
        WinMessage.Clear(Me, 200)
    End Sub

    Private Sub FileNameChange()
        With ListView

          Dim index As Integer
          Dim IsFolder As Boolean   'ファイル属性 (True:ディレクトリ)
          Dim TitleText As String

          If (.SelectedItems.Count <> 1) Then
             Beep()
             Return
          Else
             Dim fn As String = .SelectedItems(0).Text
             Dim fi As New FileInfo(fn)

             index = .SelectedIndices(0)
             .Items(index).Selected = False     'ちらつきを減らす為、選択状態を一時解除

             If fi.Attributes = -1 Then
                ShowFnfMessage(fn, False)

                .Items(index).Selected = True   '選択状態を戻す
                Return
             End If

             IsFolder = ((fi.Attributes And FileAttributes.Directory) <> 0)

             If IsFolder Then
                TitleText = "フォルダ名の変更"

             'リスト内に同じフォルダのアイテムがないかをチェック
                If Not IsNothing(.FindItemWithText(fn & "\", False, 0, True)) Then

                   ShowWarningMessage("リスト内に同じフォルダのアイテムがある為、" & vbCrLf &
                                       TitleText & "はできません。", fn, _Msg_Cap)

                   .Items(index).Selected = True   '選択状態を戻す
                   Return
                End If
             Else
                TitleText = "ファイル名の変更"
             End If
          End If

          Dim nForm As New Form_F2
          Dim bPath As String = .Items(index).Text
          Dim aPath As String = ""
          Dim bName As String = Path.GetFileName(bPath)
          Dim ext_f As Boolean

          nForm.Text = TitleText
          nForm.ToolTip.Active = Me.ToolTip.Active

          nForm.Show(Me, bName, ext_f, _ClipboardMonitor)
          Me.Update()

          If nForm.DialogResult = DialogResult.OK Then
             If ext_f Then
                aPath = Path.Combine(Path.GetDirectoryName(bPath), bName)
             Else
                aPath = Path.Combine(Path.GetDirectoryName(bPath), bName & Path.GetExtension(bPath))
             End If

             Dim s As Integer
             Do
               Try
                 If IsFolder Then
                    Directory.Move(bPath, aPath)
                 Else
                    File.Move(bPath, aPath)
                 End If

                 .BeginUpdate()
                 .Items(index).SubItems(0).Text = aPath
                 .Items(index).SubItems(1).Text = Path.GetFileName(aPath)
                 .EndUpdate()

                 Exit Do

               Catch ex As Exception

                 Dim MsgBox As New MessageBoxEx

                 MsgBox.SetButtonText(2, "中止")
                 s = MsgBox.Dialog(Me, TitleText & "に失敗しました。" & vbCr & vbCr &
                                   String.Format("  ( {0} ) ", ex.Message.Trim) & vbCr & vbCr &
                                   Path.GetFileName(bPath) & vbCr & "  ↓" & vbCr &
                                   Path.GetFileName(aPath), _App_Name,
                                   MessageBoxEx.Buttons.RetryCancel,
                                   MessageBoxIcon.Warning,
                                   MessageBoxDefaultButton.Button2, , _MessageBoxSound)
                 MsgBox.Dispose()

                 Me.Update()
                 WinMessage.Clear(Me)
               End Try
             Loop Until s = DialogResult.Cancel

          'Log Write
             If (s = 0) AndAlso _LogFileWriting Then
                If File.Exists(_Log_File) Then
                   If File.Exists(_TempFile) Then File.Delete(_TempFile)
                   File.Move(_Log_File, _TempFile)
                Else
                   Using sw As New StreamWriter(_TempFile, False, Encoding.UTF8) : End Using
                End If

                Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
                  sw.WriteLine("-- {0} : Rename --", Date.Now.ToString(_DateTimeFormat))
                  sw.WriteLine(bPath)
                  sw.WriteLine("  to")
                  sw.WriteLine(aPath)
                  sw.WriteLine()

                  Dim fi As New FileInfo(_TempFile)
                  If fi.Length > 0 Then
                     Using sr As New StreamReader(_TempFile, Encoding.UTF8)
                       sw.Write(sr.ReadToEnd())
                     End Using
                  End If
                End Using

                File.Delete(_TempFile)
             End If

          'Undoを無効にする
             If s = 0 Then Btn_Undo.Enabled = False
          End If

          nForm.Dispose()

          .Items(index).Selected = True '選択状態を戻す
        End With
    End Sub

    Private Sub FileAttributesChange()

        If ListView.SelectedItems.Count = 0 Then
           Beep()
           Return
        End If

        Dim SelectIndex As New List(Of Integer)
        Dim Select_Path As New List(Of String)

        Dim state As Boolean = (ListView.SelectedItems.Count <> 1)

        For Each LsvItem As ListViewItem In ListView.SelectedItems

          If Not PathExists(LsvItem.Text) Then

             Lsv_BackColor_Set(LsvItem, _BackColorFailure)
             If ShowFnfMessage(LsvItem.Text, state) <> DialogResult.Yes Then Return
          Else

             SelectIndex.Add(LsvItem.Index)
             Select_Path.Add(LsvItem.Text)

             ListView.Items(LsvItem.Index).Selected = False 'ちらつきを減らす為、選択状態を一時解除
          End If
        Next

        If Select_Path.Count = 0 Then Return

        Dim Fname As String
        Dim Fattr As Integer

        If Select_Path.Count = 1 Then
           Fattr = File.GetAttributes(Select_Path.Item(0)) And 23
           Fname = Path.GetFileName(Select_Path.Item(0)).TrimStart
        Else
           Fattr = -1
           Fname = String.Format("{0:###,##0} 個の項目が選択されています。", Select_Path.Count)
        End If

        Dim nForm As New Form_F3

        nForm.Show(Me, Fname, Fattr)
        Me.Update()

        If nForm.DialogResult = DialogResult.OK Then

           Dim i As Integer
           Dim c As Color = GetAttributeColor(Fattr)

           For Each p As String In Select_Path

             Try
               Dim fa As FileAttributes = DirectCast((File.GetAttributes(p) And Not 7) Or Fattr, FileAttributes)

               File.SetAttributes(p, fa)

               ListView.Items(SelectIndex(i)).SubItems(1).ForeColor = c
               i += 1

             Catch ex As Exception

               Dim msg As String = String.Format("  ( {0} ) ", ex.Message.Trim) & vbCr & vbCr & p
               ShowWarningMessage("属性の変更に失敗しました。", msg, _App_Name)
             End Try
           Next
        End If

        nForm.Dispose()

   '選択状態を戻す
      ListView.BeginUpdate()
        For Each i As Integer In SelectIndex
           ListView.Items(i).Selected = True
        Next
      ListView.EndUpdate()
    End Sub

    Private Sub GetDateTimeFormatValue()

      If _JobMode <> 0 Then Return

        Dim result As Date

        If Not DateTimeTextParse(ListView.FocusedItem.SubItems(1).Text, result) Then Return

        Pck_Date.Value = result
        Pck_Time.Value = result

        GroupBox3.ForeColor = _ColorSuccess
        GroupBox3.Update()
        Thread.Sleep(250)
        GroupBox3.ForeColor = SystemColors.ControlText
        GroupBox3.Update()
    End Sub

    <DebuggerStepThrough()>
    Private Sub DateTimeChangeLogUpdate()

        If _DateTimeValueChangeFlag Then
           _DateTimeValueChangeFlag = False

           Dim dt As Date = DateTime.Parse(Pck_Date.Text.TrimStart & " " & Pck_Time.Text.TrimStart)
           Dim ds As String = dt.ToString(_CmbDtLogFormat)
           Dim ss As String = String.Format("text = '{0}'", ds)

           If _DateTimeChangeLog.Select(ss).Length = 0 Then

              Dim dr As DataRow = _DateTimeChangeLog.NewRow

              dr("value") = dt
              dr("text") = ds

              _DateTimeChangeLog.Rows.InsertAt(dr, 0)

              If _DateTimeChangeLog.Rows.Count > _DateTimeChangeLogMaxCount Then
                 For i As Integer = _DateTimeChangeLog.Rows.Count - 1 To _DateTimeChangeLogMaxCount Step -1
                   _DateTimeChangeLog.Rows.RemoveAt(_DateTimeChangeLogMaxCount)
                 Next
              End If
           End If
           _DateTimeChangeLog.AcceptChanges()

           Cmb_DateTimeLog.SelectedIndex = 0
        End If
    End Sub

#End Region

#Region "-- FormLoad, FormClosed"

    Private Function TimeConvert(ByVal value As String, ByRef hhmm As String) As Integer
'     hhmm: [+-]hh:mm

        Dim v As String = value.Trim

        If Not Regex.IsMatch(v, "^[\+-]?\d{1,2}:\d{2}$") Then v = hhmm

        Dim i As Integer = v.IndexOf(":")
        Dim s As Integer = Convert.ToInt32(v.Substring(0, 1) = "-")     '0:正数, 1:負数
        Dim h As Integer = Math.Abs(StringToInteger(v.Substring(0, i)))
        Dim m As Integer = Convert.ToInt32(v.Substring(i + 1, v.Length - (i + 1)))

        If h > 23 Then h = 23
        If m > 59 Then m = 59

        hhmm = "+-".Substring(s, 1) & String.Format("{0:0}:{1:00}", h, m)

        Return (1 - (2 * s)) * ((h * 60) + m)
    End Function

    Private Sub iniLoad()

        If Not File.Exists(_ini_File) Then
           _OpenDirectory = My.Application.Info.DirectoryPath

           Dim MsgBox As New MessageBoxEx

           If MsgBox.Dialog(Me, "設定" & _Msg_Fnf & "設定ファイルを使用しますか？", _Msg_Cap, _
                            MessageBoxEx.Buttons.CenterYesNo, _
                            MessageBoxIcon.Warning, _
                            MessageBoxDefaultButton.Button1, , _MessageBoxSound) _
                          = DialogResult.Yes Then

              StripMenu_ToolTip.Checked = True
              StripMenu_LogWrite.Checked = True
              StripMenu_IniSave.Checked = True
           End If

           MsgBox.Dispose()
           WinMessage.Clear(Me)

        '既定値をセット (_UseEmphasisColor = 2)
           StripMenu_GridLines.Checked = True
        Else
           Dim f As Integer
           Dim sBuffer As New StringBuilder

           sBuffer.Capacity = 512   'バッファのサイズを指定

           Me.Top = GetPrivateProfileInt("Form", "Top", 0, _ini_File)
           Me.Left = GetPrivateProfileInt("Form", "Left", 0, _ini_File)

           _StartWindow_Width = GetPrivateProfileInt("Form", "Width", _StartWindow_Width, _ini_File)
           _StartWindowHeight = GetPrivateProfileInt("Form", "Height", _StartWindowHeight, _ini_File)

'   Switch
           f = GetPrivateProfileString("Form", "OfdAutoUpgrade", _OfdAutoUpgrade.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_DialogBox.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("Form", "ToolTipActive", "True", sBuffer, sBuffer.Capacity, _ini_File)

             If StringToBoolean(sBuffer.ToString) Then
                StripMenu_ToolTip.Checked = True
                ToolTip.Active = True

                GroupBox3_ToolTip_Change()
             End If

           f = GetPrivateProfileString("Form", "ItemToolTips", "True", sBuffer, sBuffer.Capacity, _ini_File)

             If StringToBoolean(sBuffer.ToString) Then
                ListView.ShowItemToolTips = True

                If Not ToolTip.Active Then
                   StripMenu_ToolTip.CheckState = CheckState.Indeterminate
                End If
             End If

           f = GetPrivateProfileString("Form", "MessageSound", _MessageBoxSound.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_MsgSound.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("Form", "LogWriting", _LogFileWriting.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_LogWrite.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("Form", "IniWriting", _IniSave.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_IniSave.Checked = StringToBoolean(sBuffer.ToString)

'   ListView
           f = GetPrivateProfileInt("ListView", "RegistLimit", _List_RegistLimit, _ini_File)
             If (f > 0) AndAlso (f <= 100000) Then _List_RegistLimit = f

           If 0 <> GetPrivateProfileString("ListView", "FontName", _Lsv_FontName, sBuffer, sBuffer.Capacity, _ini_File) Then
              _Lsv_FontName = sBuffer.ToString
           End If

           f = GetPrivateProfileInt("ListView", "FontSize", _Lsv_FontSize_Id, _ini_File)
             If (f >= 0) AndAlso (f < _Lsv_FontSize.Length) Then
                _Lsv_FontSize_Id = f
             End If

           f = GetPrivateProfileString("ListView", "GridLines", _UseGridLines.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_GridLines.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("ListView", "CtrlUpDown", _UseCtrlUpDown.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             StripMenu_CtrlUpDown.Checked = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileInt("ListView", "EmphasisColor", _UseEmphasisColor, _ini_File)
             If (f >= 0) AndAlso (f <= 3) Then _UseEmphasisColor = f

           f = GetPrivateProfileInt("ListView", "DropDelay", _DropDelay, _ini_File)
             If f < 100 Then _DropDelay = 100
'   Mode
           f = GetPrivateProfileInt("Mode", "TreeMode", _List_TreeMode, _ini_File)
             If (f >= 0) AndAlso (f <= 2) Then _List_TreeMode = f

           f = GetPrivateProfileInt("Mode", "FileMode", _List_FileMode, _ini_File)
             If (f >= 0) AndAlso (f <= 2) Then _List_FileMode = f

           f = GetPrivateProfileInt("Mode", "ChckMode", _List_ChckMode, _ini_File)
             If (f = 0) OrElse (f = 1) Then _List_ChckMode = f

'   Adjust
           f = GetPrivateProfileString("Adjust", "UTCvalue", _M_Adjust_Time, sBuffer, sBuffer.Capacity, _ini_File)
             _Adjust_Time = TimeConvert(sBuffer.ToString, _M_Adjust_Time)

        'Date
           f = GetPrivateProfileInt("Adjust", "DateMode", _M_Adjust_D_Mode, _ini_File)
             If (f = 0) OrElse (f = 1) Then
                _M_Adjust_D_Mode = f
                _Adjust_D_Mode = f
             End If

           f = GetPrivateProfileInt("Adjust", "DateValue", _M_Adjust_DValue, _ini_File)
             If (f > 0) AndAlso (f < 1000) Then _M_Adjust_DValue = f
        'Time
           f = GetPrivateProfileInt("Adjust", "TimeMode", _M_Adjust_T_Mode, _ini_File)
             If (f = 0) OrElse (f = 1) Then
                _M_Adjust_T_Mode = f
                _Adjust_T_Mode = f
             End If

           f = GetPrivateProfileInt("Adjust", "TimeValue", _M_Adjust_TValue, _ini_File)
             If (f > 0) AndAlso (f < 1000) Then _M_Adjust_TValue = f

           f = GetPrivateProfileString("Adjust", "TimeCheck", _M_Adjust_RCheck.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             _M_Adjust_RCheck = StringToBoolean(sBuffer.ToString)

           f = GetPrivateProfileString("Adjust", "TimeLow", _M_Adjust_LCheck.ToString, sBuffer, sBuffer.Capacity, _ini_File)
             _M_Adjust_LCheck = StringToBoolean(sBuffer.ToString)
             _Adjust_LCheck = _M_Adjust_LCheck

           For i = 1 To _Adjust_MpfValue.Length - 1
             f = GetPrivateProfileInt("Adjust", "TimeMpfValue" & i, _Adjust_MpfValue(i), _ini_File)
               If (f >= 100) AndAlso (f < 1000) Then _Adjust_MpfValue(i) = f
           Next

           Array.Sort(_Adjust_MpfValue, 1, _Adjust_MpfValue.Length - 1)

           f = GetPrivateProfileInt("Adjust", "TimeMpfValue0", _Adjust_MpfValue(0), _ini_File)
             If Array.IndexOf(_Adjust_MpfValue, f, 0) < 1 Then
                f = _Adjust_MpfValue(3)
             End If
             _Adjust_MpfValue(0) = f

           _M_Adjust_MValue = Convert.ToSingle(f / 100)
           _Adjust_MValue = _M_Adjust_MValue

'   Attributes
           f = GetPrivateProfileString("Attributes", "ReadOnly", "False", sBuffer, sBuffer.Capacity, _ini_File)
             If StringToBoolean(sBuffer.ToString) Then _AttrType(0) = 1
           f = GetPrivateProfileString("Attributes", "Hidden", "False", sBuffer, sBuffer.Capacity, _ini_File)
             If StringToBoolean(sBuffer.ToString) Then _AttrType(1) = 2
           f = GetPrivateProfileString("Attributes", "System", "False", sBuffer, sBuffer.Capacity, _ini_File)
             If StringToBoolean(sBuffer.ToString) Then _AttrType(2) = 4

           _List_AttrType = _AttrType(0) + _AttrType(1) + _AttrType(2)

'   Log File
           f = GetPrivateProfileInt("Log", "LogFileMaxSize", _LogFileMaxSize, _ini_File)
             If (f >= 4096) AndAlso (f <= 65536) Then _LogFileMaxSize = f

           f = GetPrivateProfileInt("Log", "LogFileLeaveCount", _LogFileLeaveCount, _ini_File)
             If (f >= 1) AndAlso (f <= 5) Then _LogFileLeaveCount = f

           f = GetPrivateProfileInt("Log", "DateTimeChangeLogMaxCount", _DateTimeChangeLogMaxCount, _ini_File)
             If (f >= 5) AndAlso (f <= 100) Then _DateTimeChangeLogMaxCount = f

'   Path
           f = GetPrivateProfileString("Path", "OpenDirectory", "", sBuffer, sBuffer.Capacity, _ini_File)
             _OpenDirectory = sBuffer.ToString

'   Color

           f = GetPrivateProfileString("Color", "AttributeR", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _AttrColorName(1) = sBuffer.ToString
           f = GetPrivateProfileString("Color", "AttributeH", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _AttrColorName(2) = sBuffer.ToString
           f = GetPrivateProfileString("Color", "AttributeS", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _AttrColorName(4) = sBuffer.ToString

           f = GetPrivateProfileString("Color", "itemsCount", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then Lbl_Items.ForeColor = Color.FromName(sBuffer.ToString)

           f = GetPrivateProfileString("Color", "Success", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorSuccess = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "Failure", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorFailure = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "Break", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorBreak = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "Undo", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorUndo = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "HoverMain", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorMain = Color.FromName(sBuffer.ToString)
           f = GetPrivateProfileString("Color", "HoverSub", "", sBuffer, sBuffer.Capacity, _ini_File)
             If Color.FromName(sBuffer.ToString).IsKnownColor Then _ColorSub = Color.FromName(sBuffer.ToString)

           sBuffer.Clear()
        End If
    End Sub

    Private Sub iniSave()

        Dim f As Integer

        If Not _IniSave Then
        '設定ファイル書込みを禁止、かつ、設定ファイルがない場合、設定を破棄
           If (Not File.Exists(_ini_File)) Then
              Return
           Else
              Dim sBuffer As New StringBuilder

              sBuffer.Capacity = 512
              f = GetPrivateProfileString("Form", "IniWriting", "True", sBuffer, sBuffer.Capacity, _ini_File)

           '設定ファイル書込みを禁止している場合でも、書き込み設定を更新
              If StringToBoolean(sBuffer.ToString) Then
                 f = WritePrivateProfileString("Form", "IniWriting", "False", _ini_File)
              End If
              sBuffer.Clear()

              Return
           End If
        End If

        f = WritePrivateProfileString("info", "Name", My.Application.Info.ProductName, _ini_File)
        f = WritePrivateProfileString("info", "Ver", My.Application.Info.Version.ToString, _ini_File)

        If Me.WindowState = FormWindowState.Normal Then
           f = WritePrivateProfileString("Form", "Top", Me.Top.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Left", Me.Left.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Width", Me.Width.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Height", Me.Height.ToString, _ini_File)
        Else
'        フォームが最大化、あるいは最小化状態
           f = WritePrivateProfileString("Form", "Top", Me.RestoreBounds.Top.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Left", Me.RestoreBounds.Left.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Width", Me.RestoreBounds.Width.ToString, _ini_File)
           f = WritePrivateProfileString("Form", "Height", Me.RestoreBounds.Height.ToString, _ini_File)
        End If

        f = WritePrivateProfileString("Form", "OfdAutoUpgrade", _OfdAutoUpgrade.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "ToolTipActive", ToolTip.Active.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "ItemToolTips", ListView.ShowItemToolTips.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "MessageSound", _MessageBoxSound.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "LogWriting", _LogFileWriting.ToString, _ini_File)
        f = WritePrivateProfileString("Form", "IniWriting", _IniSave.ToString, _ini_File)

        f = WritePrivateProfileString("ListView", "FontName", _Lsv_FontName, _ini_File)
        f = WritePrivateProfileString("ListView", "FontSize", _Lsv_FontSize_Id.ToString, _ini_File)
        f = WritePrivateProfileString("ListView", "GridLines", _UseGridLines.ToString, _ini_File)
        f = WritePrivateProfileString("ListView", "CtrlUpDown", _UseCtrlUpDown.ToString, _ini_File)
        f = WritePrivateProfileString("ListView", "EmphasisColor", _UseEmphasisColor.ToString, _ini_File)

        f = WritePrivateProfileString("Mode", "TreeMode", _List_TreeMode.ToString, _ini_File)
        f = WritePrivateProfileString("Mode", "FileMode", _List_FileMode.ToString, _ini_File)
        f = WritePrivateProfileString("Mode", "ChckMode", _List_ChckMode.ToString, _ini_File)

        f = WritePrivateProfileString("Adjust", "UTCvalue", _M_Adjust_Time, _ini_File)
        f = WritePrivateProfileString("Adjust", "DateMode", _M_Adjust_D_Mode.ToString, _ini_File)
        f = WritePrivateProfileString("Adjust", "DateValue", _M_Adjust_DValue.ToString, _ini_File)
        f = WritePrivateProfileString("Adjust", "TimeMode", _M_Adjust_T_Mode.ToString, _ini_File)
        f = WritePrivateProfileString("Adjust", "TimeValue", _M_Adjust_TValue.ToString, _ini_File)
        f = WritePrivateProfileString("Adjust", "TimeCheck", _M_Adjust_RCheck.ToString, _ini_File)
        f = WritePrivateProfileString("Adjust", "TimeLow", _M_Adjust_LCheck.ToString, _ini_File)

        For i = 0 To 5
          f = WritePrivateProfileString("Adjust", "TimeMpfValue" & i, _Adjust_MpfValue(i).ToString, _ini_File)
        Next

        f = WritePrivateProfileString("Attributes", "ReadOnly", Convert.ToBoolean(_AttrType(0) = 1).ToString, _ini_File)
        f = WritePrivateProfileString("Attributes", "Hidden", Convert.ToBoolean(_AttrType(1) = 2).ToString, _ini_File)
        f = WritePrivateProfileString("Attributes", "System", Convert.ToBoolean(_AttrType(2) = 4).ToString, _ini_File)

        f = WritePrivateProfileString("Log", "LogFileMaxSize", _LogFileMaxSize.ToString, _ini_File)
        f = WritePrivateProfileString("Log", "LogFileLeaveCount", _LogFileLeaveCount.ToString, _ini_File)
        f = WritePrivateProfileString("Log", "DateTimeChangeLogMaxCount", _DateTimeChangeLogMaxCount.ToString, _ini_File)

        f = WritePrivateProfileString("Path", "OpenDirectory", _OpenDirectory, _ini_File)

        f = WritePrivateProfileString("Color", "AttributeR", _AttrColorName(1), _ini_File)
        f = WritePrivateProfileString("Color", "AttributeH", _AttrColorName(2), _ini_File)
        f = WritePrivateProfileString("Color", "AttributeS", _AttrColorName(4), _ini_File)
        f = WritePrivateProfileString("Color", "itemsCount", Lbl_Items.ForeColor.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Success", _ColorSuccess.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Failure", _ColorFailure.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Break", _ColorBreak.Name, _ini_File)
        f = WritePrivateProfileString("Color", "Undo", _ColorUndo.Name, _ini_File)
        f = WritePrivateProfileString("Color", "HoverMain", _ColorMain.Name, _ini_File)
        f = WritePrivateProfileString("Color", "HoverSub", _ColorSub.Name, _ini_File)
    End Sub

    Private Sub CmdSendToLoad()

        If My.Application.CommandLineArgs.Count = 0 Then Return

'     設定ファイルの内容より、スイッチを優先適用。
'     同系統のスイッチを指定した場合は、後で指定したものを適用。

        Dim RootSelect As Boolean

        For Each cmd As String In My.Application.CommandLineArgs
          If cmd.StartsWith("/") Then
             Select Case cmd.TrimStart("/"c).ToLower
               Case "x" : If _IniSave Then
                             StripMenu_IniSave.Checked = False
                          End If

               Case "l" : If _LogFileWriting Then
                             StripMenu_LogWrite.Checked = False
                          End If

               Case "c0" : _List_ChckMode = 0
               Case "c1" : _List_ChckMode = 1
               Case "t0" : _List_TreeMode = 0
               Case "t1" : _List_TreeMode = 1
               Case "t2" : _List_TreeMode = 2
               Case "f0" : _List_FileMode = 0
               Case "f1" : _List_FileMode = 1
               Case "f2" : _List_FileMode = 2

               Case "b" : _DoubleBuffering = False
               Case "m" : _ClipboardMonitor = True
               Case "h" : _UseHideSelection = True
                          ListView.HideSelection = True

               Case "st" : StripMenu_ToolTip.CheckState = CheckState.Indeterminate
                           ToolTip.Active = False
                           ListView.ShowItemToolTips = True

               Case "ht" : StripMenu_TaskBar.Checked = False
                           Me.ShowInTaskbar = False

               Case Else
                 Dim msg As String = String.Format("無効なスイッチ ""{0}"" が指定されています。", cmd)
                 ShowWarningMessage(msg, "", _Msg_Cap)
             End Select
          End If
        Next

'     パス情報を取得

        Dim l As New List(Of String)
        For Each cmd As String In My.Application.CommandLineArgs
          If Not cmd.StartsWith("/") Then

          '先頭の１文字を大文字、末尾の "\" を取り除く
             cmd = cmd.Substring(0, 1).ToUpper & cmd.Substring(1).TrimEnd("\"c)

             If DriveCheck(cmd) Then
                If File.Exists(cmd) Then
                   l.Add(cmd)
                Else
                   If Directory.Exists(cmd) Then
                   'ルートチェック
                      If IsNothing(Path.GetDirectoryName(cmd)) Then
                      'ルートドライブの場合、末尾に "\" を付与
                         If Regex.Match(cmd.ToUpper, "^[A-Z]:").Success Then
                            cmd &= "\"c
                         End If

                         If RootSelectDialog(cmd) Then
                            RootSelect = True

                            l.AddRange(GetRootEntries(cmd))
                         End If
                      Else
                         l.Add(cmd)
                      End If
                   Else
                      ShowWarningMessage("無効なパスが指定されています。", cmd, _Msg_Cap)
                   End If
                End If
             End If
             Exit For
          End If
        Next

'     取得したパス情報をDropテーブルへ移す
        If l.Count <> 0 Then
           _DropFileList = l.ToArray

           If RootSelect Then

              Dim MsgForm As New Form()
              Dim MsgText As New Label()

              MsgForm.SuspendLayout()
              MsgText.AutoSize = True
              MsgText.Font = New Font(Me.Font.Name, 9.75F, FontStyle.Regular)
              MsgText.Text = "ディレクトリエントリを取得しています。"

              MsgForm.AutoScaleDimensions = New SizeF(96.0!, 96.0!)
              MsgForm.AutoScaleMode = AutoScaleMode.Dpi
              MsgForm.ClientSize = New Size(274, 34)
              MsgForm.ControlBox = False
              MsgForm.Controls.Add(MsgText)
              MsgForm.FormBorderStyle = FormBorderStyle.FixedDialog
              MsgForm.StartPosition = FormStartPosition.Manual

              MsgForm.Top = Me.Top + ((Me.Height - MsgForm.Height) \ 2)
              MsgForm.Left = Me.Left + ((Me.Width - MsgForm.Width) \ 2)

              Dim x As Integer = (MsgForm.ClientSize.Width - MsgText.Width) \ 2
              Dim y As Integer = (MsgForm.ClientSize.Height - MsgText.Height) \ 2

              MsgText.Location = New Point(x, y)

              MsgForm.ResumeLayout()
              MsgForm.PerformLayout()
              MsgForm.Show()
              MsgForm.Update()

              Thread.Sleep(1000)

           'ListView登録 (Lsv_AddRange 内で ListView.ColumnWidthChanged イベント登録)
              Lsv_AddRange()

              MsgForm.Close()
              MsgText.Dispose()
              MsgForm.Dispose()
           Else

           'ListView登録 (Lsv_AddRange 内で ListView.ColumnWidthChanged イベント登録)
              Lsv_AddRange()
           End If
        End If
    End Sub

''' <summary>
''' フォント (ListView はフォント情報のみ) をセットします。
''' </summary>

    Private Sub FontSetting()

        Dim FontSize As Single = Me.Font.Size
        Dim FontNameId As Integer = 0

'   フォントチェック
        For Each fnt As FontFamily In FontFamily.Families
          Select Case fnt.Name
            Case _Lbl_FontName(1) : FontNameId = 1
            Case _Lbl_FontName(2) : FontNameId = 2 : Exit For
          End Select
        Next

'   Dpi check
        If _FontSizeMpf <> 1.0F Then

           FontSize = GetNearFontSize(FontSize / _FontSizeMpf)

           Dim s As Single = GetNearFontSize(Lbl_Attr_R.Font.Size / _FontSizeMpf)
           Dim f As New Font("Tahoma", s)

           Lbl_Attr_R.Font = f
           Lbl_Attr_H.Font = f
           Lbl_Attr_S.Font = f

           s = GetNearFontSize(Lbl_Items.Font.Size / _FontSizeMpf)
           Lbl_Items.Font = New Font("Tahoma", s)

           s = GetNearFontSize(Pck_Date.Font.Size / _FontSizeMpf)
           f = New Font("Verdana", s)

           Pck_Date.Font = f
           Pck_Time.Font = f
           Cmb_DateTimeLog.Font = f
           Txt_Date_Ajst.Font = f
           Txt_Time_Ajst.Font = f

           s = GetNearFontSize(Lbl_DPlus.Font.Size / _FontSizeMpf)
           f = New Font("MS UI Gothic", s)

           Lbl_DPlus.Font = f
           Lbl_TPlus.Font = f

           Btn_Dummy.Font = New Font("Verdana", FontSize)

           Dim w As Integer = Convert.ToInt32(StripMenu_Cmb_FontSize.Width * _FontSizeMpf)
           Dim h As Integer = StripMenu_Cmb_FontSize.Height

           StripMenu_Cmb_FontSize.Size = New Size(w, h)
        End If

'   フォント変更
        Me.Font = New Font(_Lbl_FontName(FontNameId), FontSize)
    End Sub

    Private Sub HoverColorSet()

        For Each ctl As Control In Me.Controls
          If ctl.HasChildren Then
             For Each cctl As Control In ctl.Controls
               Select Case cctl.GetType.Name
                 Case "CheckBoxSL"
                   DirectCast(cctl, CheckBoxSL).HoverColor = _ColorMain
                 Case "LabelSL"
                   DirectCast(cctl, LabelSL).HoverColor = _ColorMain
               End Select
             Next
          Else
             If ctl.GetType.Name = "LabelSL" Then
                DirectCast(ctl, LabelSL).HoverColor = _ColorMain
             End If
          End If
        Next

        GroupBox1.HoverColor = _ColorSub
        GroupBox3.HoverColor = _ColorSub

        Btn_LogApply.HoverColor = _ColorSub
        Btn_Exec.HoverColor = _ColorMain
    End Sub

#End Region

#Region "-- Form"

    Private Sub StyleSetting()

        Dim state As Boolean = Application.RenderWithVisualStyles

     'DateTimePicker 入力フォーマット修正
        Dim fd As String = Pck_Date.CustomFormat.TrimStart
        Dim ft As String = Pck_Time.CustomFormat.TrimStart
        Dim sp As Integer = Convert.ToInt32(state)

        Pck_Date.CustomFormat = New String(" "c, 4 - sp) & fd
        Pck_Time.CustomFormat = New String(" "c, 8 - sp) & ft

     'スタイル設定
        If state Then
           If _OS_Type Then
              ListView.HeaderVisual = ListViewEx.Header.Gradation   '8, 10
           Else

           'Visual Style Setting
              ListView.HeaderVisual = ListViewEx.Header.Normal      'XP, 7
           End If
        Else

        'Classic Style Setting
           ListView.HeaderVisual = ListViewEx.Header.Gradation
        End If

        _onStyleChange = state
    End Sub

    Private Sub GroupBox3Setting()

        Const ControlSpace = 6

    'スクロールバー基準値に対するサイズ変更分（差分値）を算出
        Dim LogDifference As Integer = SystemInformation.VerticalScrollBarWidth - _ScrollBarWidth
        Dim GrpDifference As Integer

        If Application.RenderWithVisualStyles Then
           GrpDifference = (LogDifference - 1) * 2
        Else
           GrpDifference = LogDifference
        End If

    '差分値をコントロールに適用
        GroupBox3.Width = _GroupBox3Width + GrpDifference
        Cmb_DateTimeLog.Width = _DateTimeLogWidth + LogDifference

        Dim w As Integer = Cmb_DateTimeLog.Width + ControlSpace             'offset
        Dim l As Integer = (GroupBox3.Width - Btn_LogApply.Width - w) \ 2   'Left
        Dim x As Integer = GroupBox3.Location.X + l

    'コントロールの位置を修正
        Cmb_DateTimeLog.Location = New Point(x, Cmb_DateTimeLog.Location.Y)
        x += w
        Btn_LogApply.Location = New Point(x, Btn_LogApply.Location.Y)
    End Sub

    Private Sub Lbl_Items_Update()
        If ListView.Items.Count Mod 13 = 0 Then
           Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)
           Lbl_Items.Update()
        End If
    End Sub

    Private Sub Chk_Date_Control()
        Dim flg As Boolean = (Chk_Date.Checked AndAlso (_List_TreeMode = 0) AndAlso (_List_FileMode = 0))

        Pck_Date.Enabled = Chk_Date.Checked
        Btn_DtClr.Enabled = Chk_Date.Checked
        Btn_Next.Enabled = Chk_Date.Checked
        Btn_Prev.Enabled = Chk_Date.Checked
        Lbl_DPlus.Enabled = Chk_Date.Checked

        Lbl_DAdjustMode.Enabled = flg
        Txt_Date_Ajst.Enabled = flg

        Btn_Exec_Control()
    End Sub

    Private Sub Chk_Time_Control()
        Dim flg As Boolean = (Chk_Time.Checked AndAlso (_List_TreeMode = 0) AndAlso (_List_FileMode = 0))

        Pck_Time.Enabled = Chk_Time.Checked
        Btn_TmClr.Enabled = Chk_Time.Checked
        Btn_UTC0.Enabled = Chk_Time.Checked
        Btn_UTC9.Enabled = Chk_Time.Checked
        Lbl_TPlus.Enabled = Chk_Time.Checked

        Lbl_TAdjustMode.Enabled = flg
        Txt_Time_Ajst.Enabled = flg
        Chk_Rand.Enabled = flg

        Btn_Exec_Control()
    End Sub

    Private Sub ModeLamp_Control()
        Lbl_Mode_Lamp_1.Enabled = (_Lsv_DisplayMode = 1)
        Lbl_Mode_Lamp_2.Enabled = (_Lsv_DisplayMode = 2)
    End Sub

    Private Overloads Sub Btn_Exec_Control()
        Dim f As Boolean

        If _JobMode <> 0 Then
           f = (ListView.Items.Count <> 0)
        Else
           f = (ListView.Items.Count <> 0) And _
              (Chk_Make.Checked OrElse Chk_Repl.Checked) And _
              (Chk_Date.Checked OrElse Chk_Time.Checked)
        End If
        Btn_Exec_Control(f)
    End Sub

    Private Overloads Sub Btn_Exec_Control(ByVal state As Boolean)
        If state <> Btn_Exec.Enabled Then
           If state Then
              Btn_Exec.Cursor = Cursors.Hand
           Else
              Btn_Exec.Cursor = Cursors.Default
           End If

           Btn_Exec.Enabled = state
        End If
    End Sub

#End Region

#Region "-- DateTime Picker"

''' <summary>
''' DateTimePicker の次のフォーカスが対象コントロールかを調べます。
''' 対象コントロールでない場合、True を返します。
''' </summary>

    Private Function DateTimePickerExclusionControl() As Boolean

        Const ExclusionControl = "Btn_DtClr Btn_TmClr Btn_Next Btn_Prev Btn_UTC9 Btn_UTC0 Pck_Date Pck_Time"

        Return (ExclusionControl.IndexOf(Me.ActiveControl.Name) = -1)
    End Function

    Private Sub DateTimePickerButtonControl()

     'UpDownボタン変更

     'ボタンの幅をスクロールバーの幅に対応させる為の処理
        Pck_Date.ReSetShowUpDown(False)
        Pck_Time.ReSetShowUpDown(True)

        Cmb_DateTimeLog.Font = Nothing
        Cmb_DateTimeLog.Font = Pck_Time.Font
    End Sub

#End Region

#Region "-- Button"

    Private Sub Open_File()

        If Lsv_LimitCheck(False, True) Then Return

        If Not Directory.Exists(_OpenDirectory) Then
           _OpenDirectory = My.Application.Info.DirectoryPath
        End If

        Dim ofd As New OpenFileDialog()

        ofd.AutoUpgradeEnabled = _OfdAutoUpgrade
        ofd.InitialDirectory = _OpenDirectory
        ofd.RestoreDirectory = True
        ofd.Title = "リストに登録するファイルを選択してください"
        ofd.Filter = _FileDialogFilter
        ofd.Multiselect = True

        If ofd.ShowDialog() = DialogResult.OK Then
           _OpenDirectory = Path.GetDirectoryName(ofd.FileName)

           Dim FocusedIndex As Integer = ListView.Items.Count

           ListView.BeginUpdate()

             If _Lsv_DupCheckFlg Then
                For Each Fpath As String In ofd.FileNames
                  If IsNothing(ListView.FindItemWithText(Fpath, False, 0, False)) Then
                     Lsv_Add(Fpath, 0)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If
                Next
             Else
                For Each Fpath As String In ofd.FileNames
                  Lsv_Add(Fpath, 0)
                  Lbl_Items_Update()

                  If Lsv_LimitCheck(True, False) Then Exit For
                Next
             End If

          'マーカークリア
             If ListView.Items.Count <> FocusedIndex AndAlso
                ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)

             Lsv_Display_Control(FocusedIndex)

           ListView.EndUpdate()
        End If

        ofd.Dispose()
    End Sub

    Private Sub Open_Folder()

        If Lsv_LimitCheck(False, True) Then Return

        If Not Directory.Exists(_OpenDirectory) Then
           _OpenDirectory = My.Application.Info.DirectoryPath
        End If

        Dim fbd As New FolderBrowserDialog

        fbd.RootFolder = Environment.SpecialFolder.Desktop
        fbd.SelectedPath = _OpenDirectory
        fbd.Description = "リストに登録するフォルダを選択してください。"
        fbd.ShowNewFolderButton = False

        If fbd.ShowDialog() = DialogResult.OK Then
           _OpenDirectory = fbd.SelectedPath

           Dim c As Integer = ListView.Items.Count
           Dim p As String = fbd.SelectedPath
           Dim l As New List(Of String)

           If DriveCheck(p) Then
              If IsNothing(Path.GetDirectoryName(p)) Then           'ルートチェック
                 If _List_TreeMode = 0 OrElse RootSelectDialog(p) Then
                    l.AddRange(GetRootEntries(p))
                 End If
              Else
                 l.Add(p)
              End If

              If l.Count <> 0 Then
                 _DropFileList = l.ToArray

              'ListView登録
                 Lsv_AddRange()

              'マーカークリア
                 If ListView.Items.Count <> c AndAlso
                    ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)
              End If
           End If
        End If

        fbd.Dispose()
    End Sub

#End Region

#Region "-- ToolTip"

    Private Function OnOffText(ByVal value As Integer) As String
        If value = 0 Then
           Return " OFF "
        Else
           Return " ON "
        End If
    End Function

    Private Sub GroupBox3_ToolTip_Change()

        If Me.ToolTip.Active Then
           If GroupBox3.Enabled Then
              ToolTip.SetToolTip(GroupBox3.Label, "右クリックで、変更日時ログが表示されます。")
              ToolTip.SetToolTip(Btn_DtClr, "日付を今の日付にします。")
              ToolTip.SetToolTip(Btn_TmClr, "時刻をクリアします。")
          Else
              ToolTip.SetToolTip(GroupBox3.Label, "")
              ToolTip.SetToolTip(Btn_DtClr, "")
              ToolTip.SetToolTip(Btn_TmClr, "")
           End If
        End If
    End Sub

    Private Sub Btn_Exec_ToolTip_Change()

        Const tip = "[Shift] + [Enter]" & vbCr & " リストを昇順ソート後に処理を実行します。"

        If Me.ToolTip.Active Then
           If Btn_Exec.Enabled Then
              ToolTip.SetToolTip(Btn_Exec, tip)
           Else
              ToolTip.SetToolTip(Btn_Exec, "")
           End If
        End If
    End Sub

    Private Sub Attr_ToolTip_Change()

        Const AttrText = vbCr & "  読み込むファイルの属性を設定します。"

        Dim fmt As String() = {"Read Only", "Hide", "System"}
        Dim tip As String

        For i As Integer = 0 To 2
          If _onToolTipActive Then
             tip = String.Format("{0} ({1})", fmt(i), OnOffText(_AttrType(i)))
          Else
             tip = String.Format("[ {0} ] -{1}-", fmt(i), OnOffText(_AttrType(i))) & AttrText
          End If

          ToolTip.SetToolTip(Me.Controls("Lbl_Attr_" & "RHS".Substring(i, 1)), tip)
        Next
    End Sub

    Private Sub ListMode_ToolTip_Change()

        Const openText = "を開きます。"

        Dim flg As Boolean
        Dim tip As String = _ToolTip_Text(0, _List_FileMode) & openText & vbCrLf & _
                            _ToolTip_Text(0, Convert.ToInt32(_List_FileMode = 0)) & openText

        flg = (Chk_Date.Checked AndAlso (_List_TreeMode = 0) AndAlso (_List_FileMode = 0))
        Lbl_DAdjustMode.Enabled = flg
        Txt_Date_Ajst.Enabled = flg

        flg = (Chk_Time.Checked AndAlso (_List_TreeMode = 0) AndAlso (_List_FileMode = 0))
        Lbl_TAdjustMode.Enabled = flg
        Txt_Time_Ajst.Enabled = flg
        Chk_Rand.Enabled = flg

        ToolTip.SetToolTip(Btn_Open, tip)
        ToolTip.SetToolTip(Btn_TreeMode, _ToolTip_Text(1, _List_TreeMode))
        ToolTip.SetToolTip(Btn_FileMode, _ToolTip_Text(2, _List_FileMode))
        ToolTip.SetToolTip(Btn_ChckMode, _ToolTip_Text(3, _List_ChckMode))
    End Sub

#End Region

#Region "-- Context Menu"

''' <summary>
''' ファイルの拡張子をチェックします。(実行可能, DLL, インストーラー)
''' </summary>

    Private Function CheckExtension(ByVal TargetPath As String) As Boolean
        Const Extension = _
          ".exe.com.obj.bat.cmd.vbs.vbe.js.jse.wsf.wsh.msc.jar.hta.scr.cpl.lnk.url" & _
          ".acm.ocx.ax.cpa.cpi.cpx.dump.ebd.etl.iec.ime.lex.lrc.man.mod.mof.mui.nls.nt" & _
          ".olb.pro.resources.resx.rom.rs.rsp.sdi.sep.tbl.tlb.tsk.tsp.uce.vp" & _
          ".dll.sys.drv.rat" & _
          ".inf.msi."

        If Not File.Exists(TargetPath) Then
           Return False     'ディレクトリ、または、ファイルが存在しない
        End If

        Return (Extension.IndexOf(Path.GetExtension(TargetPath) & ".".ToLower) = -1)
    End Function

''' <summary>
''' サブフォームが開かれているかをチェックし、適切なフォームをアクティブにします。
''' </summary>

    Private Function CheckOpenForms() As Boolean

        Dim ofm As FormCollection = Application.OpenForms
        Dim state As Boolean = ((ofm.Count = 0) OrElse
                               ((ofm.Count = 1) AndAlso (ofm(0).Name = Me.Name)))

        If state Then
           Me.Activate()
        Else
           ofm(ofm.Count - 1).Activate()
        End If

        state = (_onDialogForm OrElse Not state)
        If state Then Beep()

        Return state
    End Function

    Private Sub StripMenu_E_Control()
        For i = 0 To StripMenu_Emphasis.DropDownItems.Count - 1
          DirectCast(StripMenu_Emphasis.DropDownItems(i), ToolStripMenuItem).CheckState = _
          DirectCast(2 * Convert.ToInt32(i = _UseEmphasisColor), CheckState)
        Next
    End Sub

    Private Sub StripMenu_R_Control()
        Dim v As Single = _Adjust_MValue * 100

        For i = 1 To ContextMenu_Random.Items.Count - 2
          DirectCast(ContextMenu_Random.Items(i + 1), ToolStripMenuItem).CheckState = _
          DirectCast(2 * Convert.ToInt32(v = _Adjust_MpfValue(i)), CheckState)
        Next
    End Sub

    Private Sub StripMenu_R0_Control()
        StripMenu_R0_0.CheckState = DirectCast(2 * Convert.ToInt32(_Adjust_LCheck = False), CheckState)
        StripMenu_R0_1.CheckState = DirectCast(2 * Convert.ToInt32(_Adjust_LCheck = True), CheckState)
    End Sub

#End Region

#Region "-- ListView 関連"

''' <summary>
''' ListView の表示に関係する値を設定します。
''' </summary>

    Private Sub Lsv_SetInfo()

        Dim sz As Single = _Lsv_FontSize(_Lsv_FontSize_Id) / _FontSizeMpf
        Dim lsv As New ListView

        lsv.View = View.Details
        lsv.Font = New Font(_Lsv_FontName, sz, FontStyle.Regular)
        lsv.Scrollable = False
        lsv.SmallImageList = Me.ImageList2

        lsv.Columns.Add("")
        lsv.Columns.Add("")
        lsv.Items.Add(New ListViewItem({"", "8888/88/88 88:88:88"}))
        lsv.Items(0).ImageIndex = 0

        lsv.Columns(0).Width = -1
        lsv.Columns(1).Width = -1

'   高さ (ラベル, 行)、幅 (パス, 日付)
        _Lsv_LabelHeight = lsv.Items(0).Bounds.Y
        _Lsv_Row_Height = lsv.Items(0).Bounds.Height                '最小値＝アイコンの高さ

        _Lsv_Col0_Width = lsv.SmallImageList.ImageSize.Width + 5    'アイコン表示最小幅
        _Lsv_Col2_Width = lsv.Columns(1).Width
        _Lsv_ColA_Width = _Lsv_Col0_Width + (_Lsv_Col2_Width * 2)

'   枠のサイズ(3D境界線のサイズ:固定値)
        _Lsv_FrameSize_W = lsv.Width - lsv.ClientSize.Width
        _Lsv_FrameSize_H = lsv.Height - lsv.ClientSize.Height

'   未使用部分(枠,ラベル)のサイズ
        _Lsv_MinUnused_W = _Lsv_FrameSize_W
        _Lsv_MinUnused_H = _Lsv_FrameSize_H + _Lsv_LabelHeight

'   ScrollBarを含めた未使用部分のサイズ
'     ※スクロールバーの幅が変更された場合は、SystemColorsChanged で再設定
        _Lsv_MaxUnused_W = _Lsv_FrameSize_W + SystemInformation.VerticalScrollBarWidth
        _Lsv_MaxUnused_H = _Lsv_FrameSize_H + SystemInformation.HorizontalScrollBarHeight + _Lsv_LabelHeight

        lsv.Dispose()

     '※先頭行が表示されていない時に
     '  フォントサイズ, SmallImageListを変更すると表示が壊れる為の措置。 
        Dim Topidx As Integer

        If Not ListView.TopItem Is Nothing Then Topidx = ListView.TopItem.Index
        If ListView.Items.Count <> 0 Then ListView.TopItem = ListView.Items(0)
     '
        ListView.Font = New Font(_Lsv_FontName, sz, FontStyle.Regular)

     '※表示位置を元に戻す
        If Topidx <> 0 Then ListView.TopItem = ListView.Items(Topidx)
    End Sub

    Private Sub Lsv_Display_Control(ByVal idx As Integer)
        If ListView.Items.Count = idx Then
           Beep()
        Else
           _Lsv_DupCheckFlg = True

           With ListView
             .SelectedItems.Clear()
             .EnsureVisible(idx)
             .Items(idx).Focused = True
             .Items(idx).Selected = True
             .Select()

             Lbl_Items.Text = String.Format(_LblItemsFormat, .Items.Count)
           End With

           Btn_Erase.Enabled = True
           Btn_Exec_Control()

           Lsv_DisplayMode_Control(False)
        End If
    End Sub

    Private Sub Lsv_DisplayMode_Control(ByVal update As Boolean)

        If _Lsv_DisplayMode = 0 Then Return

        ModeLamp_Control()

        RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

        If update Then ListView.BeginUpdate()

        With ListView
          If .Columns(2).Width <> _Lsv_Col2_Width Then
             .Columns(2).Width = _Lsv_Col2_Width
             .Columns(3).Width = _Lsv_Col2_Width
          End If

          If _Lsv_DisplayMode = 1 Then

         '水平ScrollBarが表示されないように調整する
             _Lsv_DisplayRows = (.Height - _Lsv_MinUnused_H) \ _Lsv_Row_Height  'ListView内で表示できる行数(水平Scroll Barなし)

             If _Lsv_DisplayRows >= .Items.Count Then                           '垂直ScrollBarなし
                .Columns(1).Width = .Width - _Lsv_MinUnused_W - _Lsv_ColA_Width
             Else
                .Columns(1).Width = .Width - _Lsv_MaxUnused_W - _Lsv_ColA_Width
             End If
             If .Columns(0).Width <> _Lsv_Col0_Width Then
                .Columns(0).Text = ""
                .Columns(0).Width = _Lsv_Col0_Width
             End If
          Else

          '日付部分を非表示(画面外)にする⇒水平ScrollBar表示
             _Lsv_DisplayRows = (.Height - _Lsv_MaxUnused_H) \ _Lsv_Row_Height  'ListView内で表示できる行数(水平Scroll Barあり)

             If _Lsv_DisplayRows >= .Items.Count Then                           '垂直ScrollBarなし
                .Columns(0).Width = .Width - _Lsv_MinUnused_W
             Else
                .Columns(0).Width = .Width - _Lsv_MaxUnused_W
             End If

                .Columns(0).Text = "パス"
             If .Columns(1).Width <> 0 Then
                .Columns(1).Width = 0
             End If
          End If
        End With

        If update Then ListView.EndUpdate()

        AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
    End Sub

    Private Function Lsv_LimitCheck(ByVal update As Boolean, ByVal state As Boolean) As Boolean

        If ListView.Items.Count >= _List_RegistLimit Then

           If update Then
              Lsv_DisplayMode_Control(False)
              ListView.EndUpdate()
           End If

           Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

           Dim msg1 As String
           Dim msg2 As String

           If state Then
              msg1 = "リスト登録数が最大値に達しています。"
              msg2 = "これ以上リストに登録できません。"
           Else
              msg1 = "リスト登録数が最大値に達しました。"
              msg2 = "リスト登録を中止します。"
           End If
           ShowWarningMessage(msg1, msg2, _Msg_Cap)

           If update Then ListView.BeginUpdate()

           Return True
        End If

        Return False
    End Function

    Private Sub Lsv_Add(ByVal Fpath As String, ByVal id As Integer)

        If Lsv_LimitCheck(False, True) Then Return

        Dim Fattr As Integer

        Try
          Dim fa As Integer = File.GetAttributes(Fpath)

          Fattr = fa And 7  'RHS以外をﾏｽｸ
          If (Fattr And Not _List_AttrType) <> 0 Then Return

          Fattr = fa

        Catch ex As Exception
          Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
            sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
          End Using

          MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
          Return
        End Try

        Dim LsvItem As New ListViewItem

        With LsvItem
         .Text = Fpath
         .ImageIndex = id
         .SubItems.Add(Path.GetFileName(Fpath))
         .SubItems.Add(File.GetCreationTime(Fpath).ToString(_DateTimeFormat))
         .SubItems.Add(File.GetLastWriteTime(Fpath).ToString(_DateTimeFormat))

         .SubItems(1).ForeColor = GetAttributeColor(Fattr)

      'ジャンクション，シンボリックリンクを強調
         If (Fattr And FileAttributes.ReparsePoint) <> 0 Then
            .SubItems(0).ForeColor = GetAttributeColor(Fattr)
         End If
        End With
        ListView.Items.Add(LsvItem)
    End Sub

    Private Sub Lsv_AddRange()

        If Lsv_LimitCheck(False, True) Then Return

        Dim state As Boolean = (_DropFileList.Length <> 1)
        Dim break As Boolean
        Dim Fpath As String
        Dim Dpath As String

        Dim FocusedIndex As Integer = ListView.Items.Count
        Dim DirectorySearchOption As SearchOption = DirectCast(Convert.ToInt32(_List_TreeMode = 2), SearchOption)

        Me.Cursor = Cursors.WaitCursor

        ListView.BeginUpdate()

          If _Lsv_DupCheckFlg Then

             For Each Fpath In _DropFileList
               If File.Exists(Fpath) Then
                  If (_List_FileMode <> 1) AndAlso IsNothing(ListView.FindItemWithText(Fpath, False, 0, False)) Then
                     Lsv_Add(Fpath, 0)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If
               ElseIf Not Directory.Exists(Fpath) Then

                  If ShowFnfMessage(Fpath, state) <> DialogResult.Yes Then Exit For
               Else
                  If (_List_FileMode <> 0) AndAlso IsNothing(ListView.FindItemWithText(Fpath, False, 0, False)) Then
                     Lsv_Add(Fpath, 1)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If

                  If _List_TreeMode <> 0 Then
                     Try
                       Dim sFolder As String() = Directory.GetFileSystemEntries(Fpath, "*", DirectorySearchOption)
                       For Each Dpath In sFolder
                         If File.Exists(Dpath) Then
                            If (_List_FileMode <> 1) AndAlso IsNothing(ListView.FindItemWithText(Dpath, False, 0, False)) Then
                               Lsv_Add(Dpath, 0)
                               Lbl_Items_Update()
                            End If
                         Else
                            If (_List_FileMode <> 0) AndAlso IsNothing(ListView.FindItemWithText(Dpath, False, 0, False)) Then
                               Lsv_Add(Dpath, 1)
                               Lbl_Items_Update()
                            End If
                         End If

                      '登録制限値，[ESC] Key Check
                         If Lsv_LimitCheck(True, False) OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then
                            break = True
                            Exit For
                         End If
                       Next

                  '不正なジャンクション，シンボリックリンクは無視
                     Catch ex As DirectoryNotFoundException

                     Catch ex As Exception
                       Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
                         sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
                       End Using

                       MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
                     End Try
                  End If
               End If

            '登録制限値，[ESC] Key Check
               If break OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then Exit For

             Next
          Else

             For Each Fpath In _DropFileList
               If File.Exists(Fpath) Then
                  If _List_FileMode <> 1 Then
                     Lsv_Add(Fpath, 0)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If
               ElseIf Not Directory.Exists(Fpath) Then

                  If ShowFnfMessage(Fpath, state) <> DialogResult.Yes Then Exit For
               Else
                  If _List_FileMode <> 0 Then
                     Lsv_Add(Fpath, 1)           'Folder,All mode (Folder add)
                     Lbl_Items_Update()

                     If Lsv_LimitCheck(True, False) Then Exit For
                  End If

                  If _List_TreeMode <> 0 Then
                     Try
                       Dim sFolder As String() = Directory.GetFileSystemEntries(Fpath, "*", DirectorySearchOption)
                       For Each Dpath In sFolder
                         If File.Exists(Dpath) Then
                            If _List_FileMode <> 1 Then
                               Lsv_Add(Dpath, 0)
                               Lbl_Items_Update()
                            End If
                         Else
                            If _List_FileMode <> 0 Then
                               Lsv_Add(Dpath, 1)
                               Lbl_Items_Update()
                            End If
                         End If

                      '登録制限値，[ESC] Key Check
                         If Lsv_LimitCheck(True, False) OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then
                            break = True
                            Exit For
                         End If
                       Next

                  '不正なジャンクション，シンボリックリンクは無視
                     Catch ex As DirectoryNotFoundException

                     Catch ex As Exception
                       Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
                         sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
                       End Using

                       MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
                     End Try
                  End If
               End If

            '登録制限値，[ESC] Key Check
               If break OrElse WinMessage.KeyCheck(Me, Keys.Escape) Then Exit For

             Next
          End If
'
          Lsv_Display_Control(FocusedIndex)

        ListView.EndUpdate()
'
        Me.Cursor = Cursors.Default
    End Sub

    Private Sub Lsv_Set(ByVal state As Integer, ByRef Lsv As ListViewItem, ByVal itm() As String)

        Dim i1 As Integer = 2 + (2 * Convert.ToInt32(state <> 0))
        Dim i2 As Integer = i1 + 1

        If IsNothing(Lsv) Then
           Lsv = New ListViewItem
           With Lsv
             .Text = itm(1)
             .ImageIndex = Convert.ToInt32(itm(0))
             .SubItems.Add(Path.GetFileName(itm(1)))
             .SubItems.Add(itm(i1))
             .SubItems.Add(itm(i2))
           End With
           ListView.Items.Add(Lsv)
        Else
           Lsv.SubItems(2).Text = itm(i1)
           Lsv.SubItems(3).Text = itm(i2)
        End If
    End Sub

    Private Sub Lsv_Refresh()

        ListView.BeginUpdate()
        ListView.SelectedItems.Clear()

          For Each LsvItem As ListViewItem In ListView.Items

            Dim Fpath As String = LsvItem.SubItems(0).Text

            If Not File.Exists(Fpath) AndAlso Not Directory.Exists(Fpath) Then
               LsvItem.Remove()
            Else
               Try
                 Dim fa As Integer = File.GetAttributes(Fpath)

                 For i As Integer = 0 To ListView.Columns.Count - 1
                   LsvItem.SubItems(i).BackColor = SystemColors.Window
                 Next

                 LsvItem.SubItems(1).ForeColor = GetAttributeColor(fa)
                 LsvItem.SubItems(2).Text = File.GetCreationTime(Fpath).ToString(_DateTimeFormat)
                 LsvItem.SubItems(3).Text = File.GetLastWriteTime(Fpath).ToString(_DateTimeFormat)

               Catch ex As Exception
                 Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
                   sw.WriteLine(Now & vbTab & Reflection.MethodBase.GetCurrentMethod.Name & vbTab & ex.Message)
                 End Using

                 MsgBox(Reflection.MethodBase.GetCurrentMethod.Name & vbCrLf & vbCrLf & ex.Message)
               End Try
            End If
          Next

        Lsv_DisplayMode_Control(False)

        Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        If ListView.Items.Count <> 0 Then
           ListView.Items(0).Focused = True
           ListView.Items(0).Selected = True
        Else
           _Lsv_DupCheckFlg = False

           Btn_Erase.Enabled = False
        End If

        Btn_Exec_Control()

        ListView.EndUpdate()
    End Sub

    Private Sub Lsv_Delete(ByVal state As Boolean)

        Dim n As Integer = ListView.Items.Count
        Dim idx As Integer
        Dim LsvItem As ListViewItem

        With ListView
          Try
            idx = .FocusedItem.Index                        '最後に選択したフォーカス(カーソル)の位置
          Catch ex As Exception
            idx = -1
          End Try

          .BeginUpdate()

          If state Then
             For Each LsvItem In .Items                     '処理済で成功したアイテムを削除
               Select Case LsvItem.SubItems(1).BackColor
                 Case _BackColorSuccess, _BackColorUndo
                   LsvItem.Remove()
               End Select
             Next

             If n = .Items.Count Then                       '成功したアイテムがない
                .EndUpdate()

                Beep()
                Return
             End If
          Else
             For Each LsvItem In .SelectedItems
               LsvItem.Remove()
             Next
          End If

          If .Items.Count <> 0 Then
             If idx <> -1 Then
                .Items(.FocusedItem.Index).Selected = True  'フォーカス(カーソル)位置を設定
             End If
          Else
             _Lsv_DupCheckFlg = False
          End If

          Lsv_DisplayMode_Control(False)

          Lbl_Items.Text = String.Format(_LblItemsFormat, .Items.Count)

          Btn_Erase.Enabled = (.Items.Count <> 0)
          Btn_Exec_Control()

          .EndUpdate()
        End With
    End Sub

    Private Sub Lsv_Clear()

        ListView.Items.Clear()

        Lsv_DisplayMode_Control(True)

        Lbl_Items.Text = String.Format(_LblItemsFormat, 0)

        Btn_Erase.Enabled = False
        Btn_Exec_Control()

        _Lsv_DupCheckFlg = False
    End Sub

    Private Sub Lsv_BackColor_Clear()
        For Each Lsv As ListViewItem In ListView.Items
          For i As Integer = 0 To Lsv.SubItems.Count - 1
            Lsv.SubItems(i).BackColor = Nothing
          Next
        Next
    End Sub

    Private Sub Lsv_BackColor_Set(ByRef Lsv As ListViewItem, ByVal Bcolor As Color)

        If _UseEmphasisColor = 0 Then Return

        Lsv.SubItems(0).BackColor = Bcolor
        Lsv.SubItems(1).BackColor = Bcolor

        If Chk_Make.Checked Then Lsv.SubItems(2).BackColor = Bcolor
        If Chk_Repl.Checked Then Lsv.SubItems(3).BackColor = Bcolor
    End Sub

    Private Sub Lsv_DragColor_Control()
        If _Lsv_DragState Then
           _Lsv_DragState = False

           Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
        End If
    End Sub

    Private Sub Lsv_Dragcolor_Change(ByVal index As Integer, ByVal Bcolor As Color)

        If _UseEmphasisColor = 0 Then Return
        If Bcolor.IsEmpty Then Bcolor = SystemColors.Window

        Dim lsv As ListViewItem = ListView.Items(index)

        If lsv.SubItems(0).BackColor = Bcolor Then Return

        For i As Integer = 0 To lsv.SubItems.Count - 1
          lsv.SubItems(i).BackColor = Bcolor
        Next
    End Sub

    Private Sub ShowOnlyResults()
        Dim n As Integer

        ListView.BeginUpdate()
          For Each Lsv As ListViewItem In ListView.Items
            If Lsv.SubItems(1).BackColor = SystemColors.Window Then
               Lsv.Remove()
               n += 1
            End If
          Next

          If n = 0 Then
             Beep()
          Else
             Lsv_DisplayMode_Control(False)
          End If
        ListView.EndUpdate()

        Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        Dim f As Boolean = (ListView.Items.Count <> 0)

        Btn_Erase.Enabled = f
        Btn_Exec_Control()

        _Lsv_DupCheckFlg = f
    End Sub

#End Region

#End Region

#Region "// ---- Form & Timer -------------------------------------------------"

    Private Sub DelaySettingTimer_Tick(sender As Object, e As EventArgs) Handles DelaySettingTimer.Tick

        DelaySettingTimer.Stop()

        Lsv_DisplayMode_Control(True)

        If _onDelaySetting Then
           _onDelaySetting = False

           For n As Double = 0.1R To 1.0R Step 0.2R
             Me.Opacity = n
             Thread.Sleep(10)
           Next
           Me.Opacity = 1.0R
        End If
    End Sub

    Private Sub Me_Deactivate(sender As Object, e As EventArgs)

'     フォームが アクティブ でなくなった（フォーカスを失った）場合、日時変更ログを更新。

        If _DateTimeValueChangeFlag Then
           DateTimeChangeLogUpdate()
        End If
    End Sub

    Private Sub Me_Load(sender As Object, e As EventArgs) Handles Me.Load

      Me.SuspendLayout()

      Me.Text = _App_Name
      Notify_Icon.Text = _App_Name

'   OS が ８以降かを調べる
        _OS_Type = GetOsType()

'   開始ウィンドウサイズをセット
        _StartWindow_Width = Me.Width
        _StartWindowHeight = Me.Height

'   開始 GroupBox3, Cmb_DateTimeLog の幅をセット
        _GroupBox3Width = GroupBox3.Width
        _DateTimeLogWidth = Cmb_DateTimeLog.Width

'   基準となるスクロール バーのサイズをセット
        _BeforeScrollBarWidth = SystemInformation.VerticalScrollBarWidth
'-------------------------------------------------------------------------------
'   設定読み込み & フォームの表示位置変更
        iniLoad()
'-------------------------------------------------------------------------------
'   フォントチェック(変更)
        FontSetting()
'-------------------------------------------------------------------------------
'   Style, GroupBox3 Setting
        StyleSetting()
        GroupBox3Setting()
'-------------------------------------------------------------------------------
'   ContextMenu
        _CreationTimeText = StripMenu_CreationTime.Text
        _DateModifiedText = StripMenu_DateModified.Text
'-------------------------------------------------------------------------------
'   UpDownボタン変更
        DateTimePickerButtonControl()
'-------------------------------------------------------------------------------
'   画面制御値設定
        _MinClientSize = Me.ClientSize

        Me.MinimumSize = Me.Size

'   フォームのサイズ変更
        If _StartWindow_Width < Me.MinimumSize.Width Then
           Me.Width = Me.MinimumSize.Width
        Else
           Me.Width = _StartWindow_Width
        End If

        If _StartWindowHeight < Me.MinimumSize.Height Then
           Me.Height = Me.MinimumSize.Height
        Else
           Me.Height = _StartWindowHeight
        End If
'-------------------------------------------------------------------------------
'   ListView 表示制御値を設定
        Lsv_SetInfo()

'   Label 表示補正を停止
        Lbl_Items.LockedCorrection = True
        Lbl_Items.Text = String.Format(_LblItemsFormat, 0)

'   ToolTip
        Me.ToolTip.SetToolTip(GroupBox1.Label, "右クリックで" & vbCr & "動作確認 (設定) メニューが開きます。")
'-------------------------------------------------------------------------------
      Me.ResumeLayout()
'-------------------------------------------------------------------------------
'   コマンドライン、送る チェック
'    (リストが登録された場合は、ListView.ColumnWidthChanged イベントが登録される)
        CmdSendToLoad()
'-------------------------------------------------------------------------------
      Me.SuspendLayout()
'-------------------------------------------------------------------------------
'   ダブルバッファ on
        If _DoubleBuffering Then ListView.DoubleBuffering = True
'-------------------------------------------------------------------------------
'   ボタン関連
        Btn_TreeMode.ImageIndex = _List_TreeMode + 4
        Btn_FileMode.ImageIndex = _List_FileMode + 1
        Btn_ChckMode.ImageIndex = _List_ChckMode + 9

        HoverColorSet()

        ListMode_ToolTip_Change()
        ToolTip.SetToolTip(Btn_UTC9, "UTC " & _M_Adjust_Time & " に修正します。")

'   ファイル属性表示
        Lbl_Attr_R.ForeColor = Color.FromName(_AttrColorName(_AttrType(0)))
        Lbl_Attr_H.ForeColor = Color.FromName(_AttrColorName(_AttrType(1)))
        Lbl_Attr_S.ForeColor = Color.FromName(_AttrColorName(_AttrType(2)))

        Attr_ToolTip_Change()

'   強調背景色関連
        SetEmphasisBackColor()
        StripMenu_E_Control()

'   フォントサイズ (ContextMenu_Setting)

        Const DisplayFormat = "     0.00 pt"

        Dim t As String
        Dim l As Integer = DisplayFormat.Length

        For i As Integer = 0 To _Lsv_FontSize.Count - 1
          If _Lsv_FontSize(i) >= 10.0F Then
             l = DisplayFormat.Length - 1
          End If

          t = _Lsv_FontSize(i).ToString(DisplayFormat)
          t = t.Substring(t.Length - l)

          StripMenu_Cmb_FontSize.Items.Add(t)
        Next
        StripMenu_Cmb_FontSize.SelectedIndex = _Lsv_FontSize_Id

'   Adjust 関連
        Lbl_DAdjustMode.Text = _Adjust_Name(0, _M_Adjust_D_Mode)
        Lbl_TAdjustMode.Text = _Adjust_Name(1, _M_Adjust_T_Mode)
        Chk_Rand.Checked = _M_Adjust_RCheck

     '乗率値表示設定
        For i = 1 To ContextMenu_Random.Items.Count - 2
          ContextMenu_Random.Items("StripMenu_R" & i).Text = String.Format("{0} ％", _Adjust_MpfValue(i))
        Next

        StripMenu_R_Control()
        StripMenu_R0_Control()

'   日時セット
        Pck_Date.Value = Now.Date
        Pck_Time.Value = Now.Date

'   日付変更ログ初期化
        _DateTimeChangeLog.Columns.Add("value", GetType(Date))
        _DateTimeChangeLog.Columns.Add("text", GetType(String))

        Cmb_DateTimeLog.DataSource = _DateTimeChangeLog
        Cmb_DateTimeLog.ValueMember = "value"
        Cmb_DateTimeLog.DisplayMember = "text"

        Dim dt As Date = Now.Date
        Dim dr As DataRow = _DateTimeChangeLog.NewRow

        dr("value") = dt
        dr("text") = dt.ToString(_CmbDtLogFormat)

        _DateTimeChangeLog.Rows.Add(dr)
        _DateTimeChangeLog.AcceptChanges()

'   画面制御・設定
        If ListView.Items.Count <> 0 Then
           _Lsv_DupCheckFlg = True

           ListView.Items(0).Selected = True
           ListView.Select()
           Btn_Erase.Enabled = True
           Btn_Exec_Control()
        End If

     'ListView.ColumnWidthChanged イベント登録
        Lsv_DisplayMode_Control(True)

      Me.ResumeLayout()

        AddHandler Me.Resize, AddressOf Me_Resize

        AddHandler Pck_Date.ValueChanged, AddressOf Pck_ValueChanged
        AddHandler Pck_Time.ValueChanged, AddressOf Pck_ValueChanged

        AddHandler StripMenu_ToolTip.CheckedChanged, AddressOf StripMenu_ToolTip_CheckedChanged
        AddHandler StripMenu_TaskBar.CheckedChanged, AddressOf StripMenu_TaskBar_CheckedChanged
    End Sub

    Private Sub Me_Shown(sender As Object, e As EventArgs) Handles Me.Shown

        Const Msg_Log = "ログファイルが既定サイズを超えています。" & vbCrLf & vbCrLf &
                        "ログファイルを初期化します。"

        Me.Update()
        For n As Single = 0.1 To 1 Step 0.2
          Me.Opacity = n
          Thread.Sleep(10)
        Next
        Me.Opacity = 1.0R

     'Log File Clear
        Dim fi As New FileInfo(_Log_File)

        If fi.Exists Then
           If fi.Length > _LogFileMaxSize Then

              Dim MsgBox As New MessageBoxEx
              MsgBox.SetButtonText(1, "初期化")
              MsgBox.SetButtonText(2, "整 理")

              Dim f As Integer = MsgBox.Dialog(Me, Msg_Log, _Msg_Cap,
                                               MessageBoxEx.Buttons.CenterYesNoCancel,
                                               MessageBoxIcon.Warning, , , _MessageBoxSound)
              MsgBox.Dispose()

              If f = DialogResult.Yes Then
                 Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
                 End Using

              ElseIf f = DialogResult.No Then

                 Dim cnt As Integer
                 Dim buf As String

                 If File.Exists(_TempFile) Then File.Delete(_TempFile)
                 File.Move(_Log_File, _TempFile)

                 Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
                 Using sr As New StreamReader(_TempFile, Encoding.UTF8)

                   Do
                     buf = sr.ReadLine()
                     If String.IsNullOrEmpty(buf) Then cnt += 1
                     sw.WriteLine(buf)
                   Loop Until (cnt = _LogFileLeaveCount) OrElse (sr.Peek = -1)

                 End Using
                 End Using

                 File.Delete(_TempFile)
              End If

              Me.Update()
              WinMessage.Clear(Me)
           End If
        End If
    End Sub

    Private Sub Me_FormClosed(sender As Object, e As FormClosedEventArgs) Handles Me.FormClosed

        iniSave()

'     Opacity 変更時、StyleChanged が発生する為 ハンドルを削除
'       RemoveHandler Me.StyleChanged, AddressOf Form_StyleChanged

        For n As Single = 1 To 0 Step -0.2
          Me.Opacity = n
          Thread.Sleep(10)
        Next
        Me.Opacity = 0.0R
    End Sub

    Private Sub Me_SystemColorsChanged(sender As Object, e As EventArgs) Handles Me.SystemColorsChanged

        Dim State As Boolean = Application.RenderWithVisualStyles
        Dim ScrollBarWidth As Integer = SystemInformation.VerticalScrollBarWidth

     'Style Change
       If _onStyleChange <> Application.RenderWithVisualStyles Then

           StyleSetting()
           Lsv_SetInfo()
        Else

        'ListView の ScrollBarを含めた未使用部分のサイズを再設定
           _Lsv_MaxUnused_W = _Lsv_FrameSize_W + ScrollBarWidth
           _Lsv_MaxUnused_H = _Lsv_FrameSize_H + ScrollBarWidth + _Lsv_LabelHeight
        End If

     'GroupBox3, UpDownボタン変更
        If _BeforeScrollBarWidth <> ScrollBarWidth Then

           GroupBox3Setting()
           DateTimePickerButtonControl()
        End If

     'スクロールバー（垂直）が表示されている場合に、スクロールバーの幅を小さく変更すると
     'スクロールバー（水平）が残ったままになる不具合に対する措置

        If _Lsv_DisplayMode <> 0 Then
           If (_BeforeScrollBarWidth > ScrollBarWidth) AndAlso _
              (ListView.ClientSize.Width < ListView.Width - _Lsv_FrameSize_W) Then

              DelaySettingTimer.Interval = 500
              DelaySettingTimer.Start()
           Else
              Lsv_DisplayMode_Control(True)
           End If
        End If

     '基準となるスクロール バーのサイズをセット
        _BeforeScrollBarWidth = ScrollBarWidth

     '強調背景色セット
        SetEmphasisBackColor()
    End Sub

    Private Sub Me_ResizeBegin(sender As Object, e As EventArgs) Handles Me.ResizeBegin

     'リサイズ中はリスト表示は更新せず、リサイズ確定後にリスト表示を更新する為、Resize イベントを停止
        RemoveHandler Me.Resize, AddressOf Me_Resize
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs) 'Handles Me.Resize

'     最小化
        If Me.WindowState = FormWindowState.Minimized Then
           _WindowMinimized = True

           Me.Opacity = 0.0R
           Return
        End If

'     最小化⇒元に戻された
        If _WindowMinimized Then
           _WindowMinimized = False

           _onDelaySetting = True

           DelaySettingTimer.Interval = 100
           DelaySettingTimer.Start()
           Return
        End If

'     最大化
        If Me.WindowState = FormWindowState.Maximized Then
           _WindowMaximized = True

           Lsv_DisplayMode_Control(True)
           Return
        End If

'     最大化⇒元に戻された
        If _WindowMaximized Then
           _WindowMaximized = False

           Lsv_DisplayMode_Control(True)
           Return
        End If
'---------------------------------------------------------------------------------------------
'     画面のプロパティ(アクティブタイトルバー、ウィンドウの境界・間隔)が変更された時の処理
'     （最小ウィンドウサイズの変更のみ実施）
'---------------------------------------------------------------------------------------------
        Dim fw As Integer = SystemInformation.FrameBorderSize.Width * 2
        Dim fh As Integer = SystemInformation.CaptionHeight + fw
        Dim mw As Integer = _MinClientSize.Width + fw
        Dim mh As Integer = _MinClientSize.Height + fh

        If Me.Size = Me.MinimumSize Then

           If mw < Me.MinimumSize.Width OrElse mh < Me.MinimumSize.Height Then

              RemoveHandler Me.Resize, AddressOf Me_Resize

                Me.MinimumSize = New Size(mw, mh)
                Me.Size = Me.MinimumSize

              AddHandler Me.Resize, AddressOf Me_Resize
           End If
        Else
           If mw <> Me.MinimumSize.Width OrElse mh <> Me.MinimumSize.Height Then

              Me.MinimumSize = New Size(mw, mh)
           End If
        End If

     'リスト表示を更新
        Lsv_DisplayMode_Control(True)
    End Sub

    Private Sub Me_ResizeEnd(sender As Object, e As EventArgs) Handles Me.ResizeEnd

     'リサイズ確定後にリスト表示を更新
        Lsv_DisplayMode_Control(True)

     'Resize イベントを再開
        AddHandler Me.Resize, AddressOf Me_Resize
    End Sub

    Private Sub Me_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown

        If e.KeyCode = Keys.F1 Then
           e.Handled = True

           ShowSettings(e.Alt)
           Return
        End If

        If e.Control Then
           If e.KeyCode = Keys.H Then   '[ctrl]+H
              e.Handled = True
              e.SuppressKeyPress = True 'KeyPressイベントをキャンセル

              _UseHideSelection = _UseHideSelection Xor True
              ListView.HideSelection = _UseHideSelection
           End If
           Return
        End If

        If (e.KeyData = Keys.Enter) OrElse (e.KeyData = 65549) Then     '65549 (Keys.Shift + Keys.Enter)

           e.Handled = True
           e.SuppressKeyPress = True       'KeyPressイベントをキャンセル

           If Btn_Exec.Enabled Then

              _onShiftEnter = e.Shift

           'チェックボックス以外のコントロールを対象 (ボタンは既定で除外)

              Select Case Me.ActiveControl.GetType.Name
                Case "ListViewEx", "NumericTextBox"
                  Btn_Exec.Select()

               '日時変更ログ更新
                  DateTimeChangeLogUpdate()

                  Btn_Exec.PerformClick()
'------------------------------------------------------------------------------------------------------
'   Enter が押された時のFocusが 'DateTimePicker' の時、入力時の値が確定されない為、
'   Focusを移して値を確定させる。(※重要：AcceptButton の場合でも同じ)
'------------------------------------------------------------------------------------------------------
                Case "DateTimePickerEx"

'   Focusを移して値を確定。
'     フォーカスを移動すると値を変更していた場合、Validated 後に値が確定され
'     ValueChanged が発生 (_DateTimeValueChangeFlag が True になる)
'
'     値が確定されてない場合に日時変更ログが更新されるのを回避する為、ハンドルを一時解除

                  Dim ctl As Control = Me.ActiveControl
                  RemoveHandler ctl.Validated, AddressOf Pck_Validated

                  Btn_Exec.Select()

                  AddHandler ctl.Validated, AddressOf Pck_Validated

               '入力した値が確定されてない場合にコントロールの更新が遅延する為、更新
                  ctl.Update()

               '日時変更ログ更新
                  DateTimeChangeLogUpdate()

                  Btn_Exec.PerformClick()
              End Select
           Else
              Beep()
           End If
        End If
    End Sub

    Private Sub Me_DragEnter(sender As Object, e As DragEventArgs) Handles Me.DragEnter
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
           e.Effect = DragDropEffects.Copy
        Else
           e.Effect = DragDropEffects.None
        End If
    End Sub

    Private Sub Me_DragDrop(sender As Object, e As DragEventArgs) Handles Me.DragDrop

        Me.Activate()
        Me.Update()

        Dim l As New List(Of String)

        _DropFileList = DirectCast(e.Data.GetData(DataFormats.FileDrop, False), String())

'     Dropテーブル ルートチェック、ルートエントリー取得
        For Each p As String In _DropFileList

          If DriveCheck(p) Then
             If IsNothing(Path.GetDirectoryName(p)) Then            'ルートチェック

                If _List_TreeMode = 0 OrElse RootSelectDialog(p) Then
                   l.AddRange(GetRootEntries(p))
                End If
             Else
                l.Add(p)
             End If
          End If
        Next

        If l.Count <> 0 Then
'        Dropテーブル再構成
           If _DropFileList.Length <> l.Count Then
              _DropFileList = l.ToArray
           End If

        'ListView登録
           Lsv_AddRange()

        'マーカークリア
           If ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)
        End If
    End Sub

#End Region

#Region "// ListView"

    <DebuggerStepThrough()>
    Private Sub ListView_ColumnWidthChanged(sender As Object, e As ColumnWidthChangedEventArgs)

         If _Lsv_DisplayMode <> 0 Then
           _Lsv_DisplayMode = 0 : ModeLamp_Control()
         End If

         If e.ColumnIndex = 0 Then
            RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

              If ListView.Columns(0).Width <= _Lsv_Col0_Width Then
                 ListView.Columns(0).Width = _Lsv_Col0_Width
                 ListView.Columns(0).Text = ""
              Else
                 ListView.Columns(0).Text = "パス"
              End If

            AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
         End If
    End Sub

    Private Sub ListView_MouseDown(sender As Object, e As MouseEventArgs) Handles ListView.MouseDown
        With ListView
          If .Items.Count <> 0 Then
             If e.Button = MouseButtons.Middle Then
                If .SelectedItems.Count <> 0 Then
                   If .SelectedItems.Count = .Items.Count Then
                      Lsv_Clear()
                   Else
                      Lsv_Delete(False)
                   End If
                End If
             End If
         End If
        End With
    End Sub

    Private Sub ListView_MouseUp(sender As Object, e As MouseEventArgs) Handles ListView.MouseUp

     'ContextMenu_ListView_Opening で表示内容を変更すると、表示座標がずれる為の措置
        If e.Button = MouseButtons.Right Then

           Dim c0 As Boolean = (ListView.Items.Count > 0)
           Dim s0, s1, fc, bc, ec As Boolean

           If c0 Then
              Dim sc As Integer = ListView.SelectedItems.Count

              s0 = (sc > 0)

              If s0 Then
                 s1 = (sc = 1)

                 Dim fl As String = ListView.SelectedItems.Item(0).Text

                 fc = (sc > 1) OrElse (s1 AndAlso PathExists(fl))
                 ec = (s1 AndAlso File.Exists(fl)) AndAlso CheckExtension(fl)
                 bc = (s0 AndAlso PathExists(fl)) AndAlso (Btn_Exec.Enabled)
              End If
           End If

           If s1 Then
              StripMenu_CreationTime.Text = _CreationTimeText
              StripMenu_DateModified.Text = _DateModifiedText
           Else
              StripMenu_CreationTime.Text = ""
              StripMenu_DateModified.Text = ""
           End If

           StripMenu_SelectConversion.Visible = bc
           StripMenu_CreationTime.Visible = s1
           StripMenu_DateModified.Visible = s1
           StripMenu_Open.Visible = ec
           StripMenu_SelectAllItem.Visible = c0
           StripMenu_ToggleSelection.Visible = c0
           StripMenu_DeleteItem.Visible = s0
           StripMenu_AttributeChange.Visible = fc

           StripSeparator_L1.Visible = bc
           StripSeparator_L2.Visible = s1
           StripSeparator_L3.Visible = ec
           StripSeparator_L4.Visible = c0
           StripSeparator_L5.Visible = fc
        End If
    End Sub

    Private Sub ListView_KeyDown(sender As Object, e As KeyEventArgs) Handles ListView.KeyDown

        With ListView
          If .Items.Count = 0 OrElse
             e.KeyCode = Keys.Menu Then         'Altキーを無効
             e.Handled = True
             e.SuppressKeyPress = True
             Return
          End If

          If e.KeyCode = Keys.ControlKey Then Return '無駄な処理を回避

          Select Case e.KeyData
            Case Keys.Delete
              If .SelectedItems.Count <> 0 Then
                 If .SelectedItems.Count = .Items.Count Then
                    Lsv_Clear()
                 Else
                    Lsv_Delete(False)
                 End If
              End If

            Case DirectCast(131137, Keys)       '[ctrl]+A
             .BeginUpdate()

              For Each LsvItem As ListViewItem In .Items
                LsvItem.Selected = True
              Next

             .EndUpdate()

            Case DirectCast(131110, Keys), DirectCast(131112, Keys) '[ctrl]+↑,↓
              If Not _UseCtrlUpDown Then
                 e.Handled = True
                 Return
              End If

            Case Keys.F2                        '[F2]
              FileNameChange()

            Case Keys.F3                        '[F3]
              FileAttributesChange()

            Case Keys.F5                        '[F5]
              Lsv_Refresh()

            Case Keys.F7                        '[F7]

              Dim sb As New StringBuilder
              For Each LsvItem As ListViewItem In ListView.Items
                sb.Append(LsvItem.SubItems(0).Text & vbCrLf)
              Next

              SetClipboardText(sb.ToString, "ファイルパスのリスト")

              sb.Clear()

            Case Keys.F8                        '[F8]

              Dim sb As New StringBuilder
              For Each LsvItem As ListViewItem In ListView.Items
                sb.Append(LsvItem.SubItems(1).Text & vbCrLf)
              Next

              SetClipboardText(sb.ToString, "ファイル名のリスト")

              sb.Clear()

            Case DirectCast(131191, Keys)       '[Ctrl]+[F8]

              Dim sb As New StringBuilder
              For Each LsvItem As ListViewItem In ListView.Items
                sb.Append(Path.GetFileNameWithoutExtension(LsvItem.SubItems(0).Text) & vbCrLf)
              Next

              SetClipboardText(sb.ToString, "ファイル名のリスト")

              sb.Clear()

            Case DirectCast(65655, Keys)        '[Shift]+[F8]

              Const NotExistText = "** Not Found **"

              Dim sb As New StringBuilder
              Dim SizeText As String
              Dim Dir_Text As String

              For Each LsvItem As ListViewItem In ListView.Items
                With LsvItem
                  If .ImageIndex = 0 Then
                     Dim fi As New FileInfo(.SubItems(0).Text)

                     If fi.Exists Then

                        SizeText = String.Format("{0,15:#,0}", fi.Length)

                        If (fi.Attributes And FileAttributes.ReparsePoint) = 0 Then
                           Dir_Text = StrDup(2, vbTab)
                        Else
                           Dir_Text = "   <SYMLINK>" & vbTab
                        End If
                     Else
                        SizeText = NotExistText
                        Dir_Text = StrDup(2, vbTab)
                     End If
                  Else
                     Dim di As New DirectoryInfo(.SubItems(0).Text)

                     If di.Exists Then

                        SizeText = String.Format("{0,15:#,0}", GetDirectorySize(di))

                        If (di.Attributes And FileAttributes.ReparsePoint) = 0 Then
                           Dir_Text = "       <DIR>" & vbTab
                        Else
                           Dir_Text = "  <SYMLINKD>" & vbTab
                        End If
                     Else
                        SizeText = NotExistText
                        Dir_Text = StrDup(2, vbTab)
                     End If
                  End If

                  sb.Append(.SubItems(3).Text & Dir_Text & SizeText & vbTab & .SubItems(1).Text & vbCrLf)
                End With
              Next

              SetClipboardText(sb.ToString, "ファイル名のリスト（詳細）")

              sb.Clear()

         '[F9], [Ctrl]+[F9], [Altl]+[F9], [Ctrl]+[Altl]+[F9]
            Case Keys.F9,
                 DirectCast(131192, Keys),
                 DirectCast(262264, Keys),
                 DirectCast(393336, Keys)

              If Btn_Exec.Enabled Then
                 If _JobMode = 0 Then
                    _JobMode = 3

                      If Not Chk_Date.Checked Then
                         Chk_Date.Checked = True
                         Chk_Date_Control()
                      End If

                      If Not Chk_Time.Checked Then
                         Chk_Time.Checked = True
                         Chk_Time_Control()
                      End If

                      _onAltKeyDown = e.Alt     'yyMMdd(hhmmss) 固定モード
                      _onSelectItem = e.Control 'Select Item モード

                      Btn_Exec.Select()         'ちらつきを減らす為、フォーカスを移動
                      Btn_Exec.PerformClick()

                    _JobMode = 0

                    _onAltKeyDown = False
                    _onSelectItem = False
                 Else
                    Beep()
                 End If
              End If

            Case DirectCast(131154, Keys)       '[Ctrl]+R
              ShowOnlyResults()

            Case DirectCast(131156, Keys)       '[Ctrl]+T
              GetDateTimeFormatValue()

            Case DirectCast(131149, Keys)       '[Ctrl]+M
              _ClipboardMonitor = _ClipboardMonitor Xor True
          End Select
        End With
    End Sub

    Private Sub ListView_DoubleClick(sender As Object, e As EventArgs) Handles ListView.DoubleClick
        FileNameChange()
    End Sub

    Private Sub ListView_ItemDrag(sender As Object, e As ItemDragEventArgs) Handles ListView.ItemDrag

        Lsv_BackColor_Clear()

        Dim DropList As New List(Of String)

        For Each item As ListViewItem In ListView.SelectedItems

          If File.Exists(item.Text) OrElse Directory.Exists(item.Text) Then
             DropList.Add(item.Text)
          Else
          '選択されたファイルがない場合、選択状態を解除
             item.Selected = False

             Lsv_BackColor_Set(item, _BackColorFailure)
          End If
        Next

        If DropList.Count = 0 Then
           ShowWarningMessage("Drop 可能なアイテムがありません。", "", _Msg_Cap)
           Return
        End If

     'リスト表示行数セット
        _Lsv_DisplayRows = (ListView.ClientRectangle.Height - _Lsv_LabelHeight) \ _Lsv_Row_Height
        _Lsv_InnerDrop = False

        Me.AllowDrop = False
        ListView.AllowDrop = True

        Dim DataObj As New DataObject(DataFormats.FileDrop, DropList.ToArray)
        Dim Effects As DragDropEffects

        Effects = ListView.DoDragDrop(DataObj, DragDropEffects.Copy Or DragDropEffects.Move)

    'Effects が正しい値を返さない
    ' None:      Drag Cancel, File Move (Exploler)
    ' Copy:      File Copy (Exploler)
    ' Move:      Move (Vix, XnView), Listview Item Move
    ' Copy+Move: File Copy (Exploler++), File Move (Exploler++)

        If Not (_Lsv_DragCancel Or _Lsv_InnerDrop) Then
           If (Effects = DragDropEffects.None) OrElse
              ((Effects And DragDropEffects.Move) = DragDropEffects.Move) Then

          'ファイル操作の遅延を考慮し、一時処理停止
             Thread.Sleep(_DropDelay)

             With ListView
              .BeginUpdate()

              For Each item As ListViewItem In .SelectedItems
                If Not File.Exists(item.Text) Then
                   item.Remove()
                End If
              Next
              If .Items.Count = 0 Then _Lsv_DupCheckFlg = False

              Lsv_DisplayMode_Control(False)
              Lbl_Items.Text = String.Format(_LblItemsFormat, .Items.Count)
              Btn_Exec_Control()

              .EndUpdate()
             End With
           End If
        End If

If _Lsv_DragState Then
   MsgBox("Error (DragState: True)")

   Using sw As New StreamWriter(_Err_File, True, Encoding.UTF8)
     sw.WriteLine(Now & vbTab & "ItemDrag (_Lsv_DragState: True)")
   End Using
End If

        ListView.AllowDrop = False
        Me.AllowDrop = True
    End Sub

    Private Sub ListView_QueryContinueDrag(sender As Object, e As QueryContinueDragEventArgs) Handles ListView.QueryContinueDrag

'     e.KeyState の値
'       0:何も押されていない＝マウスのボタンが離された (e.Action = DragAction.Drop)
'       1:マウスの左ボタン
'       2:マウスの右ボタン  
'       4:Shift キー, 8:CTRL キー, 16:マウスの中央ボタン, 32:Alt キー

'     マウスのボタンが離された時、
'       Move:     リストビューの範囲外の場合、ドラッグをキャンセル
'       FileDrop: リストビューの範囲内  〃

        If e.Action = DragAction.Drop Then
           Dim p As Point = Me.PointToClient(Cursor.Position)

        'マウスの右ボタンが Form 内 かつ ListView 外で離された場合、ドラッグをキャンセル
           If Me.ClientRectangle.Contains(p) Then
              p = ListView.PointToClient(Cursor.Position)
              If Not ListView.ClientRectangle.Contains(p) Then
                 e.Action = DragAction.Cancel
              Else
                 _Lsv_InnerDrop = True      'ListView 内でDrop

                 Lsv_DragColor_Control()    'ドラッグ位置の表示を消去
              End If
           End If

'     マウスの右ボタンが押された場合、ドラッグをキャンセル
        Else
           If (e.KeyState And 2) = 2 Then
              e.Action = DragAction.Cancel
           End If
        End If

     'ドラッグキャンセル情報をセット
        _Lsv_DragCancel = (e.Action = DragAction.Cancel)
    End Sub

    Private Sub ListView_DragEnter(sender As Object, e As DragEventArgs) Handles ListView.DragEnter
        e.Effect = DragDropEffects.Move
    End Sub

    Private Sub ListView_DragLeave(sender As Object, e As EventArgs) Handles ListView.DragLeave
        Lsv_DragColor_Control()
    End Sub

    Private Sub ListView_DragOver(sender As Object, e As DragEventArgs) Handles ListView.DragOver

        Const ScrollWaitTime = 200    'スクロール速度制限値 (ms)

        Dim CurPos As Point = ListView.PointToClient(New Point(e.X, e.Y))
        Dim CurIdx As Integer
        Dim ScWait As Boolean

        Dim scroll As Boolean

'     ListViewの表示行数(完全に表示されている)より登録数が多い場合のみ処理
        If _Lsv_DisplayRows < ListView.Items.Count Then

        '↑Scroll

           If CurPos.Y < _Lsv_LabelHeight Then

              CurIdx = ListView.TopItem.Index

           'マウスを速く移動した際に強調表示行が残ったままになるのを回避
              If CurIdx <> _Lsv_PrevIndex Then
                 Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
              End If

              If CurIdx > 0 Then
                 scroll = True
                 ScWait = (CurPos.Y > (_Lsv_LabelHeight \ 2))

                 _Lsv_PrevIndex = CurIdx - 1
              End If

        '↓Scroll

           ElseIf CurPos.Y > (ListView.ClientRectangle.Height - _Lsv_Row_Height) Then

              CurIdx = ListView.TopItem.Index + (_Lsv_DisplayRows - 1)

           'マウスを速く移動した際に強調表示行が残ったままになるのを回避
              If CurIdx <> _Lsv_PrevIndex Then
                 Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
              End If

              If CurIdx < ListView.Items.Count - 1 Then
                 scroll = True
                 ScWait = (CurPos.Y < ListView.ClientRectangle.Height - (_Lsv_Row_Height \ 2))

                 _Lsv_PrevIndex = CurIdx + 1
              End If
           End If

        'ドラッグ位置の表示 (スクロール)

           If scroll Then
              ListView.BeginUpdate()
                _Lsv_DragState = True

                Lsv_Dragcolor_Change(CurIdx, Nothing)
                ListView.EnsureVisible(_Lsv_PrevIndex)
                Lsv_Dragcolor_Change(_Lsv_PrevIndex, _BackColorSuccess)
              ListView.EndUpdate()
              ListView.Update()

              If ScWait Then Thread.Sleep(ScrollWaitTime)

              Return
           End If
        End If

     'ドラッグ位置の表示

        Dim itm As ListViewItem = ListView.GetItemAt(CurPos.X, CurPos.Y)

        If Not itm Is Nothing Then
           If Not _Lsv_DragState Then
              _Lsv_DragState = True
           Else
              If _Lsv_PrevIndex <> itm.Index Then                '同一処理を回避
                 Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
              End If
           End If

           _Lsv_PrevIndex = itm.Index

           Lsv_Dragcolor_Change(_Lsv_PrevIndex, _BackColorSuccess)
        Else

        'リストの未登録行にマウスカーソルがある場合は強調表示を解除
           If ListView.ClientRectangle.Contains(CurPos) Then
              Lsv_Dragcolor_Change(_Lsv_PrevIndex, Nothing)
           End If
        End If
    End Sub

    Private Sub ListView_DragDrop(sender As Object, e As DragEventArgs) Handles ListView.DragDrop

        If ListView.SelectedItems.Count = ListView.Items.Count Then Return

        Dim p As Point = ListView.PointToClient(New Point(e.X, e.Y))
        Dim DistItem As ListViewItem = ListView.GetItemAt(p.X, p.Y)
        Dim DistIndex As Integer
        Dim PushIndex As Integer

        If Not (DistItem Is Nothing) Then

        '移動先の位置を記録
           If ListView.SelectedIndices.Contains(DistItem.Index) Then

              Dim sc As Integer = ListView.SelectedItems.Count - 1
              Dim i0 As Integer = ListView.SelectedItems(0).Index
              Dim i1 As Integer = ListView.SelectedItems(sc).Index

              If sc = i1 - i0 Then Return '移動先が同じ

              For i As Integer = 0 To sc
                i0 = ListView.SelectedItems(i).Index

                If i0 = DistItem.Index Then
                   i1 = i                           '移動位置までの選択行数
                   Exit For
                End If
              Next

              PushIndex = DistItem.Index - i1
           Else
              PushIndex = DistItem.Index - 1
           End If
        End If

      ListView.BeginUpdate()

        Lsv_DragColor_Control()

        Dim ItemsCount As Integer = ListView.SelectedItems.Count
        Dim PushItem As New List(Of ListViewItem)

     '移動元のアイテムを退避して削除
        For Each SrcItem As ListViewItem In ListView.SelectedItems
          PushItem.Add(SrcItem)
          ListView.Items.Remove(SrcItem)
        Next

     '移動位置を確定
        If DistItem Is Nothing Then
           If p.Y < _Lsv_LabelHeight Then
              DistIndex = 0                         'ラベルを選択した場合はリストの先頭に
           Else
              DistIndex = ListView.Items.Count      '空白行を選択した場合はリストの最後に
           End If
        Else
           If DistItem.Index = -1 Then
              DistIndex = PushIndex
           Else
              DistIndex = DistItem.Index
           End If
        End If

     '退避したアイテムを移動位置に復元
        For i As Integer = 0 To ItemsCount - 1
          ListView.Items.Insert(DistIndex + i, PushItem(i))
        Next

     '移動位置へフォーカスを移す
        ListView.Items(DistIndex).Focused = True
        ListView.Items(DistIndex).Selected = True

     'マーカークリア
        If ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)

      ListView.EndUpdate()
    End Sub

#End Region

#Region "// Context Menu"

#Region "-- Setting"

    Private Sub ContextMenu_Setting_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Setting.Opening
        StripMenu_IniOpen.Enabled = File.Exists(_ini_File)
        StripMenu_UndoOpen.Enabled = File.Exists(_UndoFile)
        StripMenu_LogOpen.Enabled = File.Exists(_Log_File)
        StripMenu_Cmb_FontSize.SelectedIndex = _Lsv_FontSize_Id
    End Sub

    Private Sub ContextMenu_Setting_Closed(sender As Object, e As ToolStripDropDownClosedEventArgs) Handles ContextMenu_Setting.Closed

     'ListView 表示変更
        If _Lsv_FontSize_Id <> StripMenu_Cmb_FontSize.SelectedIndex Then
           _Lsv_FontSize_Id = StripMenu_Cmb_FontSize.SelectedIndex

           Lsv_SetInfo()

           Lsv_DisplayMode_Control(True)
        End If
    End Sub

    Private Sub StripMenu_DialogBox_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_DialogBox.CheckedChanged
        _OfdAutoUpgrade = StripMenu_DialogBox.Checked
    End Sub

    Private Sub StripMenu_ToolTip_Click(sender As Object, e As EventArgs) Handles StripMenu_ToolTip.Click

        Dim v As Integer = StripMenu_ToolTip.CheckState

        v += 1
        If v > 2 Then v = 0

        StripMenu_ToolTip.CheckState = DirectCast(v, CheckState)
    End Sub

    Private Sub StripMenu_ToolTip_CheckedChanged(sender As Object, e As EventArgs)

        ToolTip.Active = (StripMenu_ToolTip.CheckState = CheckState.Checked)

     'ListView.ShowItemToolTips が変更されると ListView.ColumnWidthChanged が発生する

      RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
        ListView.ShowItemToolTips = (StripMenu_ToolTip.CheckState <> CheckState.Unchecked)
      AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

        GroupBox3_ToolTip_Change()
        Btn_Exec_ToolTip_Change()
    End Sub

    Private Sub StripMenu_MsgSound_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_MsgSound.CheckedChanged
        _MessageBoxSound = StripMenu_MsgSound.Checked
    End Sub

    Private Sub StripMenu_LogWrite_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_LogWrite.CheckedChanged
        _LogFileWriting = StripMenu_LogWrite.Checked
    End Sub

    Private Sub StripMenu_IniSave_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_IniSave.CheckedChanged
        _IniSave = StripMenu_IniSave.Checked
    End Sub

    Private Sub StripMenu_IniOpen_Click(sender As Object, e As EventArgs) Handles StripMenu_IniOpen.Click

'     設定ファイルをメモ帳で開く
        Diagnostics.Process.Start("Notepad", _ini_File)
    End Sub

    Private Sub StripMenu_UndoOpen_Click(sender As Object, e As EventArgs) Handles StripMenu_UndoOpen.Click

'     アンドゥファイルをメモ帳で開く
        Diagnostics.Process.Start("Notepad", _UndoFile)
    End Sub

    Private Sub StripMenu_LogOpen_Click(sender As Object, e As EventArgs) Handles StripMenu_LogOpen.Click

'     ログファイルをメモ帳で開く
        Diagnostics.Process.Start("Notepad", _Log_File)
    End Sub

    Private Sub StripMenu_GridLines_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_GridLines.CheckedChanged
        _UseGridLines = StripMenu_GridLines.Checked

        ListView.GridLines = StripMenu_GridLines.Checked
    End Sub

    Private Sub StripMenu_CtrlUpDown_CheckedChanged(sender As Object, e As EventArgs) Handles StripMenu_CtrlUpDown.CheckedChanged
        _UseCtrlUpDown = StripMenu_CtrlUpDown.Checked
    End Sub

    Private Sub StripMenu_E_Click(sender As Object, e As EventArgs) _
                Handles StripMenu_E0.Click, StripMenu_E1.Click, StripMenu_E2.Click, StripMenu_E3.Click

        Dim n As String = DirectCast(sender, ToolStripItem).Name

        _UseEmphasisColor = Convert.ToInt32(n.Substring(n.Length - 1))

        SetEmphasisBackColor()
        StripMenu_E_Control()
    End Sub

#End Region

#Region "-- ListView"

   'Private Sub ContextMenu_ListView_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_ListView.Opening
      'ここで表示内容を変更すると、表示座標がずれる為 ListView_MouseUp で処理
   'End Sub

    Private Sub StripMenu_SelectConversion_Click(sender As Object, e As EventArgs) Handles StripMenu_SelectConversion.Click
        _onSelectItem = True

        Btn_Exec.Select()
        Btn_Exec.PerformClick()

        _onSelectItem = False
    End Sub

    Private Sub StripMenu_CreationTime_Click(sender As Object, e As EventArgs) Handles StripMenu_CreationTime.Click
        Dim d As String = ListView.Items(ListView.SelectedItems(0).Index).SubItems(2).Text

        Pck_Date.Value = DateTime.Parse(d)
        Pck_Time.Value = DateTime.Parse(d)

        DateTimeChangeLogUpdate()
    End Sub

    Private Sub StripMenu_DateModified_Click(sender As Object, e As EventArgs) Handles StripMenu_DateModified.Click
        Dim d As String = ListView.Items(ListView.SelectedItems(0).Index).SubItems(3).Text

        Pck_Date.Value = DateTime.Parse(d)
        Pck_Time.Value = DateTime.Parse(d)

        DateTimeChangeLogUpdate()
    End Sub

    Private Sub StripMenu_Open_Click(sender As Object, e As EventArgs) Handles StripMenu_Open.Click

'     ファイルを関連付けされたアプリケーションで開く
        Try
          Dim p As Diagnostics.Process = Diagnostics.Process.Start(ListView.SelectedItems.Item(0).Text)

          If Not p Is Nothing Then
             p.WaitForInputIdle()
             p.Close()
             p.Dispose()
          End If

        Catch ex As Exception
          MsgBox(ex.Message)
        End Try
    End Sub

    Private Sub StripMenu_StripMenu_SelectAllItem_Click(sender As Object, e As EventArgs) Handles StripMenu_SelectAllItem.Click
        For Each LsvItem As ListViewItem In ListView.Items
          LsvItem.Selected = True
        Next
    End Sub

    Private Sub StripMenu_ToggleSelection_Click(sender As Object, e As EventArgs) Handles StripMenu_ToggleSelection.Click
        For Each LsvItem As ListViewItem In ListView.Items
          LsvItem.Selected = LsvItem.Selected Xor True
        Next
    End Sub

    Private Sub StripMenu_DeleteItem_Click(sender As Object, e As EventArgs) Handles StripMenu_DeleteItem.Click
        Lsv_Delete(False)
    End Sub

    Private Sub StripMenu_AttributeChange_Click(sender As Object, e As EventArgs) Handles StripMenu_AttributeChange.Click
        FileAttributesChange()
    End Sub

    Private Sub StripMenu_VerInfo_Click(sender As Object, e As EventArgs) Handles StripMenu_VerInfo.Click, StripMenu_About.Click
        Dim VerInfo As New AboutBoxEx

        VerInfo.Showdialog(Me)
        VerInfo.Dispose()

        Me.Update()
        WinMessage.Clear(Me, 200)
    End Sub

#End Region

#Region "-- Notify"

   'Private Sub ContextMenu_Notify_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Notify.Opening
      'ここで表示内容を変更すると、表示座標がずれる為 Notify_Icon_MouseDown で処理
   'End Sub

    Private Sub Notify_Icon_MouseDown(sender As Object, e As MouseEventArgs) Handles Notify_Icon.MouseDown

     'ContextMenu_Notify_Opening で表示内容を変更すると、表示座標がずれる為の措置

     'Notify_Icon_MouseDown→ContextMenu_Notify_Opening→
     'Notify_Icon_MouseUp→Notify_Icon_MouseClick

        If e.Button = MouseButtons.Right Then

           Dim f As Boolean = Not _WindowMinimized

           StripMenu_Setting.Visible = f
           StripSeparator_N1.Visible = f
           StripMenu_About.Visible = f
        End If
    End Sub

    Private Sub Notify_Icon_MouseClick(sender As Object, e As MouseEventArgs) Handles Notify_Icon.MouseClick

        If Me.ShowInTaskbar Then Return

        If e.Button = MouseButtons.Left Then
           If _WindowMinimized Then

           'ContextMenu が表示されている時に、最小化を解除すると
           'ContextMenu が残ったままになる場合がある為の措置

              ContextMenu_Notify.Close()

              If _WindowMaximized Then
                 Me.WindowState = FormWindowState.Maximized
              Else
                 Me.WindowState = FormWindowState.Normal
              End If

           'Normal 復帰時にフォーム表示が壊れる事がある為の措置
              Me.Refresh()
           Else
              If (Not CheckOpenForms()) Then
                 Me.WindowState = FormWindowState.Minimized
              End If
           End If
        End If
    End Sub

    Private Sub ContextMenu_Notify_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Notify.Opening

     'DialogForm (FileDialog, FolderBrowserDialog)、または、
     '他のフォームが表示されている場合はキャンセル

        If CheckOpenForms() Then

           e.Cancel = True
        End If
    End Sub

    Private Sub StripMenu_Setting_Click(sender As Object, e As EventArgs) Handles StripMenu_Setting.Click
        ShowSettings(False)
    End Sub

    Private Sub StripMenu_TaskBar_CheckedChanged(sender As Object, e As EventArgs)

        Dim ChangeState As Boolean
        Dim MaximizedState As Boolean = _WindowMaximized

     'ShowInTaskbar が変更されると フォーム表示にちらつきが発生する為の措置
        If Not _WindowMinimized Then
           ChangeState = True
           Me.WindowState = FormWindowState.Minimized
        End If

     'ShowInTaskbar が変更されると
     'ComboBoxEx: Resize ⇒ Resize ⇒ OnCreateControl が発生する

     'ShowInTaskbar が変更されると ListView.ColumnWidthChanged が発生する為の措置

      RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
        Me.ShowInTaskbar = StripMenu_TaskBar.Checked
      AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

        If ChangeState Then
           If MaximizedState Then
              Me.WindowState = FormWindowState.Maximized
           Else
              Me.WindowState = FormWindowState.Normal
           End If
        End If
    End Sub

    Private Sub StripMenu_Quit_Click(sender As Object, e As EventArgs) Handles StripMenu_Quit.Click
        Me.Close()
    End Sub

#End Region

#Region "-- Picker"

    Private Sub ContextMenu_Picker_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Picker.Opening
     'クリップボードの内容がテキスト以外の時はコンテキストメニューを表示しない
        If Not Clipboard.ContainsText Then e.Cancel = True
    End Sub

    Private Sub ContextMenu_Picker_Closed(sender As Object, e As ToolStripDropDownClosedEventArgs) Handles ContextMenu_Picker.Closed

     'メニュー選択後、メニューを消した後で試行処理を行う。
     ' (TryParse が連続で試行された場合に、メニュー表示が壊れる（Picker 再描画時にメニューが一部残る）のを防止)

        Me.Update()
    End Sub

    Private Sub StripMenu_P1_Click(sender As Object, e As EventArgs) Handles StripMenu_P1.Click
     '[Ctrl]+V
        Me.Pck_KeyDown(Me, New KeyEventArgs(DirectCast(131158, Keys)))
    End Sub

    Private Sub StripMenu_P2_Click(sender As Object, e As EventArgs) Handles StripMenu_P2.Click
     '[Alt]+V
        Me.Pck_KeyDown(Me, New KeyEventArgs(DirectCast(262230, Keys)))
    End Sub

    Private Sub StripMenu_P3_Click(sender As Object, e As EventArgs) Handles StripMenu_P3.Click
     '[Ctrl]+[Shift]+V
        Me.Pck_KeyDown(Me, New KeyEventArgs(DirectCast(196694, Keys)))
    End Sub

#End Region

#Region "-- Random"

    Private Sub ContextMenu_Random_Opening(sender As Object, e As CancelEventArgs) Handles ContextMenu_Random.Opening
        Dim f As Boolean = Chk_Rand.Checked AndAlso (Txt_Time_Ajst.Value <> 0)

        ContextMenu_Random.Enabled = f
    End Sub

    Private Sub StripMenu_R_Click(sender As Object, e As EventArgs) _
                Handles StripMenu_R1.Click, StripMenu_R2.Click, StripMenu_R3.Click, StripMenu_R4.Click, StripMenu_R5.Click

        _Adjust_MValue = Convert.ToSingle(DirectCast(sender, ToolStripItem).Text.Substring(0, 3)) / 100

        StripMenu_R_Control()
    End Sub

    Private Sub StripMenu_R0_Click(sender As Object, e As EventArgs) Handles StripMenu_R0_0.Click, StripMenu_R0_1.Click
        _Adjust_LCheck = (DirectCast(sender, ToolStripItem).Text <> "0")

        StripMenu_R0_Control()
    End Sub

#End Region

#End Region

#Region "// GroupBox, Label, CheckBox"

#Region "-- Menu (Top, GroupBox 1)"

    Private Sub GroupBox1_MouseDown(sender As Object, e As MouseEventArgs) Handles GroupBox1.MouseDown
        If e.Button = MouseButtons.Right Then
           If GroupBox1.LabelRectangle.Contains(e.Location) Then
              ContextMenu_Setting.Show(Me, New Point(0, 0))
           End If
        End If
    End Sub

    Private Sub Lbl_Attr_MouseLeave(sender As Object, e As EventArgs) Handles Lbl_Attr_R.MouseLeave, Lbl_Attr_H.MouseLeave, Lbl_Attr_S.MouseLeave
        If _onToolTipActive Then
           _onToolTipActive = False

           Me.ToolTip.Active = False
           Me.ToolTip.IsBalloon = False
           Me.ToolTip.InitialDelay = 500

           Attr_ToolTip_Change()
        End If
    End Sub

    Private Sub Lbl_Attr_MouseClick(sender As Object, e As MouseEventArgs) Handles Lbl_Attr_R.MouseClick, Lbl_Attr_H.MouseClick, Lbl_Attr_S.MouseClick
        Dim i As Integer = "RHS".IndexOf(DirectCast(sender, LabelSL).Text, StringComparison.Ordinal)

        If e.Button <> MouseButtons.Left Then Return

        _AttrType(i) = _AttrType(i) Xor Convert.ToInt32(2 ^ i)
        _List_AttrType = _AttrType(0) + _AttrType(1) + _AttrType(2)

        DirectCast(sender, LabelSL).ForeColor = Color.FromName(_AttrColorName(_AttrType(i)))

        If Not Me.ToolTip.Active Then
           _onToolTipActive = True

           Me.ToolTip.InitialDelay = 50
           Me.ToolTip.IsBalloon = True
           Me.ToolTip.Active = True
        End If

        Attr_ToolTip_Change()
    End Sub

    Private Sub Lbl_Blank2_DoubleClick(sender As Object, e As EventArgs) Handles Lbl_Blank2.DoubleClick
        Dim fi As New FileInfo(_UndoFile)

        If (Not Btn_Undo.Enabled) AndAlso (fi.Exists) AndAlso (fi.Length <> 0) Then

           Dim MsgBox As New MessageBoxEx
           If MsgBox.Dialog(Me, "Undo 情報を再読込しますか？。", _Msg_Cap, _
                            MessageBoxEx.Buttons.CenterYesNo, _
                            MessageBoxIcon.Question, _
                            MessageBoxDefaultButton.Button2, , _MessageBoxSound) =
           DialogResult.Yes Then
              Btn_Undo.Enabled = True
           End If
           MsgBox.Dispose()

           Me.Update()
        End If
    End Sub

    Private Sub Lbl_Items_DoubleClick(sender As Object, e As EventArgs) Handles Lbl_Items.DoubleClick
        Dim Reg_SZ As String = DirectCast(Microsoft.Win32.Registry.GetValue( _
                               "HKEY_CURRENT_USER\AppEvents\Schemes\Apps\Explorer\EmptyRecycleBin\.Current", "", Nothing), String)

        If Reg_SZ Is Nothing Then
           Beep()
        Else
           Dim windir As String = Environment.GetFolderPath(Environment.SpecialFolder.Windows)
           Dim F_Name As String = Path.GetFileName(Reg_SZ)
           Dim F_Path As String

           If Path.GetDirectoryName(Reg_SZ) = "" Then
              F_Path = Path.Combine(windir, "Media", F_Name)    'XP対策
           Else
              F_Path = Replace(Reg_SZ, "%SystemRoot%", windir)
           End If

           If File.Exists(F_Path) Then
              Dim player As New SoundPlayer(F_Path)
              player.PlaySync()
              player.Dispose()
              player = Nothing
           End If
        End If

        GC.Collect()
        GC.WaitForPendingFinalizers()
        GC.Collect()
    End Sub

#End Region

#Region "-- 対象 (GroupBox 2)"

    Private Sub Chk_Make_Repl_Click(sender As Object, e As EventArgs) Handles Chk_Make.Click, Chk_Repl.Click
        Btn_Exec_Control()
    End Sub

#End Region

#Region "-- 変更日時 (GroupBox 3)"

    Private Sub GroupBox3_BringToFront()

        Btn_LogApply.SendToBack()
        Btn_LogApply.Enabled = False

     '処理の重複を回避するため、Leaveイベントを無効にする
        RemoveHandler Me.Cmb_DateTimeLog.Leave, AddressOf Cmb_DateTimeLog_Leave
          Cmb_DateTimeLog.SendToBack()
          Cmb_DateTimeLog.Enabled = False
        AddHandler Me.Cmb_DateTimeLog.Leave, AddressOf Cmb_DateTimeLog_Leave

        For Each ctl As Control In GroupBox3.Controls
          ctl.Visible = True
        Next

        Me.KeyPreview = True
        Btn_Exec_Control()
    End Sub

    Private Sub GroupBox3_EnabledChanged(sender As Object, e As EventArgs) Handles GroupBox3.EnabledChanged
        GroupBox3_ToolTip_Change()
    End Sub

    Private Sub GroupBox3_MouseDown(sender As Object, e As MouseEventArgs) Handles GroupBox3.MouseDown

        If e.Button = MouseButtons.Right Then

           If GroupBox3.LabelRectangle.Contains(e.Location) Then
              If Cmb_DateTimeLog.Enabled Then
                 GroupBox3_BringToFront()

                 Pck_Date.Select()
              Else

              'GroupBoxLabel 以外を非表示
                 For Each ctl As Control In GroupBox3.Controls
                   If ctl.Name.Length <> 0 Then
                      ctl.Visible = False
                   End If
                 Next

                 Btn_LogApply.Enabled = True
                 Btn_LogApply.BringToFront()

                 Cmb_DateTimeLog.Enabled = True
                 Cmb_DateTimeLog.BringToFront()
                 Cmb_DateTimeLog.Focus()

                 Me.KeyPreview = False
                 Btn_Exec_Control(False)
              End If
           End If
        End If
    End Sub

    Private Sub Btn_LogApply_Click(sender As Object, e As EventArgs) Handles Btn_LogApply.Click
        Me.Cmb_DateTimeLog_KeyDown(sender, New KeyEventArgs(Keys.Enter))
    End Sub

    Private Sub Btn_LogApply_Leave(sender As Object, e As EventArgs) Handles Btn_LogApply.Leave
        If Me.ActiveControl.Name <> "Cmb_DateTimeLog" Then
           GroupBox3_BringToFront()
        End If
    End Sub

    Private Sub Cmb_DateTimeLog_KeyDown(sender As Object, e As KeyEventArgs) Handles Cmb_DateTimeLog.KeyDown

        If e.KeyData = Keys.Enter Then
           Pck_Date.Value = DirectCast(Cmb_DateTimeLog.SelectedValue, Date)
           Pck_Time.Value = Pck_Date.Value
        Else
           If e.KeyData <> Keys.Escape Then Return
        End If

        GroupBox3_BringToFront()

        Pck_Date.Select()
    End Sub

    Private Sub Cmb_DateTimeLog_Leave(sender As Object, e As EventArgs) Handles Cmb_DateTimeLog.Leave
        If Me.ActiveControl.Name <> "Btn_LogApply" Then
           GroupBox3_BringToFront()
        End If
    End Sub

    Private Sub Chk_Date_Click(sender As Object, e As EventArgs) Handles Chk_Date.Click
        Chk_Date_Control()
    End Sub

    Private Sub Chk_Time_Click(sender As Object, e As EventArgs) Handles Chk_Time.Click
        Chk_Time_Control()
    End Sub

    Private Sub Lbl_DPlus_MouseDown(sender As Object, e As MouseEventArgs) Handles Lbl_DPlus.MouseDown
'-------------------------------------------------------------------------------
'     日付調整・設定値記憶
'-------------------------------------------------------------------------------
        If e.Button = MouseButtons.Middle Then
           If Txt_Date_Ajst.Value <> 0 Then
              _M_Adjust_D_Mode = _Adjust_D_Mode
              _M_Adjust_DValue = Txt_Date_Ajst.Value

              ShowInfoMessage("日付調整の設定を変更しました。", _App_Name)

              Txt_Date_Ajst.SelectAll()
           Else
              Beep()
           End If
        End If
    End Sub

    Private Sub Lbl_DPlus_DoubleClick(sender As Object, e As EventArgs) Handles Lbl_DPlus.DoubleClick
        _List_TreeMode = 0
        _List_FileMode = 0
        Btn_TreeMode.ImageIndex = 4
        Btn_FileMode.ImageIndex = 1
        ListMode_ToolTip_Change()

        Txt_Date_Ajst.Value = _M_Adjust_DValue
        Txt_Date_Ajst.Select()
        Txt_Date_Ajst.SelectAll()

        _Adjust_D_Mode = _M_Adjust_D_Mode

        Lbl_DAdjustMode.Text = _Adjust_Name(0, _M_Adjust_D_Mode)
    End Sub

    Private Sub Lbl_TPlus_MouseDown(sender As Object, e As MouseEventArgs) Handles Lbl_TPlus.MouseDown
'-------------------------------------------------------------------------------
'     時刻調整・設定値記憶
'-------------------------------------------------------------------------------
        If e.Button = MouseButtons.Middle Then
           If Txt_Time_Ajst.Value <> 0 Then
              _M_Adjust_T_Mode = _Adjust_T_Mode
              _M_Adjust_TValue = Txt_Time_Ajst.Value
              _M_Adjust_RCheck = Chk_Rand.Checked
              _M_Adjust_LCheck = _Adjust_LCheck
              _M_Adjust_MValue = _Adjust_MValue

              ShowInfoMessage("時刻調整の設定を変更しました。", _App_Name)

              Txt_Time_Ajst.SelectAll()
           Else
              Beep()
           End If
        End If
    End Sub

    Private Sub Lbl_TPlus_DoubleClick(sender As Object, e As EventArgs) Handles Lbl_TPlus.DoubleClick
        _List_TreeMode = 0
        _List_FileMode = 0
        Btn_TreeMode.ImageIndex = 4
        Btn_FileMode.ImageIndex = 1
        ListMode_ToolTip_Change()

        _Adjust_T_Mode = _M_Adjust_T_Mode
        _Adjust_LCheck = _M_Adjust_LCheck
        _Adjust_MValue = _M_Adjust_MValue

        Txt_Time_Ajst.Value = _M_Adjust_TValue
        Txt_Time_Ajst.Select()
        Txt_Time_Ajst.SelectAll()

        Chk_Rand.Checked = _M_Adjust_RCheck
        Lbl_TAdjustMode.Text = _Adjust_Name(1, _M_Adjust_T_Mode)

        StripMenu_R_Control()
        StripMenu_R0_Control()
    End Sub

    Private Sub Lbl_DAdjustMode_Click(sender As Object, e As EventArgs) Handles Lbl_DAdjustMode.Click
        _Adjust_D_Mode = _Adjust_D_Mode Xor 1
        Lbl_DAdjustMode.Text = _Adjust_Name(0, _Adjust_D_Mode)
    End Sub

    Private Sub Lbl_TAdjustMode_Click(sender As Object, e As EventArgs) Handles Lbl_TAdjustMode.Click
        _Adjust_T_Mode = _Adjust_T_Mode Xor 1
        Lbl_TAdjustMode.Text = _Adjust_Name(1, _Adjust_T_Mode)
    End Sub

#End Region

#Region "-- Menu (Bottom)"

    Private Sub Lbl_Mode_Lamp_MouseClick(sender As Object, e As MouseEventArgs) Handles Lbl_Mode_Lamp_1.MouseClick, Lbl_Mode_Lamp_2.MouseClick
        If e.Button = MouseButtons.Left Then
           _Lsv_DisplayMode = 0
           ModeLamp_Control()
        End If
    End Sub

    Private Sub Lbl_Check_Lamp_MouseClick(sender As Object, e As MouseEventArgs) Handles Lbl_CheckLamp.MouseClick
        Lbl_CheckLamp.Visible = False
    End Sub

#End Region

#End Region

#Region "// DateTimePicker"

    Private Sub Pck_Enter(sender As Object, e As System.EventArgs) Handles Pck_Date.Enter, Pck_Time.Enter

        'Enter ⇒ (Deactivate) ⇒ Leave ⇒ Validated

        If Not _onDeactivate Then
          _onDeactivate = True

           AddHandler Me.Deactivate, AddressOf Me_Deactivate
        End If
    End Sub

    Private Sub Pck_Leave(sender As Object, e As System.EventArgs) Handles Pck_Date.Leave, Pck_Time.Leave

'     他の DateTimePicker へのフォーカスの移動、および、
'     外部のボタンで値が変更された（フォーカスが戻る）場合は、イベントを解除しない。

        If DateTimePickerExclusionControl() Then
           _onDeactivate = False

           RemoveHandler Me.Deactivate, AddressOf Me_Deactivate
        End If
    End Sub

    Private Sub Pck_KeyDown(sender As Object, e As KeyEventArgs) Handles Pck_Date.KeyDown, Pck_Time.KeyDown

        If e.KeyData = DirectCast(131139, Keys) Then    '[Ctrl]+C

           Dim t As String = Pck_Date.Value.Date.ToString("yyyy/MM/dd ") &
                             Pck_Time.Value.TimeOfDay.ToString

           SetClipboardText(t, "")

           e.SuppressKeyPress = True
           Return
        End If

     '[ctrl]+V 以外、または、クリップボードの内容がテキスト以外の時はキャンセル
        If Not Clipboard.ContainsText Then Return

        Dim cText As String = Clipboard.GetText
        Dim state As Boolean
        Dim Result As Date

        Select Case e.KeyData
          Case DirectCast(131158, Keys)                 '標準書式 [Ctrl]+V
            state = DateTimeTextCheck(cText, Result)

          Case DirectCast(262230, Keys)                 '簡易書式 [Alt]+V
            state = DateTimeTextParse(cText, Result)

          Case DirectCast(196694, Keys)                 'カルチャ [Ctrl]+[Shift]+V
            state = DateTimeCultureParse(cText, Result)

          Case Else
            Return
        End Select

        e.SuppressKeyPress = True       'KeyPressイベントをキャンセル

        If state Then
        'Min,MaxDate テェック (Default 1753/01/01-9998/12/31)
           Try
             Pck_Date.Value = Result
             Pck_Time.Value = Result

             GroupBox3.ForeColor = _ColorSuccess
             GroupBox3.Update()
             Thread.Sleep(250)
             GroupBox3.ForeColor = SystemColors.ControlText
             GroupBox3.Update()

           Catch ex As Exception
             Beep()
           End Try
        Else
           Beep()
        End If
    End Sub

    Private Sub Pck_KeyPress(sender As Object, e As KeyPressEventArgs) Handles Pck_Date.KeyPress, Pck_Time.KeyPress

        If e.KeyChar = vbCr Then
           e.Handled = True
           Return
        End If

        If e.KeyChar = "*"c Then
           e.Handled = True
           If DirectCast(sender, DateTimePickerEx).Name = "Pck_Date" Then
              Pck_Time.Select()
           Else
              Pck_Date.Select()
           End If
        End If
    End Sub

    Private Sub Pck_ValueChanged(sender As Object, e As EventArgs)
        _DateTimeValueChangeFlag = True
    End Sub

    Private Sub Pck_Validated(sender As Object, e As EventArgs) Handles Pck_Date.Validated, Pck_Time.Validated

'     他の DateTimePicker へのフォーカスの移動、および、
'     外部のボタンで値が変更された（値が確定されていない）場合は、日時変更ログを更新しない。

        If _DateTimeValueChangeFlag AndAlso DateTimePickerExclusionControl() Then
           DateTimeChangeLogUpdate()
        End If
    End Sub

#End Region

#Region "// Button"

#Region "-- Menu (Top)"

    Private Sub Btn_Open_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Open.MouseDown
        If e.Button = MouseButtons.Middle Then
           Btn_Dummy.Enabled = True
           Btn_Dummy.Select()
           Btn_Open.Enabled = False

           _onDialogForm = True

           If _List_FileMode <> 0 Then
              Open_File()
           Else
              Open_Folder()
           End If

           Me.Update()
           WinMessage.Clear(Me)

           ListView.Select()

           Btn_Open.Enabled = True
           Btn_Dummy.Enabled = False
        End If

        _onDialogForm = False
    End Sub

    Private Sub Btn_Open_Click(sender As Object, e As EventArgs) Handles Btn_Open.Click
        Btn_Dummy.Enabled = True
        Btn_Dummy.Select()
        Btn_Open.Enabled = False

        _onDialogForm = True

        If _List_FileMode <> 0 Then
           Open_Folder()
        Else
           Open_File()
        End If

        Me.Update()
        WinMessage.Clear(Me)

        ListView.Select()

        Btn_Open.Enabled = True
        Btn_Dummy.Enabled = False

        _onDialogForm = False
    End Sub

    Private Sub Btn_Erase_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Erase.MouseDown
        If e.Button = MouseButtons.Middle Then
           ListView.HideSelection = True

           Lsv_Delete(True)
           ListView.Select()

           ListView.HideSelection = _UseHideSelection
        End If
    End Sub

    Private Sub Btn_Erase_Click(sender As Object, e As EventArgs) Handles Btn_Erase.Click
        ListView.HideSelection = True

        Lsv_Clear()
        ListView.Select()

        ListView.HideSelection = _UseHideSelection
    End Sub

#End Region

#Region "-- 対象 (GroupBox 2)"

    Private Sub Btn_Make_Click(sender As Object, e As EventArgs) Handles Btn_Make.Click

        _JobMode = Convert.ToInt32(_JobMode <> 1)

        Dim f As Boolean = (_JobMode = 0)
        Btn_Make.ImageIndex = Convert.ToInt32(_JobMode = 1)
        Btn_Repl.ImageIndex = 0

        Chk_Make.SetDisabledColor(False, Nothing)
        Chk_Repl.SetDisabledColor(False, _ColorMain)

        Chk_Make.Checked = f
        Chk_Make.Enabled = f
        Chk_Repl.Checked = True
        Chk_Repl.Enabled = f
        GroupBox3.Enabled = f

        Btn_Exec_Control()
    End Sub

    Private Sub Btn_Repl_Click(sender As Object, e As EventArgs) Handles Btn_Repl.Click

        _JobMode = 2 * Convert.ToInt32(_JobMode <> 2)

        Dim f As Boolean = (_JobMode = 0)
        Btn_Make.ImageIndex = 0
        Btn_Repl.ImageIndex = Convert.ToInt32(_JobMode = 2)

        Chk_Make.SetDisabledColor(False, _ColorMain)
        Chk_Repl.SetDisabledColor(False, Nothing)

        Chk_Make.Checked = True
        Chk_Make.Enabled = f
        Chk_Repl.Checked = f
        Chk_Repl.Enabled = f
        GroupBox3.Enabled = f

        Btn_Exec_Control()
    End Sub

#End Region

#Region "-- 変更日時 (GroupBox 3)"

    Private Sub Btn_DtClr_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_DtClr.MouseDown
        If e.Button = MouseButtons.Middle Then
           Txt_Date_Ajst.Value = 0

           Pck_Date.Value = Now.Date
           Pck_Date.Select()
        End If
    End Sub

    Private Sub Btn_DtClr_Click(sender As Object, e As EventArgs) Handles Btn_DtClr.Click
        Pck_Date.Value = Now.Date
        Pck_Date.Select()
    End Sub

    Private Sub Btn_TmClr_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_TmClr.MouseDown
        If e.Button = MouseButtons.Middle Then
           Txt_Time_Ajst.Value = 0

           Pck_Time.Value = Now.Date
           Pck_Time.Select()
        End If
    End Sub

    Private Sub Btn_TmClr_Click(sender As Object, e As EventArgs) Handles Btn_TmClr.Click
        Pck_Time.Value = Now.Date
        Pck_Time.Select()
    End Sub

    Private Sub Btn_Next_Click(sender As Object, e As EventArgs) Handles Btn_Next.Click
        Dim d As Date = DateTime.Parse(Pck_Date.Text.TrimStart)

        Pck_Date.Value = d.AddDays(7)
        Pck_Date.Select()
    End Sub

    Private Sub Btn_Prev_Click(sender As Object, e As EventArgs) Handles Btn_Prev.Click
        Dim d As Date = DateTime.Parse(Pck_Date.Text.TrimStart)

        Pck_Date.Value = d.AddDays(-7)
        Pck_Date.Select()
    End Sub

    Private Sub Btn_UTC0_Click(sender As Object, e As EventArgs) Handles Btn_UTC0.Click
        Dim d As Date = DateTime.Parse(Pck_Date.Text.TrimStart & " " & Pck_Time.Text.TrimStart)

        Pck_Time.Value = d.AddMinutes(-_Adjust_Time)
        Pck_Date.Value = d.AddMinutes(-_Adjust_Time)
        Pck_Time.Select()
    End Sub

    Private Sub Btn_UTC9_Click(sender As Object, e As EventArgs) Handles Btn_UTC9.Click
        Dim d As Date = DateTime.Parse(Pck_Date.Text.TrimStart & " " & Pck_Time.Text.TrimStart)

        Pck_Time.Value = d.AddMinutes(_Adjust_Time)
        Pck_Date.Value = d.AddMinutes(_Adjust_Time)
        Pck_Time.Select()
    End Sub

#End Region

#Region "-- Menu (Bottom)"

    Private Sub Btn_Col0Reset_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Col0Reset.MouseDown

        If _Lsv_DisplayMode <> 1 Then Return

        If (e.Button = MouseButtons.Middle) OrElse (e.Button = MouseButtons.Right) Then

           _Lsv_DisplayMode = 0
           ModeLamp_Control()

           RemoveHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged

           With ListView
             .BeginUpdate()

             Select Case e.Button
               Case MouseButtons.Middle
                 .Columns(1).Width += _Lsv_Col2_Width
                 .Columns(2).Width = 0

               Case MouseButtons.Right
                 .Columns(1).Width += (_Lsv_Col2_Width * 2)
                 .Columns(2).Width = 0
                 .Columns(3).Width = 0
             End Select

             .EndUpdate()
           End With

           AddHandler ListView.ColumnWidthChanged, AddressOf ListView_ColumnWidthChanged
        End If
    End Sub

    Private Sub Btn_Col0Reset_Click(sender As Object, e As EventArgs) Handles Btn_Col0Reset.Click
        _Lsv_DisplayMode = 1

        Lsv_DisplayMode_Control(True)
        ListView.Select()
    End Sub

    Private Sub Btn_Col2Reset_Click(sender As Object, e As EventArgs) Handles Btn_Col2Reset.Click
        _Lsv_DisplayMode = 2

        Lsv_DisplayMode_Control(True)
        ListView.Select()
    End Sub

    Private Sub Btn_ChckMode_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_ChckMode.MouseDown
        If e.Button = MouseButtons.Middle Then
           _List_ChckMode = _List_ChckMode Xor 1
           Btn_ChckMode.ImageIndex = _List_ChckMode + 9
           ListMode_ToolTip_Change()

           Lbl_CheckLamp.Visible = False
        End If
    End Sub

    Private Sub Btn_ChckMode_Click(sender As Object, e As EventArgs) Handles Btn_ChckMode.Click
        _List_ChckMode = _List_ChckMode Xor 1
        Btn_ChckMode.ImageIndex = _List_ChckMode + 9
        ListMode_ToolTip_Change()

        Lbl_CheckLamp.Visible = True
    End Sub

    Private Sub Btn_TreeMode_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_TreeMode.MouseDown
        If e.Button = MouseButtons.Middle Then
           _List_TreeMode = (_List_TreeMode Xor 2) Mod 3
           Btn_TreeMode.ImageIndex = _List_TreeMode + 4
           ListMode_ToolTip_Change()
        End If
    End Sub

    Private Sub Btn_TreeMode_Click(sender As Object, e As EventArgs) Handles Btn_TreeMode.Click
        _List_TreeMode = (_List_TreeMode + 1) Mod 3
        Btn_TreeMode.ImageIndex = _List_TreeMode + 4
        ListMode_ToolTip_Change()
    End Sub

    Private Sub Btn_FileMode_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_FileMode.MouseDown
        If e.Button = MouseButtons.Middle Then
           _List_FileMode = (_List_FileMode Xor 2) Mod 3
           Btn_FileMode.ImageIndex = _List_FileMode + 1
           ListMode_ToolTip_Change()
        End If
    End Sub

    Private Sub Btn_FileMode_Click(sender As Object, e As EventArgs) Handles Btn_FileMode.Click
        _List_FileMode = (_List_FileMode + 1) Mod 3
        Btn_FileMode.ImageIndex = _List_FileMode + 1
        ListMode_ToolTip_Change()
    End Sub

#End Region

#End Region

#Region "// ---- Main Button --------------------------------------------------"

    Private Sub Btn_Exec_EnabledChanged(sender As Object, e As EventArgs) Handles Btn_Exec.EnabledChanged
        Btn_Exec_ToolTip_Change()
    End Sub

    Private Sub Btn_Exec_MouseDown(sender As Object, e As MouseEventArgs) Handles Btn_Exec.MouseDown

        If e.Button = MouseButtons.Middle Then

           Dim cMode As Integer = _List_ChckMode
           Dim mMode As Boolean = Lbl_CheckLamp.Visible

           If mMode Then Lbl_CheckLamp.Enabled = False

           _List_ChckMode = 0   '0:Ignore

           If Me.ActiveControl.GetType.Name = "DateTimePickerEx" Then

           '値が確定されてない場合に日時変更ログが更新されるのを回避する為、ハンドルを一時解除

              Dim ctl As Control = Me.ActiveControl
              RemoveHandler ctl.Validated, AddressOf Pck_Validated

              Btn_Exec.Select()     '値が確定されてない場合、ValueChanged 発生

              AddHandler ctl.Validated, AddressOf Pck_Validated

           '入力した値が確定されてない場合にコントロールの更新が遅延する為、更新
              ctl.Update()

           Else
              Btn_Exec.Select()
           End If

        '日時変更ログ更新
           DateTimeChangeLogUpdate()

           Btn_Exec.PerformClick()

           If mMode Then Lbl_CheckLamp.Enabled = True

           _List_ChckMode = cMode
        End If
    End Sub

    Private Sub Btn_Exec_Click(sender As Object, e As EventArgs) Handles Btn_Exec.Click

        Dim n As Integer                '処理件数
        Dim l, h As Integer             '時刻調整値の下限，上限値
        Dim r As New Random

        Dim DateAdjustFlag As Boolean
        Dim TimeAdjustFlag As Boolean
        Dim DateAdjustValue As Integer
        Dim TimeAdjustValue As Integer

        Dim DateGetLength As Integer    'ListViewの日付取得用の桁数
        Dim TimeGetLength As Integer    'ListViewの時刻取得用の桁数

        Dim dt As Date                  '作業用の日時の値
        Dim dw As String
        Dim d1, d2 As Date              '変更後の日時 (作成日時，更新日時)
        Dim p1, p2 As String            '変更前の日時 (作成日時，更新日時)

        Dim Lsv As ListViewItem

        Me.Cursor = Cursors.WaitCursor

        Btn_Dummy.Enabled = True
        Btn_Exec_Control(False)                 'Btn_Dummy (NextControl) へ Focusを移す (フォーカスを消す)

        Btn_Undo.Enabled = False
'-------------------------------------------------------------------------------
'     Show ProgressBar (ListView.BeginUpdate の前に表示しないと再描画により
'                       ListView 内の表示がクリアされる為)
'-------------------------------------------------------------------------------
        ProgressBar.Value = 0
        ProgressBar.Visible = True
'-------------------------------------------------------------------------------
'     表示色クリア
'-------------------------------------------------------------------------------
        ListView.BeginUpdate()
        ListView.HideSelection = True

        Lsv_BackColor_Clear()
'-------------------------------------------------------------------------------
'     前処理(パスを昇順ソート)
'-------------------------------------------------------------------------------
        If _onShiftEnter Then
           ListView.MarkerSet(0, 0)
        End If
'-------------------------------------------------------------------------------
'     Job Start
'-------------------------------------------------------------------------------
        Using sw As New StreamWriter(_UndoFile, False, Encoding.UTF8)

          n = 0

       'Target Item Set
          If Not _onSelectItem Then
             For Each Lsv In ListView.Items
               Lsv.Selected = True
             Next
          End If

          Dim state As Boolean = (ListView.SelectedItems.Count <> 1)    'MsgBox 内での処理継続判定用
          Dim Result As Integer

       'ProgressBar Control value setting
          Dim pVal As Single = 0.0F
          Dim pAdd As Single = Convert.ToSingle(100 / ListView.SelectedItems.Count)

          Select Case _JobMode
            Case 0
              DateAdjustFlag = ((_List_FileMode = 0) AndAlso (_List_TreeMode = 0) AndAlso (Txt_Date_Ajst.Value <> 0))
              TimeAdjustFlag = ((_List_FileMode = 0) AndAlso (_List_TreeMode = 0) AndAlso (Txt_Time_Ajst.Value <> 0))

              DateAdjustValue = Txt_Date_Ajst.Value
              TimeAdjustValue = Txt_Time_Ajst.Value

              TimeGetLength = _DateTimeFormat.Length - 8
              DateGetLength = TimeGetLength

              If Chk_Date.Checked And Not Chk_Time.Checked Then

          '日付のみ
                 dt = DateTime.Parse(Pck_Date.Text.TrimStart)

                 For Each Lsv In ListView.SelectedItems
                   With Lsv
                     dw = dt.Date.ToString("yyyy/MM/dd ")
                     d1 = DateTime.Parse(dw & .SubItems(2).Text.Substring(TimeGetLength))
                     d2 = DateTime.Parse(dw & .SubItems(3).Text.Substring(TimeGetLength))
                     Do
                       Result = DateUpdate(.SubItems(0).Text, d1, d2, state)
                     Loop While Result = DialogResult.Retry

                     Select Case Result
                       Case DialogResult.None
                         p1 = .SubItems(2).Text
                         p2 = .SubItems(3).Text
                         If Chk_Make.Checked Then .SubItems(2).Text = d1.ToString(_DateTimeFormat)
                         If Chk_Repl.Checked Then .SubItems(3).Text = d2.ToString(_DateTimeFormat)
                         sw.WriteLine(.ImageIndex & vbTab & .SubItems(0).Text & vbTab & _
                                      p1 & vbTab & p2 & vbTab & .SubItems(2).Text & vbTab & .SubItems(3).Text)
                         If _List_ChckMode <> 0 Then
                            Lsv_BackColor_Set(Lsv, _BackColorSuccess)
                         Else
                            .Remove()
                         End If
                         n += 1

                       Case DialogResult.OK, DialogResult.Ignore
                         Lsv_BackColor_Set(Lsv, _BackColorFailure)

                       Case DialogResult.Cancel
                         Lsv_BackColor_Set(Lsv, _BackColorBreak)
                         Exit For

                       Case DialogResult.Abort
                         Exit For
                     End Select

                  '日付調整
                     If Result <= DialogResult.None Then
                        If DateAdjustFlag Then
                           If _Adjust_D_Mode = 0 Then
                              dt = dt.AddDays(DateAdjustValue)
                           Else
                              dt = dt.AddMonths(DateAdjustValue)
                           End If
                        End If
                     End If

                     pVal += pAdd
                     ProgressBar.Value = Convert.ToInt32(pVal)
                   End With
                 Next
              Else

          '時刻の調整値(下限，上限値)・設定
                 If TimeAdjustFlag Then
                    l = Txt_Time_Ajst.Value * (1 + (59 * _Adjust_T_Mode))

                    If Chk_Rand.Checked Then
                       h = Convert.ToInt32(Math.Round(l * _Adjust_MValue) + 1)  '最大値 (最近接偶数丸め)
                    Else
                       h = l + 1                                                '固定
                    End If
                    If Not _Adjust_LCheck Then l = 0
                 End If

                 If Chk_Time.Checked And Not Chk_Date.Checked Then

          '時刻のみ
                    dt = DateTime.Parse(Pck_Time.Text.TrimStart)

                    For Each Lsv In ListView.SelectedItems
                      With Lsv
                        dw = dt.TimeOfDay.ToString
                        d1 = DateTime.Parse(.SubItems(2).Text.Substring(0, DateGetLength) & dw)
                        d2 = DateTime.Parse(.SubItems(3).Text.Substring(0, DateGetLength) & dw)

                        Do
                          Result = DateUpdate(.SubItems(0).Text, d1, d2, state)
                        Loop While Result = DialogResult.Retry

                        Select Case Result
                          Case DialogResult.None
                            p1 = .SubItems(2).Text
                            p2 = .SubItems(3).Text
                            If Chk_Make.Checked Then .SubItems(2).Text = d1.ToString(_DateTimeFormat)
                            If Chk_Repl.Checked Then .SubItems(3).Text = d2.ToString(_DateTimeFormat)
                            sw.WriteLine(.ImageIndex & vbTab & .SubItems(0).Text & vbTab & _
                                         p1 & vbTab & p2 & vbTab & .SubItems(2).Text & vbTab & .SubItems(3).Text)
                            If _List_ChckMode <> 0 Then
                               Lsv_BackColor_Set(Lsv, _BackColorSuccess)
                            Else
                               .Remove()
                            End If
                            n += 1

                          Case DialogResult.OK, DialogResult.Ignore                                 'DialogResult.OK / Ignore
                            Lsv_BackColor_Set(Lsv, _BackColorFailure)

                          Case DialogResult.Cancel                                    'DialogResult.Cancel
                            Lsv_BackColor_Set(Lsv, _BackColorBreak)
                            Exit For

                          Case DialogResult.Abort
                            Exit For
                        End Select

                     '時刻調整
                        If Result <= DialogResult.None Then
                           If TimeAdjustFlag Then dt = dt.AddSeconds(r.Next(l, h))
                        End If

                        pVal += pAdd
                        ProgressBar.Value = Convert.ToInt32(pVal)
                      End With
                    Next
                 Else

          '全て
                    dt = DateTime.Parse(Pck_Date.Text.TrimStart & " " & Pck_Time.Text.TrimStart)
                    For Each Lsv In ListView.SelectedItems
                      With Lsv
                        Do
                          Result = DateUpdate(.SubItems(0).Text, dt, dt, state)
                        Loop While Result = DialogResult.Retry

                        Select Case Result
                          Case DialogResult.None
                            p1 = .SubItems(2).Text
                            p2 = .SubItems(3).Text
                            If Chk_Make.Checked Then .SubItems(2).Text = dt.ToString(_DateTimeFormat)
                            If Chk_Repl.Checked Then .SubItems(3).Text = dt.ToString(_DateTimeFormat)
                            sw.WriteLine(.ImageIndex & vbTab & .SubItems(0).Text & vbTab & _
                                         p1 & vbTab & p2 & vbTab & .SubItems(2).Text & vbTab & .SubItems(3).Text)
                            If _List_ChckMode <> 0 Then
                               Lsv_BackColor_Set(Lsv, _BackColorSuccess)
                            Else
                               .Remove()
                            End If
                            n += 1

                          Case DialogResult.OK, DialogResult.Ignore
                            Lsv_BackColor_Set(Lsv, _BackColorFailure)

                          Case DialogResult.Cancel
                            Lsv_BackColor_Set(Lsv, _BackColorBreak)
                            Exit For

                          Case DialogResult.Abort
                            Exit For
                        End Select

                     '日時調整
                        If Result <= DialogResult.None Then
                           If DateAdjustFlag Then
                              If _Adjust_D_Mode = 0 Then
                                 dt = dt.AddDays(DateAdjustValue)
                              Else
                                 dt = dt.AddMonths(DateAdjustValue)
                              End If
                           End If
                           If TimeAdjustFlag Then dt = dt.AddSeconds(r.Next(l, h))
                        End If

                        pVal += pAdd
                        ProgressBar.Value = Convert.ToInt32(pVal)
                      End With
                    Next
                 End If
              End If

         '作成日時⇒更新日時 (Chk_Make.Checked = False, Chk_Repl.Checked = True)
            Case 1
              For Each Lsv In ListView.SelectedItems
                With Lsv
                  dt = DateTime.Parse(.SubItems(2).Text)
                  Do
                    Result = DateUpdate(.SubItems(0).Text, dt, dt, state)
                  Loop While Result = DialogResult.Retry

                  Select Case Result
                    Case DialogResult.None
                      p1 = .SubItems(2).Text
                      p2 = .SubItems(3).Text
                      If _List_ChckMode <> 0 Then
                         .SubItems(3).Text = .SubItems(2).Text
                         Lsv_BackColor_Set(Lsv, _BackColorSuccess)
                      Else
                         .Remove()
                      End If
                      sw.WriteLine(.ImageIndex & vbTab & .SubItems(0).Text & vbTab & _
                                   p1 & vbTab & p2 & vbTab & p1 & vbTab & p1)
                      n += 1

                    Case DialogResult.OK, DialogResult.Ignore
                      Lsv_BackColor_Set(Lsv, _BackColorFailure)

                    Case DialogResult.Cancel
                      Lsv_BackColor_Set(Lsv, _BackColorBreak)
                      Exit For

                    Case DialogResult.Abort
                      Exit For
                  End Select

                  pVal += pAdd
                  ProgressBar.Value = Convert.ToInt32(pVal)
                End With
              Next

         '更新日時⇒作成日時 (Chk_Make.Checked = True, Chk_Repl.Checked = False)
            Case 2
              For Each Lsv In ListView.SelectedItems
                With Lsv
                  dt = DateTime.Parse(.SubItems(3).Text)
                  Do
                    Result = DateUpdate(.SubItems(0).Text, dt, dt, state)
                  Loop While Result = DialogResult.Retry

                  Select Case Result
                    Case DialogResult.None
                      p1 = .SubItems(2).Text
                      p2 = .SubItems(3).Text
                      If _List_ChckMode <> 0 Then
                         .SubItems(2).Text = .SubItems(3).Text
                         Lsv_BackColor_Set(Lsv, _BackColorSuccess)
                      Else
                         .Remove()
                      End If
                      sw.WriteLine(.ImageIndex & vbTab & .SubItems(0).Text & vbTab & _
                                   p1 & vbTab & p2 & vbTab & p2 & vbTab & p2)
                      n += 1

                    Case DialogResult.OK, DialogResult.Ignore
                      Lsv_BackColor_Set(Lsv, _BackColorFailure)

                    Case DialogResult.Cancel
                      Lsv_BackColor_Set(Lsv, _BackColorBreak)
                      Exit For

                    Case DialogResult.Abort
                      Exit For
                  End Select

                  pVal += pAdd
                  ProgressBar.Value = Convert.ToInt32(pVal)
                End With
              Next

         'ファイル名から日時取得 (ファイル名(数字のみ)の先頭が、[YY]YYMMDD[hhmmss] の場合)
            Case 3
              For Each Lsv In ListView.SelectedItems
                With Lsv
                  If DateTimePathCheck(.Text, dt) Then
                     Do
                       Result = DateUpdate(.SubItems(0).Text, dt, dt, state)
                     Loop While Result = DialogResult.Retry

                     Select Case Result
                       Case DialogResult.None
                         p1 = .SubItems(2).Text
                         p2 = .SubItems(3).Text

                         If Chk_Make.Checked Then .SubItems(2).Text = dt.ToString(_DateTimeFormat)
                         If Chk_Repl.Checked Then .SubItems(3).Text = dt.ToString(_DateTimeFormat)

                         sw.WriteLine(.ImageIndex & vbTab & .SubItems(0).Text & vbTab & _
                                      p1 & vbTab & p2 & vbTab & .SubItems(2).Text & vbTab & .SubItems(3).Text)

                         If _List_ChckMode <> 0 Then
                            Lsv_BackColor_Set(Lsv, _BackColorSuccess)
                         Else
                            .Remove()
                         End If
                         n += 1

                       Case DialogResult.OK, DialogResult.Ignore
                         Lsv_BackColor_Set(Lsv, _BackColorFailure)

                       Case DialogResult.Cancel
                         Lsv_BackColor_Set(Lsv, _BackColorBreak)
                         Exit For

                       Case DialogResult.Abort
                         Exit For
                     End Select

                     pVal += pAdd
                     ProgressBar.Value = Convert.ToInt32(pVal)
                  End If
                End With
              Next
          End Select

          Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        End Using

        If Not _onSelectItem Then
           ListView.SelectedItems.Clear()
        End If

        Lsv_DisplayMode_Control(False)

        ListView.EndUpdate()
'-------------------------------------------------------------------------------
'     Log Write
'-------------------------------------------------------------------------------
        If (n <> 0) AndAlso _LogFileWriting Then

           If File.Exists(_Log_File) Then
              If File.Exists(_TempFile) Then File.Delete(_TempFile)
              File.Move(_Log_File, _TempFile)
           Else
              Using sw As New StreamWriter(_TempFile, False, Encoding.UTF8) : End Using
           End If

           Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
           Using sr As New StreamReader(_UndoFile, Encoding.UTF8)

             Dim msg1 As String = ""
             Dim msg2 As String = ""

             Select Case _JobMode
               Case 0
                 If Chk_Make.Checked And Chk_Repl.Checked Then
                    msg1 = "Make Update] "
                 ElseIf Chk_Make.Checked Then
                    msg1 = "Make] "
                 Else
                    msg1 = "Update] "
                 End If

                 If Chk_Date.Checked And Chk_Time.Checked Then
                    msg1 = msg1 & "[Date Time"
                 ElseIf Chk_Date.Checked Then
                    msg1 = msg1 & "[Date"
                 Else
                    msg1 = msg1 & "[Time"
                 End If

               Case 1 : msg1 = "Make -> Update"
               Case 2 : msg1 = "Update -> Make"
               Case 3 : msg1 = "Pick Up"
             End Select

             If n > 1 Then
                msg2 = String.Format(_LblItemsFormat, n) & " --"
             Else
                msg2 = "--"
             End If
             sw.WriteLine("-- {0} [{1}] : Change {2}", Date.Now.ToString(_DateTimeFormat), msg1, msg2)

             Dim buf, itm() As String

             Do While Not sr.EndOfStream
               buf = sr.ReadLine
               itm = buf.Split(CChar(vbTab))
               sw.WriteLine("{0}  {1} --> {2}  {3} [{4}] {5}", itm(2), itm(3), itm(4), itm(5), itm(0), itm(1))
             Loop
             sw.WriteLine()
           End Using

           Dim fi As New FileInfo(_TempFile)
           If fi.Length > 0 Then
              Using sr As New StreamReader(_TempFile, Encoding.UTF8)
                sw.Write(sr.ReadToEnd())
              End Using
           End If
           End Using

           File.Delete(_TempFile)
        End If
'-------------------------------------------------------------------------------
'     Message Disp. 
'-------------------------------------------------------------------------------
        Me.Update()
        Me.Cursor = Cursors.Default

        If n <> 0 Then
           ProgressBar.Value = 100

           If (Not Lbl_CheckLamp.Visible) OrElse (Not Lbl_CheckLamp.Enabled) Then
              WinMessage.Clear(Me, 200)
           Else
              Dim msg As String = String.Format("{0:#,##0} 個のタイムスタンプを変更しました。", n)
              ShowInfoMessage(msg, _App_Name)
           End If

           Btn_Undo.Enabled = True
        Else
           Beep()
        End If

        ProgressBar.Visible = False
        Btn_Exec_Control()

        Btn_Dummy.Enabled = False
        Btn_Erase.Enabled = (ListView.Items.Count <> 0)

        ListView.HideSelection = _UseHideSelection
        ListView.Select()

        _Lsv_DupCheckFlg = (ListView.Items.Count <> 0)
    End Sub

    Private Sub Btn_Undo_Click(sender As Object, e As EventArgs) Handles Btn_Undo.Click

        Btn_Dummy.Enabled = True
        Btn_Dummy.Select()          'Btn_Dummy へ Focusを移す (フォーカスを消す)
        Btn_Exec.Enabled = False
        Btn_Undo.Enabled = False

        Dim fi As New FileInfo(_UndoFile)

        If Not (fi.Exists) OrElse (fi.Length = 0) Then

           ShowWarningMessage("アンドゥファイルがありません。", "", _Msg_Cap)
        Else
           Btn_Undo_Execute()
        End If

        Btn_Exec_Control()

        Btn_Dummy.Enabled = False
        Btn_Erase.Enabled = (ListView.Items.Count <> 0)

        ListView.Select()

        _Lsv_DupCheckFlg = (ListView.Items.Count <> 0)
    End Sub

    Private Sub Btn_Undo_Execute()

        Dim n As Integer
        Dim c As Integer = ListView.Items.Count
        Dim Result As Integer
        Dim msg As String
        Dim pAdd, pVal As Single
        Dim Lsv As ListViewItem
        Dim buf, itm() As String

        Me.Cursor = Cursors.WaitCursor

'-------------------------------------------------------------------------------
'     処理レコード数取得
'-------------------------------------------------------------------------------
        Dim i As Integer = 0
        Using sr As New StreamReader(_UndoFile, Encoding.UTF8)
          Do While Not sr.EndOfStream
            buf = sr.ReadLine
            i += 1
          Loop
        End Using
'-------------------------------------------------------------------------------
'     ProgressBar init.
'-------------------------------------------------------------------------------
        ProgressBar.Value = 0
        ProgressBar.Visible = True

        pVal = 0.0F
        pAdd = Convert.ToSingle(100 / i)
'-------------------------------------------------------------------------------
'     表示色クリア
'-------------------------------------------------------------------------------
        ListView.BeginUpdate()
        ListView.HideSelection = True

        Lsv_BackColor_Clear()
'-------------------------------------------------------------------------------
'     Undo Start
'-------------------------------------------------------------------------------
        Using sr As New StreamReader(_UndoFile, Encoding.UTF8)
        Using sw As New StreamWriter(_WorkFile, False, Encoding.UTF8)

          Do While Not sr.EndOfStream
            buf = sr.ReadLine
            itm = buf.Split(CChar(vbTab))

            If _Lsv_DupCheckFlg Then
               Try
                 Lsv = ListView.FindItemWithText(itm(1), False, 0, False)
               Catch ex As ArgumentOutOfRangeException
                 Lsv = Nothing
               End Try
            Else
               Lsv = Nothing
            End If

            Do
              Result = DateUpdate(itm(1), DateTime.Parse(itm(2)), DateTime.Parse(itm(3)), True)
            Loop While Result = DialogResult.Retry

            If Result = DialogResult.Abort Then Exit Do

            If _List_ChckMode <> 0 Then

            '結果を問わずすべて表示
               Lsv_Set(Result, Lsv, itm)

               With Lsv
                 Select Case Result
                   Case 0
                     n += 1
                     Lsv_BackColor_Set(Lsv, _BackColorUndo)

                  'Logファイル書込み用の為に、Undoに成功したデータを一時保存
                     If _LogFileWriting Then sw.WriteLine(buf)

                   Case DialogResult.OK, DialogResult.Ignore
                     Lsv_BackColor_Set(Lsv, _BackColorFailure)

                   Case DialogResult.Cancel
                     Lsv_BackColor_Set(Lsv, _BackColorBreak)
                     Exit Do
                 End Select
               End With
            Else

            'Undoに失敗した場合のみ表示
               If Result <> DialogResult.None Then
                  Lsv_Set(Result, Lsv, itm)

                  With Lsv
                    If Result = DialogResult.Cancel Then
                       Lsv_BackColor_Set(Lsv, _BackColorBreak)
                       Exit Do
                    Else
                       Lsv_BackColor_Set(Lsv, _BackColorFailure)
                    End If
                  End With
               Else
                  n += 1
                  If Not IsNothing(Lsv) Then Lsv.Remove()

               'Logファイル書込み用の為に、Undoに成功したデータを一時保存
                  If _LogFileWriting Then sw.WriteLine(buf)

               End If
            End If

            pVal += pAdd
            ProgressBar.Value = Convert.ToInt32(pVal)
          Loop

          Lbl_Items.Text = String.Format(_LblItemsFormat, ListView.Items.Count)

        End Using
        End Using

     'マーカークリア
        If ListView.Items.Count <> c AndAlso
           ListView.MarkerIndex <> -1 Then ListView.MarkerClear(True)

        ListView.SelectedItems.Clear()

        Lsv_DisplayMode_Control(False)

        ListView.EndUpdate()
'-------------------------------------------------------------------------------
'     Log Write
'-------------------------------------------------------------------------------
        If (n <> 0) AndAlso _LogFileWriting Then

           If File.Exists(_Log_File) Then
              If File.Exists(_TempFile) Then File.Delete(_TempFile)
              File.Move(_Log_File, _TempFile)
           Else
              Using sw As New StreamWriter(_TempFile, False, Encoding.UTF8) : End Using
           End If

           Using sw As New StreamWriter(_Log_File, False, Encoding.UTF8)
           Using sr As New StreamReader(_WorkFile, Encoding.UTF8)

             If n > 1 Then
                msg = String.Format(_LblItemsFormat, n) & " --"
             Else
                msg = "--"
             End If
             sw.WriteLine("-- {0} : Undo {1}", Date.Now.ToString(_DateTimeFormat), msg)

             Do While Not sr.EndOfStream
               buf = sr.ReadLine
               itm = buf.Split(CChar(vbTab))
               sw.WriteLine("{0}  {1} --> {2}  {3} [{4}] {5}", itm(4), itm(5), itm(2), itm(3), itm(0), itm(1))
             Loop
             sw.WriteLine()
           End Using

           Using sr As New StreamReader(_TempFile, Encoding.UTF8)
             sw.Write(sr.ReadToEnd())
           End Using
           End Using

           File.Delete(_TempFile)
        End If

        File.Delete(_WorkFile)
'-------------------------------------------------------------------------------
'     Message Disp. 
'-------------------------------------------------------------------------------
        Me.Update()
        Me.Cursor = Cursors.Default

        If n <> 0 Then
           ProgressBar.Value = 100

           msg = String.Format("{0:#,##0} 個のファイルを復元しました。", n)
           ShowInfoMessage(msg, _App_Name)
        Else
           ShowWarningMessage("ファイルの復元に失敗しました。", "", _App_Name)
        End If

        ProgressBar.Visible = False

        ListView.HideSelection = _UseHideSelection
        ListView.Select()
    End Sub

#End Region

End Class
