﻿'
'   Corner trimming of Groupbox 1.0.5.0 (GDI+ Label Custum Version)
'
'       Property
'         BorderEraseWidth
'         LabelRectangle
'         HoverColor
'         HoverEffect
'
'       Method
'         None
'
Imports System.ComponentModel
Imports System.Text

Namespace UserLib

<DebuggerStepThrough()>
Public Class GroupBoxEx
    Inherits GroupBox

    Public Event LabelMouseDown(sender As Object, e As MouseEventArgs)

    Public WithEvents Label As Label

    Private Const _LabelPadding = 2

    Private _onCreate As Boolean
    Private _onMouse As Boolean

    Private _pn_Dark As New Pen(SystemColors.ControlDark)
    Private _pn_Light As New Pen(SystemColors.ControlLightLight)
    Private _pt_DarkLine As Point()
    Private _pt_LightLine As Point()

    Private _DrawText As String

#Region "// Property, Method"

    Private _BorderEraseWidth As Integer
    Private _LabelRectangle As Rectangle
    Private _HoverColor As Color = SystemColors.Highlight
    Private _HoverEffect As Boolean

#Region "** 表示"

    ''' <summary>
    ''' ラベル位置の枠線を消すサイズを指定します。
    ''' </summary>

    <DefaultValue(0), Category("表示"),
     Description("ラベル位置の枠線を消すサイズを指定します。")> _
    Public Property BorderEraseWidth As Integer
        Get
          Return _BorderEraseWidth
        End Get

        Set(value As Integer)
          If value < 0 Then
             value = 0
          End If

          _BorderEraseWidth = value

          If _onCreate Then
             SetDrawPoint()

             Me.Invalidate()
          End If
        End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor() As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。")> _
    Public Property HoverEffect() As Boolean
      Get
        Return _HoverEffect
      End Get

      Set(ByVal value As Boolean)
        _HoverEffect = value

        If Not _onCreate Then Return

        If Me.Enabled Then
           If value Then
              _onMouse = _LabelRectangle.Contains(Me.PointToClient(Cursor.Position))

              If _onMouse Then
                 Me.Label.ForeColor = _HoverColor

              'MouseEnter を発生
                 Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
              End If
           Else
              If _onMouse Then
                 _onMouse = False

                 Me.Label.ForeColor = Me.ForeColor
              End If
           End If
        Else
           _onMouse = False

           Me.Label.ForeColor = Me.ForeColor
        End If
      End Set
    End Property

#End Region

#Region "** その他"

''' <summary>
''' ラベルの位置、サイズを取得します。
''' </summary>

    <Browsable(False)>
    Public ReadOnly Property LabelRectangle() As Rectangle
      Get
        Return _LabelRectangle
      End Get
    End Property

#End Region

#End Region

#Region "// Function, Sub"

''' <summary>
''' 現在ラベルで使用されているフォントの高さを取得します。
''' </summary>
''' 
    <DebuggerStepThrough()>
    Private Function GetFontHeight() As Integer

        Dim w As Integer = Me.Font.Height
        Dim h As Integer = Me.Font.Height

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)
        Dim c As Color = b.GetPixel(0, 0)

        Dim sf As New StringFormat

        g.DrawString("▏", Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

        Dim BackColorArgb As Integer = c.ToArgb()

        Dim y0 As Integer = -1
        Dim y1 As Integer = -1

        For yp As Integer = 0 To h - 1
          For xp As Integer = 0 To w - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               y0 = yp
               Exit For
            End If
          Next
          If y0 >= 0 Then Exit For
        Next

        For yp As Integer = h - 1 To y0 Step -1
          For xp As Integer = 0 To w - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               y1 = yp
               Exit For
            End If
          Next
          If y1 >= 0 Then Exit For
        Next

        sf.Dispose()
        g.Dispose()
        b.Dispose()

        sf = Nothing
        g = Nothing
        b = Nothing

        Return (y1 - y0) + 1
    End Function

''' <summary>
''' ラベルの上余白の値を取得します。
''' </summary>
''' 
    <DebuggerStepThrough()>
    Private Function GetTopMargin() As Integer

        Dim w As Integer = Me.Label.Width
        Dim h As Integer = Me.Label.Height + 4

        If w > 0 Then

           Dim b As New Bitmap(w, h)
           Dim g As Graphics = Graphics.FromImage(b)
           Dim c As Color = b.GetPixel(0, 0)

           Dim sf As New StringFormat

           g.DrawString(Me.Text, Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

           Dim BackColorArgb As Integer = c.ToArgb()

           Dim y As Integer = -1

           For yp As Integer = 0 To h - 1
             For xp As Integer = 0 To w - 1
               If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                  y = yp
                  Exit For
               End If
             Next
             If y >= 0 Then Exit For
           Next

           sf.Dispose()
           g.Dispose()
           b.Dispose()

           sf = Nothing
           g = Nothing
           b = Nothing

           Dim enc As Encoding = Encoding.GetEncoding("EUC-JP")

           If Me.Text.Length = enc.GetByteCount(Me.Text) Then
              y = -(1 - y)  '上余白 (1 Pxcel 固定) 
           End If

           If y >= 0 Then Return y
        End If

        Return 0
    End Function

''' <summary>
''' ラベルの右端 または 左端の余白の値を取得します。
''' </summary>
''' 
    <DebuggerStepThrough()>
    Private Function GetSideMargin() As Integer

        Dim w As Integer = Me.Label.Width
        Dim h As Integer = Me.Label.Height
        Dim x As Integer = 0

        If w > 0 Then

           Dim b As New Bitmap(w, h)
           Dim g As Graphics = Graphics.FromImage(b)
           Dim c As Color = b.GetPixel(0, 0)

           Dim sf As New StringFormat

           If Me.RightToLeft = RightToLeft.Yes Then
              sf.FormatFlags = StringFormatFlags.DirectionRightToLeft
           End If

           g.DrawString(Me.Text, Me.Font, Brushes.White, New RectangleF(x, 0, w, h), sf)

           Dim BackColorArgb As Integer = c.ToArgb()

           x = -1

           If Me.RightToLeft = RightToLeft.Yes Then
              For xp As Integer = 0 To w - 1
                For yp As Integer = 0 To h - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     x = xp
                     Exit For
                  End If
                Next
                If x >= 0 Then Exit For
              Next
           Else
              For xp As Integer = w - 1 To 0 Step -1
                For yp As Integer = 0 To h - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     x = w - xp
                     Exit For
                  End If
                Next
                If x >= 0 Then Exit For
              Next
           End If

           sf.Dispose()
           g.Dispose()
           b.Dispose()

           sf = Nothing
           g = Nothing
           b = Nothing

           Return x
        End If

        Return 0
    End Function

''' <summary>
''' テキスト内の [CR], [LF] を [CRLF] へ置換します。また、テキスト末尾の 空白，[CRLF] を削除します。
''' </summary>

    Private Sub SetTrimText()

        _DrawText = Me.Text.TrimEnd
        _DrawText = _DrawText.Replace(vbCrLf, vbLf).TrimEnd
        _DrawText = _DrawText.Replace(vbCr, vbLf)
        _DrawText = _DrawText.Replace(vbLf, vbCrLf)

        If _DrawText.EndsWith(vbCrLf) Then _DrawText = _DrawText.Substring(0, _DrawText.Length - 2)

        Me.Label.Text = _DrawText
    End Sub

''' <summary>
''' コントロールの Region を設定します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetRegion()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(2, 0, w - 4, 1))
        path.AddRectangle(New Rectangle(1, 1, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 2, w, h - 4))
        path.AddRectangle(New Rectangle(1, h - 2, w - 2, 1))
        path.AddRectangle(New Rectangle(2, h - 1, w - 4, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

''' <summary>
''' ラベル、外枠の設定を行います。
''' </summary>

    Private Sub SetDrawPoint()

        Const _LabelOffset As Integer = 8                   '枠からラベル迄の距離(X)
        Const _ExceptWidth As Integer = _LabelOffset * 2    '枠のラベルに対するパディングの合計

        Dim x, y As Integer
        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim dw As Integer = w - _ExceptWidth                'ラベルの描画可能な幅

        Dim x0, x1 As Integer

        Dim RtoL As Boolean = (Me.Label.RightToLeft = RightToLeft.Yes)

        If w < (_ExceptWidth + 4) OrElse h < (Me.Font.Height + 2) Then

        '枠線のみ
           Me.Label.Visible = False
           _LabelRectangle = Nothing

           x0 = 2
           x1 = x0

        '枠 (Top) の位置
           If _DrawText.Length > 0 Then
              y = (GetFontHeight() \ 2) - 1
           End If
        Else

        'ラベルあり
           If _DrawText.Length > 0 Then

              If RtoL Then
                 If (Me.Font.Style And 2) <> 0 Then
                 'Italic
                    Me.Label.Padding = New Padding(_LabelPadding, 0, _LabelPadding, 0)
                 Else
                    Me.Label.Padding = New Padding(_LabelPadding, 0, 1, 0)
                 End If
              Else
                 If (Me.Font.Style And 2) <> 0 Then
                 'Italic
                    Me.Label.Padding = New Padding(0, 0, _LabelPadding, 0)
                 Else
                    Me.Label.Padding = New Padding(0)
                 End If
              End If

              Me.Label.Visible = True
              Me.Label.MaximumSize = New Size(dw, Me.Height - 3)
              Me.Label.AutoSize = True

           '枠線座標(開始，終了)、ラベル位置設定
              Dim SmCut As Integer = GetSideMargin()

              If SmCut > 4 Then     'ラベルと枠線の余白の補正値
                 SmCut -= 4
              Else
                 SmCut = 0
              End If

              Dim xl As Integer     'ラベルの表示位置
              Dim xr As Integer     'ラベルの表示（有効部分）位置
              Dim ps As Size = Me.Label.PreferredSize

              If RtoL Then
                 xl = w - (ps.Width + _LabelOffset)
                 x0 = xl + SmCut - 1
                 x1 = w - _LabelOffset
                 xr = x0
              Else
                 xl = _LabelOffset
                 x0 = _LabelOffset - 1
                 x1 = _LabelOffset + ps.Width - SmCut
                 xr = _LabelOffset
              End If

          ' ラベルの表示情報セット

              Dim tm As Integer = GetTopMargin()
              Dim tl As Integer = ps.Height \ Me.Label.Font.Height
              Dim th As Integer = tl * Me.Label.Font.Height

              Dim path As New Drawing2D.GraphicsPath()

              If RtoL Then x = SmCut

           'ラベルで消された余白部分の枠線を表示（ラベルの Region 変更）
              path.AddRectangle(New Rectangle(x, 0, ps.Width - SmCut, th))

              Me.Label.Region = New Region(path)

              path.Dispose()
              path = Nothing

              Me.Label.AutoSize = False
              Me.Label.SetBounds(xl, -tm, ps.Width, th)

              _LabelRectangle = New Rectangle(xr, -tm, ps.Width - SmCut, th)

           'ラベル位置の枠線を消去
              If _BorderEraseWidth > 0 Then
                 If RtoL Then
                    If dw <= _BorderEraseWidth Then
                       x0 = x1 - dw - 1
                       'xl = x1 - dw - 1
                    Else
                       x0 = x1 - _BorderEraseWidth - 1
                       'xl = x1 - _BorderEraseWidth - 1
                    End If
                 Else
                    If dw <= _BorderEraseWidth Then
                       x1 = x0 + dw + 1
                    Else
                       x1 = x0 + _BorderEraseWidth + 1
                    End If
                 End If
              End If

           '枠 (Top) の位置
              If _DrawText.Length <> 0 Then
                 y = (GetFontHeight() \ 2) - 1
              End If

           Else

           'ラベルなし
              Me.Label.Visible = False
              _LabelRectangle = Nothing

              If _BorderEraseWidth = 0 Then
              '枠線消去なし
                 x0 = 8
                 x1 = x0
              Else
              '枠線消去位置設定
                 If RtoL Then
                    x1 = w - _LabelOffset

                    If dw <= _BorderEraseWidth Then
                       x0 = _LabelOffset - 1
                    Else
                       x0 = x1 - _BorderEraseWidth - 1
                    End If
                 Else
                    x0 = _LabelOffset - 1

                    If dw <= _BorderEraseWidth Then
                       x1 = w - _LabelOffset
                    Else
                       x1 = _LabelOffset + _BorderEraseWidth
                    End If
                 End If
              End If
           End If
        End If

    ' 枠線 (Dark)
        _pt_DarkLine = {
            New Point(x0, y), New Point(2, y), New Point(0, y + 2), New Point(0, h - 4),
            New Point(2, h - 2), New Point(w - 3, h - 2), New Point(w - 1, h - 4), New Point(w - 1, y + 2),
            New Point(w - 3, y), New Point(x1, y)}

    ' 枠線 (Light)
        y += 1
        _pt_LightLine = {
            New Point(x0, y), New Point(2, y), New Point(1, y + 1), New Point(1, h - 4),
            New Point(0, h - 3), New Point(2, h - 1), New Point(w - 3, h - 1), New Point(w - 1, h - 3),
            New Point(w - 2, h - 4), New Point(w - 2, y + 1), New Point(w - 3, y), New Point(x1, y)}
    End Sub

#End Region

    Public Sub New()

        Me.Label = New Label()
        Me.SuspendLayout()

        Me.Label.UseCompatibleTextRendering = True
        Me.Controls.Add(Label)

        Me.ResumeLayout(False)
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        SetTrimText()
        SetRegion()
        SetDrawPoint()

        AddHandler Me.ControlAdded, AddressOf Me_ControlAdded
        AddHandler Me.RightToLeftChanged, AddressOf Me_RightToLeftChanged

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler Me.FontChanged, AddressOf Me_FontChanged
        AddHandler Me.TextChanged, AddressOf Me_TextChanged

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        e.Graphics.DrawLines(_pn_Dark, _pt_DarkLine)
        e.Graphics.DrawLines(_pn_Light, _pt_LightLine)
    End Sub

#Region "// Me"

    Private Sub Me_ControlAdded(ByVal sender As Object, ByVal e As ControlEventArgs)
        Me.Label.SendToBack()
    End Sub

    Private Sub Me_RightToLeftChanged(ByVal sender As Object, ByVal e As EventArgs)

  ' RightToLeft Change
  '  ⇒ OnCreateControl()
  '  ⇒ RightToLeftChanged()

        SetDrawPoint()
    End Sub

    Private Sub Me_FontChanged(sender As Object, e As EventArgs)
        SetDrawPoint()
    End Sub

    Private Sub Me_TextChanged(ByVal sender As Object, ByVal e As EventArgs)

        SetTrimText()
        SetDrawPoint()
    End Sub

    Private Sub Me_Resize(ByVal sender As Object, ByVal e As EventArgs)

        If Me.FindForm.WindowState = FormWindowState.Minimized Then
           Return
        Else
           SetRegion()
           SetDrawPoint()
        End If
    End Sub

#End Region

#Region "// Label"

    Private Sub Label_EnabledChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Label.EnabledChanged
        If Not _HoverEffect Then Return

        If Me.Enabled Then
           _onMouse = _LabelRectangle.Contains(Me.PointToClient(Cursor.Position))

           If _onMouse Then
              Me.Label.ForeColor = _HoverColor

           'MouseEnter を発生
              Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
           End If
        Else
           _onMouse = False

           Me.Label.ForeColor = Nothing
        End If
    End Sub

    Private Sub Label_MouseEnter(ByVal sender As Object, ByVal e As EventArgs) Handles Label.MouseEnter
        If Not _HoverEffect Then Return

        _onMouse = True

        Me.Label.ForeColor = _HoverColor
    End Sub

    Private Sub Label_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Label.MouseLeave
        If Not _HoverEffect Then Return

        _onMouse = False

        Me.Label.ForeColor = Nothing
    End Sub

' コントロール配置元にイベント通知

    Private Sub Label_MouseDown(ByVal sender As Object, ByVal e As MouseEventArgs) Handles Label.MouseDown
        Me.OnMouseDown(e)

        If _HoverEffect Then
           RaiseEvent LabelMouseDown(sender, e)
        End If
    End Sub

    Private Sub Label_Click(ByVal sender As Object, ByVal e As EventArgs) Handles Label.Click
        Me.OnClick(e)
    End Sub

    Private Sub Label_MouseClick(sender As Object, e As MouseEventArgs) Handles Label.MouseClick
        Me.OnMouseClick(e)
    End Sub

    Private Sub Label_DoubleClick(ByVal sender As Object, ByVal e As EventArgs) Handles Label.DoubleClick
        Me.OnDoubleClick(e)
    End Sub

    Private Sub Label_MouseDoubleClick(ByVal sender As Object, ByVal e As MouseEventArgs) Handles Label.MouseDoubleClick
        Me.OnMouseDoubleClick(e)
    End Sub

#End Region

End Class

End Namespace
