﻿'
'   CheckBoxML (Simple Check Box) 1.0.2.0 limitation センタリング仕様
'
'       Required Class: 
'         None
'
'       Property:
'         AutoScale
'         DoubleBuffering
'         Text
'
'         DisabledColorState
'         MarkType
'         ShowFocusRect
'
'         CheckMarkColor    マークの表示色
'         UnCheckedColor    ラベルの表示色
'         DisabledColor
'         HoverColor
'         HoverEffect
'
'       Method
'         SetDisabledColorState
'         SetDisabledColor

Imports System.ComponentModel
Imports System.Drawing.Text
Imports System.Drawing.Design
Imports System.Text

Namespace UserLib

<DebuggerStepThrough()>
Public Class CheckBoxML
    Inherits UserControl

    Public Event CheckedChanged(sender As Object, e As EventArgs)
    Public Event CheckStateChanged(sender As Object, e As EventArgs)

    Public Shadows Event TextChanged(sender As Object, e As EventArgs)

Public Enum Alignment As Integer
    Left = 16
    Center = 32
    Right = 64
End Enum

Public Enum Mark As Integer
    Original = 0
    System = 1
End Enum

#Region "// constant, variable declaration"

    Private Const _Base_MarkBoxSize As Integer = 11  'チェックボックスの基準枠サイズ (FlatStyle: Flat, Popup)
    Private Const _BaseFocusBoxSize As Integer = 14  'チェックボックスのFocus枠サイズ (FlatStyle: Standard)

    Private Const _Mark_Text_Margin As Integer = 3
    Private Const _FocusText_Margin As Integer = 3

    Private _onCreate As Boolean
    Private _onResize As Boolean        '内部で ReSize する場合: False
    Private _onMouse As Boolean
    Private _onFocus As Boolean
    Private _onSpaceKey As Boolean      'スペース(空白) キー
    Private _onLeftButton As Boolean    'マウスの左ボタン

    Private _ResizeHandler As Boolean   'Add(Remove)Handler の代替フラグ

    Private _FontSizeMpf As Single
    Private _FontSizeInPointsMpf As Single
    Private _FontPixelSize As Single

    Private _PushSize As Size           'AutoSize: False 時のサイズ復元用
    Private _PushAutoSize As Size       'AutoSize:  True 時のサイズ復元用
    Private _BaseAutoSize As Size       'Padding を含まない AutoSize値

    Private _AutoVerticalAdjust As Integer

    Private _Draw_MarkBoxSize As Integer
    Private _DrawFocusBoxSize As Integer

    Private _MarkText As String
    Private _MarkFont As Font
    Private _MarkFontName As String
    Private _MarkFontSize As Single
    Private _MarkFontStyle As FontStyle
    Private _MarkFontState As Boolean       'True: Meiryo UI

    Private _DrawText As String = ""
    Private _DrawTextRectangle As Rectangle
    Private _DrawTextRectangleF As RectangleF

#End Region

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AutoValidate As AutoValidate
    <Browsable(False)> _
    Public Shadows Property ImeMode As ImeMode

    <Browsable(False)> _
    Public Shadows Property AutoScroll As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize As Size
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode As AutoSizeMode
    <Browsable(False)> _
    Public Shadows Property BorderStyle As BorderStyle

#End Region

    Private _Text As String = ""
    Private _TextValue As String = ""

    Private _DoubleBuffering As Boolean
    Private _ThreeState As Boolean

    Private _AutoSize As Boolean = True
    Private _AutoScale As Boolean
    Private _MinimumSize As Size
    Private _MaximumSize As Size
    Private _Padding As Padding

    Private _Checked As Boolean
    Private _CheckState As CheckState = CheckState.Unchecked
    Private _FlatStyle As FlatStyle = FlatStyle.Standard
    Private _MarkType As Mark = Mark.Original

    Private _CheckAlign As Alignment = Alignment.Left
    Private _TextAlign As Alignment = Alignment.Left

    Private _HoverEffect As Boolean
    Private _ShowFocusRect As Boolean
    Private _DisabledColorState As Boolean
    Private _CheckMarkColorState As Boolean
    Private _UnCheckedColorState As Boolean

    Private _CheckMarkColor As Color = Me.ForeColor
    Private _UnCheckedColor As Color = Me.ForeColor
    Private _DisabledColor As Color = SystemColors.GrayText
    Private _HoverColor As Color = SystemColors.Highlight

#Region "** 制御用"

''' <summary>
''' デザイナー で設定した Text プロパティの値をデザイナーに渡す為のプロパティです。
''' </summary>

    <Browsable(False), DefaultValue("")> _
    Public Property TextValue As String
       Get
         Return _Text
       End Get

      Set(value As String)
        Me.Text = value
      End Set
    End Property

#End Region

#Region "** 動作"

''' <summary>
''' ダブルバッファリングの状態を取得または設定します。
''' </summary>

    <DefaultValue(False), Browsable(True), Category("動作"),
     Description("ダブルバッファリングの有効／無効を設定します。")> _
    Public Property DoubleBuffering As Boolean
      Get
        Return _DoubleBuffering
      End Get

      Set(ByVal value As Boolean)
        _DoubleBuffering = value

        Me.DoubleBuffered = value
      End Set
    End Property

''' <summary>
''' チェックボックスで3つのチェック状態が表示できるかどうかを示す値を取得または設定します。
''' </summary>

    <Category("動作"), DefaultValue(False),
     Description("チェックボックスで3つのチェックされた状態を許可するかどうかを示します。")> _
    Public Property ThreeState As Boolean
      Get
        Return _ThreeState
      End Get

      Set(value As Boolean)
        _ThreeState = value
      End Set
    End Property

#End Region

#Region "** 配置"

''' <summary>
''' フォント サイズを基にして、サイズ（幅）の自動調整を行います。
''' </summary>

    <DefaultValue(True), Category("配置"),
     Description("コントロールが、その内容に合わせて自動的にサイズを変更するかどうかを指定します。")> _
    Public Overrides Property AutoSize As Boolean

   'デザイナで配置時、値を True にする為定義
      Get
        Return _AutoSize
      End Get

      Set(value As Boolean)
        _AutoSize = value

        If Not _onCreate Then Return

        If _AutoSize Then
           SetAutoSize()
             If Not _onResize Then
                _onResize = True

                SetTextRectangle()
             End If
        Else
           _onResize = False
             Me.Size = SizeCheck(_PushSize)
           _onResize = True
        End If

     '親フォームに通知
        MyBase.OnAutoSizeChanged(New EventArgs)
      End Set
    End Property

''' <summary>
''' AutoScaleの有効／無効を変更します。規定値は True です。
''' 無効時は、画面の解像度(Dpi)が変わった場合でもチェックのサイズは同じです。
''' </summary>

    <DefaultValue(True), Category("配置"),
     Description("AutoScaleの有効／無効を指定します。")> _
    Public Property AutoScale As Boolean
      Get
        Return _AutoScale
      End Get

      Set(ByVal value As Boolean)
        _AutoScale = value

        If _onCreate Then
           SetMarkInfo()
           AutoSize_Control()
        End If
      End Set
    End Property

''' <summary>
''' コントロールが指定できる下限のサイズを取得または設定します。
''' </summary>

    <DefaultValue(GetType(Size), "0, 0"), Category("配置"), _
     RefreshProperties(RefreshProperties.Repaint)>
    Public Shadows Property MinimumSize As Size
      Get
        Return _MinimumSize
      End Get

      Set(value As Size)
        If value.Width < 0 Then value.Width = 0
        If value.Height < 0 Then value.Height = 0

        If value.IsEmpty Then
           _MinimumSize = value
        Else
           Dim w As Integer = value.Width
           Dim h As Integer = value.Height

           Dim mw As Integer = _MaximumSize.Width
           Dim mh As Integer = _MaximumSize.Width

           If (w <> 0) AndAlso (mw <> 0) AndAlso (w > mw) Then w = mw
           If (h <> 0) AndAlso (mh <> 0) AndAlso (h > mh) Then h = mh

           _MinimumSize = New Size(w, h)
        End If

        If Not _onCreate Then Return

        _onResize = False
          If _AutoSize Then
             Me.Size = SizeCheck(_PushAutoSize)
          Else
             Me.Size = SizeCheck(_PushSize)
          End If
        _onResize = True
      End Set
    End Property

''' <summary>
''' コントロールが指定できる上限のサイズを取得または設定します。
''' </summary>

    <DefaultValue(GetType(Size), "0, 0"), Category("配置"), _
     RefreshProperties(RefreshProperties.Repaint)>
    Public Shadows Property MaximumSize As Size
      Get
        Return _MaximumSize
      End Get

      Set(value As Size)
        If value.Width < 0 Then value.Width = 0
        If value.Height < 0 Then value.Height = 0

        If value.IsEmpty Then
           _MaximumSize = value
        Else
           Dim w As Integer = value.Width
           Dim h As Integer = value.Height

           If (w <> 0) AndAlso (w < _MinimumSize.Width) Then w = _MinimumSize.Width
           If (h <> 0) AndAlso (h < _MinimumSize.Height) Then h = _MinimumSize.Height

           _MaximumSize = New Size(w, h)
        End If

        If Not _onCreate Then Return

        _onResize = False
          If _AutoSize Then
             AutoSize_Control()
          Else
             Me.Size = SizeCheck(_PushSize)
          End If
        _onResize = True
      End Set
    End Property

''' <summary>
''' コントロールの内部スペースを指定します。
''' </summary>

    <DefaultValue(GetType(Padding), "0,0,0,0"), Category("配置"),
     Description("コントロールの内部スペースを指定します。")> _
    Public Shadows Property Padding As Padding
      Get
        Return _Padding
      End Get

      Set(ByVal value As Padding)
        _Padding = value

        If _onCreate Then
           AutoSize_Control()

           MyBase.OnPaddingChanged(New EventArgs)
        End If
      End Set
    End Property

#End Region

#Region "** 表示"

#Region " （基本）"

''' <summary>
''' コントロールのチェック マークの配置を取得または設定します。
''' </summary>

    <Category("表示"), DefaultValue(Alignment.Left),
     Description("コントロールの中のチェックボックスの場所を決定します。")> _
    Public Property CheckAlign As Alignment
      Get
        Return _CheckAlign
      End Get

      Set(value As Alignment)
        _CheckAlign = value

        If _onCreate Then AutoSize_Control()
      End Set
    End Property

''' <summary>
''' チェックボックス がチェックされた状態であるかどうかをを示す値を取得または設定します。
''' </summary>

    <Category("表示"), DefaultValue(False),
     Description("コンポーネントがチェックされた状態かどうかを示します。")> _
    Public Property Checked As Boolean
      Get
        Return _Checked
      End Get

      Set(value As Boolean)
        _Checked = value

        If value Then
           _CheckState = CheckState.Checked
        Else
           _CheckState = CheckState.Unchecked
        End If

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' チェックボックス の状態を取得または設定します。
''' </summary>

    <Category("表示"), DefaultValue(CheckState.Unchecked),
     Description("コンポーネントの状態を示します。")> _
    Public Property CheckState As CheckState
      Get
        Return _CheckState
      End Get

      Set(value As CheckState)
        _CheckState = value
        _Checked = (value <> Windows.Forms.CheckState.Unchecked)

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' コントロールのフラット スタイルの外観を取得または設定します。
''' </summary>

    <Category("表示"), DefaultValue(FlatStyle.Standard),
     Description("ユーザーがマウス ポインターをコントロールの上に動かした時の表示を決定します。")> _
    Public Property FlatStyle As FlatStyle
      Get
        Return _FlatStyle
      End Get

      Set(value As FlatStyle)
        If value = FlatStyle.System Then
           value = FlatStyle.Flat
        End If

        _FlatStyle = value

        If _onCreate Then
           SetMarkInfo()
           AutoSize_Control()
        End If
      End Set
    End Property

 ''' <summary>
 '''  チェックボックスの現在のラベル（テキスト）を取得または設定します。
 ''' </summary>

    <Browsable(True), DefaultValue(""), Category("表示"),
     Editor("System.ComponentModel.Design.MultilineStringEditor, System.Design", GetType(UITypeEditor)),
     Description("コントロールに関連付けされたテキストです。")> _
    Public Shadows Property Text As String
      Get
        Return _Text
      End Get

      Set(value As String)
        If value Is Nothing Then value = ""

        _Text = value
        _TextValue = value

        SetTrimText()

        If _onCreate Then AutoSize_Control()

        RaiseEvent TextChanged(Me, New EventArgs)
      End Set
    End Property

''' <summary>
''' コントロールのラベル（テキスト）の配置を取得または設定します。
''' </summary>

    <Category("表示"), DefaultValue(Alignment.Left),
     Description("コントロールに表示される文字列の配置です。")> _
    Public Property TextAlign As Alignment
      Get
        Return _TextAlign
      End Get

      Set(value As Alignment)
        _TextAlign = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

#End Region

#Region " （拡張）"

''' <summary>
''' FlatStyle が Standard 時のマークの色を指定します。
''' </summary>

    <Category("表示"),
     Description("FlatStyle が Standard 時のマークの色を指定します。")> _
    Public Property CheckMarkColor As Color
      Get
        Return _CheckMarkColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           value = Me.ForeColor

           _CheckMarkColorState = False
        Else
           _CheckMarkColorState = True
        End If

        _CheckMarkColor = value

        If _onCreate AndAlso (_FlatStyle = FlatStyle.Standard) Then Me.Invalidate()
      End Set
    End Property

    Private Function ShouldSerializeCheckMarkColor() As Boolean
        Return Not _CheckMarkColor.Equals(Me.ForeColor)
    End Function

    Private Sub ResetCheckMarkColor()
        _CheckMarkColor = Me.ForeColor
        _CheckMarkColorState = False

        Me.Invalidate()
    End Sub

''' <summary>
''' Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）
''' </summary>

    <Category("表示"), DefaultValue(GetType(Color), "GrayText"),
     Description("Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）")> _
    Public Property DisabledColor As Color
      Get
        Return _DisabledColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           _DisabledColor = SystemColors.GrayText
        Else
           _DisabledColor = value
        End If

        If _onCreate AndAlso (Not Me.Enabled) Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。
''' </summary>

    <Category("表示"), DefaultValue(False),
     Description("Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。")> _
    Public Property DisabledColorState As Boolean
      Get
        Return _DisabledColorState
      End Get

      Set(ByVal value As Boolean)
        _DisabledColorState = value

        If _onCreate AndAlso (Not Me.Enabled) Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' コントロール上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <Category("表示"), DefaultValue(GetType(Color), "Highlight"),
     Description("コントロール上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。")> _
    Public Property HoverEffect As Boolean
      Get
        Return _HoverEffect
      End Get

      Set(ByVal value As Boolean)
        _HoverEffect = value

        If Not _onCreate Then Return

        If Me.Enabled Then
           _onMouse = Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position))

           If _onMouse Then Me.Invalidate()
        Else
           _onMouse = False
        End If
      End Set
    End Property

''' <summary>
''' マークのタイプを指定します。
''' </summary>

    <Category("表示"), DefaultValue(Mark.Original),
     Description("マークのタイプを指定します。")> _
    Public Property MarkType As Mark
      Get
        Return _MarkType
      End Get

      Set(value As Mark)

      '"・" ハングル互換字母   UTF-16 0x318d
      '"☒" その他の記号       UTF-16 0x2612
      '"✓" 装飾記号           UTF-16 0x2713
      '"✔" 装飾記号           UTF-16 0x2714
      '"？" 半角形/全角形      UTF-16 0xff1f

        _MarkType = value

        If _MarkFontState Then
           If value = Mark.System Then
              _MarkText = " ✔・ ✔✔"
           Else
              _MarkText = "・✔？ ✔✔"
           End If
        Else
           If value = Mark.System Then
              _MarkText = " ✓ㆍ ✓✓"
           Else
              _MarkText = "ㆍ✓？ ✓✓"
           End If
        End If

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' フォーカスを示す枠を表示するかを指定します。枠の色は HoverColor で描画されます。
''' </summary>

    <Category("表示"), DefaultValue(False),
     Description("フォーカスを示す枠を表示するかを指定します。枠の色は HoverColor で描画されます。")> _
    Public Property ShowFocusRect As Boolean
      Get
        Return _ShowFocusRect
      End Get

      Set(value As Boolean)
        _ShowFocusRect = value

        If _onCreate AndAlso _onFocus Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' Unchecked, Indeterminate 時のラベルの色を指定します。
''' </summary>

    <Category("表示"),
     Description("Unchecked, Indeterminate 時のラベルの色を指定します。")> _
    Public Property UnCheckedColor As Color
      Get
        Return _UnCheckedColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           value = Me.ForeColor

           _UnCheckedColorState = False
        Else
           _UnCheckedColorState = True
        End If

        _UnCheckedColor = value

        If _onCreate AndAlso (_CheckState <> CheckState.Checked) Then Me.Invalidate()
      End Set
    End Property

    Private Function ShouldSerializeUnCheckedColor() As Boolean
        Return Not _UnCheckedColor.Equals(Me.ForeColor)
    End Function

    Private Sub ResetUnCheckedColor()
        _UnCheckedColor = Me.ForeColor
        _UnCheckedColorState = False

        Me.Invalidate()
    End Sub

#End Region

#End Region

#End Region

#Region "// Method"

''' <summary>
''' Enabled が False 時のラベル表示状態と色を設定します。
''' </summary>

    Public Sub SetDisabledColor(ByVal state As Boolean, ByVal color As Color)
        If color.IsEmpty Then color = SystemColors.GrayText

        _DisabledColorState = state
        _DisabledColor = color
    End Sub

''' <summary>
''' Enabled が False 時のラベル表示状態を設定します。Flase 時には DisabledColor の値が適用されます。
''' </summary>

    Public Sub SetDisabledColorState(ByVal state As Boolean)
        _DisabledColorState = state
    End Sub

#End Region

#Region "// Sub-Routine"

''' <summary>
''' 対象のサイズが MaximumSize, MinimumSize の範囲内にあるかをチェックします。
''' </summary>

    Private Function SizeCheck(ByVal TargetSize As Size) As Size

        Dim w As Integer = TargetSize.Width
        Dim h As Integer = TargetSize.Height

        If w <= 0 Then
           w = 0
        Else
           Dim MinW As Integer = _MinimumSize.Width
           Dim MaxW As Integer = _MaximumSize.Width

           If (MinW <> 0) AndAlso (w < MinW) Then w = MinW
           If (MaxW <> 0) AndAlso (w > MaxW) Then w = MaxW
        End If

        If h <= 0 Then
           h = 0
        Else
           Dim MinH As Integer = _MinimumSize.Height
           Dim MaxH As Integer = _MaximumSize.Height

           If (MinH <> 0) AndAlso (h < MinH) Then h = MinH
           If (MaxH <> 0) AndAlso (h > MaxH) Then h = MaxH
        End If

        Return New Size(w, h)
    End Function

''' <summary>
''' AutoSizeが True の時、サイズ変更，表示位置の設定を行います。
''' </summary>

    Private Sub AutoSize_Control()
        If _AutoSize Then
           SetAutoSize()
             If Not _onResize Then  'Resize が発生しなかった場合でも、表示位置の補正を行う
                _onResize = True

                SetTextRectangle()

                Me.Invalidate()
             End If
        Else
           SetTextRectangle()

           Me.Invalidate()
        End If
    End Sub

''' <summary>
''' コントロールのサイズを取得します。
'''   幅は文字数, Padding により決定されます。
'''   高さは FlatStyle, Padding の値により決定されます。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetAutoSize()

        Dim w As Integer
        Dim h As Integer

        Dim MarkWidth As Integer
        Dim MarkHeight As Integer
        Dim TextMargin As Integer

        If _FlatStyle = FlatStyle.Standard Then
           MarkWidth = _DrawFocusBoxSize
           MarkHeight = _DrawFocusBoxSize + 2
           TextMargin = _FocusText_Margin
        Else
           MarkWidth = _Draw_MarkBoxSize
           MarkHeight = _Draw_MarkBoxSize + 2
           TextMargin = _Mark_Text_Margin
        End If

        If _DrawText.Length > 0 Then

           If (Me.MaximumSize.Width = 0) OrElse (Me.MaximumSize.Height = 0) Then

              Dim LineCount As Integer = _DrawText.Length - _DrawText.Replace(vbLf, "").Length + 1

              w = Convert.ToInt32(Math.Ceiling(_FontPixelSize * (_DrawText.Length + 1)))
              h = Convert.ToInt32(Math.Ceiling(_FontPixelSize))

           'MS UI Gothic (6.75 - 10.5p) 対策
              If (h < Me.Font.Height) OrElse (h > 15) Then
                 h = Me.Font.Height
              End If

              h *= LineCount
           Else
              w = Me.MaximumSize.Width - MarkWidth - TextMargin
              h = Me.MaximumSize.Height
           End If

           Dim b As New Bitmap(w, h)
           Dim g As Graphics = Graphics.FromImage(b)
           Dim s As SizeF = g.MeasureString(_DrawText, Me.Font, New SizeF(w, h))

           w = Convert.ToInt32(Math.Ceiling(s.Width))
           h = Convert.ToInt32(Math.Ceiling(s.Height))

           g.Dispose()
           b.Dispose()

           g = Nothing
           b = Nothing

           If (Me.Font.Style And FontStyle.Italic) <> 0 Then
              w += 2
           End If

           If (_CheckAlign And 80) <> 0 Then                'Left or Right
              If _FlatStyle = FlatStyle.Standard Then
                 w += (MarkWidth + _FocusText_Margin)
              Else
                 w += (MarkWidth + _Mark_Text_Margin)
              End If
           End If
        End If

     'マークが欠けない幅，高さを確保
        If w < MarkWidth Then w = MarkWidth
        If h < MarkHeight Then h = MarkHeight

        _BaseAutoSize = New Size(w, h)  'AutoSize の基準値

        w += _Padding.Horizontal
        h += _Padding.Vertical

        _PushAutoSize = New Size(w, h)  'AutoSize: True 時に Resize を実行した際の復元用

        _onResize = False
          Me.Size = SizeCheck(New Size(w, h))
    End Sub

''' <summary>
''' テキストの描画情報をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetTextRectangle()

        Dim x As Integer = _Padding.Left
        Dim y As Integer = _Padding.Top
        Dim w As Integer = Me.Width - _Padding.Horizontal
        Dim h As Integer = Me.Height - _Padding.Vertical

        If (w <= 0) OrElse (h <= 0) Then Return

        Dim MarkWidth As Integer

        If _FlatStyle = FlatStyle.Standard Then

           MarkWidth = _DrawFocusBoxSize

           If _CheckAlign = Alignment.Center Then
              MarkWidth = 0
           Else
              MarkWidth = _DrawFocusBoxSize + _FocusText_Margin
           End If
        Else
           MarkWidth = _Draw_MarkBoxSize + _Mark_Text_Margin
        End If

     'RightToLeft
        Dim ChkHorizontalAlign As Integer = _CheckAlign

        If Me.RightToLeft = RightToLeft.Yes Then
           If _CheckAlign = Alignment.Left Then         'Left⇒Right
               ChkHorizontalAlign = Alignment.Right
           ElseIf _CheckAlign = Alignment.Right Then    'Right⇒Left
               ChkHorizontalAlign = Alignment.Left
           End If
        End If

        Select Case ChkHorizontalAlign
          Case Alignment.Left
            x += MarkWidth
            w -= MarkWidth

          Case Alignment.Right
            w -= MarkWidth
        End Select

        If (w <= 0) OrElse (h <= 0) Then Return

        _DrawTextRectangle = New Rectangle(x, y, w, h)

        GetAutoVerticalAdjust()

        _DrawTextRectangle = New Rectangle(x, y + _AutoVerticalAdjust, w, h)
        _DrawTextRectangleF = _DrawTextRectangle
    End Sub

''' <summary>
''' コントロールの Region を設定します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetRegion()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

''' <summary>
''' AutoSize が True の時、テキストの位置（垂直方向）の調整値を取得します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub GetAutoVerticalAdjust()

        If _DrawText.Length > 0 Then

           Dim w As Integer = _DrawTextRectangle.Width
           Dim h As Integer = _DrawTextRectangle.Height

           Dim b As New Bitmap(w, h)
           Dim g As Graphics = Graphics.FromImage(b)
           Dim c As Color = b.GetPixel(0, 0)

           Dim sf As New StringFormat
           sf.LineAlignment = StringAlignment.Center

           g.DrawString(_DrawText, Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

           Dim BackColorArgb As Integer = c.ToArgb()

           Dim y0 As Integer = -1
           Dim y1 As Integer = -1

           For yp As Integer = 0 To h - 1
             For xp As Integer = 0 To w - 1
               If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                  y0 = yp
                  Exit For
               End If
             Next
             If y0 >= 0 Then Exit For
           Next

           If y0 = -1 Then
             _AutoVerticalAdjust = 0
           Else
              For yp As Integer = h - 1 To y0 Step -1
                For xp As Integer = 0 To w - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     y1 = yp
                     Exit For
                  End If
                Next
                If y1 >= 0 Then Exit For
              Next

              Dim TextHeight As Integer = (y1 - y0) + 1

              If h <= TextHeight Then
                 _AutoVerticalAdjust = 0
              Else
                 Dim CheckHeight As Integer

                 If _FlatStyle = FlatStyle.Standard Then
                    CheckHeight = _DrawFocusBoxSize
                 Else
                    CheckHeight = _Draw_MarkBoxSize
                 End If

                 Dim CheckMargin As Integer = h - CheckHeight
                 Dim TextMargin As Integer = h - TextHeight

                 If (CheckMargin Mod 2) = 0 Then
                    _AutoVerticalAdjust = ((TextMargin + 1) \ 2) - y0
                 Else
                    _AutoVerticalAdjust = (TextMargin \ 2) - y0
                 End If
              End If
           End If

           sf.Dispose()
           g.Dispose()
           b.Dispose()

           sf = Nothing
           g = Nothing
           b = Nothing
        Else
           _AutoVerticalAdjust = 0
        End If
    End Sub

''' <summary>
''' テキスト内の [CR], [LF] を [CRLF] へ置換します。また、テキスト末尾の [CRLF] を削除します。
''' </summary>

    Private Sub SetTrimText()
        _DrawText = _Text.Trim

        If _DrawText.Length > 0 Then
           _DrawText = _Text.Replace(vbCrLf, vbLf)
           _DrawText = _DrawText.Replace(vbCr, vbLf)
           _DrawText = _DrawText.Replace(vbLf, vbCrLf)

           If _DrawText.EndsWith(vbCrLf) Then _DrawText = _DrawText.Substring(0, _DrawText.Length - 2)
        End If
    End Sub

''' <summary>
''' チェックマーク情報をセットします。
''' </summary>

    Private Sub SetMarkInfo()
        If _FlatStyle = FlatStyle.Standard Then
           _MarkFontSize = 9.0F
        Else
           _MarkFontSize = 8.25F
        End If

        If _AutoScale Then
           _Draw_MarkBoxSize = Convert.ToInt32(Math.Ceiling(_Base_MarkBoxSize * _FontSizeMpf))
           _DrawFocusBoxSize = Convert.ToInt32(Math.Ceiling(_BaseFocusBoxSize * _FontSizeMpf))
        Else
           _MarkFontSize /= _FontSizeMpf

           _Draw_MarkBoxSize = _Base_MarkBoxSize
           _DrawFocusBoxSize = _BaseFocusBoxSize
        End If

        _MarkFont = New Font(_MarkFontName, _MarkFontSize, _MarkFontStyle)
    End Sub

''' <summary>
''' チェックボックスの状態を更新します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub CheckState_Update()

        Dim c As Boolean = _Checked
        Dim s As Integer = _CheckState

        If _ThreeState Then
            Dim v As Integer = (_CheckState + 1) Mod 3
           _CheckState = DirectCast(v, CheckState)
           _Checked = (v <> 0)
        Else
           _Checked = _Checked Xor True
           _CheckState = DirectCast(Convert.ToInt32(_Checked), CheckState)
        End If

        If _onCreate Then Me.Invalidate()

     '親フォームに通知
        If c <> _Checked Then RaiseEvent CheckedChanged(Me, New EventArgs)
        If s <> _CheckState Then RaiseEvent CheckStateChanged(Me, New EventArgs)

        MyBase.OnClick(New EventArgs)
    End Sub

#End Region

    Public Sub New()

        InitializeComponent()

        For Each ff In FontFamily.Families
          If ff.Name = "Meiryo UI" Then
             _MarkFontState = True
             Exit For
          End If
        Next

        Me.MarkType = Mark.Original

        If _MarkFontState Then
           _MarkFontName = "Meiryo UI"
           _MarkFontStyle = FontStyle.Regular
        Else
           _MarkFontName = "MS UI Gothic"
           _MarkFontStyle = FontStyle.Bold
        End If
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _FontSizeMpf = Me.CreateGraphics.DpiX / 96.0F
        _FontSizeInPointsMpf = _FontSizeMpf * 1.333334F
        _FontPixelSize = Me.Font.SizeInPoints * _FontSizeInPointsMpf

        If Not Me.FindForm Is Nothing Then
           If Me.FindForm.AutoScaleMode = AutoScaleMode.None Then
              Me.AutoScale = False
           End If
        End If

        _PushSize = Me.Size

        SetMarkInfo()

        If _AutoSize Then
           SetAutoSize()    '_ResizeHandler が False の為、Resize は発生しない
        End If

        SetTextRectangle()
        SetRegion()

        _onCreate = True
        _onResize = True        '外部からのリサイズ時に _PushSize を更新する為に True をセット

        _ResizeHandler = True   'Resize イベント ON

        AddHandler Me.FontChanged, AddressOf Me_FontChanged
        AddHandler Me.RightToLeftChanged, AddressOf Me_RightToLeftChanged

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        Dim ChkHorizontalAlign As Integer = _CheckAlign

     'RightToLeft
        If Me.RightToLeft = RightToLeft.Yes Then
           Select Case ChkHorizontalAlign
             Case Alignment.Left : ChkHorizontalAlign = Alignment.Right
             Case Alignment.Right : ChkHorizontalAlign = Alignment.Left
           End Select
        End If

     'マーク位置算出
        Dim mx, my As Integer   'マーク位置
        Dim fx, fy As Integer   'Focus 位置
        Dim bx, by As Integer   'Box   位置

        Dim sb As SolidBrush = New SolidBrush(Color.Empty)

        Dim FontAdjust As Integer = Convert.ToInt32(_MarkFontState)

        If _FlatStyle = FlatStyle.Standard Then
           If ChkHorizontalAlign = Alignment.Left Then

              fx = _Padding.Left
           ElseIf ChkHorizontalAlign = Alignment.Center Then

              fx = _Padding.Left + ((Me.Width - _Padding.Horizontal - _DrawFocusBoxSize) \ 2)
           Else

              fx = Me.Width - _Padding.Right - _DrawFocusBoxSize
           End If

           fy = _Padding.Top + ((Me.Height - _Padding.Vertical - _DrawFocusBoxSize) \ 2)

        'マーク位置
           mx = fx - FontAdjust
           my = fy + 1 - (3 * FontAdjust)

           If _CheckState = CheckState.Checked Then
              my += 1
           Else
              If (Not _AutoScale) OrElse (_FontSizeMpf = 1.0F) Then
                 If (_MarkType = Mark.Original) AndAlso
                    (_CheckState = CheckState.Indeterminate) Then
                    my += 1     '？
                 End If
              Else

              'Dpi変更時の端数誤差修正
                 Select Case _CheckState
                   Case CheckState.Indeterminate
                     If _MarkType = Mark.System Then
                        my += 1     '・
                     Else
                        mx += 1     '？
                        my += 1
                     End If
                   Case CheckState.Unchecked
                     If _MarkType = Mark.Original Then
                        my += 1     '・
                     End If
                 End Select
              End If
           End If
        Else
           If ChkHorizontalAlign = Alignment.Left Then
              bx = _Padding.Left

           ElseIf ChkHorizontalAlign = Alignment.Center Then
              bx = _Padding.Left + ((Me.Width - _Padding.Left - _Padding.Right - _Draw_MarkBoxSize) \ 2)

           Else
              bx = Me.Width - _Padding.Right - _Draw_MarkBoxSize
           End If

           by = _Padding.Top + ((Me.Height - _Padding.Vertical - _Draw_MarkBoxSize) \ 2)

        '枠(Back)描画
           If _onMouse Then
              sb.Color = SystemColors.Window
           Else
              sb.Color = SystemColors.ControlLightLight
           End If
           e.Graphics.FillRectangle(sb, bx + 1, by + 1, _Draw_MarkBoxSize - 2, _Draw_MarkBoxSize - 2)

        'マーク位置
           mx = bx - FontAdjust
           my = by + 1 - (3 * FontAdjust)

        'Dpi変更時の端数誤差修正
           If _AutoScale AndAlso (_FontSizeMpf = 1.25F) Then
              mx -= 1
              my -= 1
           End If
        End If

     'マーク描画
        Dim mt As String
        Dim sf As New StringFormat

        If _FlatStyle = FlatStyle.Standard Then
           mt = _MarkText.Substring(_CheckState, 1)
        Else
           mt = _MarkText.Substring(_CheckState + 3, 1)
        End If

        sf.Alignment = StringAlignment.Center

        If Me.Enabled Then
           If _FlatStyle <> FlatStyle.Standard Then
              If _CheckState = CheckState.Checked Then
                 sb.Color = SystemColors.ControlText
              Else
                 sb.Color = SystemColors.GrayText
              End If
           Else
              If _onMouse AndAlso _HoverEffect Then
                 sb.Color = _HoverColor
              Else
                 sb.Color = _CheckMarkColor
              End If
           End If

           e.Graphics.DrawString(mt, _MarkFont, sb,
             New RectangleF(mx, my, _MarkFont.Height, _MarkFont.Height), sf)
        Else
           If _DisabledColorState Then
              ControlPaint.DrawStringDisabled(e.Graphics, mt, _MarkFont,
                Me.BackColor,
                New RectangleF(mx, my, _MarkFont.Height, _MarkFont.Height), sf)
           Else
              sb.Color = _DisabledColor
              e.Graphics.DrawString(mt, _MarkFont, sb,
                New RectangleF(mx, my, _MarkFont.Height, _MarkFont.Height), sf)
           End If
        End If

     '枠描画
        If _FlatStyle = FlatStyle.Standard Then

        'Focus 枠描画
           If _onFocus AndAlso _ShowFocusRect Then
              e.Graphics.DrawRectangle(New Pen(_HoverColor), fx, fy, _DrawFocusBoxSize - 1, _DrawFocusBoxSize - 1)
           End If
        Else

           Dim p As Pen

           If _FlatStyle = FlatStyle.Flat Then
              If (_onFocus And _ShowFocusRect And Not _onMouse) Then
                 p = New Pen(_HoverColor)
              Else
                 p = New Pen(SystemColors.WindowText)
              End If

              e.Graphics.DrawRectangle(p, New Rectangle(bx, by, _Draw_MarkBoxSize - 1, _Draw_MarkBoxSize - 1))
           Else
              If (_onFocus And _ShowFocusRect) Then
                 p = New Pen(_HoverColor)
              Else
                 p = New Pen(SystemColors.ControlDark)
              End If

              Dim s As Integer = Convert.ToInt32(Not _onFocus)

              If _onMouse Then
                 e.Graphics.DrawLines(New Pen(SystemColors.ControlDark), {
                   New Point(bx, by + _Draw_MarkBoxSize),
                   New Point(bx, by),
                   New Point(bx + _Draw_MarkBoxSize - 1, by)})
                 e.Graphics.DrawLines(New Pen(SystemColors.Control), {
                   New Point(bx + 1, by + _Draw_MarkBoxSize - 1),
                   New Point(bx + _Draw_MarkBoxSize - 1, by + _Draw_MarkBoxSize - 1),
                   New Point(bx + _Draw_MarkBoxSize - 1, by + 1)})
                 e.Graphics.DrawLines(New Pen(SystemColors.ControlLightLight), {
                   New Point(bx + s, by + _Draw_MarkBoxSize),
                   New Point(bx + _Draw_MarkBoxSize, by + _Draw_MarkBoxSize),
                   New Point(bx + _Draw_MarkBoxSize, by)})
              Else
                 e.Graphics.DrawRectangle(p, New Rectangle(bx, by, _Draw_MarkBoxSize - 1, _Draw_MarkBoxSize - 1))
              End If
           End If

           p.Dispose()
           p = Nothing
        End If

     'Text描画
        If (_DrawText.Length > 0) AndAlso
           (_DrawTextRectangle.Width > 4) AndAlso
           (_DrawTextRectangle.Height > 4) Then

           If _TextAlign = Alignment.Left Then          'Left
              sf.Alignment = StringAlignment.Near
           ElseIf _TextAlign = Alignment.Center Then    'Center
              sf.Alignment = StringAlignment.Center
           Else                                         'Right
              sf.Alignment = StringAlignment.Far
           End If

           sf.LineAlignment = StringAlignment.Center
           sf.Trimming = StringTrimming.Word

           If _DrawTextRectangle.Height >= Me.FontHeight Then
              sf.FormatFlags = StringFormatFlags.LineLimit
           End If

           If Me.RightToLeft = RightToLeft.Yes Then
              sf.FormatFlags = sf.FormatFlags Or StringFormatFlags.DirectionRightToLeft
           End If

           sf.HotkeyPrefix = HotkeyPrefix.Show

           If Me.Enabled Then
              If _onMouse AndAlso _HoverEffect Then
                 sb.Color = _HoverColor
              Else
                 If _CheckState = CheckState.Checked Then
                    sb.Color = Me.ForeColor
                 Else
                    sb.Color = _UnCheckedColor
                 End If
              End If

              e.Graphics.DrawString(_DrawText, Me.Font, sb, _DrawTextRectangleF, sf)
           Else
              If _DisabledColorState Then
                 ControlPaint.DrawStringDisabled(e.Graphics, _DrawText, Me.Font, Me.BackColor,
                   _DrawTextRectangleF, sf)
              Else
                 sb.Color = _DisabledColor
                 e.Graphics.DrawString(_DrawText, Me.Font, sb, _DrawTextRectangleF, sf)
              End If
           End If
        End If

        sb.Dispose()
        sf.Dispose()

        sb = Nothing
        sf = Nothing
    End Sub

    Protected Overrides Sub OnResize(e As EventArgs)

        If Not _ResizeHandler Then Return

        If Not _onResize Then
           _onResize = True
        Else
           _ResizeHandler = False
             If _AutoSize Then
                Me.Size = SizeCheck(_PushAutoSize)
             Else
                _PushSize = Me.Size

                Me.Size = SizeCheck(Me.Size)
             End If
           _ResizeHandler = True
        End If

        SetTextRectangle()
        SetRegion()

        Me.Invalidate()

     'Resize 確定後に親フォームに通知
        MyBase.OnResize(e)
    End Sub

    Protected Overrides Sub OnClick(e As EventArgs)
     'CheckState_Update 処理後に親フォームに通知
        'MyBase.OnClick(e)
    End Sub

    Protected Overrides Sub OnPaddingChanged(e As EventArgs)
        If _onCreate Then
           AutoSize_Control()

           MyBase.OnPaddingChanged(e)
        End If
    End Sub

#Region "// Me"

    Private Sub Me_RightToLeftChanged(sender As Object, e As EventArgs)

        SetTextRectangle()
    End Sub

    Private Sub Me_FontChanged(sender As Object, e As EventArgs)
        _FontPixelSize = Me.Font.SizeInPoints * _FontSizeInPointsMpf

        AutoSize_Control()
    End Sub

    Private Sub Me_System_ForeColorChanged(ByVal sender As Object, ByVal e As EventArgs) _
                Handles Me.SystemColorsChanged, Me.ForeColorChanged

        If Not _CheckMarkColorState Then
           _CheckMarkColor = Me.ForeColor
        End If

        If Not _UnCheckedColorState Then
           _UnCheckedColor = Me.ForeColor
        End If
    End Sub

    Private Sub Me_EnabledChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.EnabledChanged

        If Me.Enabled Then
           If Not _onMouse Then
              If Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position)) Then
                 _onMouse = True

                 If _HoverEffect Then Me.Invalidate()
              End If
           End If
        Else
           _onMouse = False
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_Enter(sender As Object, e As EventArgs) Handles Me.Enter
        If _ShowFocusRect Then
           _onFocus = True

           Me.Invalidate()
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_Leave(sender As Object, e As EventArgs) Handles Me.Leave
        If _ShowFocusRect Then
           _onFocus = False

           Me.Invalidate()
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseEnter(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseEnter
        _onMouse = True

        If _HoverEffect OrElse (_FlatStyle <> FlatStyle.Standard) Then
           Me.Invalidate()
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseLeave
        _onMouse = False

        If _HoverEffect OrElse (_FlatStyle <> FlatStyle.Standard) Then
           Me.Invalidate()
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseDown(sender As Object, e As MouseEventArgs) Handles Me.MouseDown
        If (Not _onLeftButton) AndAlso (e.Button = MouseButtons.Left) Then
           _onLeftButton = True
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseUp(sender As Object, e As MouseEventArgs) Handles Me.MouseUp
        If _onLeftButton Then
           _onLeftButton = False

           If Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position)) Then
               CheckState_Update()
           End If
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub me_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        If (Not _onSpaceKey) AndAlso (e.KeyCode = Keys.Space) Then
           _onSpaceKey = True
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_KeyUp(sender As Object, e As KeyEventArgs) Handles Me.KeyUp
        If _onSpaceKey Then
           _onSpaceKey = False

           CheckState_Update()
        End If
    End Sub

#End Region

End Class

End Namespace
