﻿'
'   CheckBoxEx (Simple Check Box) 1.2.7.0 センタリング仕様
'
'       Required Class: 
'         None
'
'       Property:
'         AutoScale
'         DoubleBuffering
'
'         MarkType
'         ShowFocusRect
'         DisabledColorState
'
'         CheckMarkColor    マークの表示色
'         UnCheckedColor    ラベルの表示色
'         DisabledColor
'         HoverColor
'         HoverEffect
'
'       Method
'         SetDisabledColorState
'         SetDisabledColor
'
'       下記のプロパティは無効です。
'       また、デザイナでの設定もできません。
'
'         BackgroundImage
'         BackgroundImageLayout
'         TextImageRelation (Overlay固定)
'         UseVisualStyleBackColor
'
Imports System.ComponentModel
Imports System.Drawing.Drawing2D
Imports System.Drawing.Text

Namespace UserLib

<DebuggerStepThrough()>
Public Class CheckBoxEx
    Inherits CheckBox

Public Enum Mark As Integer
    Original = 0
    System = 1
End Enum

#Region "// constant, variable declaration"

    Private Const _Base_MarkBoxSize As Integer = 11  'チェックボックスの基準枠サイズ (FlatStyle: Flat, Popup)
    Private Const _BaseFocusBoxSize As Integer = 14  'チェックボックスのFocus枠サイズ (FlatStyle: Standard)

    Private _onCreate As Boolean
    Private _onMouse As Boolean
    Private _onFocus As Boolean

    Private _FontSizeMpf As Single
    Private _BaseCheckAlign As ContentAlignment

    Private _Draw_MarkBoxSize As Integer
    Private _DrawFocusBoxSize As Integer

    Private _AutoTextAdjust As Integer

    Private _MarkText As String
    Private _MarkFont As Font
    Private _MarkFontName As String
    Private _MarkFontSize As Single
    Private _MarkFontStyle As FontStyle
    Private _MarkFontState As Boolean       'True: Meiryo UI

    Private _DrawText As String
    Private _DrawTextRectangle As Rectangle
    Private _DrawTextRectangleF As RectangleF

#End Region

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout

    <Browsable(False), DefaultValue(TextImageRelation.Overlay)> _
    Public Shadows Property TextImageRelation() As TextImageRelation
    <Browsable(False), DefaultValue(False)> _
    Public Shadows Property UseVisualStyleBackColor() As Boolean

#End Region

    Private _AutoScale As Boolean = True
    Private _DoubleBuffering As Boolean

    Private _MarkType As Mark = Mark.Original
    Private _FlatStyle As FlatStyle = FlatStyle.Standard

    Private _HoverEffect As Boolean
    Private _ShowFocusRect As Boolean
    Private _DisabledColorState As Boolean = True
    Private _CheckMarkColorState As Boolean
    Private _UnCheckedColorState As Boolean

    Private _CheckMarkColor As Color = Me.ForeColor
    Private _UnCheckedColor As Color = Me.ForeColor
    Private _DisabledColor As Color = SystemColors.GrayText
    Private _HoverColor As Color = SystemColors.Highlight

#Region "** 動作"

''' <summary>
''' ダブルバッファリングの状態を取得または設定します。
''' </summary>

    <DefaultValue(False), Browsable(True), Category("動作"),
     Description("ダブルバッファリングの有効／無効を設定します。")> _
    Public Property DoubleBuffering() As Boolean
      Get
        Return _DoubleBuffering
      End Get

      Set(ByVal value As Boolean)
        _DoubleBuffering = value

        Me.DoubleBuffered = value
      End Set
    End Property

#End Region

#Region "** 配置"

''' <summary>
''' AutoScaleの有効／無効を変更します。規定値は True です。
''' 無効時は、画面の解像度(Dpi)が変わった場合でもチェックのサイズは同じです。
''' </summary>

    <DefaultValue(True), Category("配置"),
     Description("AutoScaleの有効／無効を変更します。")> _
    Public Property AutoScale() As Boolean
      Get
        Return _AutoScale
      End Get

      Set(ByVal value As Boolean)
        _AutoScale = value

        If _onCreate Then
           SetMarkInfo()
           SetTextRectangle()

           Me.Invalidate()
        End If
      End Set
    End Property

#End Region

#Region "** 表示"

  ''' <summary>
  ''' コントロールのフラット スタイルの外観を取得または設定します。
  ''' </summary>

    Public Shadows Property FlatStyle As FlatStyle
      Get
        Return _FlatStyle
      End Get

      Set(value As FlatStyle)
        If value = FlatStyle.System Then
           value = FlatStyle.Flat
        End If

        _FlatStyle = value

        If _onCreate Then
           SetMarkInfo()
           SetTextRectangle()

           Me.Invalidate()
        End If
      End Set
    End Property

''' <summary>
''' FlatStyle が Standard 時のマークの色を指定します。
''' </summary>

    <Category("表示"),
     Description("FlatStyle が Standard 時のマークの色を指定します。")> _
    Public Property CheckMarkColor() As Color
      Get
        Return _CheckMarkColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           value = Me.ForeColor

           _CheckMarkColorState = False
        Else
           _CheckMarkColorState = True
        End If

        _CheckMarkColor = value

        If _onCreate AndAlso (_FlatStyle = FlatStyle.Standard) Then Me.Invalidate()
      End Set
    End Property

    Private Function ShouldSerializeCheckMarkColor() As Boolean
        Return Not _CheckMarkColor.Equals(Me.ForeColor)
    End Function

    Private Sub ResetCheckMarkColor()
        _CheckMarkColor = Me.ForeColor
        _CheckMarkColorState = False

        Me.Invalidate()
    End Sub

''' <summary>
''' Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）
''' </summary>

    <DefaultValue(GetType(Color), "GrayText"), Category("表示"),
     Description("Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）")> _
    Public Property DisabledColor() As Color
      Get
        Return _DisabledColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           _DisabledColor = SystemColors.GrayText
        Else
           _DisabledColor = value
        End If

        If _onCreate AndAlso (Not Me.Enabled) Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。
''' </summary>

    <DefaultValue(True), Category("表示"),
     Description("Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。")> _
    Public Property DisabledColorState() As Boolean
      Get
        Return _DisabledColorState
      End Get

      Set(ByVal value As Boolean)
        _DisabledColorState = value

        If _onCreate AndAlso (Not Me.Enabled) Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' コントロール上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("コントロール上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor() As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。")> _
    Public Property HoverEffect() As Boolean
      Get
        Return _HoverEffect
      End Get

      Set(ByVal value As Boolean)
        _HoverEffect = value

        If Not _onCreate Then Return

        If Me.Enabled Then
           _onMouse = Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position))

           If _onMouse Then Me.Invalidate()
        Else
           _onMouse = False
        End If
      End Set
    End Property

''' <summary>
''' マークのタイプを指定します。
''' </summary>

    <DefaultValue(Mark.Original), Category("表示"),
     Description("マークのタイプを指定します。")> _
    Public Property MarkType() As Mark
      Get
        Return _MarkType
      End Get

      Set(value As Mark)

      '"・" ハングル互換字母   UTF-16 0x318d
      '"☒" その他の記号       UTF-16 0x2612
      '"✓" 装飾記号           UTF-16 0x2713
      '"✔" 装飾記号           UTF-16 0x2714
      '"？" 半角形/全角形      UTF-16 0xff1f

        _MarkType = value

        If _MarkFontState Then
           If value = Mark.System Then
              _MarkText = " ✔・ ✔✔"
           Else
              _MarkText = "・✔？ ✔✔"
           End If
        Else
           If value = Mark.System Then
              _MarkText = " ✓ㆍ ✓✓"
           Else
              _MarkText = "ㆍ✓？ ✓✓"
           End If
        End If

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' フォーカスを示す枠を表示するかを指定します。枠の色は HoverColor で描画されます。
''' </summary>

    <Category("表示"), DefaultValue(False),
     Description("フォーカスを示す枠を表示するかを指定します。枠の色は HoverColor で描画されます。")> _
    Public Property ShowFocusRect() As Boolean
      Get
        Return _ShowFocusRect
      End Get

      Set(value As Boolean)
        _ShowFocusRect = value

        If _onCreate AndAlso _onFocus Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' Unchecked, Indeterminate 時のラベルの色を指定します。
''' </summary>

    <Category("表示"),
     Description("Unchecked, Indeterminate 時のラベルの色を指定します。")> _
    Public Property UnCheckedColor() As Color
      Get
        Return _UnCheckedColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           value = Me.ForeColor

           _UnCheckedColorState = False
        Else
           _UnCheckedColorState = True
        End If

        _UnCheckedColor = value

        If _onCreate AndAlso (Me.CheckState <> CheckState.Checked) Then Me.Invalidate()
      End Set
    End Property

    Private Function ShouldSerializeUnCheckedColor() As Boolean
        Return Not _UnCheckedColor.Equals(Me.ForeColor)
    End Function

    Private Sub ResetUnCheckedColor()
        _UnCheckedColor = Me.ForeColor
        _UnCheckedColorState = False

        Me.Invalidate()
    End Sub

#End Region

#End Region

#Region "// Method"

''' <summary>
''' Enabled が False 時のラベル表示状態と色を設定します。
''' </summary>

    Public Sub SetDisabledColor(ByVal state As Boolean, ByVal color As Color)
        If color.IsEmpty Then color = SystemColors.GrayText

        _DisabledColorState = state
        _DisabledColor = color
    End Sub

''' <summary>
''' Enabled が False 時のラベル表示状態を設定します。Flase 時には DisabledColor の値が適用されます。
''' </summary>

    Public Sub SetDisabledColorState(ByVal state As Boolean)
        _DisabledColorState = state
    End Sub

#End Region

#Region "// Sub-Routine"

''' <summary>
''' コントロールの Region を設定します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetRegion()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

''' <summary>
''' チェックマーク情報をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetMarkInfo()
        If Me.FlatStyle = FlatStyle.Standard Then
           _MarkFontSize = 9.0F
        Else
           _MarkFontSize = 8.25F
        End If

        If _AutoScale Then
           _Draw_MarkBoxSize = Convert.ToInt32(Math.Ceiling(_Base_MarkBoxSize * _FontSizeMpf))
           _DrawFocusBoxSize = Convert.ToInt32(Math.Ceiling(_BaseFocusBoxSize * _FontSizeMpf))
        Else
           _MarkFontSize /= _FontSizeMpf

           _Draw_MarkBoxSize = _Base_MarkBoxSize
           _DrawFocusBoxSize = _BaseFocusBoxSize
        End If

        _MarkFont = New Font(_MarkFontName, _MarkFontSize, _MarkFontStyle)
    End Sub

''' <summary>
''' テキスト内の [CR], [LF] を [CRLF] へ置換します。また、テキスト末尾の [CRLF] を削除します。
''' </summary>

    Private Sub SetTrimText()

        _DrawText = Me.Text.Replace(vbCrLf, vbLf)
        _DrawText = _DrawText.Replace(vbCr, vbLf)
        _DrawText = _DrawText.Replace(vbLf, vbCrLf)

        If _DrawText.EndsWith(vbCrLf) Then _DrawText = _DrawText.Substring(0, _DrawText.Length - 2)
    End Sub

''' <summary>
''' テキストの描画情報をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetTextRectangle()

        Dim x As Integer = Me.Padding.Left
        Dim y As Integer = Me.Padding.Top
        Dim w As Integer = Me.Width - Me.Padding.Horizontal
        Dim h As Integer = Me.Height - Me.Padding.Vertical

        Dim MarkSize As Integer

        Const _Mark_Text_Margin As Integer = 3
        Const _FocusText_Margin As Integer = 3 '4

        If Me.FlatStyle = FlatStyle.Standard Then

           If (Me.CheckAlign = ContentAlignment.TopCenter) OrElse
              (Me.CheckAlign = ContentAlignment.BottomCenter) Then

              MarkSize = _DrawFocusBoxSize + _Mark_Text_Margin
           ElseIf (Me.CheckAlign = ContentAlignment.MiddleCenter) Then
              MarkSize = 0
           Else
              MarkSize = _DrawFocusBoxSize + _FocusText_Margin
           End If
        Else
           MarkSize = _Draw_MarkBoxSize + _Mark_Text_Margin
        End If

     'RightToLeft
        Dim ChkHorizontalAlign As Integer = Me.CheckAlign

        If Me.RightToLeft = RightToLeft.Yes Then
           If (ChkHorizontalAlign And 273) <> 0 Then        'Left⇒Right
               ChkHorizontalAlign = 4
           ElseIf (ChkHorizontalAlign And 1092) <> 0 Then   'Right⇒Left
               ChkHorizontalAlign = 1
           End If
        End If

        If (ChkHorizontalAlign And 273) <> 0 Then           'Left
            x += MarkSize
            w -= MarkSize

        ElseIf (ChkHorizontalAlign And 1092) <> 0 Then      'Right
            w -= MarkSize

        ElseIf Me.CheckAlign = ContentAlignment.TopCenter Then
            y += MarkSize
            h -= MarkSize

        ElseIf Me.CheckAlign = ContentAlignment.BottomCenter Then
            h -= MarkSize
        End If

        _DrawTextRectangle = New Rectangle(x, y, w, h)

        GetAutoTextAdjust()

        _DrawTextRectangle = New Rectangle(x, y + _AutoTextAdjust, w, h)
        _DrawTextRectangleF = _DrawTextRectangle
    End Sub

''' <summary>
''' TextAlign が Middle の時にテキストの位置（垂直方向）の調整値を取得します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub GetAutoTextAdjust()

        Dim w As Integer = _DrawTextRectangle.Width
        Dim h As Integer = _DrawTextRectangle.Height

     'TextAlignがMiddleの時、位置を調整
        If (Me.Text.Length > 0) AndAlso ((Me.TextAlign And 112) <> 0) AndAlso (w > 4) AndAlso (h > 4) Then

           Dim b As New Bitmap(w, h)
           Dim g As Graphics = Graphics.FromImage(b)
           Dim c As Color = b.GetPixel(0, 0)

           Dim sf As New StringFormat
           sf.LineAlignment = StringAlignment.Center
           g.DrawString(Me.Text, Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

           Dim BackColorArgb As Integer = c.ToArgb()

           Dim y0 As Integer = -1
           Dim y1 As Integer = -1

           For yp As Integer = 0 To h - 1
             For xp As Integer = 0 To w - 1
               If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                  y0 = yp
                  Exit For
               End If
             Next
             If y0 >= 0 Then Exit For
           Next

           If y0 = -1 Then
             _AutoTextAdjust = 0
           Else
              For yp As Integer = h - 1 To y0 Step -1
                For xp As Integer = 0 To w - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     y1 = yp
                     Exit For
                  End If
                Next
                If y1 >= 0 Then Exit For
              Next

              Dim TextHeight As Integer = (y1 - y0) + 1

              If h <= TextHeight Then
                 _AutoTextAdjust = 0
              Else
                 Dim CheckHeight As Integer

                 If _FlatStyle = FlatStyle.Standard Then
                    CheckHeight = _DrawFocusBoxSize
                 Else
                    CheckHeight = _Draw_MarkBoxSize
                 End If

                 Dim CheckMargin As Integer = h - CheckHeight
                 Dim TextMargin As Integer = h - TextHeight

                 If (CheckMargin Mod 2) = 0 Then
                    _AutoTextAdjust = ((TextMargin + 1) \ 2) - y0
                 Else
                    _AutoTextAdjust = (TextMargin \ 2) - y0
                 End If
              End If
           End If

           sf.Dispose()
           g.Dispose()
           b.Dispose()

           sf = Nothing
           g = Nothing
           b = Nothing
        Else
           _AutoTextAdjust = 0
        End If
    End Sub

#End Region

    <DebuggerStepThrough()>
    Public Sub New()

        For Each ff In FontFamily.Families
          If ff.Name = "Meiryo UI" Then
             _MarkFontState = True
             Exit For
          End If
        Next

        Me.MarkType = Mark.Original

        If _MarkFontState Then
           _MarkFontName = "Meiryo UI"
           _MarkFontStyle = FontStyle.Regular
        Else
           _MarkFontName = "MS UI Gothic"
           _MarkFontStyle = FontStyle.Bold
        End If

     '使用しないプロパテｨをリセット
        MyBase.UseVisualStyleBackColor = False
    End Sub

    Protected Overrides Sub OnCreateControl()
        MyBase.OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        _FontSizeMpf = Me.CreateGraphics.DpiX / 96.0F

        If Not Me.FindForm Is Nothing Then
           If Me.FindForm.AutoScaleMode = AutoScaleMode.None Then
              Me.AutoScale = False
           End If
        End If

        If Me.BackgroundImage IsNot Nothing Then
           Me.BackgroundImage = Nothing
           Me.BackgroundImageLayout = Nothing
        End If

        'Me.UseCompatibleTextRendering = True

        SetTrimText()
        SetMarkInfo()
        SetTextRectangle()
        SetRegion()

        _BaseCheckAlign = Me.CheckAlign     '基準値をセット

        AddHandler Me.FontChanged, AddressOf Me_FontChanged
        AddHandler Me.TextChanged, AddressOf Me_TextChanged

        AddHandler Me.RightToLeftChanged, AddressOf Me_RightToLeftChanged
        AddHandler Me.Resize, AddressOf Me_Resize

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

'     FlatStyle が System 時は発生しない

        If Me.Appearance = Appearance.Button Then
           MyBase.OnPaint(e)
           Return
        End If

    'BackColor 設定
        Dim c As Color

        If Me.BackColor = Color.Transparent Then

           Dim ctl As Control = Me.Parent
           Dim BaseParentName As String = Me.FindForm.Name

           Do
              If Not ctl.BackColor = Color.Transparent Then
                 c = ctl.BackColor

                 Exit Do
              End If

              If ctl.GetType.Name = "TabPage" Then
                 If Application.RenderWithVisualStyles Then

                 'システム既定で UseVisualStyleBackColor が True の時は 
                 'Color.White (固定) となる為、仕様を合わせる。
                    c = Color.White
                 Else

                 'システム既定で UseVisualStyleBackColor が False の時は 
                 'SystemColors.Control (固定) となる為、仕様を合わせる。
                    c = SystemColors.Control
                 End If

                 Exit Do
              End If

              ctl = ctl.Parent
           Loop Until ctl.Name = BaseParentName

        Else
           c = Me.BackColor
        End If

        e.Graphics.Clear(c)

        Dim ImgHorizontalAlign As Integer = Me.ImageAlign
        Dim ChkHorizontalAlign As Integer = Me.CheckAlign
        Dim TxtHorizontalAlign As Integer = Me.TextAlign

     'RightToLeft
        If Me.RightToLeft = RightToLeft.Yes Then
           If (ImgHorizontalAlign And 273) <> 0 Then        'Left⇒Right
              ImgHorizontalAlign = 4
           ElseIf (ImgHorizontalAlign And 1092) <> 0 Then   'Right⇒Left
              ImgHorizontalAlign = 1
           End If

           If (ChkHorizontalAlign And 273) <> 0 Then
              ChkHorizontalAlign = 4
           ElseIf (ChkHorizontalAlign And 1092) <> 0 Then
              ChkHorizontalAlign = 1
           End If

           If (TxtHorizontalAlign And 273) <> 0 Then
              TxtHorizontalAlign = 4
           ElseIf (TxtHorizontalAlign And 1092) <> 0 Then
              TxtHorizontalAlign = 1
           End If
        End If

     'Image描画
        Dim x, y As Integer

        If Not Me.Image Is Nothing Then

           If (ImgHorizontalAlign And 273) <> 0 Then        'Left
              x = Me.Padding.Left

           ElseIf (ImgHorizontalAlign And 546) <> 0 Then    'Center
              x = Me.Padding.Left + (Me.Width - Me.Padding.Horizontal - Me.Image.Width) \ 2

           Else                                             'Right
              x = (Me.Width - Me.Padding.Right - Me.Image.Width)
           End If

           If (Me.ImageAlign And 7) <> 0 Then               'Top
              y = Me.Padding.Top

           ElseIf (Me.ImageAlign And 112) <> 0 Then         'Center
              y = Me.Padding.Top + (Me.Height - Me.Padding.Vertical - Me.Image.Height) \ 2

           Else                                             'Bottom
              y = (Me.Height - Me.Padding.Bottom - Me.Image.Height)
           End If

           If Me.Enabled Then
              e.Graphics.DrawImage(Me.Image, New Rectangle(x, y, Me.Image.Width, Me.Image.Height))
           Else
              ControlPaint.DrawImageDisabled(e.Graphics, Me.Image, x, y, SystemColors.ControlLightLight)
           End If
        End If

     'マーク位置算出
        Dim mx, my As Integer   'マーク位置
        Dim fx, fy As Integer   'Focus 位置
        Dim bx, by As Integer   'Box   位置

        Dim sb As SolidBrush = New SolidBrush(Color.Empty)

        Dim FontAdjust As Integer = Convert.ToInt32(_MarkFontState)

        If Me.FlatStyle = FlatStyle.Standard Then
           If (ChkHorizontalAlign And 273) <> 0 Then        'Left

              fx = Me.Padding.Left
           ElseIf (ChkHorizontalAlign And 546) <> 0 Then    'Center

              fx = Me.Padding.Left + ((Me.Width - Me.Padding.Horizontal - _DrawFocusBoxSize) \ 2)
           Else                                             'Right

              fx = Me.Width - Me.Padding.Right - _DrawFocusBoxSize
           End If

           If (Me.CheckAlign And 7) <> 0 Then               'Top

              fy = Me.Padding.Top
           ElseIf (Me.CheckAlign And 112) <> 0 Then         'Center

              fy = Me.Padding.Top + ((Me.Height - Me.Padding.Vertical - _DrawFocusBoxSize) \ 2)
           Else                                             'Bottom

              fy = Me.Height - Me.Padding.Bottom - _DrawFocusBoxSize
           End If

        'マーク位置
           mx = fx - FontAdjust
           my = fy + 1 - (3 * FontAdjust)

           If Me.Checked Then my += 1
        Else
           If (ChkHorizontalAlign And 273) <> 0 Then                'Left
              bx = Me.Padding.Left

           ElseIf (ChkHorizontalAlign And 546) <> 0 Then            'Center
              bx = Me.Padding.Left + ((Me.Width - Me.Padding.Left - Me.Padding.Right - _Draw_MarkBoxSize) \ 2)

           Else                                                     'Right
              bx = Me.Width - Me.Padding.Right - _Draw_MarkBoxSize
           End If

           If (Me.CheckAlign And 7) <> 0 Then                       'Top
              by = Me.Padding.Top

           ElseIf (Me.CheckAlign And 112) <> 0 Then                 'Center
              by = Me.Padding.Top + ((Me.Height - Me.Padding.Top - Me.Padding.Bottom - _Draw_MarkBoxSize) \ 2)

           Else                                                     'Bottom
              by = Me.Height - Me.Padding.Bottom - _Draw_MarkBoxSize
           End If

        '枠(Back)描画
           If _onMouse Then
              sb.Color = SystemColors.Window
           Else
              sb.Color = SystemColors.ControlLightLight
           End If
           e.Graphics.FillRectangle(sb, bx + 1, by + 1, _Draw_MarkBoxSize - 2, _Draw_MarkBoxSize - 2)

        'マーク位置
           mx = bx - FontAdjust
           my = by + 1 - (3 * FontAdjust)

        'Dpi変更時の端数誤差修正
           If _AutoScale AndAlso _FontSizeMpf = 1.25F Then
              mx -= 1
              my -= 1
           End If
        End If

     'マーク描画
        Dim mt As String
        Dim sf As New StringFormat

        If Me.FlatStyle = FlatStyle.Standard Then
           mt = _MarkText.Substring(Me.CheckState, 1)
        Else
           mt = _MarkText.Substring(Me.CheckState + 3, 1)
        End If

        sf.Alignment = StringAlignment.Center

        If Me.Enabled Then
           If Me.FlatStyle <> FlatStyle.Standard Then
              If Me.CheckState = CheckState.Checked Then
                 sb.Color = SystemColors.ControlText
              Else
                 sb.Color = SystemColors.GrayText
              End If
           Else
              If _onMouse AndAlso _HoverEffect Then
                 sb.Color = _HoverColor
              Else
                 sb.Color = _CheckMarkColor
              End If
           End If

           e.Graphics.DrawString(mt, _MarkFont, sb,
             New RectangleF(mx, my, _MarkFont.Height, _MarkFont.Height), sf)
        Else
           If _DisabledColorState Then
              ControlPaint.DrawStringDisabled(e.Graphics, mt, _MarkFont,
                Me.BackColor,
                New RectangleF(mx, my, _MarkFont.Height, _MarkFont.Height), sf)
           Else
              sb.Color = _DisabledColor
              e.Graphics.DrawString(mt, _MarkFont, sb,
                New RectangleF(mx, my, _MarkFont.Height, _MarkFont.Height), sf)
           End If
        End If

     '枠描画
        If Me.FlatStyle = FlatStyle.Standard Then

        'Focus 枠描画
           If _onFocus AndAlso _ShowFocusRect Then
              e.Graphics.DrawRectangle(New Pen(_HoverColor), fx, fy, _DrawFocusBoxSize - 1, _DrawFocusBoxSize - 1)
           End If
        Else

           Dim p As Pen

           If Me.FlatStyle = FlatStyle.Flat Then
              If (_onFocus And _ShowFocusRect And Not _onMouse) Then
                 p = New Pen(_HoverColor)
              Else
                 p = New Pen(SystemColors.WindowText)
              End If

              e.Graphics.DrawRectangle(p, New Rectangle(bx, by, _Draw_MarkBoxSize - 1, _Draw_MarkBoxSize - 1))
           Else
              If (_onFocus And _ShowFocusRect) Then
                 p = New Pen(_HoverColor)
              Else
                 p = New Pen(SystemColors.ControlDark)
              End If

              Dim s As Integer = Convert.ToInt32(Not _onFocus)

              If _onMouse Then
                 e.Graphics.DrawLines(New Pen(SystemColors.ControlDark), {
                   New Point(bx, by + _Draw_MarkBoxSize),
                   New Point(bx, by),
                   New Point(bx + _Draw_MarkBoxSize - 1, by)})
                 e.Graphics.DrawLines(New Pen(SystemColors.Control), {
                   New Point(bx + 1, by + _Draw_MarkBoxSize - 1),
                   New Point(bx + _Draw_MarkBoxSize - 1, by + _Draw_MarkBoxSize - 1),
                   New Point(bx + _Draw_MarkBoxSize - 1, by + 1)})
                 e.Graphics.DrawLines(New Pen(SystemColors.ControlLightLight), {
                   New Point(bx + s, by + _Draw_MarkBoxSize),
                   New Point(bx + _Draw_MarkBoxSize, by + _Draw_MarkBoxSize),
                   New Point(bx + _Draw_MarkBoxSize, by)})
              Else
                 e.Graphics.DrawRectangle(p, New Rectangle(bx, by, _Draw_MarkBoxSize - 1, _Draw_MarkBoxSize - 1))
              End If
           End If

           p.Dispose()
           p = Nothing
        End If

     'Text描画

        If (_DrawText.Length > 0) AndAlso
           (_DrawTextRectangle.Width > 4) AndAlso
           (_DrawTextRectangle.Height > 4) Then

           If (TxtHorizontalAlign And 273) <> 0 Then        'Left
              sf.Alignment = StringAlignment.Near
           ElseIf (TxtHorizontalAlign And 546) <> 0 Then    'Center
              sf.Alignment = StringAlignment.Center
           Else                                             'Right
              sf.Alignment = StringAlignment.Far
           End If

           If (Me.TextAlign And 7) <> 0 Then                'Top
              sf.LineAlignment = StringAlignment.Near
           ElseIf (Me.TextAlign And 112) <> 0 Then          'Center
              sf.LineAlignment = StringAlignment.Center
           Else                                             'Bottom
              sf.LineAlignment = StringAlignment.Far
           End If

           If _DrawTextRectangle.Height >= Me.FontHeight Then
              sf.FormatFlags = StringFormatFlags.LineLimit
           End If

        'CheckAlign 変更チェック
           If _BaseCheckAlign <> Me.CheckAlign Then
              _BaseCheckAlign = Me.CheckAlign

              SetTextRectangle()
           End If

           sf.HotkeyPrefix = HotkeyPrefix.Show

           If Me.Enabled Then
              If _onMouse AndAlso _HoverEffect Then
                 sb.Color = _HoverColor
              Else
                 If Me.CheckState = CheckState.Checked Then
                    sb.Color = Me.ForeColor
                 Else
                    sb.Color = _UnCheckedColor
                 End If
              End If

              e.Graphics.DrawString(_DrawText, Me.Font, sb, _DrawTextRectangleF, sf)
           Else
              If _DisabledColorState Then
                 ControlPaint.DrawStringDisabled(e.Graphics, _DrawText, Me.Font, Me.BackColor,
                   _DrawTextRectangleF, sf)
              Else
                 sb.Color = _DisabledColor
                 e.Graphics.DrawString(_DrawText, Me.Font, sb, _DrawTextRectangleF, sf)
              End If
           End If
        End If

        sb.Dispose()
        sf.Dispose()

        sb = Nothing
        sf = Nothing
    End Sub

    Protected Overrides Sub OnEnabledChanged(e As EventArgs)
        If Me.Enabled Then
           If Not _onMouse Then
              _onMouse = Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position))
           End If
        Else
           _onMouse = False
        End If

     'onPaint called to system
        MyBase.OnEnabledChanged(e)
    End Sub

#Region "// Me"

    Private Sub Me_RightToLeftChanged(sender As Object, e As EventArgs)

        SetTextRectangle()
    End Sub

    Private Sub Me_AutoSizeChanged(sender As Object, e As EventArgs) Handles Me.AutoSizeChanged

    'AutoSize Change
    ' (True → False) : Resize (確定) ⇒ AutoSizeChanged ⇒ UseCompatibleTextRendering
    ' (False → True) : Resize ⇒ AutoSizeChanged ⇒ UseCompatibleTextRendering ⇒ Resize (確定) 
    End Sub

    Private Sub Me_FontChanged(sender As Object, e As EventArgs)

     'Font が変わると Resize イベントが発生する為、処理の二重化を防止
        If Me.AutoSize Then Return

        SetTextRectangle()
    End Sub

    Private Sub Me_TextChanged(sender As Object, e As EventArgs)

        SetTrimText()

     'Text が変わると Resize イベントが発生する為、処理の二重化を防止
        If Me.AutoSize Then Return

        SetTextRectangle()
    End Sub

    Private Sub Me_System_ForeColorChanged(ByVal sender As Object, ByVal e As EventArgs) _
                Handles Me.ForeColorChanged, Me.SystemColorsChanged

        If Not _CheckMarkColorState Then
           _CheckMarkColor = Me.ForeColor
        End If

        If Not _UnCheckedColorState Then
           _UnCheckedColor = Me.ForeColor
        End If
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        SetRegion()
        SetTextRectangle()
    End Sub

    Private Sub Me_PaddingChanged(sender As Object, e As EventArgs) Handles Me.PaddingChanged
        SetTextRectangle()
    End Sub

    Private Sub Me_Enter(sender As Object, e As EventArgs) Handles Me.Enter
        _onFocus = True
    End Sub

    Private Sub Me_Leave(sender As Object, e As EventArgs) Handles Me.Leave
        _onFocus = False
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseEnter(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseEnter
        _onMouse = True

     'onPaint called to system
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseLeave
        _onMouse = False

     'onPaint called to system
    End Sub

#End Region

End Class

End Namespace
