﻿'
'   ComboBoxEx 1.1.1.1 for Classic Style Custom Ver. 貼り付け文字修正
'
'       Property:
'         None
'
'       Method
'         None
'
'     (注意事項)
'       MouseEnter, MouseHover, MouseLeave は、正常な動作が不可能な為、使用できません。

Imports System.Runtime.InteropServices
Imports System.Security.Permissions
Imports System.Text.RegularExpressions

Namespace UserLib

<DebuggerStepThrough()>
Public Class ComboBoxEx
    Inherits ComboBox

    Private WithEvents Frame As PictureBox

#Region "// 貼り付け文字の先頭から、TAB,CR,LFを削除する為の記述"

#Region "** ComboBoxInfo 構造体"

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure ComboBoxInfo
        Public Size As Integer
        Public RectItem As Rectangle
        Public RectButton As Rectangle
        Public ButtonState As Integer
        Public ComboBoxHandle As IntPtr
        Public EditBoxHandle As IntPtr
        Public ListBoxHandle As IntPtr
    End Structure

#End Region

#Region "** P/Invoke - GetComboBox 関数"

    <DllImport("USER32.DLL")> _
    Private Shared Function GetComboBoxInfo( _
        ByVal comboBoxHandle As IntPtr, _
        ByRef pComboBoxInfo As ComboBoxInfo) As Boolean
    End Function

#End Region

#Region "** EditBoxWindow クラス"

    <DebuggerStepThrough()>
    Private Class EditBoxWindow
        Inherits NativeWindow

        Public Sub New(ByVal parentComboBox As ComboBox)
            AddHandler parentComboBox.HandleDestroyed, AddressOf Me.OnHandleDestroyed
        End Sub

        Private Sub OnHandleDestroyed(ByVal sender As Object, ByVal e As EventArgs)
            Me.ReleaseHandle()
        End Sub

        <PermissionSetAttribute(SecurityAction.Demand)> _
        Protected Overrides Sub WndProc(ByRef m As Message)
            Const WM_PASTE As Integer = &H302

            Select Case m.Msg
                Case WM_PASTE

                  Dim cText = Clipboard.GetText

               '先頭の TAB,CR,LFを削除
                  If Not String.IsNullOrEmpty(cText) Then
                     cText = Regex.Replace(cText, "^[\t\r\n]+", "")

                     Clipboard.SetText(cText)
                  End If
            End Select

            MyBase.WndProc(m)
        End Sub

    End Class

#End Region

#Region "** OnHandleCreated メソッド (Override)"

<DebuggerStepThrough()>
    Protected Overrides Sub OnHandleCreated(ByVal e As EventArgs)
        MyBase.OnHandleCreated(e)

        Dim tComboBoxInfo As New ComboBoxInfo()

        tComboBoxInfo.Size = Marshal.SizeOf(tComboBoxInfo)

        If GetComboBoxInfo(Me.Handle, tComboBoxInfo) = False Then
           Return
        End If

        If tComboBoxInfo.EditBoxHandle.Equals(IntPtr.Zero) Then
           Return
        End If

        Dim hEditBoxWindow As New EditBoxWindow(Me)
        hEditBoxWindow.AssignHandle(tComboBoxInfo.EditBoxHandle)
    End Sub

#End Region

#End Region

    Private _onCreate As Boolean

    Private _FramePoint As Point()

#Region "// Sub-Routine"

''' <summary>
''' 外枠の描画座標をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetDrawPoint()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        _FramePoint = {
          New Point(1, 0), New Point(w - 2, 0), New Point(w - 2, 1), New Point(w - 1, 1),
          New Point(w - 1, h - 2), New Point(w - 2, h - 2), New Point(w - 2, h - 1), New Point(1, h - 1),
          New Point(1, h - 2), New Point(0, h - 2), New Point(0, 1), New Point(1, 1)}
    End Sub

''' <summary>
''' Region (Me, Frame) の設定、および、外枠を描画します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub DrawFrame()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        If (w < 1) OrElse (h < 1) Then Return

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        g.DrawRectangle(New Pen(SystemColors.ControlLightLight), 1, 1, w - 3, h - 3)
        g.DrawLines(New Pen(SystemColors.ControlDark), _FramePoint)

        Me.Frame.Image = DirectCast(b.Clone, Drawing.Image)

        path = New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(0, 0, w, h))
        path.AddRectangle(New Rectangle(2, 2, w - 4, h - 4))

        Me.Frame.Region = New Region(path)

        path.Dispose()
        path = Nothing

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing
    End Sub

#End Region

    Public Sub New()

        Me.Frame = New PictureBox

        CType(Me.Frame, System.ComponentModel.ISupportInitialize).BeginInit()
        Me.SuspendLayout()
        Me.Frame.Visible = Not Application.RenderWithVisualStyles
        Me.Controls.Add(Me.Frame)

        CType(Me.Frame, System.ComponentModel.ISupportInitialize).EndInit()
        Me.ResumeLayout(False)
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        If Not Application.RenderWithVisualStyles Then
           Me.Frame.Size = Me.Size

           SetDrawPoint()
           DrawFrame()
        End If

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler Me.SystemColorsChanged, AddressOf Me_SystemColorsChanged

        MyBase.OnCreateControl()
    End Sub

#Region "** Unavailable Event"

'    Protected Overrides Sub OnEnter(ByVal e As EventArgs)
'
'   ここでComboboxの操作をすると不正な処理になる為、GotFocusで処理を行うことを推奨。
'     ・Enterイベント開始時に、SelectedText が "" になる
'     ・Entetイベント終了後に、SelectedText, SelectionStart, SelectionLength が強制的にセットされる 
'     のが原因
'   
'     例)                        [Enter]       [GotFocus]
'     Cmb_Text_1.Text             "Test"        "Test"
'     Cmb_Text_1.SelectionStart   0             0
'     Cmb_Text_1.SelectionLength  0             4
'     Cmb_Text_1.SelectedText     ""            "Test"
'
'     Cmb_Text_1.SelectedText = "Test" を実行
'
'     Cmb_Text_1.Text             "TestTest"    "Test"
'     Cmb_Text_1.SelectionStart   4             4
'     Cmb_Text_1.SelectionLength  0             0
'     Cmb_Text_1.SelectedText     ""            ""
'
'       MyBase.OnEnter(e)
'    End Sub

    Protected Overrides Sub OnMouseEnter(e As EventArgs)
        'MyBase.OnMouseEnter(e)
    End Sub

    Protected Overrides Sub OnMouseHover(e As EventArgs)
        'MyBase.OnMouseHover(e)
    End Sub

    Protected Overrides Sub OnMouseLeave(e As EventArgs)
        'MyBase.OnMouseLeave(e)
    End Sub

#End Region

#Region "// Me"

    Private Sub Me_SystemColorsChanged(sender As Object, e As EventArgs)

        Dim s As Boolean = Not Application.RenderWithVisualStyles

        If s Then
           If Not (Me.Frame.Size = Me.Size) Then
              Me.Frame.Size = Me.Size

              SetDrawPoint()
           End If

           DrawFrame()
        Else
           Me.Region = Nothing
        End If

        Me.Frame.Visible = s
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        If Not Application.RenderWithVisualStyles Then
           Me.Frame.Size = Me.Size

           SetDrawPoint()
           DrawFrame()
        End If
'
'     リサイズ時に”入力された文字”、および、フォーカスがないコンボボックスが
'     強制的に選択状態になる不具合を回避。
'
'     ※対象: DropDownStyle= 'DropDown'

        If Me.DropDownStyle = ComboBoxStyle.DropDown Then
           If Not Me.Focused Then
              Me.SelectionLength = 0
           End If
        End If
    End Sub

#End Region

End Class

End Namespace
