﻿'
'   Flat (Gradation) ProgressBar 1.0.1.1
'
'       Property:
'         BorderStyle
'
'         Maximum
'         Minimum
'         Value
'
'         GradationEffect
'
'       Method
'         None
'
Imports System.ComponentModel
Imports System.Drawing.Drawing2D

Namespace UserLib

<DebuggerStepThrough()>
Public Class FlatProgressBar

Public Enum Gradation As Integer
    None = 0
    Simple = 1
    Normal = 2
    Strength = 3
End Enum

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AllowDrop() As Boolean
    <Browsable(False)> _
    Public Shadows Property AutoValidate() As AutoValidate
    <Browsable(False)> _
    Public Shadows Property ImeMode() As ImeMode
    <Browsable(False), DefaultValue(False)> _
    Public Shadows Property TabStop() As Boolean

    <Browsable(False)> _
    Public Shadows Property AutoScroll() As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin() As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize() As Size
    <Browsable(False)> _
    Public Shadows Property AutoSize() As Boolean
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode() As AutoSizeMode
    <Browsable(False)> _
    Public Shadows Property Padding() As Padding

    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout
    <Browsable(False)> _
    Public Shadows Property Font() As Font
    <Browsable(False)> _
    Public Shadows Property RightToLeft() As RightToLeft

#End Region

    Private _Value As Integer
    Private _Maximum As Integer = 100
    Private _Minimum As Integer

    Private _BorderStyle As BorderStyle = BorderStyle.FixedSingle

    Private _DefaultForeColor As Color = SystemColors.Highlight
    Private _ForeColor As Color = _DefaultForeColor

    Private _GradationEffect As Gradation = Gradation.None

#Region "** 動作"

''' <summary>
''' コントロールの範囲の最大値を取得または設定します。
''' </summary>

    <DefaultValue(100), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("この ProgressBar が処理している範囲の上限です。")> _
    Public Property Maximum() As Integer
      Get
        Return _Maximum
      End Get

      Set(value As Integer)
        If value < 0 Then value = 0
        If value < _Minimum Then _Minimum = value

        _Maximum = value

        If value < _Minimum OrElse value < _Value Then
           Me.Value = value
        Else
           If _onCreate Then Me.Invalidate()
        End If
      End Set
    End Property

''' <summary>
''' コントロールの範囲の最小値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("この ProgressBar が処理している範囲の下限です。")> _
    Public Property Minimum() As Integer
      Get
        Return _Minimum
      End Get

      Set(value As Integer)
        If value < 0 Then value = 0
        If value > Maximum Then _Maximum = value

        _Minimum = value

        If value > Maximum OrElse value > _Value Then
           Me.Value = value
        Else
           If _onCreate Then Me.Invalidate()
        End If
      End Set
    End Property

''' <summary>
''' FlatProgressBar の現在の値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("最小および最大プロパティで指定された範囲内の ProgressBar の現在の値です。")> _
    Public Property Value() As Integer
      Get
        Return _Value
      End Get

      Set(value As Integer)
        If value = _Value Then Return

        If value < _Minimum Then
           _Value = _Minimum
        ElseIf value > _Maximum Then
           _Value = _Maximum
        Else
           _Value = value
        End If

        If _onCreate Then
           Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
        End If
      End Set
    End Property

#End Region

#Region "** 表示"

''' <summary>
''' コントロールの境界線スタイルを取得または設定します。
''' </summary>

    <DefaultValue(BorderStyle.FixedSingle), Category("表示")> _
    Public Shadows Property BorderStyle() As BorderStyle
      Get
        Return _BorderStyle
      End Get

      Set(value As BorderStyle)
        _BorderStyle = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' コントロールの前景色を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示")> _
    Public Shadows Property ForeColor() As Color
      Get
        Return _ForeColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           value = _DefaultForeColor
        End If
        _ForeColor = value

        SetBarColor()

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' バーのグラデーションの強さを取得または設定します。
''' </summary>

    <DefaultValue(Gradation.None), Category("表示"),
     Description("バーを描画する際のグラデーションの強さを指定します。")> _
    Public Property GradationEffect() As Gradation
      Get
        Return _GradationEffect
      End Get

      Set(ByVal value As Gradation)
        _GradationEffect = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

#End Region

#End Region

    Private _onCreate As Boolean

    Private _pt_DarkLine As Point()
    Private _pt_LightLine As Point()

    Private _BackImage As Drawing.Image

    Private _cl_Light1 As Color
    Private _cl_Light2 As Color
    Private _cl_Light3 As Color
    Private _cl_Loght4 As Color
    Private _cl_Dark1 As Color
    Private _cl_Dark2 As Color
    Private _cl_Dark3 As Color

#Region "// Function, Sub"

''' <summary>
''' 色値をHSV(HSB)からRGBへ変換します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Function ColorHSVtoRGB(ByVal H As Single, ByVal S As Single, ByVal V As Single) As Color
        If H < 0.0F Then
           H = 0.0F
        ElseIf H >= 360.0F Then
           H = 359.9999F
        End If

        If S < 0.0F Then
           S = 0.0F
        ElseIf S > 1.0F Then
           S = 1.0F
        End If

        If V < 0.0F Then
           V = 0.0F
        ElseIf V > 1.0F Then
           V = 1.0F
        End If

        Dim R As Single, G As Single, B As Single
        If S = 0 Then
            R = V
            G = V
            B = V
        Else
            Dim d As Single = H / 60.0F
            Dim i As Integer = Convert.ToInt32(Math.Floor(d))   '=0-5
            Dim f As Single = d - i                             '=0-0.9999
            Dim c As Single

            If V < 0.5F Then
                c = 2.0F * S * V
            Else
                c = 2.0F * S * (1.0F - V)
            End If

            Dim m As Single = V - c / 2.0F
            Dim p As Single = c + m
            Dim q As Single

            If i Mod 2 = 0 Then
                q = V + c * (f - 0.5F)
            Else
                q = V - c * (f - 0.5F)
            End If

            Select Case i
                Case 0 : R = p : G = q : B = m
                Case 1 : R = q : G = p : B = m
                Case 2 : R = m : G = p : B = q
                Case 3 : R = m : G = q : B = p
                Case 4 : R = q : G = m : B = p
                Case 5 : R = p : G = m : B = q
            End Select
        End If

        Return Color.FromArgb(Convert.ToInt32(Math.Round(R * 255.0F)), _
                              Convert.ToInt32(Math.Round(G * 255.0F)), _
                              Convert.ToInt32(Math.Round(B * 255.0F)))
    End Function

''' <summary>
''' ForeColor を基に、明暗色を生成します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub SetBarColor()

        Dim H As Single = Me.ForeColor.GetHue
        Dim S As Single = Me.ForeColor.GetSaturation
        Dim B As Single = Me.ForeColor.GetBrightness

        Dim c0 As Single = 1.0F - B
        Dim c1 As Single = B * 0.665F
        Dim c2 As Single = B - c1

        If c0 > 0.4F Then
           c0 = 0.4F
        End If

        _cl_Light1 = ColorHSVtoRGB(H, S, B + (c0 * 0.15F))
        _cl_Light2 = ColorHSVtoRGB(H, S, B + (c0 * 0.3F))
        _cl_Light3 = ColorHSVtoRGB(H, S, B + (c0 * 0.45F))
        _cl_Loght4 = ColorHSVtoRGB(H, S, B + (c0 * 0.75F))
        _cl_Dark1 = ColorHSVtoRGB(H, S, c1 + (c2 * 0.8F))
        _cl_Dark2 = ColorHSVtoRGB(H, S, c1 + (c2 * 0.4F))
        _cl_Dark3 = ColorHSVtoRGB(H, S, c1)
    End Sub

''' <summary>
''' コントロールの描画位置、および、領域を設定します。
''' </summary>

    Private Sub SetDrawPoint()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

     '枠線 (Dark)
        _pt_DarkLine = {
            New Point(0, h - 2), New Point(0, 1), New Point(1, 0), New Point(w - 2, 0)}

     '枠線 (Light)
        _pt_LightLine = {
            New Point(w - 1, 1), New Point(w - 1, h - 2), New Point(w - 2, h - 1), New Point(1, h - 1)}

     'Region Set
        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

#End Region

    <DebuggerStepThrough()>
    Public Sub New()

        InitializeComponent()

        SetBarColor()

        Me.DoubleBuffered = True

     '使用しないプロパテｨをリセット
        MyBase.TabStop = False
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        SetDrawPoint()

        AddHandler Me.Resize, AddressOf Me_Resize

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        Dim w As Integer = Me.Width - 1
        Dim h As Integer = Me.Height - 1

        Dim pn_Standard As New Pen(SystemColors.Control)
        Dim pn_Dark As New Pen(SystemColors.ControlDark)
        Dim pn_Light As New Pen(SystemColors.ControlLightLight)

        If Parent.BackColor <> Me.BackColor Then
           e.Graphics.DrawRectangle(New Pen(Parent.BackColor), 0, 0, w, h)
        End If

        Dim x, y As Integer

        Select Case Me.BorderStyle
          Case BorderStyle.FixedSingle
            e.Graphics.DrawLines(pn_Light, _pt_LightLine)
            e.Graphics.DrawLines(pn_Light, _pt_DarkLine)

            w -= 1
            h -= 1
            x = 1
            y = 1

          Case BorderStyle.Fixed3D

            e.Graphics.DrawLines(pn_Dark, _pt_DarkLine)
            e.Graphics.DrawLines(pn_Light, _pt_LightLine)
            e.Graphics.DrawRectangle(pn_Standard, 1, 1, w - 2, h - 2)

            w -= 3
            h -= 3
            x = 2
            y = 2

          Case Else

            If _Value = _Maximum Then
               e.Graphics.DrawRectangle(New Pen(_cl_Dark1), 0, 1, w, h - 2)
            ElseIf _Value = _Minimum Then
               e.Graphics.DrawRectangle(New Pen(Me.BackColor), 0, 1, w, h - 2)
            Else
               e.Graphics.DrawLine(New Pen(_cl_Dark1), 0, 1, 0, h - 1)
               e.Graphics.DrawLine(New Pen(Me.BackColor), w, 1, w, h - 1)
            End If

            w -= 1
            h += 1
            x = 1
            y = 0

        End Select

        Dim v As Integer
        Dim p As Integer = _Maximum - _Minimum

        If p = 0 Then
           v = w
        Else
           v = Convert.ToInt32((w / p) * (_Value - _Minimum))
        End If

        Dim gb As New LinearGradientBrush(New Rectangle(x, y, 1, h), Color.Black, Color.White, LinearGradientMode.Vertical)
        Dim bl As New ColorBlend()

        Select Case GradationEffect
          Case Gradation.Simple

            bl.Positions = New Single() {0.0F, 0.4F, 1.0F}
            bl.Colors = New Color() {_cl_Light2, Me.ForeColor, _cl_Dark1}

          Case Gradation.Normal

            bl.Positions = New Single() {0.0F, 0.5F, 0.75F, 1.0F}
            bl.Colors = New Color() {_cl_Light3, Me.ForeColor, _cl_Dark1, _cl_Dark2}

          Case Gradation.Strength

            bl.Positions = New Single() {0.0F, 0.35F, 0.65F, 0.85F, 1.0F}
            bl.Colors = New Color() {_cl_Light3, _cl_Loght4, _cl_Light1, Me.ForeColor, _cl_Dark2}
        End Select

        If p = 0 OrElse Not (_Value = _Minimum) Then
           If GradationEffect = Gradation.None Then
              e.Graphics.FillRectangle(New SolidBrush(Me.ForeColor), New Rectangle(x, y, v, h))
           Else
              gb.InterpolationColors = bl
              e.Graphics.FillRectangle(gb, New Rectangle(x, y, v, h))
           End If
        End If

        If Not (_Value = _Maximum) Then
           e.Graphics.FillRectangle(New SolidBrush(Me.BackColor), New Rectangle(v + x, y, w - v, h))
        End If

        gb.Dispose()
        gb = Nothing

        pn_Standard.Dispose()
        pn_Dark.Dispose()
        pn_Light.Dispose()

        pn_Standard = Nothing
        pn_Dark = Nothing
        pn_Light = Nothing
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        SetDrawPoint()

        Me.Invalidate()
    End Sub

End Class

End Namespace
