﻿'
'   Flat ProgressBar 1.0.1.1
'
'       Property:
'         BorderStyle
'
'         Maximum
'         Minimum
'         Value
'
'       Method
'         None
'
Imports System.ComponentModel

Namespace UserLib

<DebuggerStepThrough()>
Public Class FlatProgressBar

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AllowDrop() As Boolean
    <Browsable(False)> _
    Public Shadows Property AutoValidate() As AutoValidate
    <Browsable(False)> _
    Public Shadows Property ImeMode() As ImeMode
    <Browsable(False), DefaultValue(False)> _
    Public Shadows Property TabStop() As Boolean

    <Browsable(False)> _
    Public Shadows Property AutoScroll() As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin() As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize() As Size
    <Browsable(False)> _
    Public Shadows Property AutoSize() As Boolean
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode() As AutoSizeMode
    <Browsable(False)> _
    Public Shadows Property Padding() As Padding

    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout
    <Browsable(False)> _
    Public Shadows Property Font() As Font
    <Browsable(False)> _
    Public Shadows Property RightToLeft() As RightToLeft

#End Region

    Private _Value As Integer
    Private _Maximum As Integer = 100
    Private _Minimum As Integer

    Private _BorderStyle As BorderStyle = BorderStyle.FixedSingle

    Private _DefaultForeColor As Color = SystemColors.Highlight
    Private _ForeColor As Color = _DefaultForeColor

#Region "** 動作"

''' <summary>
''' コントロールの範囲の最大値を取得または設定します。
''' </summary>

    <DefaultValue(100), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("この ProgressBar が処理している範囲の上限です。")> _
    Public Property Maximum() As Integer
      Get
        Return _Maximum
      End Get

      Set(value As Integer)
        If value < 0 Then value = 0
        If value < _Minimum Then _Minimum = value

        _Maximum = value

        If value < _Minimum OrElse value < _Value Then
           Me.Value = value
        Else
           If _onCreate Then Me.Invalidate()
        End If
      End Set
    End Property

''' <summary>
''' コントロールの範囲の最小値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("この ProgressBar が処理している範囲の下限です。")> _
    Public Property Minimum() As Integer
      Get
        Return _Minimum
      End Get

      Set(value As Integer)
        If value < 0 Then value = 0
        If value > Maximum Then _Maximum = value

        _Minimum = value

        If value > Maximum OrElse value > _Value Then
           Me.Value = value
        Else
           If _onCreate Then Me.Invalidate()
        End If
      End Set
    End Property

''' <summary>
''' FlatProgressBar の現在の値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("最小および最大プロパティで指定された範囲内の ProgressBar の現在の値です。")> _
    Public Property Value() As Integer
      Get
        Return _Value
      End Get

      Set(value As Integer)
        If value = _Value Then Return

        If value < _Minimum Then
           _Value = _Minimum
        ElseIf value > _Maximum Then
           _Value = _Maximum
        Else
           _Value = value
        End If

        If _onCreate Then
           Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
        End If
      End Set
    End Property

#End Region

#Region "** 表示"

''' <summary>
''' コントロールの境界線スタイルを取得または設定します。
''' </summary>

    <DefaultValue(GetType(BorderStyle), "FixedSingle"), Category("表示")> _
    Public Shadows Property BorderStyle() As BorderStyle
      Get
        Return _BorderStyle
      End Get

      Set(value As BorderStyle)
        _BorderStyle = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' コントロールの前景色を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示")> _
    Public Shadows Property ForeColor() As Color
      Get
        Return _ForeColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           value = _DefaultForeColor
        End If
        _ForeColor = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

#End Region

#End Region

#Region "// Sub"

''' <summary>
''' コントロールの描画位置、および、領域を設定します。
''' </summary>

    Private Sub SetDrawPoint()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

     '枠線 (Dark)
        _pt_DarkLine = {
            New Point(0, h - 2), New Point(0, 1), New Point(1, 0), New Point(w - 2, 0)}

     '枠線 (Light)
        _pt_LightLine = {
            New Point(w - 1, 1), New Point(w - 1, h - 2), New Point(w - 2, h - 1), New Point(1, h - 1)}

     'Region Set
        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

#End Region

    Private _onCreate As Boolean

    Private _pt_DarkLine As Point()
    Private _pt_LightLine As Point()

    <DebuggerStepThrough()>
    Public Sub New()

        InitializeComponent()

        Me.DoubleBuffered = True

     '使用しないプロパテｨをリセット
        MyBase.TabStop = False
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        SetDrawPoint()

        AddHandler Me.Resize, AddressOf Me_Resize

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        Dim w As Integer = Me.Width - 1
        Dim h As Integer = Me.Height - 1

        Dim pn_Standard As New Pen(SystemColors.Control)
        Dim pn_Dark As New Pen(SystemColors.ControlDark)
        Dim pn_Light As New Pen(SystemColors.ControlLightLight)

        Dim x, y As Integer

        Select Case Me.BorderStyle
          Case BorderStyle.FixedSingle
            e.Graphics.DrawLines(pn_Light, _pt_LightLine)
            e.Graphics.DrawLines(pn_Light, _pt_DarkLine)

            w -= 1
            h -= 1
            x = 1
            y = 1

          Case BorderStyle.Fixed3D

            e.Graphics.DrawLines(pn_Dark, _pt_DarkLine)
            e.Graphics.DrawLines(pn_Light, _pt_LightLine)
            e.Graphics.DrawRectangle(pn_Standard, 1, 1, w - 2, h - 2)

            w -= 3
            h -= 3
            x = 2
            y = 2

          Case Else

            If _Value = _Maximum Then
               e.Graphics.DrawRectangle(New Pen(Me.ForeColor), 0, 1, w, h - 2)
            ElseIf _Value = _Minimum Then
               e.Graphics.DrawRectangle(New Pen(Me.BackColor), 0, 1, w, h - 2)
            Else
               e.Graphics.DrawLine(New Pen(Me.ForeColor), 0, 1, 0, h - 1)
               e.Graphics.DrawLine(New Pen(Me.BackColor), w, 1, w, h - 1)
            End If

            w -= 1
            h += 1
            x = 1
            y = 0

        End Select

        Dim v As Integer
        Dim p As Integer = _Maximum - _Minimum

        If p = 0 Then
           v = w
        Else
           v = Convert.ToInt32((w / p) * (_Value - _Minimum))
        End If

        If p = 0 OrElse Not (_Value = _Minimum) Then
           e.Graphics.FillRectangle(New SolidBrush(Me.ForeColor), New Rectangle(x, y, v, h))
        End If

        If Not (_Value = _Maximum) Then
           e.Graphics.FillRectangle(New SolidBrush(Me.BackColor), New Rectangle(v + x, y, w - v, h))
        End If

        pn_Standard.Dispose()
        pn_Dark.Dispose()
        pn_Light.Dispose()

        pn_Standard = Nothing
        pn_Dark = Nothing
        pn_Light = Nothing
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        SetDrawPoint()

        Me.Invalidate()
    End Sub

End Class

End Namespace
