﻿'
'   SingleLine, Ellipsis (DrawText) / Corner trimming of Groupbox 1.0.2.1
'
'       Property:
'         LabelRectangle
'         HoverColor
'         HoverEffect
'
'       Method
'         None
'
Imports System.ComponentModel

Namespace UserLib

<DebuggerStepThrough()>
Public Class GroupBoxEx
    Inherits GroupBox

#Region "// Property, Method"

    Private _LabelRectangle As Rectangle
    Private _HoverColor As Color = SystemColors.Highlight
    Private _HoverEffect As Boolean

#Region "** 表示"

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("ボタン上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor() As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。")> _
    Public Property HoverEffect() As Boolean
      Get
        Return _HoverEffect
      End Get

      Set(ByVal value As Boolean)
        _HoverEffect = value

        If Not _onCreate Then Return

        If Me.Enabled Then
           If value Then
              _onMouse = _LabelRectangle.Contains(Me.PointToClient(Cursor.Position))

              If _onMouse Then

                 Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))

              'MouseEnter を発生
                 Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
              End If
           Else
              If _onMouse Then
                 _onMouse = False

                 Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
              End If
           End If
        Else
           _onMouse = False
        End If
      End Set
    End Property

#End Region

#Region "** その他"

''' <summary>
''' ラベルの位置を取得します。
''' </summary>

    <Browsable(False)>
    Public ReadOnly Property LabelRectangle() As Rectangle
      Get
        Return _LabelRectangle
      End Get
    End Property

#End Region

#End Region

    Private _onCreate As Boolean
    Private _onMouse As Boolean

    Private _LabelOffset As Integer = 8                   '枠からラベル迄の距離(X)
    Private _ExceptWidth As Integer = _LabelOffset * 2    '枠のラベルに対するパディングの合計
    Private _MeasureTextWidth As Integer

    Private _pn_Dark As New Pen(SystemColors.ControlDark)
    Private _pn_Light As New Pen(SystemColors.ControlLightLight)
    Private _pt_DarkLine As Point()
    Private _pt_LightLine As Point()

    Private _tf_Default As TextFormatFlags = TextFormatFlags.SingleLine Or TextFormatFlags.EndEllipsis
    Private _tf_Drawing As TextFormatFlags = _tf_Default

#Region "// Function, Sub"

''' <summary>
''' ラベルの文字列の幅 (左右余白含む) を取得します。
''' </summary>

    Private Sub SetMeasureTextWidth()

        If String.IsNullOrEmpty(Me.Text) Then
           _MeasureTextWidth = 0
           Return
        End If

        Dim h As Integer = Me.Font.Height
        Dim w As Integer = Me.Text.Length * Convert.ToInt32(h * 1.2F)
        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        _MeasureTextWidth = TextRenderer.MeasureText(g, Me.Text, Me.Font, b.Size, TextFormatFlags.SingleLine).Width

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing
    End Sub

    Private Sub SetDrawPoint()

        Dim y As Integer = (Me.Font.Height \ 2) - 1
        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim dw As Integer = w - _ExceptWidth    'ラベル最大描画幅
        Dim lx As Integer                       'ラベル表示座標(X)
        Dim lw As Integer = _MeasureTextWidth   'ラベル表示に必要な幅
        Dim lh As Integer = Me.Font.Height      'ラベル表示高さ

        Dim x0, x1 As Integer

     'ラベル表示幅，高さ修正
        If lw > dw Then

        'ラベル最低描画幅判定
           If dw < 6 Then
              lw = 0
           Else
              lw = dw
           End If
        End If

        '枠線座標(開始，終了)、ラベル位置設定
           If Me.RightToLeft = RightToLeft.Yes Then

              x0 = w - (lw + _LabelOffset) - 1
              x1 = w - _LabelOffset
              lx = x0 + 1
           Else

              x0 = _LabelOffset - 1
              x1 = x0 + lw + 1
              lx = x0 + 1
           End If

    ' ラベルの描画・表示位置セット
        _LabelRectangle = New Rectangle(lx, 0, lw, lh)

    ' Dark Line
        _pt_DarkLine = {
            New Point(x0, y), New Point(2, y), New Point(0, y + 2), New Point(0, h - 4),
            New Point(2, h - 2), New Point(w - 3, h - 2), New Point(w - 1, h - 4), New Point(w - 1, y + 2),
            New Point(w - 3, y), New Point(x1, y)}

    ' Light Line
        y += 1

        _pt_LightLine = {
            New Point(x0, y), New Point(2, y), New Point(1, y + 1), New Point(1, h - 4),
            New Point(0, h - 3), New Point(2, h - 1), New Point(w - 3, h - 1), New Point(w - 1, h - 3),
            New Point(w - 2, h - 4), New Point(w - 2, y + 1), New Point(w - 3, y), New Point(x1, y)}
    End Sub

#End Region

    Public Sub New()

        Me.DoubleBuffered = True
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        SetMeasureTextWidth()
        SetDrawPoint()

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler Me.FontChanged, AddressOf Me_FontTextChanged
        AddHandler Me.TextChanged, AddressOf Me_FontTextChanged

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        If _LabelRectangle.Width > 0 Then

          Dim c As Color

          If Me.Enabled Then
              If _HoverEffect AndAlso _onMouse Then
                 c = _HoverColor
              Else
                 c = Me.ForeColor
              End If

              TextRenderer.DrawText(e.Graphics, Me.Text, Me.Font, _
                _LabelRectangle, c, Me.BackColor, _tf_Drawing)
           Else

              TextRenderer.DrawText(e.Graphics, Me.Text, Me.Font, _
                _LabelRectangle, SystemColors.GrayText, Me.BackColor, _tf_Drawing)
           End If
        End If

        e.Graphics.DrawLines(_pn_Dark, _pt_DarkLine)
        e.Graphics.DrawLines(_pn_Light, _pt_LightLine)
    End Sub

#Region "// Me"

    Private Sub Me_RightToLeftChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.RightToLeftChanged

  ' RightToLeft Change
  '  ⇒ OnCreateControl()
  '  ⇒ RightToLeftChanged()

        If Me.RightToLeft = RightToLeft.Yes Then
           _tf_Drawing = _tf_Default Or TextFormatFlags.RightToLeft
        Else
           _tf_Drawing = _tf_Default
        End If

        If _onCreate Then
           SetMeasureTextWidth()
           SetDrawPoint()
        End If
    End Sub

    Private Sub Me_FontTextChanged(ByVal sender As Object, ByVal e As EventArgs)
        SetMeasureTextWidth()
        SetDrawPoint()
    End Sub

    Private Sub Me_EnabledChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.EnabledChanged

        If Not _HoverEffect Then Return

        If Me.Enabled Then
           _onMouse = _LabelRectangle.Contains(Me.PointToClient(Cursor.Position))

           If _onMouse Then
              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))

           'MouseEnter を発生
              Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
           End If
        Else
           _onMouse = False
        End If
    End Sub

    Private Sub Me_Resize(ByVal sender As Object, ByVal e As EventArgs)

        SetDrawPoint()
    End Sub

    Private Sub Me_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs) Handles Me.MouseMove

        If Not _HoverEffect Then Return

        If _LabelRectangle.Contains(Me.PointToClient(Cursor.Position)) Then
           If Not _onMouse Then
              _onMouse = True

              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
           End If
        Else
           If _onMouse Then
              _onMouse = False

              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
           End If
        End If
    End Sub

    Private Sub Me_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseLeave

        If Not _HoverEffect Then Return

        If _onMouse Then
           _onMouse = False

           If Me.Enabled Then

           'MouseLeave では onPaint が発生しない為、強制呼出し
              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
           End If
        End If
    End Sub

#End Region

End Class

End Namespace
