﻿'
'   Corner trimming of Groupbox 1.2.7.1
'
'       Property:
'         LabelRectangle
'         TrailingSpaces
'         DisabledColorState
'         DisabledColor
'         HoverColor
'         HoverEffect
'
'       Method
'         None
'
Imports System.ComponentModel

Namespace UserLib

<DebuggerStepThrough()>
Public Class GroupBoxEx
    Inherits GroupBox

#Region "// Property, Method"

    Private _DrawRectangle As RectangleF
    Private _LabelRectangle As Rectangle
    Private _TrailingSpaces As Boolean
    Private _HoverColor As Color = SystemColors.Highlight
    Private _HoverEffect As Boolean

#Region "** 表示"

''' <summary>
''' Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）
''' </summary>

    <DefaultValue(GetType(Color), "GrayText"), Category("表示"),
     Description("Enabled が False 時のラベルの色を指定します。（DisabledColorState が False の時のみ有効です。）")> _
    Public Property DisabledColor() As Color
      Get
        Return _DisabledColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           _DisabledColor = SystemColors.GrayText
        Else
           _DisabledColor = value
        End If
      End Set
    End Property

''' <summary>
''' Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。
''' </summary>

    <DefaultValue(True), Category("表示"),
     Description("Enabled が False 時のラベル表示状態を変更します。Flase 時には DisabledColor の値が適用されます。")> _
    Public Property DisabledColorState() As Boolean
      Get
        Return _DisabledColorState
      End Get

      Set(ByVal value As Boolean)
        _DisabledColorState = value

        If _onCreate AndAlso (Not Me.Enabled) Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor() As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。")> _
    Public Property HoverEffect() As Boolean
      Get
        Return _HoverEffect
      End Get

      Set(ByVal value As Boolean)
        _HoverEffect = value

        If Not _onCreate Then Return

        If Me.Enabled Then
           If value Then
              _onMouse = _LabelRectangle.Contains(Me.PointToClient(Cursor.Position))

              If _onMouse Then
                 Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))

              'MouseEnter を発生
                 Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
              End If
           Else
              If _onMouse Then
                 _onMouse = False

                 Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
              End If
           End If
        Else
           _onMouse = False
        End If
      End Set
    End Property

''' <summary>
''' ラベル末尾のスペース (最初の行のみ) を有効にするかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル末尾のスペース (最初の行のみ) を有効にするかを指定します。")> _
    Public Property TrailingSpaces() As Boolean
      Get
        Return _TrailingSpaces
      End Get

      Set(ByVal value As Boolean)
        _TrailingSpaces = value

        SetMeasureTextWidth()
        SetDrawPoint()

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

#End Region

#Region "** その他"

''' <summary>
''' ラベルの位置を取得します。
''' </summary>

    <Browsable(False)>
    Public ReadOnly Property LabelRectangle() As Rectangle
      Get
        Return _LabelRectangle
      End Get
    End Property

#End Region

#End Region

    Private _onCreate As Boolean
    Private _onMouse As Boolean

    Private _LabelOffset As Integer = 7                         '枠からラベル迄の距離(X)
    Private _ExceptWidth As Integer = (_LabelOffset + 1) * 2    '枠のラベルに対するパディングの合計
    Private _MeasureTextWidth As Integer

    Private _DisabledColorState As Boolean = True
    Private _DisabledColor As Color = SystemColors.GrayText

    Private _pn_Dark As New Pen(SystemColors.ControlDark)
    Private _pn_Light As New Pen(SystemColors.ControlLightLight)
    Private _pt_DarkLine As Point()
    Private _pt_LightLine As Point()

    Private _sf_Default As New StringFormat
    Private _sf_Drawing As New StringFormat
    Private _sb_Drawing As New SolidBrush(Me.ForeColor)
    Private _tf_Default As TextFormatFlags
    Private _tf_Drawing As TextFormatFlags

#Region "// Function, Sub"

''' <summary>
''' ラベルの文字列の幅 (左右余白含む) を取得します。
''' </summary>

    Private Sub SetMeasureTextWidth()

        If String.IsNullOrEmpty(Me.Text) OrElse
          (String.IsNullOrEmpty(Me.Text.Trim) AndAlso Not _TrailingSpaces) Then
           _MeasureTextWidth = 0
           Return
        End If

        Dim h As Integer = Me.Font.Height
        Dim w As Integer = Me.Text.Length * Convert.ToInt32(h * 1.2) + h    'italic対策 (x1.2)
        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        If _TrailingSpaces AndAlso Me.Text.EndsWith(" ") Then
           _MeasureTextWidth = TextRenderer.MeasureText(g, Me.Text, Me.Font, b.Size, _tf_Drawing).Width

        Else

           Dim mw As Integer = Convert.ToInt32(Math.Ceiling(
                               g.MeasureString(Me.Text.TrimEnd, Me.Font, w, _sf_Drawing).Width))

           g.DrawString(Me.Text, Me.Font, New SolidBrush(Color.White), New RectangleF(0, 0, w, h), _sf_Drawing)

           Dim c As Color = b.GetPixel(0, 0)
           Dim BackColorArgb As Integer = c.ToArgb()

           _MeasureTextWidth = -1

           If Me.RightToLeft = RightToLeft.Yes Then
              For xp As Integer = w - mw To w - 1
                For yp As Integer = 0 To h - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     _MeasureTextWidth = (w - xp) + 3                      '左余白(1+2)を加算

                  '斜体の余白分を加算
                     If Me.Font.Italic Then _MeasureTextWidth += 2

                     Exit For
                  End If
                Next
                If _MeasureTextWidth >= 0 Then Exit For
              Next
           Else
              For xp As Integer = mw To 0 Step -1
                For yp As Integer = 0 To h - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     _MeasureTextWidth = xp + 3                           '右余白(1+2)を加算
                     Exit For
                  End If
                Next
                If _MeasureTextWidth >= 0 Then Exit For
              Next
           End If
        End If

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing
    End Sub

    Private Function GetMeasureText(ByVal width As Integer, ByVal height As Integer) As TextInfo

        If String.IsNullOrEmpty(Me.Text.Trim) Then
           Return New TextInfo
        End If

        Dim b As New Bitmap(width, height)
        Dim g As Graphics = Graphics.FromImage(b)

        Dim m As SizeF = g.MeasureString(Me.Text, Me.Font, New Size(width, height))
        Dim s As Integer
        Dim h As Integer = Convert.ToInt32(Math.Ceiling(m.Height))
        Dim w As Integer = Convert.ToInt32(Math.Ceiling(m.Width))

        If height > h Then height = h

        If width > w Then
           s = width - w
           width = w
        End If

        g.DrawString(Me.Text, Me.Font, New SolidBrush(Color.White), New RectangleF(0, 0, width, height), _sf_Drawing)

        Dim l As Integer = -1
        Dim r As Integer = -1
        Dim c As Color = b.GetPixel(0, 0)
        Dim BackColorArgb As Integer = c.ToArgb()

        For xp As Integer = 0 To width - 1
          For yp As Integer = 0 To height - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               l = xp
               Exit For
            End If
          Next
          If l >= 0 Then Exit For
        Next

        For xp As Integer = width - 1 To l Step -1
          For yp As Integer = 0 To height - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               r = (width - 1) - xp
               Exit For
            End If
          Next
          If r >= 0 Then Exit For
        Next

        w = width - (l + r)

        If Me.RightToLeft = RightToLeft.Yes Then
           l += s
        Else
           r += s
        End If

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing

        Return New TextInfo(l, r, w, height)
    End Function

    Private Sub SetDrawPoint()

        Dim y As Integer = (Me.Font.Height \ 2) - 1
        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim dw As Integer = w - _ExceptWidth    'ラベル最大描画幅
        Dim dx As Integer                       'ラベル描画座標(X)
        Dim sx As Integer                       'ラベル描画シフト値
        Dim lx As Integer                       'ラベル表示座標(X)
        Dim lw As Integer = _MeasureTextWidth   'ラベル表示に必要な幅
        Dim lh As Integer                       'ラベル表示高さ

        Dim ti As New TextInfo
        Dim x0, x1 As Integer

     'ラベル表示幅，高さ修正
        If lw > dw Then

     'ラベル複数行 (左右余白均等表示／TrailingSpaces 有効時で末尾の空白をすべて描画できない場合は１行)

        '左右の余白分を確保
           dw -= 4

        'ラベル最低描画幅判定
           If dw < 6 Then
              lw = 0
           Else
              ti = GetMeasureText(dw, Me.Height)
              lw = ti.Width + 4
              lh = ti.Height
           End If

        '枠線座標(開始，終了)、ラベル位置(描画，表示)設定
           If Me.RightToLeft = RightToLeft.Yes Then

              x0 = _LabelOffset + ti.Horizontal
              x1 = (w - 1) - _LabelOffset
              lx = x0 + 1
              dx = _LabelOffset + (ti.Right + 3)
           Else
              x0 = _LabelOffset
              x1 = x0 + lw + 1
              lx = x0 + 1
              dx = lx + (2 - ti.Left)
           End If
        Else

     'ラベル１行のみ (簡易表示)

        '文字描画時（RightToLeft)の先頭に付与する余白(グリフ突出対策)をシフト値として設定
           sx = Me.Font.Height + 2

        '文字描画時、一部のフォントで余白があるのに改行する不具合に対する措置
        ' (描画領域内に確実に文字を描画する為の措置)
           dw = lw + sx

           lh = Me.Font.Height

        '枠線座標(開始，終了)、ラベル位置(描画，表示)設定
           If Me.RightToLeft = RightToLeft.Yes Then

              x0 = w - (lw + _LabelOffset) - 2
              x1 = w - _LabelOffset - 1
              lx = x0 + 1
              dx = lx - sx             '余白分を左へシフト

           '斜体突出分を左へシフト
              If Me.Font.Italic Then dx -= 2
           Else
              x0 = _LabelOffset
              x1 = x0 + lw + 1
              lx = x0 + 1
              dx = lx
           End If
        End If

    ' ラベルの描画・表示位置セット
        _DrawRectangle = New RectangleF(dx, 0, dw, lh)
        _LabelRectangle = New Rectangle(lx, 0, lw, lh)

    ' Dark Line
        _pt_DarkLine = {
            New Point(x0, y), New Point(2, y), New Point(0, y + 2), New Point(0, h - 4),
            New Point(2, h - 2), New Point(w - 3, h - 2), New Point(w - 1, h - 4), New Point(w - 1, y + 2),
            New Point(w - 3, y), New Point(x1, y)}

    ' Light Line
        y += 1
        _pt_LightLine = {
            New Point(x0, y), New Point(2, y), New Point(1, y + 1), New Point(1, h - 4),
            New Point(0, h - 3), New Point(2, h - 1), New Point(w - 3, h - 1), New Point(w - 1, h - 3),
            New Point(w - 2, h - 4), New Point(w - 2, y + 1), New Point(w - 3, y), New Point(x1, y)}
    End Sub

#End Region

    Public Sub New()

        _tf_Default = TextFormatFlags.WordBreak Or TextFormatFlags.TextBoxControl
        _tf_Drawing = _tf_Default

        _sf_Drawing.FormatFlags = _sf_Default.FormatFlags

        Me.DoubleBuffered = True
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        SetMeasureTextWidth()
        SetDrawPoint()

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler Me.FontChanged, AddressOf Me_FontTextChanged
        AddHandler Me.TextChanged, AddressOf Me_FontTextChanged

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        If _LabelRectangle.Width > 0 Then

          e.Graphics.FillRectangle(New SolidBrush(Me.BackColor), _LabelRectangle)

          If Me.Enabled Then
              If _HoverEffect AndAlso _onMouse Then
                 _sb_Drawing.Color = _HoverColor
              Else
                 _sb_Drawing.Color = Me.ForeColor
              End If

              e.Graphics.DrawString(Me.Text, Me.Font, _sb_Drawing, _DrawRectangle, _sf_Drawing)
           Else
              If _DisabledColorState Then
                 ControlPaint.DrawStringDisabled(e.Graphics, Me.Text, Me.Font,
                   Me.BackColor,
                   _DrawRectangle, _sf_Drawing)
              Else
                 _sb_Drawing.Color = _DisabledColor
                 e.Graphics.DrawString(Me.Text, Me.Font, _sb_Drawing, _DrawRectangle, _sf_Drawing)
              End If
           End If
        End If

        e.Graphics.DrawLines(_pn_Dark, _pt_DarkLine)
        e.Graphics.DrawLines(_pn_Light, _pt_LightLine)
    End Sub

#Region "// Me"

    Private Sub Me_RightToLeftChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.RightToLeftChanged

  ' RightToLeft Change
  '  ⇒ OnCreateControl()
  '  ⇒ RightToLeftChanged()

        If Me.RightToLeft = RightToLeft.Yes Then
           _tf_Drawing = _tf_Default Or TextFormatFlags.RightToLeft
           _sf_Drawing.FormatFlags = _sf_Default.FormatFlags Or StringFormatFlags.DirectionRightToLeft
        Else
           _tf_Drawing = _tf_Default
           _sf_Drawing.FormatFlags = _sf_Default.FormatFlags
        End If

        If _onCreate Then
           SetMeasureTextWidth()
           SetDrawPoint()
        End If
    End Sub

    Private Sub Me_FontTextChanged(ByVal sender As Object, ByVal e As EventArgs)
        SetMeasureTextWidth()
        SetDrawPoint()
    End Sub

    Private Sub Me_EnabledChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.EnabledChanged

        If Not _HoverEffect Then Return

        If Me.Enabled Then
           _onMouse = _LabelRectangle.Contains(Me.PointToClient(Cursor.Position))

           If _onMouse Then
              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))

           'MouseEnter を発生
              Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
           End If
        Else
           _onMouse = False
        End If
    End Sub

    Private Sub Me_Resize(ByVal sender As Object, ByVal e As EventArgs)

        SetDrawPoint()
    End Sub

    Private Sub Me_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs) Handles Me.MouseMove

        If Not _HoverEffect Then Return

        If _LabelRectangle.Contains(Me.PointToClient(Cursor.Position)) Then
           If Not _onMouse Then
              _onMouse = True

              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
           End If
        Else
           If _onMouse Then
              _onMouse = False

              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
           End If
        End If
    End Sub

    Private Sub Me_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseLeave

        If Not _HoverEffect Then Return

        If _onMouse Then
           _onMouse = False

           If Me.Enabled Then
           'MouseLeave では onPaint が発生しない為、強制呼出し
              Me.OnPaint(New PaintEventArgs(Me.CreateGraphics, Me.Bounds))
           End If
        End If
    End Sub

#End Region

End Class

<DebuggerStepThrough()>
Class TextInfo

    Public Property Left() As Integer
    Public Property Right() As Integer
    Public Property Width() As Integer
    Public Property Height() As Integer

    Public ReadOnly Property Horizontal() As Integer
        Get
          Return Me.Left + Me.Right
        End Get
    End Property

    Public Sub New()
        Me.Left = 0
        Me.Right = 0
        Me.Width = 0
        Me.Height = 0
    End Sub

    Public Sub New(ByVal Left As Integer, ByVal Right As Integer, ByVal Width As Integer, ByVal Height As Integer)
        Me.Left = Left
        Me.Right = Right
        Me.Width = Width
        Me.Height = Height
    End Sub

End Class

End Namespace
