﻿'
'   Horizontal TrackBar 1.1.1.1 (Slider Tip Support Ver.)
'
'       Required Class: 
'         ColorEx.vb
'
'       Property:
'         Minimum,     Maximum
'         SmallChange, LargeChange
'         Value
'         ReferenceValue (Read Only)
'
'         Padding
'
'         BarHeight
'         HoverColor
'         SliderSize
'         TickFrequency
'         TickStyle
'
'       Method
'         None

Imports System.ComponentModel
Imports System.Drawing.Drawing2D

Namespace UserLib

<DebuggerStepThrough()>
Public Class HorizontalTrackBar

    Public Event ValueChanged(sender As Object, e As EventArgs)
    Public Event ReferenceValueChanged(sender As Object, e As EventArgs)

    Public Event SliderMouseEnter(sender As Object, e As EventArgs)
    Public Event SliderMouseLeave(sender As Object, e As EventArgs)
    Public Event SliderMouseMove(sender As Object, e As MouseEventArgs)

Public Enum Slider As Integer
    Small = 0
    MediumShort = 1
    Medium = 2
    MediumLong = 3
    Large = 4
End Enum

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AutoValidate() As AutoValidate
    <Browsable(False)> _
    Public Shadows Property ImeMode() As ImeMode

    <Browsable(False)> _
    Public Shadows Property AutoScroll() As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin() As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize() As Size
    <Browsable(False)> _
    Public Shadows Property AutoSize() As Boolean
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode() As AutoSizeMode

    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout
    <Browsable(False)> _
    Public Shadows Property BorderStyle() As BorderStyle
   <Browsable(False)> _
    Public Shadows Property Font() As Font
    <Browsable(False)> _
    Public Shadows Property RightToLeft() As RightToLeft

#End Region

    Private _Padding As Padding

    Private _Value As Integer
    Private _Maximum As Integer = 10
    Private _Minimum As Integer
    Private _ReferenceValue As Integer      '参考値

    Private _SmallChange As Integer = 1
    Private _LargeChange As Integer = 5

    Private _BarHeight As Integer = 4
    Private _SliderSize As Slider = Slider.Medium
    Private _TickFrequency As Integer = 1
    Private _TickStyle As TickStyle = TickStyle.BottomRight

    Private _HoverColor As Color = SystemColors.Highlight

#Region "** 動作"

''' <summary>
''' キーボードの入力（Page Up および Page Downキー）に対して、スライダーが移動する位置の数を取得または設定します。
''' </summary>

    <DefaultValue(5), Category("動作"),
     Description("キーボードの入力（Page Up および Page Downキー）に対して、スライダーが移動する位置の数です。")> _
    Public Property LargeChange() As Integer
      Get
        Return _LargeChange
      End Get

      Set(value As Integer)
        If value < 1 Then value = 1

        _LargeChange = value
      End Set
    End Property

''' <summary>
''' このコントロールが使用する範囲の上限値を取得または設定します。
''' </summary>

    <DefaultValue(10), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("トラック バー 上のスライダー位置の最大値です。")> _
    Public Property Maximum() As Integer
      Get
        Return _Maximum
      End Get

      Set(value As Integer)
        If value <= _Minimum Then Return

        _Maximum = value

        If value < _Value Then
           Me.Value = value
        Else
           If _onCreate Then SetSliderLocation()
        End If

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' このコントロールが使用する範囲の下限値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("トラック バー 上のスライダー位置の最小値です。")> _
    Public Property Minimum() As Integer
      Get
        Return _Minimum
      End Get

      Set(value As Integer)
        If value >= _Maximum Then Return

        _Minimum = value

        If value > _Value Then
           Me.Value = value
        Else
           If _onCreate Then SetSliderLocation()
        End If

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' キーボードの入力（方向キー）に対して、スライダーが移動する位置の数を取得または設定します。
''' </summary>

    <DefaultValue(1), Category("動作"),
     Description("キーボードの入力（方向キー）に対して、スライダーが移動する位置の数です。")> _
    Public Property SmallChange() As Integer
      Get
        Return _SmallChange
      End Get

      Set(value As Integer)
        If value < 1 Then value = 1

        _SmallChange = value
      End Set
    End Property

''' <summary>
''' トラック バーの上に Tip を表示する際に必要になる処理を使用するかを示す値を取得または設定します。
''' </summary>

    <DefaultValue(True), Category("動作"),
     Description("トラック バー上に Tip を表示する際に必要な処理を使用するかを指定します。")> _
    Public Property TipSupport As Boolean = True

''' <summary>
''' トラック バーにおけるスライダーの現在位置を表す数値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("動作"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("最小および最大プロパティで指定された範囲内のスライダーの現在の位置です。")> _
    Public Property Value() As Integer
      Get
        Return _Value
      End Get

      Set(value As Integer)

        If value < _Minimum Then
           If _Value = _Minimum Then Return

           _Value = _Minimum

        ElseIf value > _Maximum Then
           If _Value = _Maximum Then Return

           _Value = _Maximum

        Else
           If value = _Value Then Return

           _Value = value
        End If

        If _onCreate Then
           SetSliderLocation()

           Me.Invalidate()
        End If

        RaiseEvent ValueChanged(Me, New EventArgs)
      End Set
    End Property

''' <summary>
''' トラック バーにおけるスライダーの現在位置を表す数値（未確定値）を取得します。
''' </summary>

    <Browsable(False), Category("動作")> _
    Public ReadOnly Property ReferenceValue As Integer
        Get
          Return _ReferenceValue
        End Get
    End Property

#End Region

#Region "** 配置"

''' <summary>
''' コントロールの内部スペースを指定する値を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Padding), "0,0,0,0"), Category("配置"),
     Description("コントロールの内部スペースを指定します。")> _
    Public Shadows Property Padding() As Padding
      Get
        Return _Padding
      End Get

      Set(value As Padding)
        _Padding = value

        If _onCreate Then

           SetMeSize()
           SetSliderInfo()
           SetSliderLocation()

           DrawSlider(Me.Enabled)

           Me.Invalidate()
        End If
      End Set
    End Property

#End Region

#Region "** 表示"

''' <summary>
''' トラック（バー）の高さを示す値を取得または設定します。
''' </summary>

    <DefaultValue(4), Category("表示"),
     Description("トラック（バー）の高さを指定します。")> _
    Public Property BarHeight As Integer
      Get
        Return _BarHeight
      End Get

      Set(value As Integer)
        If value < 4 Then
           value = 4
        ElseIf value > 8 Then
           value = 8
        End If

        _BarHeight = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' スライダー上にマウスカーソルがある事を示す値を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("スライダー上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor() As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If

        If _onCreate Then SetHoverColor()
      End Set
    End Property

''' <summary>
''' スライダーのサイズを取得または設定します。
''' </summary>

    <DefaultValue(Slider.Medium), Category("表示"),
     Description("スライダーのサイズを指定します。")> _
    Public Property SliderSize() As Slider
      Get
        Return _SliderSize
      End Get

      Set(value As Slider)
        _SliderSize = value

        If _onCreate Then

           SetMeSize()
           SetSliderInfo()
           SetSliderLocation()

           DrawSlider(Me.Enabled)

           Me.Invalidate()
        End If
      End Set
    End Property

''' <summary>
''' コントロールに描画された目盛り間のデルタを指定する値を取得または設定します。
''' </summary>

    <DefaultValue(1), Category("表示"),
     Description("目盛りマーク間の位置の数です。")> _
    Public Property TickFrequency() As Integer
      Get
        Return _TickFrequency
      End Get

      Set(value As Integer)
        If value < 1 Then
           value = 1
        Else
           Dim diff As Integer = _Maximum - _Minimum

           If value > diff Then
              value = diff
           End If
        End If

        _TickFrequency = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' トラックバー上に目盛りを表示する方法を示す値を取得または設定します。
''' </summary>

    <DefaultValue(TickStyle.BottomRight), Category("表示"),
     Description("目盛りをトラックバーのどこに表示するかを指定します。")> _
    Public Property TickStyle() As TickStyle
      Get
        Return _TickStyle
      End Get

      Set(value As TickStyle)
        _TickStyle = value

        If _onCreate Then

           SetMeSize()
           SetSliderInfo()

           DrawSlider(Me.Enabled)

           Me.Invalidate()
        End If
      End Set
    End Property

#End Region

#End Region

#Region "// constant, variable declaration"

    Private _onCreate As Boolean
    Private _onEnter As Boolean
    Private _onSliderMouseEnter As Boolean
    Private _onSliderMouseMove As Boolean

    Private Const _ScaleHeight = 5  '(Scale:4, Space:1)

    Private _Slider_Size As Size() = {
              New Size(7, 14), New Size(9, 18), New Size(11, 22),
              New Size(13, 26), New Size(15, 30)}

    Private _CursorPosition As Integer
    Private _SliderLocation As Point
    Private _BarRectangle As Rectangle

    Private _cl_HoverDark As Color
    Private _cl_HoverLight1 As Color
    Private _cl_HoverLight2 As Color

    Private _pn_Contours As New Pen(SystemColors.ControlDarkDark)
    Private _pn_Standard As New Pen(SystemColors.Control)
    Private _pn_Dark As New Pen(SystemColors.ControlDark)
    Private _pn_Light As New Pen(SystemColors.ControlLightLight)

    Private _pt_Contours As Point()
    Private _pt_DarkLine As Point()
    Private _pt_LightLine As Point()

#End Region

#Region "// Sub-Routine & Function"

''' <summary>
''' カーソルキーを押した時に KeyDownイベントを発生させます。
''' </summary>

    Protected Overrides Function IsInputKey(keyData As Keys) As Boolean

'*******************************************************************************
'   UserControl でカーソルキーを押した時に KeyDown イベントを発生させる
'*******************************************************************************

        If keyData = Keys.Right OrElse keyData = Keys.Left OrElse
            keyData = Keys.Up OrElse keyData = Keys.Down Then
            Return True
        End If

        Return MyBase.IsInputKey(keyData)
    End Function

''' <summary>
''' value で指定された数値の小数部を切り上げて Int32 へ変換します。 
''' </summary>

    Private Function CeilingToInt32(ByVal value As Double) As Integer
        Return Convert.ToInt32(Math.Ceiling(value))
    End Function

''' <summary>
''' HoverColor を基に、明暗色を生成します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub SetHoverColor()

        Dim H As Single = _HoverColor.GetHue
        Dim S As Single = _HoverColor.GetSaturation
        Dim B As Single = _HoverColor.GetBrightness

        Dim c0 As Single = 1.0F - B
        Dim c1 As Single = B * 0.665F
        Dim c2 As Single = B - c1

        If c0 > 0.4F Then
           c0 = 0.4F
        End If

        _cl_HoverDark = ColorEx.ColorHSVtoRGB(H, S, c1)
        _cl_HoverLight1 = ColorEx.ColorHSVtoRGB(H, S, B + (c0 * 0.6F))
        _cl_HoverLight2 = ColorEx.ColorHSVtoRGB(H, S, B + (c0 * 0.9F))
    End Sub

''' <summary>
''' コントロールの高さ、および、Region を設定します。
''' </summary>

    Private Sub SetMeSize()

    'Me.ReSize, TickStyle, SliderSize

        Dim h As Integer = Me.Padding.Vertical + _Slider_Size(_SliderSize).Height

        If _TickStyle <> TickStyle.None Then
           If _TickStyle = TickStyle.Both Then
              h += (_ScaleHeight * 2)
           Else
              h += _ScaleHeight
           End If
        End If

        Dim w As Integer = Me.Padding.Horizontal + (_Slider_Size(_SliderSize).Width * 4)

      RemoveHandler Me.Resize, AddressOf Me_Resize
        If Me.Width < w Then
           Me.Width = w
        End If

        Me.Height = h
      AddHandler Me.Resize, AddressOf Me_Resize

        w = Me.Width
        h = Me.Height

        Dim path As New GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

''' <summary>
''' スライダーのサイズ、Region、位置(y)、および、描画情報を設定します。
''' </summary>

    Private Sub SetSliderInfo()

    'TickStyle, SliderSize

    'Size
        Dim w As Integer = _Slider_Size(_SliderSize).Width
        Dim h As Integer = _Slider_Size(_SliderSize).Height

        Pic_Slider.Width = w
        Pic_Slider.Height = h

    'Region
        Dim path As New GraphicsPath()

        Dim d0 As Integer = w \ 2
        Dim d1 As Integer = h - d0 - 1
        Dim d2 As Integer = w - 2
        Dim d3 As Integer = h - 1

        Select Case _TickStyle
          Case TickStyle.TopLeft
            path.AddLines({New Point(0, d0), New Point(d0, -1), New Point(w, d0)})

            path.AddRectangle(New Rectangle(0, d0, w, d1))
            path.AddRectangle(New Rectangle(1, d3, d2, 1))

          Case TickStyle.BottomRight
            path.AddRectangle(New Rectangle(1, 0, d2, 1))
            path.AddRectangle(New Rectangle(0, 1, w, d1 - 1))

            path.AddLines({New Point(0, d1), New Point(d0, h), New Point(w, d1)})

          Case Else
            path.AddRectangle(New Rectangle(1, 0, d2, 1))
            path.AddRectangle(New Rectangle(0, 1, w, d3 - 1))
            path.AddRectangle(New Rectangle(1, d3, d2, 1))
        End Select

        Pic_Slider.Region = New Region(path)

        path.Dispose()
        path = Nothing

    'Location (y)
        If _TickStyle Mod 2 = 0 Then    'None, Right
           Pic_Slider.Top = Me.Padding.Top
        Else
           Pic_Slider.Top = Me.Padding.Top + _ScaleHeight
        End If

    'Line Point
        w = _Slider_Size(_SliderSize).Width - 1
        h = _Slider_Size(_SliderSize).Height - 1

        d0 = _Slider_Size(_SliderSize).Width \ 2
        d1 = h - d0
        d2 = w - 1
        d3 = h - 1

        Select Case _TickStyle
          Case TickStyle.TopLeft

            _pt_Contours = {
              New Point(0, d0), New Point(d0, 0), New Point(w, d0), New Point(w, d3),
              New Point(d2, h), New Point(1, h), New Point(0, d3), New Point(0, d0)}

            _pt_DarkLine = {
              New Point(1, d3), New Point(1, d0), New Point(d0, 1), New Point(d2, d0),
              New Point(d2, d3)}

            _pt_LightLine = {
              New Point(1, d0 + 1), New Point(d0, 2), New Point(d2, d0 + 1), New Point(d2, h - 2),
              New Point(w - 2, d3), New Point(2, d3), New Point(1, h - 2), New Point(1, d0 + 1)}

          Case TickStyle.BottomRight

            _pt_Contours = {
              New Point(1, 0), New Point(d2, 0), New Point(w, 1), New Point(w, d1),
              New Point(d0, h), New Point(0, d1), New Point(0, 1), New Point(1, 0)}

            _pt_DarkLine = {
              New Point(1, 1), New Point(1, d1), New Point(d0, d3), New Point(d2, d1),
              New Point(d2, 1)}

            _pt_LightLine = {
              New Point(2, 1), New Point(w - 2, 1), New Point(d2, 2), New Point(d2, d1 - 1),
              New Point(d0, h - 2), New Point(1, d1 - 1), New Point(1, 2), New Point(2, 1)}

          Case Else

            _pt_Contours = {
              New Point(1, 0), New Point(d2, 0), New Point(w, 1), New Point(w, d3),
              New Point(d2, h), New Point(1, h), New Point(0, d3), New Point(0, 1),
              New Point(1, 0)}

            _pt_DarkLine = {
              New Point(1, 1), New Point(1, d3), New Point(d2, d3), New Point(d2, 1)}

            _pt_LightLine = {
              New Point(2, 1), New Point(w - 2, 1), New Point(d2, 2), New Point(d2, h - 2),
              New Point(w - 2, d3), New Point(2, d3), New Point(1, h - 2), New Point(1, 2),
              New Point(2, 1)}
        End Select
    End Sub

''' <summary>
''' スライダーを描画します。
''' </summary>

    Private Sub DrawSlider(ByVal state As Boolean)

    'Me.Enter, Me.Leave, TickStyle, SliderSize

        Dim w As Integer = Pic_Slider.Width
        Dim h As Integer = Pic_Slider.Height

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)
        Dim r As New Rectangle(0, 0, w, h)

        Dim p0, p1, p2 As Pen

        If state Then
           p0 = New Pen(_pn_Contours.Color)

           Dim Brush As New LinearGradientBrush(r, Color.Black, Color.White, LinearGradientMode.Vertical)
           Dim Blend As New ColorBlend()

           Dim c0, c1 As Color

           If _onSliderMouseMove Then

              c0 = _cl_HoverLight1
              c1 = _cl_HoverDark
              p1 = New Pen(_cl_HoverDark)
              p2 = New Pen(_cl_HoverLight1)

           ElseIf _onSliderMouseEnter Then

              c0 = _cl_HoverLight2
              c1 = _HoverColor
              p1 = New Pen(_HoverColor)
              p2 = New Pen(_cl_HoverLight2)

           ElseIf _onEnter Then

              c0 = _pn_Light.Color
              c1 = _pn_Standard.Color
              p1 = New Pen(_pn_Dark.Color)
              p2 = New Pen(_pn_Standard.Color)

           Else

              c0 = _pn_Standard.Color
              c1 = _pn_Dark.Color
              p1 = New Pen(_pn_Dark.Color)
              p2 = New Pen(_pn_Light.Color)

           End If

           Blend.Positions = New Single() {0.0F, 0.4F, 1.0F}
           Blend.Colors = New Color() {c0, c0, c1}
           Brush.InterpolationColors = Blend

           g.FillRectangle(Brush, r)

           Brush.Dispose()
           Brush = Nothing
        Else

           p0 = New Pen(_pn_Dark.Color)
           p1 = New Pen(_pn_Dark.Color)
           p2 = New Pen(_pn_Light.Color)

           g.FillRectangle(New SolidBrush(_pn_Standard.Color), r)
        End If

        g.DrawLines(p0, _pt_Contours)
        g.DrawLines(p1, _pt_DarkLine)
        g.DrawLines(p2, _pt_LightLine)

        Pic_Slider.Image = DirectCast(b.Clone, Drawing.Image)

        p0.Dispose()
        p1.Dispose()
        p2.Dispose()

        p0 = Nothing
        p1 = Nothing
        p2 = Nothing

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing
    End Sub

''' <summary>
''' スライダーの位置(x)をセットします。
''' </summary>

    Private Sub SetSliderLocation()

        Dim width As Integer = Me.Width - Me.Padding.Horizontal - _Slider_Size(_SliderSize).Width
        Dim interval As Double = width / (_Maximum - _Minimum)

        Pic_Slider.Left = Me.Padding.Left + Convert.ToInt32((_Value - _Minimum) * interval)
    End Sub

#End Region

    <DebuggerStepThrough()>
    Public Sub New()

        InitializeComponent()
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        AddHandler Me.Resize, AddressOf Me_Resize   'SetMeSize で Remove、AddHandler している為

        SetMeSize()
        SetHoverColor()
        SetSliderInfo()
        SetSliderLocation()

        DrawSlider(Me.Enabled)

        AddHandler Me.EnabledChanged, AddressOf Me_EnabledChanged
        AddHandler Me.SystemColorsChanged, AddressOf Me_SystemColorsChanged

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height
        Dim x As Integer

        Dim wx As Integer = _Slider_Size(_SliderSize).Width \ 2

        Dim x0 As Integer = Me.Padding.Left + wx
        Dim x1 As Integer = w - Me.Padding.Right - wx - 1
        Dim y0 As Integer
        Dim y1 As Integer

        Dim p As Pen

        If Me.Enabled Then
           p = New Pen(Me.ForeColor)
        Else
           p = New Pen(_pn_Dark.Color)
        End If

        Dim diff As Integer = _Maximum - _Minimum

        Dim ScaleLines As Integer = CeilingToInt32(diff / _TickFrequency) - 1
        Dim ScaleInterval As Double = (w - Me.Padding.Horizontal - _Slider_Size(_SliderSize).Width) / diff * _TickFrequency

        If _TickStyle <> TickStyle.None Then

        'Top Scale
           If (_TickStyle And 1) = 1 Then

              y0 = Me.Padding.Top
              y1 = y0 + 3

              e.Graphics.DrawLine(p, x0, y0, x0, y1)
              e.Graphics.DrawLine(p, x1, y0, x1, y1)

              y0 += 1
              For i = 1 To ScaleLines
                x = x0 + Convert.ToInt32(i * ScaleInterval)
                e.Graphics.DrawLine(p, x, y0, x, y1)
              Next
           End If

        'Bottom Scale
           If (_TickStyle And 2) = 2 Then

              y0 = h - Me.Padding.Bottom - 4
              y1 = y0 + 3

              e.Graphics.DrawLine(p, x0, y0, x0, y1)
              e.Graphics.DrawLine(p, x1, y0, x1, y1)

              y1 -= 1
              For i = 1 To ScaleLines
                x = x0 + Convert.ToInt32(i * ScaleInterval)
                e.Graphics.DrawLine(p, x, y0, x, y1)
              Next
           End If
        End If

     'Bar
        Dim pt As Integer = Me.Padding.Top

        Select Case _TickStyle
          Case TickStyle.TopLeft
            y0 = pt + _ScaleHeight + (_Slider_Size(_SliderSize).Height - _BarHeight) \ 2 + 1

          Case TickStyle.BottomRight

            y0 = pt + (_Slider_Size(_SliderSize).Height - _BarHeight) \ 2 - 1
          Case Else

            y0 = pt + (h - Me.Padding.Vertical - _BarHeight) \ 2
        End Select

        x0 = Me.Padding.Left
        x1 = w - Me.Padding.Right - 1
        y1 = y0 + (_BarHeight - 1)

        wx = w - Me.Padding.Horizontal - 2

        e.Graphics.DrawLines(_pn_Dark, {New Point(x0, y1), New Point(x0, y0), New Point(x1 - 1, y0)})
        e.Graphics.DrawLines(_pn_Light, {New Point(x0 + 1, y1), New Point(x1, y1), New Point(x1, y0)})
        e.Graphics.FillRectangle(New SolidBrush(SystemColors.Window), x0 + 1, y0 + 1, wx, _BarHeight - 2)

        p.Dispose()
        p = Nothing

     'Click イベント用の領域情報をセット
        '_BarRectangle = New Rectangle(x0 + 1, y0 - 3, wx, _BarHeight + 6)
        _BarRectangle = New Rectangle(x0 + 1, _Padding.Top, wx, h - Padding.Vertical)
    End Sub

#Region "// Me"

    Private Sub Me_SystemColorsChanged(sender As Object, e As EventArgs)
        SetHoverColor()

        DrawSlider(Me.Enabled)
    End Sub

    Private Sub Me_EnabledChanged(sender As Object, e As EventArgs)
        DrawSlider(Me.Enabled)
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        SetMeSize()
        SetSliderLocation()

      '目盛更新の為、初期化
        If _TickStyle <> TickStyle.None Then Me.Invalidate()
    End Sub

    Private Sub Me_Enter(sender As Object, e As EventArgs) Handles Me.Enter
        _onEnter = True

        DrawSlider(Me.Enabled)
    End Sub

    Private Sub Me_Leave(sender As Object, e As EventArgs) Handles Me.Leave
        _onEnter = False

        DrawSlider(Me.Enabled)
    End Sub

    Private Sub Me_MouseDown(sender As Object, e As MouseEventArgs) Handles Me.MouseDown

        If e.Button = MouseButtons.Left Then

           Dim p As Point = Me.PointToClient(Cursor.Position)

           If Not _BarRectangle.Contains(p) Then Return

           Dim x As Integer = p.X - Me.Padding.Left - (_Slider_Size(_SliderSize).Width \ 2)
           Dim w As Integer = Me.Width - Me.Padding.Horizontal - _Slider_Size(_SliderSize).Width

           Dim interval As Double = w / (_Maximum - _Minimum)

           If x < 0 Then
              x = 0
           ElseIf x > w Then
              x = w
           End If

           _Value = _Minimum + Convert.ToInt32(x / interval)

           RaiseEvent ValueChanged(Me, New EventArgs)

           Pic_Slider.Left = Me.Padding.Left + Convert.ToInt32((_Value - _Minimum) * interval)
        End If
    End Sub

    Private Sub Me_MouseMove(sender As Object, e As MouseEventArgs) Handles Me.MouseMove

        If Not _TipSupport Then Return

        Dim p As Point = Me.PointToClient(Cursor.Position)
        Dim x As Integer = p.X - Me.Padding.Left - (_Slider_Size(_SliderSize).Width \ 2)
        Dim w As Integer = Me.Width - Me.Padding.Horizontal - _Slider_Size(_SliderSize).Width

        Dim interval As Double = w / (_Maximum - _Minimum)

        If x < 0 Then
           x = 0
        ElseIf x > w Then
           x = w
        End If

        Dim v As Integer = _ReferenceValue
        _ReferenceValue = _Minimum + Convert.ToInt32(x / interval)

        If v <> _ReferenceValue Then
           RaiseEvent ReferenceValueChanged(Me, New EventArgs)
        End If
    End Sub

    Private Sub Me_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown

        Select Case e.KeyData
          Case Keys.Up, Keys.Right
            Me.Value += _SmallChange

          Case Keys.Down, Keys.Left
            Me.Value -= _SmallChange

          Case Keys.PageUp
            Me.Value += _LargeChange

          Case Keys.PageDown
            Me.Value -= _LargeChange
        End Select
    End Sub

#End Region

#Region "// Pic_Slider"

    Private Sub Pic_Slider_MouseEnter(sender As Object, e As EventArgs) Handles Pic_Slider.MouseEnter
        _onSliderMouseEnter = True

        DrawSlider(True)

        RaiseEvent SliderMouseEnter(Me, e)
    End Sub

    Private Sub Pic_Slider_MouseLeave(sender As Object, e As EventArgs) Handles Pic_Slider.MouseLeave
        _onSliderMouseEnter = False

        If Not _onSliderMouseMove Then DrawSlider(True)

        RaiseEvent SliderMouseLeave(Me, e)
    End Sub

    Private Sub Pic_Slider_MouseDown(sender As Object, e As MouseEventArgs) Handles Pic_Slider.MouseDown

        If e.Button = MouseButtons.Left Then

           If Not Me.Focused Then Me.Select()

        'ドラッグ開始時点の位置を取得
           _CursorPosition = Me.PointToClient(Cursor.Position).X
           _SliderLocation = Pic_Slider.Location
        End If
    End Sub

    Private Sub Pic_Slider_MouseUp(sender As Object, e As MouseEventArgs) Handles Pic_Slider.MouseUp

        If Not _onSliderMouseMove Then Return

        _onSliderMouseMove = False

        DrawSlider(True)

        SetSliderLocation()
    End Sub

    Private Sub Pic_Slider_MouseMove(sender As Object, e As MouseEventArgs) Handles Pic_Slider.MouseMove

        RaiseEvent SliderMouseMove(Me, e)

        Static v As Integer
        Static w As Integer
        Static MaxLimit As Integer
        Static interval As Double

        If e.Button <> MouseButtons.Left Then Return

        If Not _onSliderMouseMove Then
           _onSliderMouseMove = True

           v = Me.Width - _Slider_Size(_SliderSize).Width
           w = v - Me.Padding.Horizontal

           MaxLimit = v - Me.Padding.Right
           interval = w / (_Maximum - _Minimum)

           v = _Value

           DrawSlider(True)
        End If

     'ドラッグ中の位置情報を取得して、その位置に表示

        Dim cx As Integer = Me.PointToClient(Cursor.Position).X
        Dim sx As Integer = _SliderLocation.X + (cx - _CursorPosition)

        If sx < Me.Padding.Left Then
           sx = Me.Padding.Left         'カーソルの左端位置
        Else
          If sx > MaxLimit Then
             sx = MaxLimit              'カーソルの右端位置
          End If
        End If

        _Value = _Minimum + Convert.ToInt32((sx - Me.Padding.Left) / interval)

        If v <> _Value Then
           v = _Value

           RaiseEvent ValueChanged(Me, New EventArgs)
        End If

        Pic_Slider.Location = New Point(sx, _SliderLocation.Y)
    End Sub

    Private Sub Pic_Slider_Click(sender As Object, e As EventArgs) Handles Pic_Slider.Click
        If Not Me.Focused Then Me.Select()
    End Sub

#End Region

End Class

End Namespace
