﻿'
'   Label 1.3.2.2
'
'       Property:
'         BorderStyle
'
'         AllowClipboard
'         LabelColor
'         HoverColor
'         HoverEffect
'
'       Method
'         None
'
'   ※  Visual Style を有効にしている場合に、ラベルをダブルクリックすると
'       ラベル内容がクリップボードにコピーされる事に対する措置
'
Imports System.ComponentModel
Imports System.Windows.Forms.VisualStyles

Namespace UserLib

<DebuggerStepThrough()>
Public Class LabelEx
    Inherits Label

#Region "// Property, Method"

    Private _LabelColor As Color
    Private _HoverColor As Color = SystemColors.Highlight
    Private _HoverEffect As Boolean

#Region "** 動作"

''' <summary>
''' ダブルクリック時のクリップボードへのコピーを許可するかを決定します。
''' </summary>
''' <value> False:不許可, True: 許可
''' </value>

    <DefaultValue(False), Category("動作"),
     Description("ダブルクリック時のクリップボードへのコピーを許可するかを決定します。")> _
    Public Property AllowClipboard() As Boolean

#End Region

#Region "** 表示"

''' <summary>
''' コントロールの境界線スタイルを取得または設定します。
''' </summary>

    <DefaultValue(BorderStyle.None), Category("表示")> _
    Public Shadows Property BorderStyle() As BorderStyle

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("ボタン上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor() As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。")> _
    Public Property HoverEffect() As Boolean
      Get
        Return _HoverEffect
      End Get

      Set(ByVal value As Boolean)
        _HoverEffect = value

        If Not _onCreate Then Return

        _EventCancel = True
          If Me.Enabled Then
             If value Then
                _onMouse = Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position))

                If _onMouse Then
                   Me.ForeColor = _HoverColor

                'MouseEnter を発生
                   Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
                End If
             Else
                _onMouse = False
             End If
          Else
             _onMouse = False
          End If

          If Not _onMouse Then
             If _DefaultColor Then
                Me.ForeColor = Nothing
             Else
                Me.ForeColor = _LabelColor
             End If
          End If

        _EventCancel = False
      End Set
    End Property

 ''' <summary>
 ''' ラベルの色を取得します。
 ''' </summary>

    <Browsable(False), DefaultValue(GetType(Color), "ControlText"), Category("表示"),
     Description("ラベルの色を取得します。")> _
    Public ReadOnly Property LabelColor() As Color
      Get
        Return _LabelColor
      End Get
    End Property

#End Region

#End Region

    Private _onCreate As Boolean
    Private _onMouse As Boolean
    Private _EventCancel As Boolean         '[ForeColorChanged] キャンセルフラグ
    Private _DefaultColor As Boolean        '既定色判定フラグ

    Private _PushClipBoardText As String    'クリップボードのバックアップ

#Region "// Sub-Routine"

''' <summary>
''' コントロールの Region を設定します。
''' </summary>

    Private Sub SetRegion()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

#End Region

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

        _LabelColor = Me.ForeColor
        _DefaultColor = (Me.ForeColor = Parent.ForeColor)

        SetRegion()

        AddHandler Me.Resize, AddressOf Me_Resize

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        MyBase.OnPaint(e)

        Dim w As Integer = Me.Width - 1
        Dim h As Integer = Me.Height - 1

        Dim p As New Pen(SystemColors.ControlDark)

        If Me.BorderStyle <> BorderStyle.None Then
           e.Graphics.DrawLines(p,
            {New Point(0, h - 1), New Point(0, 1), New Point(1, 1), New Point(1, 0), New Point(w - 1, 0)})

           If Me.BorderStyle = BorderStyle.Fixed3D Then
              p = New Pen(SystemColors.ControlLightLight)
           End If

           e.Graphics.DrawLines(p,
            {New Point(w - 1, 1), New Point(w, 1), New Point(w, h - 1), New Point(w - 1, h - 1),
             New Point(w - 1, h), New Point(1, h), New Point(1, h - 1)})

        End If

        p.Dispose()
        p = Nothing
    End Sub

#Region "// Me"

    Private Sub Me_ForeColorChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.ForeColorChanged

        If _EventCancel OrElse (Not _onCreate) Then Return

        _EventCancel = True

          _DefaultColor = (Me.ForeColor = Parent.ForeColor)

          If _HoverEffect Then
             _LabelColor = Me.ForeColor

             If _onMouse Then
                Me.ForeColor = _HoverColor
             Else
                If _DefaultColor Then Me.ForeColor = Nothing
             End If
          Else
             If _DefaultColor Then Me.ForeColor = Nothing
          End If

        _EventCancel = False
    End Sub

    Private Sub Me_EnabledChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.EnabledChanged

        If Not _HoverEffect Then Return

        _EventCancel = True

          If Me.Enabled Then
             _onMouse = Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position))

             If _onMouse Then
                Me.ForeColor = _HoverColor

             'MouseEnter を発生
                Me.OnMouseMove(New MouseEventArgs(MouseButtons.None, 0, 0, 0, 0))
             End If
          Else
             _onMouse = False

             If _DefaultColor Then
                Me.ForeColor = Nothing
             Else
                Me.ForeColor = _LabelColor
             End If
          End If

        _EventCancel = False
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        SetRegion()
    End Sub

    Private Sub Me_MouseEnter(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseEnter
        If _HoverEffect Then
           _onMouse = True

           _EventCancel = True
             Me.ForeColor = _HoverColor
           _EventCancel = False
        End If
    End Sub

    Private Sub Me_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseLeave
        If _HoverEffect Then
           _onMouse = False

           _EventCancel = True
             If _DefaultColor Then
                Me.ForeColor = Nothing
             Else
                Me.ForeColor = _LabelColor
             End If
           _EventCancel = False
        End If
    End Sub

    Private Sub Me_Click(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Click
        If Not _AllowClipboard Then
           _PushClipBoardText = Clipboard.GetText
        End If
    End Sub

    Private Sub Me_DoubleClick(ByVal sender As Object, ByVal e As EventArgs) Handles Me.DoubleClick
        If _AllowClipboard Then
           If Not Application.RenderWithVisualStyles Then
              Clipboard.SetText(Me.Text)
           End If
        Else
           If String.IsNullOrEmpty(_PushClipBoardText) Then
           'ダブルクリックでクリップボードにセットされた値をクリア
              Clipboard.Clear()
           Else
           'ダブルクリックでクリップボードにセットされた値を元の値へ戻す
              Clipboard.SetText(_PushClipBoardText)
           End If
        End If
    End Sub

#End Region

End Class

End Namespace
