﻿'
'   LabelML 1.0.0.0 limitation センタリング仕様
'
'       Required Class: 
'         None
'
'       Event invalidity
'         MouseDown
'
'       Property:
'         AllowClipboard
'         AutoEllipsis
'         DoubleBuffering
'         LockedCorrection
'
'         AutoTrim
'
'         HoverColor
'         HoverEffect
'
'       Method
'         None

Imports System.ComponentModel
Imports System.Drawing.Text
Imports System.Drawing.Design

Namespace UserLib

<DebuggerStepThrough()>
Public Class LabelML
    Inherits UserControl

    Public Shadows Event MouseDown(sender As Object, e As MouseEventArgs)
    Public Shadows Event TextChanged(sender As Object, e As EventArgs)

Public Enum Alignment As Integer
    Left = 16
    Center = 32
    Right = 64
End Enum

#Region "// constant, variable declaration"

    Private Const _TextLimitLength = 32     'ラベルの制限文字数
    Private Const _SideMargin = 2           'ラベルの両端余白
    Private Const _MinFixHeight = 6         'ラベルの固定高さ指定値

    Private _onCreate As Boolean
    Private _onResize As Boolean
    Private _onMouse As Boolean
    Private _onFocus As Boolean

    Private _ResizeHandler As Boolean   'Add(Remove)Handler の代替フラグ

    Private _FontSizeMpf As Single
    Private _FontSizeInPointsMpf As Single
    Private _FontPixelSize As Single

    Private _PushSize As Size           'AutoSize: False 時のサイズ復元用
    Private _PushAutoSize As Size       'AutoSize:  True 時のサイズ復元用
    Private _BaseAutoSize As Size       'Padding を含まない AutoSize値

    Private _AutoVerticalAdjust As Integer
    Private _LeftShift As Integer
    Private _TrimWidth As Integer

    Private _DrawText As String = ""
    Private _DrawTextAlign As Alignment = Alignment.Left
    Private _DrawTextRectangle As Rectangle
    Private _DrawTextRectangleF As RectangleF

#End Region

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AutoValidate As AutoValidate
    <Browsable(False)> _
    Public Shadows Property ImeMode As ImeMode

    <Browsable(False)> _
    Public Shadows Property AutoScroll As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize As Size
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode As AutoSizeMode

    <Browsable(False)> _
    Public Shadows Property BorderStyle As BorderStyle
    <Browsable(False), DefaultValue(False)> _
    Public Shadows Property TabStop As Boolean

#End Region

    Private _Text As String = ""
    Private _TextValue As String = ""

    Private _AutoEllipsis As Boolean
    Private _DoubleBuffering As Boolean = True    'システム既定値

    Private _AutoSize As Boolean = True
    Private _AutoTrim As Boolean
    Private _MinimumSize As Size
    Private _MaximumSize As Size
    Private _Padding As Padding

    Private _TextAlign As Alignment = Alignment.Left
    Private _HoverColor As Color = SystemColors.Highlight
    Private _HoverEffect As Boolean

#Region "** 制御用"

''' <summary>
''' デザイナー で設定した Text プロパティの値をデザイナーに渡す為のプロパティです。
''' </summary>

    <Browsable(False), DefaultValue("")> _
    Public Property TextValue As String
       Get
         Return _Text
       End Get

      Set(value As String)
        Me.Text = value
      End Set
    End Property

#End Region

#Region "** 動作"

''' <summary>
''' ダブルクリック時のクリップボードへのコピーを許可するかを決定します。
''' </summary>
''' <value> False:不許可, True: 許可
''' </value>

    <DefaultValue(False), Category("動作"),
    Description("ダブルクリック時のクリップボードへのコピーを許可するかを決定します。")> _
    Public Property AllowClipboard As Boolean

''' <summary>
''' コントロールのテキストが、指定されたコントロールの長さを超えることを示す省略記号文字 (…) を
''' コントロールの右端に表示するかどうかを指定します。
''' </summary>

    <DefaultValue(False), Category("動作"),
    Description("ラベル コントロールの幅を超えるテキストの自動処理を有効にします。")> _
    Public Property AutoEllipsis As Boolean
      Get
        Return _AutoEllipsis
      End Get

      Set(value As Boolean)
        _AutoEllipsis = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

''' <summary>
''' ダブルバッファリングの状態を取得または設定します。
''' </summary>

    <DefaultValue(True), Browsable(True), Category("動作"),
     Description("ダブルバッファリングの有効／無効を設定します。")> _
    Public Property DoubleBuffering As Boolean
      Get
        Return _DoubleBuffering
      End Get

      Set(ByVal value As Boolean)
        _DoubleBuffering = value

        Me.DoubleBuffered = value
      End Set
    End Property

''' <summary>
''' ラベル（テキスト）変更時の表示補正処理をロックします。
''' </summary>

    <Browsable(False), DefaultValue(False), Category("動作")>
    Public Property LockedCorrection As Boolean

#End Region

#Region "** 配置"

''' <summary>
''' フォント サイズを基にして、サイズの自動調整を行います。
''' </summary>

    <DefaultValue(True), Category("配置"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("コントロールが、その内容に合わせて自動的にサイズを変更するかどうかを指定します。")> _
    Public Overrides Property AutoSize As Boolean
      Get
        Return _AutoSize
      End Get

      Set(value As Boolean)
        _AutoSize = value

        If Not _onCreate Then Return

        If _AutoSize Then
           SetAutoSize(True)
        Else
           _onResize = False
             Me.Size = SizeCheck(_PushSize)
        End If

        If Not _onResize Then
           _onResize = True

           SetTextRectangle(True)
           SetRegion()
        End If

     '親フォームに通知
        MyBase.OnAutoSizeChanged(New EventArgs)
      End Set
    End Property

''' <summary>
''' AutoSize が True, TextAlign が Left の時、または、AutoSize が False, TextAlign が Right の時、
''' ラベル先頭および末尾の余白を自動的にトリムするかどうかを指定します。
''' </summary>

    <DefaultValue(False), Category("配置"),
     Description("AutoSize が True, TextAlign が Left の時、ラベル先頭および末尾の余白を自動的にトリムするかどうかを指定します。")> _
    Public Property AutoTrim As Boolean
      Get
        Return _AutoTrim
      End Get

      Set(value As Boolean)
        _AutoTrim = value

        If _onCreate Then
           SetTextRectangle(False)
           SetRegion()

           Me.Invalidate()
        End If
      End Set
    End Property

''' <summary>
''' コントロールが指定できる下限のサイズを取得または設定します。
''' </summary>

    <DefaultValue(GetType(Size), "0, 0"), Category("配置"), _
     RefreshProperties(RefreshProperties.Repaint)>
    Public Shadows Property MinimumSize As Size
      Get
        Return _MinimumSize
      End Get

      Set(value As Size)
        If value.Width < 0 Then value.Width = 0
        If value.Height < 0 Then value.Height = 0

        If value.IsEmpty Then
           _MinimumSize = value
        Else
           Dim w As Integer = value.Width
           Dim h As Integer = value.Height

           Dim mw As Integer = _MaximumSize.Width
           Dim mh As Integer = _MaximumSize.Width

           If (w <> 0) AndAlso (mw <> 0) AndAlso (w > mw) Then w = mw
           If (h <> 0) AndAlso (mh <> 0) AndAlso (h > mh) Then h = mh

           _MinimumSize = New Size(w, h)
        End If

        If Not _onCreate Then Return

        _onResize = False
          If _AutoSize Then
             Me.Size = SizeCheck(_PushAutoSize)
          Else
             Me.Size = SizeCheck(_PushSize)
          End If
        _onResize = True
      End Set
    End Property

''' <summary>
''' コントロールが指定できる上限のサイズを取得または設定します。
''' </summary>

    <DefaultValue(GetType(Size), "0, 0"), Category("配置"), _
     RefreshProperties(RefreshProperties.Repaint)>
    Public Shadows Property MaximumSize As Size
      Get
        Return _MaximumSize
      End Get

      Set(value As Size)
        If value.Width < 0 Then value.Width = 0
        If value.Height < 0 Then value.Height = 0

        If value.IsEmpty Then
           _MaximumSize = value
        Else
           Dim w As Integer = value.Width
           Dim h As Integer = value.Height

           If (w <> 0) AndAlso (w < _MinimumSize.Width) Then w = _MinimumSize.Width
           If (h <> 0) AndAlso (h < _MinimumSize.Height) Then h = _MinimumSize.Height

           _MaximumSize = New Size(w, h)
        End If

        If Not _onCreate Then Return

        _onResize = False
          If _AutoSize Then
             AutoSize_Control(True)
          Else
             Me.Size = SizeCheck(_PushSize)
          End If
        _onResize = True
      End Set
    End Property

''' <summary>
''' コントロールの内部スペースを指定します。
''' </summary>

    <DefaultValue(GetType(Padding), "0,0,0,0"), Category("配置"),
     Description("コントロールの内部スペースを指定します。")> _
    Public Shadows Property Padding As Padding
      Get
        Return _Padding
      End Get

      Set(ByVal value As Padding)
        _Padding = value

        If _onCreate Then
           AutoSize_Control(False)

           MyBase.OnPaddingChanged(New EventArgs)
        End If
      End Set
    End Property

#End Region

#Region "** 表示"

#Region " （基本）"

 ''' <summary>
 '''  現在のラベル（テキスト）を取得または設定します。
 ''' </summary>

    <Browsable(True), DefaultValue(""), Category("表示"),
     Editor("System.ComponentModel.Design.MultilineStringEditor, System.Design", GetType(UITypeEditor)),
     RefreshProperties(RefreshProperties.Repaint),
     Description("コントロールに関連付けされたテキストです。")> _
    Public Shadows Property Text As String
      Get
        Return _Text
      End Get

      Set(value As String)
        If value Is Nothing Then value = ""

        _Text = value
        _TextValue = value

        SetTrimText()

        If _onCreate Then
           If Not _LockedCorrection Then
              AutoSize_Control(True)
           Else
              Me.Invalidate()
           End If
        End If

        RaiseEvent TextChanged(Me, New EventArgs)
      End Set
    End Property

''' <summary>
''' コントロールのラベル（テキスト）の配置を取得または設定します。
''' </summary>

    <DefaultValue(Alignment.Left), Category("表示"),
     Description("コントロールに表示される文字列の配置です。")> _
    Public Property TextAlign As Alignment
      Get
        Return _TextAlign
      End Get

      Set(value As Alignment)
        _TextAlign = value

        SetDrawTextAlign()

        If _onCreate Then
           If _AutoTrim Then
              SetTextRectangle(False)
              SetRegion()
           End If

           Me.Invalidate()
        End If
      End Set
    End Property

#End Region

#Region " （拡張）"

''' <summary>
''' コントロール上にマウスカーソルがある時のホバー色を指定します。
''' </summary>

    <DefaultValue(GetType(Color), "Highlight"), Category("表示"),
     Description("コントロール上にマウスカーソルがある時のホバー色を指定します。")> _
    Public Property HoverColor As Color
      Get
        Return _HoverColor
      End Get

      Set(ByVal value As Color)
        If value.IsEmpty Then
           _HoverColor = SystemColors.Highlight
        Else
           _HoverColor = value
        End If
      End Set
    End Property

''' <summary>
''' ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("ラベル上にマウスカーソルがある時のホバー効果を使用するかを指定します。")> _
    Public Property HoverEffect As Boolean
      Get
        Return _HoverEffect
      End Get

      Set(ByVal value As Boolean)
        _HoverEffect = value

        If Not _onCreate Then Return

        If Me.Enabled Then
           _onMouse = Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position))

           If _onMouse Then Me.Invalidate()
        Else
           _onMouse = False
        End If
      End Set
    End Property

#End Region

#End Region

#End Region

#Region "// Sub-Routine"

''' <summary>
''' 対象のサイズが MaximumSize, MinimumSize の範囲内にあるかをチェックします。
''' </summary>

    Private Function SizeCheck(ByVal TargetSize As Size) As Size

        Dim w As Integer = TargetSize.Width
        Dim h As Integer = TargetSize.Height

        If w <= 0 Then
           w = 0
        Else
           Dim MinW As Integer = _MinimumSize.Width
           Dim MaxW As Integer = _MaximumSize.Width

           If (MinW <> 0) AndAlso (w < MinW) Then w = MinW
           If (MaxW <> 0) AndAlso (w > MaxW) Then w = MaxW
        End If

        If h <= 0 Then
           h = 0
        Else
           Dim MinH As Integer = _MinimumSize.Height
           Dim MaxH As Integer = _MaximumSize.Height

           If (MinH <> 0) AndAlso (h < MinH) Then h = MinH
           If (MaxH <> 0) AndAlso (h > MaxH) Then h = MaxH
        End If

        Return New Size(w, h)
    End Function

''' <summary>
''' AutoSize が True の時、サイズ変更，表示位置の設定を行います。
''' </summary>

    Private Sub AutoSize_Control(ByVal state As Boolean)
        If _AutoSize Then

           SetAutoSize(state)
             If Not _onResize Then      'Resize が発生しなかった場合でも、表示位置の補正を行う

                SetTextRectangle(True)
                SetRegion()

                Me.Invalidate()
             End If
        Else
           SetTextRectangle(True)

           Me.Invalidate()
        End If
    End Sub

''' <summary>
''' コントロールのサイズを取得します。
'''   幅は Max(Min)imumSize, 文字数, Padding により決定されます。
'''   高さは Max(Min)imumSize, FixHeight , Padding の値により決定されます。
''' </summary>

    Private Sub SetAutoSize(ByVal state As Boolean)

        Dim w As Integer
        Dim h As Integer

        If state Then
           If _DrawText.Length > 0 Then

              Dim LineCount As Integer = _DrawText.Length - _DrawText.Replace(vbLf, "").Length + 1

              w = Convert.ToInt32(Math.Ceiling(_FontPixelSize * (_DrawText.Length + 1)))
              h = Convert.ToInt32(Math.Ceiling(_FontPixelSize))

           'MS UI Gothic (6.75 - 10.5p) 対策
              If (h < Me.Font.Height) OrElse (h > 15) Then
                 h = Me.Font.Height
              End If

              h *= LineCount

              If w < Me.MinimumSize.Width Then w = Me.MinimumSize.Width
              If h < Me.MinimumSize.Height Then h = Me.MinimumSize.Height

              If Me.MaximumSize.Width <> 0 Then w = Me.MaximumSize.Width
              If Me.MaximumSize.Height <> 0 Then h = Me.MaximumSize.Height

              Dim b As New Bitmap(w, h)
              Dim g As Graphics = Graphics.FromImage(b)
              Dim s As SizeF = g.MeasureString(_DrawText, Me.Font, New SizeF(w, h))

              w = Convert.ToInt32(Math.Ceiling(s.Width))

              If (Me.Font.Style And 3) <> 0 Then  'Bold Or Italic
                 w += 2
              End If

              h = Convert.ToInt32(Math.Ceiling(s.Height))

              g.Dispose()
              b.Dispose()

              g = Nothing
              b = Nothing

              _BaseAutoSize = New Size(w, h)   'AutoSize の基準値
           End If
        Else
           w = _BaseAutoSize.Width
           h = _BaseAutoSize.Height
        End If

        w += _Padding.Horizontal
        h += _Padding.Vertical

        _PushAutoSize = New Size(w, h)   'AutoSize: True 時に Resize を実行した際の復元用

        _onResize = False
          Me.Size = SizeCheck(New Size(w, h))
    End Sub

''' <summary>
''' 描画用の TextAlign をセットします
''' </summary>

    Private Sub SetDrawTextAlign()
        If Me.RightToLeft = RightToLeft.Yes Then
           If _TextAlign = Alignment.Left Then
              _DrawTextAlign = Alignment.Right
           ElseIf _TextAlign = Alignment.Right Then
              _DrawTextAlign = Alignment.Left
           End If
        Else
           _DrawTextAlign = _TextAlign
        End If
    End Sub

''' <summary>
''' 描画用の StringFormat をセットします
''' </summary>

    Private Sub SetStringFormat(ByRef sf As StringFormat, Optional ByVal state As Boolean = False)

        If state Then
           If _DrawTextAlign = Alignment.Left Then          'Left
              sf.Alignment = StringAlignment.Near
           ElseIf _DrawTextAlign = Alignment.Center Then    'Center
              sf.Alignment = StringAlignment.Center
           Else                                             'Right
              sf.Alignment = StringAlignment.Far
           End If
        End If

        sf.LineAlignment = StringAlignment.Center
        sf.Trimming = StringTrimming.Word

        If _AutoEllipsis Then
           sf.Trimming = sf.Trimming Or StringTrimming.EllipsisCharacter
        End If

        If _DrawTextRectangle.Height >= Me.FontHeight Then
           sf.FormatFlags = StringFormatFlags.LineLimit
        End If

        If Me.RightToLeft = RightToLeft.Yes Then
           sf.FormatFlags = sf.FormatFlags Or StringFormatFlags.DirectionRightToLeft
        End If

        sf.HotkeyPrefix = HotkeyPrefix.Show
    End Sub

''' <summary>
''' テキストの描画情報をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetTextRectangle(ByVal state As Boolean)

        Dim x As Integer = _Padding.Left
        Dim y As Integer = _Padding.Top
        Dim w As Integer = Me.Width - _Padding.Horizontal
        Dim h As Integer = Me.Height - _Padding.Vertical

        If (w <= 0) OrElse (h <= 0) Then Return

        If state Then GetAutoVerticalAdjust(w, h)

        If (_AutoSize AndAlso _AutoTrim) Then
           _TrimWidth = GetAutoTrimWidth(w, h)

        ElseIf _AutoTrim AndAlso (_DrawTextAlign = Alignment.Right) Then
           _LeftShift = GetLeftShift(w, h)
           _TrimWidth = 0
        Else
           _LeftShift = 0
           _TrimWidth = 0
        End If

        _DrawTextRectangle = New Rectangle(x - _LeftShift, y + _AutoVerticalAdjust, w, h)
        _DrawTextRectangleF = _DrawTextRectangle
    End Sub

''' <summary>
''' コントロールの Region を設定します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetRegion()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        If (_AutoSize AndAlso _AutoTrim) AndAlso
           (_DrawTextAlign = Alignment.Left) Then w -= (_TrimWidth + _LeftShift)

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        path.Dispose()
        path = Nothing
    End Sub

''' <summary>
''' TextAlign が Middle の時にテキストの位置（垂直方向）の調整値を取得します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub GetAutoVerticalAdjust(ByVal w As Integer, ByVal h As Integer)

     'TextAlignがMiddleの時、位置を調整
        If (_DrawText.Length > 0) AndAlso ((_DrawTextAlign And 112) <> 0) AndAlso
           (w > 4) AndAlso (h > 4) Then

           Dim b As New Bitmap(w, h)
           Dim g As Graphics = Graphics.FromImage(b)
           Dim c As Color = b.GetPixel(0, 0)

           Dim sf As New StringFormat

           SetStringFormat(sf)

           g.DrawString(_DrawText, Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

           Dim BackColorArgb As Integer = c.ToArgb()

           Dim y0 As Integer = -1
           Dim y1 As Integer = -1

           For yp As Integer = 0 To h - 1
             For xp As Integer = 0 To w - 1
               If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                  y0 = yp
                  Exit For
               End If
             Next
             If y0 >= 0 Then Exit For
           Next

           If y0 = -1 Then
             _AutoVerticalAdjust = 0
           Else
              For yp As Integer = h - 1 To y0 Step -1
                For xp As Integer = 0 To w - 1
                  If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                     y1 = yp
                     Exit For
                  End If
                Next
                If y1 >= 0 Then Exit For
              Next

              Dim TextHeight As Integer = (y1 - y0) + 1

              If h <= TextHeight Then
                 _AutoVerticalAdjust = 0
              Else
                 Dim TextMargin As Integer = h - TextHeight

                 _AutoVerticalAdjust = ((TextMargin + 1) \ 2) - y0  '下位置合わせ
              End If
           End If

           sf.Dispose()
           g.Dispose()
           b.Dispose()

           sf = Nothing
           g = Nothing
           b = Nothing
        Else
           _AutoVerticalAdjust = 0
        End If
    End Sub

''' <summary>
''' AutoSize が True, TextAlign が Left の時の、ラベル末尾のトリムする幅を取得します。
''' </summary>

    Private Function GetAutoTrimWidth(ByVal w As Integer, ByVal h As Integer) As Integer

     'Text が空白の場合は無視
        If _DrawText.Trim.Length = 0 Then Return 0

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)
        Dim c As Color = b.GetPixel(0, 0)

        Dim sf As New StringFormat

        SetStringFormat(sf)

        g.DrawString(_DrawText, Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

        Dim BackColorArgb As Integer = c.ToArgb()

        Dim x0 As Integer = -1
        Dim x1 As Integer = -1

        For xp As Integer = 0 To w - 1
          For yp As Integer = 0 To h - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               x0 = xp
               Exit For
            End If
          Next
          If x0 >= 0 Then Exit For
        Next

        If x0 <> -1 Then
           For xp As Integer = w - 1 To 0 Step -1
             For yp As Integer = 0 To h - 1
               If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
                  x1 = xp
                  Exit For
               End If
             Next
             If x1 >= 0 Then Exit For
           Next
        End If

        sf.Dispose()
        g.Dispose()
        b.Dispose()

        sf = Nothing
        g = Nothing
        b = Nothing

        If x0 = -1 Then Return 0

        If x0 > _SideMargin Then
           _LeftShift = (x0 - _SideMargin)
        Else
           _LeftShift = 0
        End If

        Dim TrimWidth As Integer = (w - 1) - x1 - _SideMargin

        If TrimWidth <= 0 Then Return 0

        Return TrimWidth
    End Function

''' <summary>
''' AutoSize が False, TextAlign が Right の時の、ラベルの調整値を取得します。
''' </summary>

    Private Function GetLeftShift(ByVal w As Integer, ByVal h As Integer) As Integer

     'Text が空白の場合は無視
        If _DrawText.Trim.Length = 0 Then Return 0

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)
        Dim c As Color = b.GetPixel(0, 0)

        Dim sf As New StringFormat

        SetStringFormat(sf, True)

        g.DrawString(_DrawText, Me.Font, Brushes.White, New RectangleF(0, 0, w, h), sf)

        Dim BackColorArgb As Integer = c.ToArgb()

        Dim x1 As Integer = -1

        For xp As Integer = w - 1 To 0 Step -1
          For yp As Integer = 0 To h - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               x1 = xp
               Exit For
            End If
          Next
          If x1 >= 0 Then Exit For
        Next

        sf.Dispose()
        g.Dispose()
        b.Dispose()

        sf = Nothing
        g = Nothing
        b = Nothing

        Return _SideMargin - (w - 1 - x1)
    End Function

''' <summary>
''' テキスト内の [CR], [LF] を [CRLF] へ置換します。また、テキスト末尾の [CRLF] を削除します。
''' </summary>

    Private Sub SetTrimText()
        _DrawText = _Text.Trim

        If _DrawText.Length > 0 Then
           _DrawText = _Text.Replace(vbCrLf, vbLf)
           _DrawText = _DrawText.Replace(vbCr, vbLf)
           _DrawText = _DrawText.Replace(vbLf, vbCrLf)

           If _DrawText.EndsWith(vbCrLf) Then _DrawText = _DrawText.Substring(0, _DrawText.Length - 2)
        End If
    End Sub

#End Region

    Public Sub New()

        InitializeComponent()

        MyBase.TabStop = False
        MyBase.SetStyle(ControlStyles.Selectable, False)
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _FontSizeMpf = Me.CreateGraphics.DpiX / 96.0F
        _FontSizeInPointsMpf = _FontSizeMpf * 1.333334F
        _FontPixelSize = Me.Font.SizeInPoints * _FontSizeInPointsMpf

        _PushSize = Me.Size

        If _AutoSize Then
           SetAutoSize(True)    '_ResizeHandler が False の為、Resize は発生しない
        End If

        SetDrawTextAlign()
        SetTextRectangle(True)
        SetRegion()

        _onCreate = True
        _onResize = True        '外部からのリサイズ時に _PushSize を更新する為に True をセット

        _ResizeHandler = True   'Resize イベント ON

        AddHandler Me.FontChanged, AddressOf Me_FontChanged
        AddHandler Me.RightToLeftChanged, AddressOf Me_RightToLeftChanged

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        Dim sf As New StringFormat
        Dim sb As SolidBrush = New SolidBrush(Color.Empty)

     'Text描画
        If (_DrawText.Length > 0) AndAlso
           (_DrawTextRectangle.Width > 4) AndAlso
           (_DrawTextRectangle.Height > 4) Then

           SetStringFormat(sf, True)

           If Me.Enabled Then
              If _onMouse AndAlso _HoverEffect Then
                 sb.Color = _HoverColor
              Else
                 sb.Color = Me.ForeColor
              End If
           Else
              sb.Color = SystemColors.GrayText
           End If

           e.Graphics.DrawString(_DrawText, Me.Font, sb, _DrawTextRectangleF, sf)
        End If

        sb.Dispose()
        sf.Dispose()

        sb = Nothing
        sf = Nothing
    End Sub

    Protected Overrides Sub OnResize(e As EventArgs)

        If Not _ResizeHandler Then Return

        If Not _onResize Then
           _onResize = True
        Else
           _ResizeHandler = False
             If _AutoSize Then
                Me.Size = SizeCheck(_PushAutoSize)
             Else
                _PushSize = Me.Size

                Me.Size = SizeCheck(Me.Size)
             End If
           _ResizeHandler = True
        End If

        SetTextRectangle(True)
        SetRegion()

        Me.Invalidate()

     'Resize 確定後に親フォームに通知
        MyBase.OnResize(e)
    End Sub

    Protected Overrides Sub OnPaddingChanged(e As EventArgs)
        If _onCreate Then
           AutoSize_Control(True)

           MyBase.OnPaddingChanged(e)
        End If
    End Sub

    Protected Overrides Sub OnMouseDown(e As MouseEventArgs)
     'Focus を持たせない為にイベントを無効にする
       'MyBase.OnMouseDown(e)

        RaiseEvent MouseDown(Me, e)
    End Sub

#Region "// Me"

    Private Sub Me_RightToLeftChanged(sender As Object, e As EventArgs)

        SetDrawTextAlign()
        SetTextRectangle(True)
    End Sub

    Private Sub Me_FontChanged(sender As Object, e As EventArgs)
        _FontPixelSize = Me.Font.SizeInPoints * _FontSizeInPointsMpf

        AutoSize_Control(True)
    End Sub

    Private Sub Me_EnabledChanged(ByVal sender As Object, ByVal e As EventArgs) Handles Me.EnabledChanged

        If Me.Enabled Then
           If Me.ClientRectangle.Contains(Me.PointToClient(Cursor.Position)) Then
              _onMouse = True

              If _HoverEffect Then Me.Invalidate()
           End If
        Else
           _onMouse = False
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseEnter(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseEnter
        If _HoverEffect Then
           _onMouse = True

           Me.Invalidate()
        End If
    End Sub

    <DebuggerStepThrough()>
    Private Sub Me_MouseLeave(ByVal sender As Object, ByVal e As EventArgs) Handles Me.MouseLeave
        If _HoverEffect Then
           _onMouse = False

           Me.Invalidate()
        End If
    End Sub

    Private Sub Me_MouseDoubleClick(sender As Object, e As MouseEventArgs) Handles Me.MouseDoubleClick
        If _AllowClipboard Then
           Clipboard.SetText(_DrawText)
        End If
    End Sub

#End Region

End Class

End Namespace
