﻿'
'   Adding Marker,Select Sort Format for ListView 1.5.7.0b (Consolidated edition, add to AutoSort)
'
'       Property:
'         AutoSort
'         DoubleBuffering
'         LabelDrawingMode
'         MarkerIndex
'         MarkerSize   (Small, Normal, Large)
'         HeaderAdjust
'         HeaderVisual
'
'         ColumnPaddingWidth
'         MarkerPaddingWidth
'         MinDrawTextWidth
'
'       Method
'         SetAutoSort
'         SetComparerMode
'
'         Marker [Set, Check, Clear]
'         State  [Add, Insert, Remove, RemoveAt]
'
'       ※注意事項
'         コントロール配置時にカラムが右詰めに設定されている場合、コンペアモードは 
'         数値に変更されます。
'
'         デザイナー以外で、カラムの追加・削除を行う場合は下記の様に
'         追加・削除命令の前に必ずソート情報の設定をして下さい。
'
'         ListViewEx1.StateInsert(index as integer)
'         ListViewEx1.Columns.Insert(index as integer, text as string)
'
'         ListViewEx1.StateRemoveAt(index as integer)
'         ListViewEx1.Columns.RemoveAt(index as integer)
'
Imports System.ComponentModel
Imports System.Drawing.Drawing2D
Imports System.Globalization

Namespace UserLib

<DebuggerStepThrough()>
Public Class ListViewEx
    Inherits ListView

#Region "// Enum"

Public Enum DrawMode As Integer
    Simple = 0
    Detail = 1
End Enum

Public Enum Marker As Integer
    Small = 0
    Normal = 1
    Large = 2
End Enum

Public Enum Header As Integer
    Normal = 0
    Gradation = 1
End Enum

Public Enum ComparerMode As Integer
    Text = 0
    Numeric = 1
    DateTime = 2
End Enum

Public Enum SortOrder As Integer
    Ascending = 0
    Descending = 1
End Enum

#End Region

#Region "// Property"

    Private _AutoSort As Boolean
    Private _DoubleBuffering As Boolean
    Private _LabelDrawingMode As DrawMode = DrawMode.Detail

    Private _MarkerIndex As Integer = -1
    Private _MarkerSize As Marker = Marker.Normal
    Private _HeaderAdjust As Integer
    Private _HeaderVisual As Header

    Private Const _Default_ColumnPaddingWidth As Integer = 6    '列の両端空白部分のサイズ
    Private Const _Default_MarkerPaddingWidth As Integer = 8    'マーカー(矢印)の両端空白部分のサイズ
    Private Const _Default_MinDrawTextWidth As Integer = 4      '文字の最小描画幅

    Private _ColumnPaddingWidth As Integer = _Default_ColumnPaddingWidth
    Private _MarkerPaddingWidth As Integer = _Default_MarkerPaddingWidth
    Private _MinDrawTextWidth As Integer = _Default_MinDrawTextWidth

#Region "** 動作"

''' <summary>
''' ダブルバッファリングの状態を取得または設定します。
''' </summary>

    <DefaultValue(False), Browsable(True), Category("動作"),
     Description("ダブルバッファリングの有効／無効を設定します。")> _
    Public Property DoubleBuffering() As Boolean
      Get
        Return _DoubleBuffering
      End Get

      Set(ByVal value As Boolean)
        _DoubleBuffering = value

        Me.DoubleBuffered = value
      End Set
    End Property

''' <summary>
''' ラベルの描画方法を取得または設定します。
''' </summary>

    <DefaultValue(DrawMode.Detail), Browsable(True), Category("動作"),
     Description("ラベルの描画方法を設定します。")> _
    Public Property LabelDrawingMode() As DrawMode
      Get
        Return _LabelDrawingMode
      End Get

      Set(value As DrawMode)
        If Not [Enum].IsDefined(GetType(DrawMode), value) Then
           _LabelDrawingMode = DrawMode.Detail
        Else
           _LabelDrawingMode = value
        End If

        If _LabelDrawingMode = DrawMode.Detail Then
           _DrawTextFormatFlags = TextFormatFlags.NoPadding
        Else
           _DrawTextFormatFlags = TextFormatFlags.NoPadding Or TextFormatFlags.EndEllipsis
        End If
      End Set
    End Property

#End Region

#Region "** 表示"

''' <summary>
''' ヘッダーのラベル位置(y)の補正値を取得または設定します。
''' </summary>

    <DefaultValue(0), Browsable(True), Category("表示"),
     Description("ヘッダーのラベル位置(y)の補正値を設定します。")> _
    Public Property HeaderAdjust() As Integer
      Get
        Return _HeaderAdjust
      End Get

      Set(ByVal value As Integer)
        _HeaderAdjust = value
      End Set
    End Property

''' <summary>
''' ヘッダーの表示形態を取得または設定します。
''' </summary>

    <DefaultValue(Header.Normal), Browsable(True), Category("表示"),
     Description("ヘッダーの表示形態を設定します。")> _
    Public Property HeaderVisual() As Header
      Get
        Return _HeaderVisual
      End Get

      Set(ByVal value As Header)
        If value = _HeaderVisual Then Return

        _HeaderVisual = value

        ColumnHeaderDraw()
      End Set
    End Property

''' <summary>
''' マーカーのサイズを取得または設定します。
''' </summary>

    <DefaultValue(Marker.Normal), Browsable(True), Category("表示"),
     Description("マーカーの大きさを設定します。")> _
    Public Property MarkerSize() As Marker
      Get
        Return _MarkerSize
      End Get

      Set(ByVal value As Marker)
        If Not [Enum].IsDefined(GetType(Marker), value) Then
           _MarkerSize = Marker.Normal
        Else
           _MarkerSize = value
        End If

        _Arrow_Width = _ArrowSize(_MarkerSize, 0)
        _ArrowHeight = _ArrowSize(_MarkerSize, 1)
        _MarkerWidth = _Arrow_Width + (_MarkerPaddingWidth * 2)
      End Set
    End Property

#End Region

#Region "** その他"

''' <summary>
''' 自動ソート状態を取得します。
''' </summary>

    <Browsable(False)> _
    Public ReadOnly Property AutoSort() As Boolean
      Get
        Return _AutoSort
      End Get
    End Property

''' <summary>
''' 列の両端空白部分のサイズ(幅）を取得または設定します。
''' </summary>

    <Browsable(False)> _
    Public Property ColumnPaddingWidth() As Integer
      Get
        Return _ColumnPaddingWidth
      End Get

      Set(value As Integer)
        If value < 4 Then
           _ColumnPaddingWidth = _Default_ColumnPaddingWidth
        Else
           _ColumnPaddingWidth = value
        End If
      End Set
    End Property

''' <summary>
''' マーカーのある列のインデックス値を取得します。
''' </summary>

    <Browsable(False)> _
    Public ReadOnly Property MarkerIndex() As Integer
      Get
        Return _MarkerIndex
      End Get
    End Property

''' <summary>
''' マーカー(矢印)の両端空白部分のサイズ(幅）を取得または設定します。
''' </summary>

    <Browsable(False)> _
    Public Property MarkerPaddingWidth() As Integer
      Get
        Return _MarkerPaddingWidth
      End Get

      Set(value As Integer)
        If value < 4 Then
           _MarkerPaddingWidth = _Default_MarkerPaddingWidth
        Else
           _MarkerPaddingWidth = value
        End If

        _MarkerWidth = _Arrow_Width + (_MarkerPaddingWidth * 2)
      End Set
    End Property

''' <summary>
''' ラベル文字の最小描画幅を取得または設定します。
''' </summary>

    <Browsable(False)> _
    Public Property MinDrawTextWidth() As Integer
      Get
        Return _MinDrawTextWidth
      End Get

      Set(value As Integer)
        If value < 2 Then
           _MinDrawTextWidth = _Default_MinDrawTextWidth
        Else
           _MinDrawTextWidth = value
        End If
      End Set
    End Property

#End Region

#End Region

#Region "// Method"

''' <summary>
''' 自動ソートを設定します。
''' State に True をセットした時は、マーカーがある（マーカーがない場合は最初の）列を
''' ソート（ソート情報がない場合は昇順）後、マーカーをセットします。
''' State に False をセットした時は、リストにデータがない場合、マーカーはクリアされます。
''' </summary>

    Public Overloads Sub SetAutoSort(ByVal state As Boolean)

        _AutoSort = state

        If state Then
           If _MarkerIndex = -1 Then
              _MarkerIndex = 0
           End If

           Me.ListViewItemSorter = New ListViewItemComparer(_MarkerIndex, _SortInfo(_MarkerIndex))
        Else
           Me.ListViewItemSorter = Nothing

           MarkerCheck()
        End If

        ColumnHeaderDraw()
    End Sub

''' <summary>
''' 自動ソートを条件を付けて設定します。
''' 指定した（指定した列がない場合は最初の）列をソート後、マーカーをセットします。
''' </summary>

    Public Overloads Sub SetAutoSort(ByVal column As Integer, ByVal order As SortOrder)

        _AutoSort = True

        If (column < 0) OrElse (column > Me.Columns.Count - 1) Then
           column = 0
        End If

        If Not [Enum].IsDefined(GetType(SortOrder), order) Then
           order = SortOrder.Ascending
        End If

        _MarkerIndex = column
        _SortInfo(column).Order = order

        Me.ListViewItemSorter = New ListViewItemComparer(column, _SortInfo(column))

        ColumnHeaderDraw()
    End Sub

''' <summary>
''' カラムのコンペアモードを設定します。
''' 存在しない列を指定した場合、設定は無視されます。
''' </summary>

    Public Sub SetComparerMode(ByVal column As Integer, ByVal mode As ComparerMode)
        If (column < 0) OrElse (column > Me.Columns.Count) Then Return

        _SortInfo(column).Mode = mode
    End Sub

''' <summary>
''' 指定した列をソート後、マーカーをセットします。
''' 指定した列がない場合、マーカーはクリアされます。
''' </summary>
''' <param name="column">列     (0-n)</param>
''' <param name="order"> ソート (0:昇順,1:降順)</param>

    Public Sub MarkerSet(ByVal column As Integer, ByVal order As SortOrder)
        If (column < 0) OrElse (column > Me.Columns.Count - 1) Then
           _MarkerIndex = -1
        Else
           If Not [Enum].IsDefined(GetType(SortOrder), order) Then
              order = SortOrder.Ascending
           End If

           _MarkerIndex = column
           _SortInfo(column).Order = order

           Me.ListViewItemSorter = New ListViewItemComparer(column, _SortInfo(column))

           If Not _AutoSort Then
              Me.ListViewItemSorter = Nothing
           End If
        End If

        ColumnHeaderDraw()
    End Sub

''' <summary>
''' リストにデータがない場合、マーカーをクリアします。各列の状態もクリアできます。
''' </summary>

    Public Sub MarkerCheck(Optional ByVal StateClear As Boolean = False)
        If (_MarkerIndex <> -1) AndAlso (Me.Items.Count = 0) Then
           MarkerClear(StateClear)
        End If
    End Sub

''' <summary>
''' マーカーをクリアします。各列の状態もクリアできます。
''' </summary>

    Public Sub MarkerClear(Optional ByVal StateClear As Boolean = False)

        _MarkerIndex = -1

        If StateClear Then
           For i As Integer = 0 To _SortInfo.Count - 1
             _SortInfo(i).Order = SortOrder.Ascending
           Next
        End If

        ColumnHeaderDraw()
    End Sub

'-------------------------------------------------------------------------------
'   ソート情報操作
'-------------------------------------------------------------------------------

''' <summary>
''' カラムのコンペアモードを指定して、ソート情報をリストの末尾に追加します。
''' </summary>

    Public Overloads Sub StateAdd(Optional ByVal mode As ComparerMode = ComparerMode.Text)

        _SortInfo.Add(New SortInfo(SortOrder.Ascending, mode))
    End Sub

''' <summary>
''' カラムのコンペアモードを指定して、指定した数のソート情報をリストの末尾に追加します。
''' </summary>

    Public Overloads Sub StateAdd(ByVal count As Integer, Optional ByVal mode As ComparerMode = ComparerMode.Text)
        For i As Integer = 1 To count
          _SortInfo.Add(New SortInfo(SortOrder.Ascending, mode))
        Next
    End Sub

''' <summary>
''' カラムのコンペアモードを指定して、ソート情報を指定した位置に追加（挿入）します。
''' </summary>

    Public Sub StateInsert(ByVal index As Integer,
                  Optional ByVal mode As ComparerMode = ComparerMode.Text)

        If index < 0 OrElse index > Me.Columns.Count Then
           Throw New IndexOutOfRangeException
        End If

        _SortInfo.Insert(index, New SortInfo(SortOrder.Ascending, mode))

        If (_MarkerIndex <> -1) AndAlso (index <= _MarkerIndex) Then
           _MarkerIndex += 1
        End If
    End Sub

''' <summary>
''' ソート情報をリストの末尾から削除します。
''' </summary>

    Public Sub StateRemove()

        If Me.Columns.Count = 0 Then
           Throw New IndexOutOfRangeException
        End If

        Dim idx As Integer = Me.Columns.Count - 1

        _SortInfo.RemoveAt(idx)

        If _MarkerIndex <> -1 Then
           If _MarkerIndex = idx Then MarkerClear()
        End If
    End Sub

''' <summary>
''' ソート情報を指定した位置から削除します。
''' </summary>

    Public Sub StateRemoveAt(ByVal index As Integer)

        If index < 0 OrElse index > Me.Columns.Count - 1 Then
           Throw New IndexOutOfRangeException
        End If

        _SortInfo.RemoveAt(index)

        If _MarkerIndex <> -1 Then
           If index < _MarkerIndex Then
              _MarkerIndex -= 1
           ElseIf index = _MarkerIndex Then
              MarkerClear()
           End If
        End If
    End Sub

#End Region

#Region "// constant, variable declaration"

    Private Const _EllipsisTxt = "..."

    Private _OsType As Boolean

    Private _SortInfo As New List(Of SortInfo)

    Private _ArrowSize(,) As Integer = {{7, 4}, {9, 5}, {11, 6}}

    Private _Arrow_Width As Integer = _ArrowSize(_MarkerSize, 0)                'マーカー(▲)の幅
    Private _ArrowHeight As Integer = _ArrowSize(_MarkerSize, 1)                '      〃      高さ
    Private _MarkerWidth As Integer = _Arrow_Width + (_MarkerPaddingWidth * 2)  '      〃      描画領域の幅

    Private _DefaultMeasureText As Integer
    Private _DrawTextFormatFlags As TextFormatFlags = TextFormatFlags.NoPadding

    Private _DrawStringWidth As Integer     '文字描画領域幅
    Private _ExceptWidth As Integer         '非描画領域幅の合計
    Private _PossibleWidth As Integer       '描画可能領域幅

    Private _L_offset As Integer            'ラベル位置   (相対Ｘ座標)
    Private _M_offset As Integer            'マーカー位置 (    〃    )

' InitializeComponent, TextAlign 変更時、OnColumnWidthChanged をキャンセルする際に使用。
    Private _EventCancelFlag As Boolean = True

#End Region

#Region "// Sub-Routine & Function"

    Private Function GetMeasureTextWidth(text As String, ByVal font As Font) As Integer

        Dim h As Integer = font.Height
        Dim w As Integer = (text.Length + 1) * h
        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        Dim width As Integer = TextRenderer.MeasureText(g, text, Me.Font, New Size(w, h), TextFormatFlags.Default).Width

        TextRenderer.DrawText(g, text, Me.Font, _
          New Rectangle(0, 0, width, h), Color.White, Color.Empty, TextFormatFlags.NoPadding)

        Dim c As Color = b.GetPixel(0, 0)
        Dim BackColorArgb As Integer = c.ToArgb()

        Dim MeasureTextWidth As Integer = -1

        For xp As Integer = width - 1 To 0 Step -1
          For yp As Integer = 0 To h - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               MeasureTextWidth = xp + 1
               Exit For
            End If
          Next
          If MeasureTextWidth >= 0 Then Exit For
        Next

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing

        Return MeasureTextWidth
    End Function

    Private Function GetMeasureTextPadding(ByVal text As String, ByVal width As Integer) As VerticalPadding

        If text.Trim.Length = 0 Then Return New VerticalPadding()

        Dim h As Integer = Me.Font.Height
        Dim b As New Bitmap(width, h)
        Dim g As Graphics = Graphics.FromImage(b)

        TextRenderer.DrawText(g, text, Me.Font, _
          New Rectangle(0, 0, width, h), Color.White, Color.Empty, TextFormatFlags.NoPadding)

        Dim c As Color = b.GetPixel(0, 0)
        Dim BackColorArgb As Integer = c.ToArgb()

        Dim y0 As Integer = -1
        Dim y1 As Integer = -1

        For yp As Integer = 0 To h - 1
          For xp As Integer = 0 To width - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               y0 = yp
               Exit For
            End If
          Next
          If y0 >= 0 Then Exit For
        Next

        For yp As Integer = h - 1 To 0 Step -1
          For xp As Integer = 0 To width - 1
            If b.GetPixel(xp, yp).ToArgb() <> BackColorArgb Then
               y1 = (h - 1) - yp
               Exit For
            End If
          Next
          If y1 >= 0 Then Exit For
        Next

        g.Dispose()
        b.Dispose()

        g = Nothing
        b = Nothing

        Return New VerticalPadding(y0, y1)
    End Function

    Private Sub ColumnHeaderDraw()

     'TextAlign を変更すると OnColumnWidthChanged が発生する為、キャンセルする。
        _EventCancelFlag = True
          Me.Columns(0).TextAlign = Me.Columns(0).TextAlign     'DrawColumnHeader 強制発動
        _EventCancelFlag = False
    End Sub

#End Region

    Public Sub New()
        Me.View = View.Details
        Me.OwnerDraw = True

        Dim OS As OperatingSystem = System.Environment.OSVersion
        Dim OS_Ver As Single = Convert.ToSingle(OS.Version.Major + (OS.Version.Minor / 10))

        _OsType = (OS_Ver > 6.1F)
    End Sub

    Protected Overrides Sub OnCreateControl()

        _EventCancelFlag = False

        Dim m As ComparerMode

        For i = 0 To Me.Columns.Count - 1

       'カラムが右詰めの時は、数値とみなす。
          If Me.Columns(i).TextAlign = HorizontalAlignment.Right Then
             m = ComparerMode.Numeric
          Else
             m = ComparerMode.Text
          End If

          _SortInfo.Add(New SortInfo(SortOrder.Ascending, m))
        Next

        MyBase.OnCreateControl()
    End Sub

    <DebuggerStepThrough()>
    Protected Overrides Sub OnColumnWidthChanged(ByVal e As ColumnWidthChangedEventArgs)
        If _EventCancelFlag Then Return

        MyBase.OnColumnWidthChanged(e)
    End Sub

    Protected Overrides Sub OnColumnClick(ByVal e As ColumnClickEventArgs)

        If (Not _AutoSort) AndAlso (Me.Items.Count = 0) Then Return

        If e.Column = _MarkerIndex Then
           _SortInfo(e.Column).Order = _SortInfo(e.Column).Order Xor SortOrder.Descending
        End If

        Me.ListViewItemSorter = New ListViewItemComparer(e.Column, _SortInfo(e.Column))

        If Not _AutoSort Then
           Me.ListViewItemSorter = Nothing
        End If

        _MarkerIndex = e.Column

        ColumnHeaderDraw()

        MyBase.OnColumnClick(e)
    End Sub

    Protected Overrides Sub OnDrawColumnHeader(ByVal e As DrawListViewColumnHeaderEventArgs)

        If e.Bounds.Width = 0 Then Return

        Dim FontHeight As Integer = Me.Font.Height
        Dim HeaderText As String = e.Header.Text.Trim

        Dim Bounds_X As Integer = e.Bounds.X
        Dim Bounds_Y As Integer = e.Bounds.Y
        Dim BoundsWidth As Integer = e.Bounds.Width
        Dim BoundsHeight As Integer = e.Bounds.Height
'-------------------------------------------------------------------------------
'   基本制御値設定
'-------------------------------------------------------------------------------
     '非描画領域幅設定
        If e.ColumnIndex = _MarkerIndex Then
           _ExceptWidth = _ColumnPaddingWidth + _MarkerWidth
        Else
           _ExceptWidth = _ColumnPaddingWidth * 2
        End If

     '描画領域幅設定
        If (BoundsWidth < _ExceptWidth) OrElse (HeaderText.Length = 0) Then
           _DrawStringWidth = 0
        Else

           _PossibleWidth = BoundsWidth - _ExceptWidth                      '描画可能領域幅
           _DefaultMeasureText = GetMeasureTextWidth(HeaderText, Me.Font)   '全文字列の正しい描画幅

           Dim mt As Integer

           If _DefaultMeasureText <= _PossibleWidth Then
              _DrawStringWidth = _DefaultMeasureText
           Else

              If _LabelDrawingMode = DrawMode.Detail Then

                 Dim ct As String
                 Dim cw As Integer = Convert.ToInt32(FontHeight * 2)

                 For i As Integer = 1 To HeaderText.Length
                   ct = HeaderText.Substring(0, i)
                   mt = GetMeasureTextWidth(ct, Me.Font)

                   If mt < _PossibleWidth Then
                      cw += FontHeight

                      _DrawStringWidth = mt
                   Else
                      _DrawStringWidth = _PossibleWidth

                      If (mt = _PossibleWidth) OrElse (i = 1) Then
                         HeaderText = ct
                      Else
                         HeaderText = ct.Substring(0, i - 1) & _EllipsisTxt
                      End If
                      Exit For
                   End If
                 Next
              Else

                 mt = TextRenderer.MeasureText(e.Graphics, HeaderText, Me.Font, _
                                     New Size(_PossibleWidth, FontHeight), _DrawTextFormatFlags).Width

                 If mt <= _PossibleWidth Then
                    _DrawStringWidth = mt
                 Else
                    _DrawStringWidth = _PossibleWidth
                 End If
              End If
           End If
        End If

     'ラベル・マーカー描画位置(x)設定
        If e.ColumnIndex = _MarkerIndex Then
           Select Case e.Header.TextAlign
             Case HorizontalAlignment.Left
               _L_offset = _ColumnPaddingWidth
               _M_offset = _ColumnPaddingWidth + _DrawStringWidth

             Case HorizontalAlignment.Center
               _L_offset = (BoundsWidth - _DrawStringWidth - _MarkerWidth + _ColumnPaddingWidth) \ 2
               _M_offset = _L_offset + _DrawStringWidth

             Case HorizontalAlignment.Right
               _L_offset = BoundsWidth - _DrawStringWidth - _MarkerWidth
               _M_offset = BoundsWidth - _MarkerWidth

           End Select
        Else
           Select Case e.Header.TextAlign
             Case HorizontalAlignment.Left
               _L_offset = _ColumnPaddingWidth

             Case HorizontalAlignment.Center
               _L_offset = (BoundsWidth - _DrawStringWidth) \ 2

             Case HorizontalAlignment.Right
               _L_offset = BoundsWidth - _DrawStringWidth - _ColumnPaddingWidth

           End Select
        End If

     'ラベル描画位置(y)設定

        Dim y As Integer

        If _DrawStringWidth >= _MinDrawTextWidth Then

           Dim tp As VerticalPadding = GetMeasureTextPadding(HeaderText, _DrawStringWidth + FontHeight)
           Dim op As Single = (BoundsHeight - FontHeight) / 2.0F
           Dim ip As Single = -(tp.Top - tp.Bottom) / 2.0F

           y = Convert.ToInt32(Math.Truncate(op + ip)) + _HeaderAdjust  '小数部切捨て
        End If
'-------------------------------------------------------------------------------
'   ラベル描画
'-------------------------------------------------------------------------------
        Dim DrawTextColor As Color = SystemColors.ControlText

        If _HeaderVisual = Header.Gradation Then

           Dim gb As New LinearGradientBrush(e.Bounds, Color.Black, Color.White, LinearGradientMode.Vertical)
           Dim bl As New ColorBlend()

           If Application.RenderWithVisualStyles Then
              bl.Positions = New Single() {0, 0.5, 1}
           Else
              bl.Positions = New Single() {0, 0.4, 1}
           End If

           bl.Colors = New Color() {SystemColors.ControlLightLight, SystemColors.ControlLightLight, SystemColors.Control}
           gb.InterpolationColors = bl

           Dim w As Integer = Bounds_X + BoundsWidth - 1
           Dim p As Point() = {New Point(Bounds_X, BoundsHeight - 1), New Point(w, BoundsHeight - 1), New Point(w, 0)}

           e.Graphics.FillRectangle(gb, e.Bounds)
           e.Graphics.DrawLines(New Pen(SystemColors.ControlDarkDark), p)   '外枠 」

           If (e.State And 1) = 1 Then  'カラムクリック (Mouse Down)

              p = {New Point(Bounds_X, BoundsHeight - 2), New Point(Bounds_X, -1), New Point(w - 1, -1)}
              e.Graphics.DrawLines(New Pen(SystemColors.ControlDark), p)    '影枠「

              If _DrawStringWidth >= _MinDrawTextWidth Then
                 TextRenderer.DrawText(e.Graphics, HeaderText, Me.Font, _
                   New Rectangle(Bounds_X + _L_offset + 1, y + 1, _DrawStringWidth, FontHeight), _
                   DrawTextColor, _DrawTextFormatFlags _
                   )
              End If
           Else
              p = {New Point(Bounds_X, BoundsHeight - 2), New Point(Bounds_X, 0), New Point(w - 2, 0)}
              e.Graphics.DrawLines(New Pen(SystemColors.ControlLightLight), p)  '光枠「

              p = {New Point(Bounds_X + 1, BoundsHeight - 2), New Point(w - 1, BoundsHeight - 2), New Point(w - 1, 1)}
              e.Graphics.DrawLines(New Pen(SystemColors.ControlDark), p)        '影枠 」

              If _DrawStringWidth >= _MinDrawTextWidth Then
                 TextRenderer.DrawText(e.Graphics, HeaderText, Me.Font, _
                   New Rectangle(Bounds_X + _L_offset, y, _DrawStringWidth, FontHeight), _
                   DrawTextColor, _DrawTextFormatFlags _
                   )
              End If
           End If

           gb.Dispose()
           gb = Nothing
        Else
           e.DrawBackground()           'BackGroundを描画

           If (e.State And 1) = 1 Then  'カラムクリック (Mouse Down)

              If Not _OsType Then
                 e.Graphics.DrawRectangle(
                   New Pen(New SolidBrush(SystemColors.ControlDark), 1),
                   New Rectangle(Bounds_X, Bounds_Y, BoundsWidth - 1, BoundsHeight - 2)
                   )
              End If

              If _DrawStringWidth >= _MinDrawTextWidth Then
                 TextRenderer.DrawText(e.Graphics, HeaderText, Me.Font, _
                   New Rectangle(Bounds_X + _L_offset + 1, y + 1, _DrawStringWidth, FontHeight), _
                   DrawTextColor, _DrawTextFormatFlags _
                   )
              End If
           Else
              If _DrawStringWidth >= _MinDrawTextWidth Then
                 TextRenderer.DrawText(e.Graphics, HeaderText, Me.Font, _
                   New Rectangle(Bounds_X + _L_offset, y, _DrawStringWidth, FontHeight), _
                   DrawTextColor, _DrawTextFormatFlags _
                   )
              End If
           End If

           If _OsType Then
              e.Graphics.DrawLines(New Pen(SystemColors.ControlLight), {New Point(Bounds_X, BoundsHeight - 1), New Point(BoundsWidth - 1, BoundsHeight - 1)})
           End If
        End If
'-------------------------------------------------------------------------------
'   マーカー描画
'-------------------------------------------------------------------------------
        If Not _AutoSort Then MarkerCheck(True)

        If e.ColumnIndex = _MarkerIndex Then

           If BoundsWidth >= _Arrow_Width + _MinDrawTextWidth Then

              Dim xp As Integer
              Dim yp As Integer = Convert.ToInt32(Math.Ceiling((BoundsHeight - _ArrowHeight) / 2.0F))
              Dim wp As Point()

              If BoundsWidth >= _ExceptWidth Then
                 xp = Bounds_X + _M_offset + _MarkerPaddingWidth
              Else
                 xp = Bounds_X + ((BoundsWidth - _Arrow_Width - 1) \ 2)
              End If

              If _SortInfo(e.ColumnIndex).Order = SortOrder.Ascending Then
                '▲
                 wp = {
                    New Point(xp - 1, yp + _ArrowHeight),
                    New Point(xp + (_Arrow_Width \ 2), yp - 1),
                    New Point(xp + _Arrow_Width, yp + _ArrowHeight)}
              Else
                '▼
                 wp = {
                    New Point(xp, yp),
                    New Point(xp + (_Arrow_Width \ 2), yp + _ArrowHeight),
                    New Point(xp + _Arrow_Width, yp)}
              End If

              e.Graphics.FillPolygon(New SolidBrush(SystemColors.GrayText), wp, FillMode.Winding)
           End If
        End If

        MyBase.OnDrawColumnHeader(e)
    End Sub

    Protected Overrides Sub OnDrawSubItem(ByVal e As DrawListViewSubItemEventArgs)
        e.DrawDefault = True    'システムに描画を任せる

        MyBase.OnDrawSubItem(e)
    End Sub

End Class

<DebuggerStepThrough()>
Class VerticalPadding

    Public Top As Integer
    Public Bottom As Integer

    Public Sub New()
        Me.Top = 0
        Me.Bottom = 0
    End Sub

    Public Sub New(ByVal top As Integer, ByVal bottom As Integer)
        Me.Top = top
        Me.Bottom = bottom
    End Sub

End Class

<DebuggerStepThrough()>
Class SortInfo

    Public Order As ListViewEx.SortOrder
    Public Mode As ListViewEx.ComparerMode

    Public Sub New()
        Me.Order = ListViewEx.SortOrder.Ascending
        Me.Mode = ListViewEx.ComparerMode.Text
    End Sub

    Public Sub New(ByVal order As ListViewEx.SortOrder, ByRef mode As ListViewEx.ComparerMode)
        Me.Order = order
        Me.Mode = mode
    End Sub

End Class

<DebuggerStepThrough()>
Class ListViewItemComparer
    Implements IComparer

    Private Column As Integer
    Private Order As ListViewEx.SortOrder
    Private Mode As ListViewEx.ComparerMode

    Public Sub New()
        Column = 0
        Mode = ListViewEx.ComparerMode.Text
        Order = ListViewEx.SortOrder.Ascending
    End Sub

    Public Sub New(ByVal column As Integer, ByVal info As SortInfo)
        Me.Column = column
        Me.Mode = info.Mode
        Me.Order = info.Order
    End Sub

    Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements IComparer.Compare

        Dim result As Integer

        Dim itemX As String = DirectCast(x, ListViewItem).SubItems(Column).Text
        Dim itemY As String = DirectCast(y, ListViewItem).SubItems(Column).Text

        Select Case Mode
          Case ListViewEx.ComparerMode.Text
            result = String.Compare(itemX, itemY)

          Case ListViewEx.ComparerMode.Numeric
            result = Integer.Parse(itemX, NumberStyles.AllowThousands).CompareTo(Integer.Parse(itemY, NumberStyles.AllowThousands))

          Case ListViewEx.ComparerMode.DateTime
            result = DateTime.Compare(DateTime.Parse(itemX), DateTime.Parse(itemY))
        End Select

     '降順の時は結果を+-逆にする
        If Order = ListViewEx.SortOrder.Descending Then result = -result

        Return result
    End Function

End Class

End Namespace
