﻿'
'   NumericTextBox 1.1.3.3 Int32
'
'       Property:
'         BackColor   / ForeColor
'         Value
'         Maximum     / Minimum
'         Increment
'         DisplayZero
'         ThousandsSeparator
'
'         Text         
'         TextLength
'
'       Method
'         Clear
'         SelectAll
'
'       Event Notification
'         ValueChanged

Imports System.ComponentModel
Imports System.Text.RegularExpressions
Imports System.Runtime.InteropServices

Namespace UserLib

<DebuggerStepThrough()>
Public Class NumericTextBox

    Public Event ValueChanged(sender As Object, e As EventArgs)

    Private Enum State As Integer
        None = 0
        Paste = 1
    End Enum

    Private _onCreate As Boolean
    Private _onResize As Boolean

    Private _CaretState As Boolean = True       'カーソル表示状態
    Private _KeyPressState As Integer
    Private _KeyPressRegStr As String

    Private _OldValue As Integer

    Private _BothFrameBorderHeight As Integer = SystemInformation.FixedFrameBorderSize.Height * 2

    Private _FramePoint As Point()

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AllowDrop() As Boolean
    <Browsable(False)> _
    Public Shadows Property AutoValidate() As AutoValidate
    <Browsable(False)> _
    Public Shadows Property ImeMode() As ImeMode

    <Browsable(False)> _
    Public Shadows Property AutoScroll() As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin() As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize() As Size
    <Browsable(False)> _
    Public Shadows Property AutoSize() As Boolean
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode() As AutoSizeMode
    <Browsable(False)> _
    Public Shadows Property Padding() As Padding

    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout
    <Browsable(False), DefaultValue(BorderStyle.Fixed3D)> _
    Public Shadows Property BorderStyle() As BorderStyle = BorderStyle.Fixed3D

#End Region

    Private _Value As Integer
    Private _Minimum As Integer
    Private _Maximum As Integer = 999
    Private _Increment As Integer = 1

    Private _DisplayZero As Boolean
    Private _ThousandsSeparator As Boolean

    Private _DefaultBackColor As Color = SystemColors.Window
    Private _DefaultForeColor As Color = SystemColors.WindowText

#Region "** データ"

''' <summary>
''' コントロールの範囲の最小値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("データ"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("この NumericTextBox が処理している範囲の下限です。")> _
    Public Property Minimum() As Integer
      Get
        Return _Minimum
      End Get

      Set(value As Integer)
        If value > Maximum Then _Maximum = value

        _Minimum = value

        If value > Maximum OrElse value > _Value Then
           Me.Value = value
        End If

        SetKeyPressRegStr()
      End Set
    End Property

''' <summary>
''' コントロールの範囲の最大値を取得または設定します。
''' </summary>

    <DefaultValue(999), Category("データ"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("この NumericTextBox が処理している範囲の上限です。")> _
    Public Property Maximum() As Integer
      Get
        Return _Maximum
      End Get

      Set(value As Integer)
        If value < _Minimum Then _Minimum = value

        _Maximum = value

        If value < _Minimum OrElse value < _Value Then
           Me.Value = value
        End If
      End Set
    End Property

''' <summary>
''' マウスホイールで増分または減分する値を取得または設定します。
''' </summary>

    <DefaultValue(1), Category("データ"),
     Description("マウスホイールで増減する値を示します。")> _
    Public Property Increment() As Integer
      Get
        Return _Increment
      End Get

      Set(value As Integer)
        _Increment = value

        If value < 1 Then _Increment = 1
      End Set
    End Property

''' <summary>
''' 桁区切り記号を表示するかどうかを示す値を取得または設定します。
''' </summary>

    <DefaultValue(False), Category("データ"),
     Description("３桁ごとに区切り記号を挿入するかどうかを示します。")> _
    Public Property ThousandsSeparator() As Boolean
      Get
        Return _ThousandsSeparator
      End Get

      Set(value As Boolean)
        _ThousandsSeparator = value

        ValueTextUpdate()

        SetKeyPressRegStr()
      End Set
    End Property

#End Region

#Region "** 表示"

''' <summary>
''' コントロールの背景色を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Color), "Window"), Category("表示"),
     Description("コンポーネントの背景色です。")> _
    Public Shadows Property BackColor() As Color
      Get
        Return TextBox.BackColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           value = _DefaultBackColor
        End If

        TextBox.BackColor = value
      End Set
    End Property

''' <summary>
''' コントロールの前景色を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Color), "WindowText"), Category("表示"),
     Description("テキストを表示するのに使用される、このコンポーネントの前景色です。")> _
    Public Shadows Property ForeColor() As Color
      Get
        Return TextBox.ForeColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           value = _DefaultForeColor
        End If

        TextBox.ForeColor = value
      End Set
    End Property

''' <summary>
'''  NumericTextBox の現在のテキストを取得または設定します。
''' </summary>

    Public Shadows Property Text() As String
      Get
        Return TextBox.Text
      End Get

      Set(value As String)
        TextBox.Text = value
      End Set
    End Property

''' <summary>
''' Value が 0 の時の表示方法を指定します。
''' </summary>

    <DefaultValue(False), Category("表示"),
     Description("Value が 0 の時に 0 を表示するかを指定します。")> _
    Public Property DisplayZero() As Boolean
      Get
        Return _DisplayZero
      End Get

      Set(value As Boolean)
        _DisplayZero = value

        If _Value = 0 Then
           If _DisplayZero Then
              TextBox.Text = "0"
           Else
              TextBox.Clear()
           End If
        End If
      End Set
    End Property

''' <summary>
''' NumericTextBox の現在の値を取得または設定します。
''' </summary>

    <DefaultValue(0), Category("表示"),
     RefreshProperties(RefreshProperties.Repaint),
     Description("最小および最大プロパティで指定された範囲内の NumericTextBox の現在の値です。")> _
    Public Property Value() As Integer
      Get
        TextValueCheck(0)

        Return _Value
      End Get

      Set(value As Integer)

        _OldValue = _Value

        If value < _Minimum Then
           _Value = _Minimum
        ElseIf value > _Maximum Then
           _Value = _Maximum
        Else
           _Value = value
        End If

        If (_Value = 0) AndAlso (Not _DisplayZero) Then
           TextBox.Clear()
        Else
           ValueTextUpdate()
        End If

        If _OldValue <> _Value Then
           RaiseEvent ValueChanged(Me, New EventArgs)
        End If
      End Set
    End Property

#End Region

#Region "** その他"

''' <summary>
''' NumericTextBox のテキストの長さを取得します。
''' </summary>

    <Browsable(False)> _
    Public ReadOnly Property TextLength() As Integer
      Get
        Return TextBox.TextLength
      End Get
    End Property

#End Region

#End Region

#Region "// Method"

''' <summary>
''' NumericTextBox のすべてのテキストを削除します。
''' </summary>

    Public Sub Clear()
        TextBox.Clear()
    End Sub

''' <summary>
''' NumericTextBox のすべてのテキストを選択します。
''' </summary>

    Public Sub SelectAll()
        TextBox.SelectAll()
    End Sub

#End Region

#Region "// Sub-Routine & Function"

    <DllImport("USER32.DLL", CharSet:=CharSet.Auto)> _
    Private Shared Function HideCaret( _
        ByVal hwnd As IntPtr) As Integer
    End Function

    <DllImport("USER32.DLL", CharSet:=CharSet.Auto)> _
    Private Shared Function ShowCaret( _
        ByVal hwnd As IntPtr) As Integer
    End Function

    <DebuggerStepThrough()> _
    Private Function StringToInt32Parse(ByVal value As String, ByRef state As Boolean) As Integer

        state = True

        If String.IsNullOrEmpty(value) Then Return 0

        value = value.Replace(",", "")

        Dim v As Integer
        If Integer.TryParse(value, v) Then Return v

        state = False
        Return 0
    End Function

''' <summary>
''' OnCreateControl 発生前に、Me.Height を修正します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetMeHeight()

        RemoveHandler Me.Resize, AddressOf BeforeOnCreateReSize
          If Application.RenderWithVisualStyles Then
             Me.Height = TextBox.Height
          Else
             Me.Height = TextBox.Font.Height + _BothFrameBorderHeight
          End If
        AddHandler Me.Resize, AddressOf BeforeOnCreateReSize
    End Sub

''' <summary>
''' コントロールの高さを設定、および、外枠を描画します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub MeUpdate(ByVal state As Boolean)

        If state Then
           RemoveHandler Me.Resize, AddressOf Me_Resize
           RemoveHandler TextBox.Resize, AddressOf Me_Resize
        End If

        If Not Application.RenderWithVisualStyles Then

        'TextBox の高さ(-1) を固定
           Me.Height = TextBox.Font.Height + _BothFrameBorderHeight

           If _onCreate Then
              Frame.Size = Me.Size

              SetDrawPoint()
              DrawFrame()
           End If
        Else
           Me.Region = Nothing
           Me.Height = TextBox.Height
        End If

     'TextBox のサイズを Me に会わせる
        TextBox.Size = Me.Size

        If state Then
           AddHandler Me.Resize, AddressOf Me_Resize
           AddHandler TextBox.Resize, AddressOf Me_Resize
        End If
    End Sub

''' <summary>
''' 外枠の描画座標をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetDrawPoint()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        _FramePoint = {
          New Point(1, 0), New Point(w - 2, 0), New Point(w - 2, 1), New Point(w - 1, 1),
          New Point(w - 1, h - 2), New Point(w - 2, h - 2), New Point(w - 2, h - 1), New Point(1, h - 1),
          New Point(1, h - 2), New Point(0, h - 2), New Point(0, 1), New Point(1, 1)}
    End Sub

''' <summary>
''' Region (Me, Frame) の設定、および、外枠を描画します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub DrawFrame()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        If (w < 1) OrElse (h < 1) Then Return

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        g.DrawRectangle(New Pen(SystemColors.ControlLightLight), 1, 1, w - 3, h - 3)
        g.DrawLines(New Pen(SystemColors.ControlDark), _FramePoint)

        Me.Frame.Image = DirectCast(b.Clone, Drawing.Image)

        path = New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(0, 0, w, h))
        path.AddRectangle(New Rectangle(2, 2, w - 4, h - 4))

        Me.Frame.Region = New Region(path)

        path.Dispose()
        g.Dispose()
        b.Dispose()

        path = Nothing
        g = Nothing
        b = Nothing
    End Sub

''' <summary>
''' 入力文字のフィルターを設定します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub SetKeyPressRegStr()
        _KeyPressRegStr = "["

        If _Minimum < 0 Then _KeyPressRegStr &= "-"
        If _ThousandsSeparator Then _KeyPressRegStr &= ","

        _KeyPressRegStr &= "\d\b]"
    End Sub

''' <summary>
''' TextBox の表示（値）を更新します。
''' </summary>

    <DebuggerStepThrough()> _
    Private Sub ValueTextUpdate()
        If _ThousandsSeparator Then
           TextBox.Text = String.Format("{0:#,##0}", _Value)
        Else
           TextBox.Text = _Value.ToString
        End If
    End Sub

''' <summary>
''' 入力（ペースト）された値をチェックし、TextBox の表示（値）を更新します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub TextValueCheck(ByVal delta As Integer)

        Dim state As Boolean

        _OldValue = _Value

        _Value = StringToInt32Parse(TextBox.Text, state) + delta

        If state Then
           state = False
           If _Value < _Minimum Then
              _Value = _Minimum
           ElseIf _Value > Maximum Then
              _Value = Maximum
           Else
              state = True
           End If
        Else
           _Value = _OldValue
        End If

        If (_Value = 0) AndAlso Not _DisplayZero Then
           TextBox.Clear()
        Else
           ValueTextUpdate()
        End If

        If Not state Then Beep()

        If _OldValue <> _Value Then
           RaiseEvent ValueChanged(Me, New EventArgs)
        End If
    End Sub

#End Region

    Public Sub New()

        InitializeComponent()

        Me.Frame.BringToFront()
        Me.Frame.Visible = Not Application.RenderWithVisualStyles
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

     'OnCreateControl 発生前のイベントを削除
        RemoveHandler Me.Resize, AddressOf BeforeOnCreateReSize
        RemoveHandler TextBox.Resize, AddressOf BeforeOnCreateTextBoxResize

        MeUpdate(False)
        SetKeyPressRegStr()

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler TextBox.Resize, AddressOf Me_Resize

        MyBase.OnCreateControl()
    End Sub

#Region "// Me"

''' <summary>
''' OnCreateControl 発生前に、TextBox のサイズを取得する為のイベント。
''' </summary>

    Private Sub BeforeOnCreateReSize(sender As Object, e As EventArgs) Handles Me.Resize

        _onResize = True

        SetMeHeight()
    End Sub

''' <summary>
''' OnCreateControl 発生前に、TextBox のサイズを取得する為のイベント。
''' </summary>

    Private Sub BeforeOnCreateTextBoxResize(sender As Object, e As EventArgs) Handles TextBox.Resize

        If Not _onResize Then Return

        SetMeHeight()
    End Sub

    Private Sub Me_SystemColorsChanged(sender As Object, e As EventArgs) Handles Me.SystemColorsChanged

        MeUpdate(True)

        Me.Frame.Visible = Not Application.RenderWithVisualStyles
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        MeUpdate(True)
    End Sub

    Private Sub Me_Enter(sender As Object, e As EventArgs) Handles Me.Enter
        TextBox.Select()
    End Sub

#End Region

#Region "// TextBox"

    Private Sub TextBox_Enter(sender As Object, e As EventArgs) Handles TextBox.Enter

        _CaretState = True  '＝ Form_Activated
    End Sub

    Private Sub TextBox_Leave(sender As Object, e As EventArgs) Handles TextBox.Leave

        TextValueCheck(0)
    End Sub

    Private Sub TextBox_MouseLeave(sender As Object, e As EventArgs) Handles TextBox.MouseLeave
        If Not _CaretState AndAlso TextBox.Focused Then
        'カーソルを右端に移動
           If TextBox.SelectionLength = 0 Then
              TextBox.SelectionStart = TextBox.Text.Length
           End If

           _CaretState = True

           ShowCaret(TextBox.Handle)
        End If
    End Sub

    Private Sub TextBox_MouseDown(sender As Object, e As MouseEventArgs) Handles TextBox.MouseDown
        If e.Button = MouseButtons.Middle Then

           If Not _CaretState Then
              _CaretState = True

              ShowCaret(TextBox.Handle)
           End If

           Me.Value = 0
        End If
    End Sub

    Private Sub TextBox_MouseWheel(sender As Object, e As MouseEventArgs) Handles TextBox.MouseWheel

        If _CaretState AndAlso TextBox.Focused Then
            _CaretState = False

            HideCaret(TextBox.Handle)
        End If

        TextValueCheck((_Increment * Math.Sign(e.Delta)))
    End Sub

    Private Sub TextBox_Click(sender As Object, e As EventArgs) Handles TextBox.Click
        If Not _CaretState Then
           _CaretState = True

           ShowCaret(TextBox.Handle)
        End If
    End Sub

    Private Sub TextBox_PreviewKeyDown(sender As Object, e As PreviewKeyDownEventArgs) Handles TextBox.PreviewKeyDown

    '[Enter] キーによる値の確定、および、 
    '配置元のフォームで、KeyPreview が True になっている場合に対処する措置
    '( PreviewKeyDown ⇒ Form:KeyDown ⇒ KeyDown )

        If e.KeyData = Keys.Enter Then

           TextValueCheck(0)
        End If
    End Sub

    Private Sub TextBox_KeyDown(sender As Object, e As KeyEventArgs) Handles TextBox.KeyDown

        Dim delta As Integer

        Select Case e.KeyData

          Case Keys.Up
            delta = _Increment

          Case Keys.Down
            delta = -_Increment

          Case Keys.Enter

            e.SuppressKeyPress = True       'KeyPressイベントをキャンセル

          Case Keys.Right, Keys.Left
            If Not _CaretState Then
              _CaretState = True

              ShowCaret(TextBox.Handle)
            End If

          Case DirectCast(131137, Keys)     '[Ctrl]+A
            TextBox.SelectAll()

            e.SuppressKeyPress = True       'KeyPressイベントをキャンセル

          Case DirectCast(131139, Keys)     '[Ctrl]+C
            If TextBox.SelectedText.Length <> 0 Then
               Clipboard.SetText(TextBox.SelectedText)
            End If

            e.SuppressKeyPress = True       'KeyPressイベントをキャンセル

          Case DirectCast(131158, Keys)     '[Ctrl]+V
            _KeyPressState = State.Paste

        End Select

        If delta <> 0 Then
           Me.TextBox_MouseWheel(Me, New MouseEventArgs(MouseButtons.None, 0, 0, 0, delta))
        End If
    End Sub

    Private Sub TextBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles TextBox.KeyPress

        If _KeyPressState = State.None Then

           If e.KeyChar = vbCr Then
              e.Handled = True      '※ Handle を True にしないと Beep 音が鳴る
              Return
           End If

        '[-,]0-9,[BS] 以外、KeyPressイベントをキャンセル
           If Not Regex.IsMatch(e.KeyChar.ToString, _KeyPressRegStr) Then
              e.Handled = True

              Beep()
           End If
        Else
           _KeyPressState = State.None
        End If
    End Sub

#End Region

End Class

End Namespace
