﻿'
'   TextBoxEx 1.0.3.1 for Classic Style (Multiline, Text 対応 Ver.)
'
'       Property:
'         BackColor       / ForeColor
'         Multiline
'         Read_Only
'         Text
'         TextAlign
'         TextLength      / MaxLength
'         SelectionStart  / SelectionLength
'
'       Method
'         Clear
'         SelectAll
'
'       Event Notification
'         KeyDown      / KeyPress
'         MouseDown
'         TextChanged
'         Click
'
'       注）デザイナー以外で本コントロールを生成する際は、
'           必ず 下記の様に Multiline の設定を先に行って下さい。
'
'           Font → Multiline → Size
'           Multiline → Size → Font
'           Multiline → Font → Size

Imports System.ComponentModel
Imports System.Drawing.Design
Imports System.Text.RegularExpressions
Imports System.Runtime.InteropServices

Namespace UserLib

<DebuggerStepThrough()>
Public Class TextBoxEx

    Public Shadows Event TextChanged(sender As Object, e As EventArgs)

    Private _onCreate As Boolean
    Private _onResize As Boolean
    Private _onResizeEventState As Boolean = True

    Private _BothFrameBorderHeight As Integer = SystemInformation.FixedFrameBorderSize.Height * 2

    Private _FramePoint As Point()
    Private _MultilineHeight As Integer

#Region "// Property"

#Region "** Shadows Property"

    <Browsable(False)> _
    Public Shadows Property AutoValidate() As AutoValidate

    <Browsable(False)> _
    Public Shadows Property AutoScroll() As Boolean
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMargin() As Size
    <Browsable(False), DefaultValue(GetType(Size), "0, 0")> _
    Public Shadows Property AutoScrollMinSize() As Size
    <Browsable(False)> _
    Public Shadows Property AutoSize() As Boolean
    <Browsable(False), DefaultValue(AutoSizeMode.GrowAndShrink)> _
    Public Shadows Property AutoSizeMode() As AutoSizeMode
    <Browsable(False)> _
    Public Shadows Property Padding() As Padding

    <Browsable(False)> _
    Public Shadows Property BackgroundImage() As Image
    <Browsable(False), DefaultValue(ImageLayout.None)> _
    Public Shadows Property BackgroundImageLayout() As ImageLayout
    <Browsable(False), DefaultValue(BorderStyle.Fixed3D)> _
    Public Shadows Property BorderStyle() As BorderStyle = BorderStyle.Fixed3D

#End Region

    Private _Text As String
    Private _TextValue As String
    Private _Multiline As Boolean

    Private _DefaultBackColor As Color = SystemColors.Window
    Private _DefaultForeColor As Color = SystemColors.WindowText

#Region "** 制御用"

''' <summary>
''' デザイナー で設定した Text プロパティの値をデザイナーに渡す為のプロパティです。
''' </summary>

    <Browsable(False), DefaultValue("")> _
    Public Property TextValue As String
      Get
        Return _Text
      End Get

      Set(value As String)
        Me.Text = value
      End Set
    End Property

#End Region

#Region "** 動作"

''' <summary>
''' ユーザーがテキスト ボックス コントロールに入力または貼り付けできる最大文字数を取得または設定します。
''' </summary>

    <Browsable(True), DefaultValue(32767), Category("動作"),
     Description("エディット コントロールに入力できる最大文字数を指定します。")> _
    Public Property MaxLength() As Integer
      Get
        Return TextBox.MaxLength
      End Get

      Set(value As Integer)
        TextBox.MaxLength = value
      End Set
    End Property

''' <summary>
''' これが複数行のテキスト ボックス コントロールかどうかを示す値を取得または設定します。
''' </summary>

    <Browsable(True), DefaultValue(False), Category("動作"),
     Description("エディット コントロールのテキストが複数行にわたることができるかを設定します。")> _
    Public Property Multiline() As Boolean
      Get
        Return _Multiline
      End Get

      Set(value As Boolean)
        If value = _Multiline Then Return

        _Multiline = value

        If Not value Then
           _MultilineHeight = Me.Height
        End If

     'Multiline 変更時、全てのResize イベントをキャンセル
        _onResizeEventState = False
          TextBox.Multiline = value     'TextBox.Resize 複数回発生 (キャンセル)
        _onResizeEventState = True

        If Not _onCreate Then Return

        If value Then
           If Me.Height <> _MultilineHeight Then
              Me.Height = _MultilineHeight
           End If
        Else
           If Application.RenderWithVisualStyles Then
              Me.Height = TextBox.Height
           Else
              Me.Height = TextBox.Font.Height + _BothFrameBorderHeight
           End If
        End If
      End Set
    End Property

''' <summary>
''' テキスト ボックスのテキストが読み取り専用かどうかを示す値を取得または設定します。
''' </summary>

    <DefaultValue(False), Category("動作"),
     Description("エディット コントロールの中の文字列を変更できるかどうかを設定します。")> _
    Public Property Read_Only() As Boolean
      Get
        Return TextBox.ReadOnly
      End Get

      Set(value As Boolean)
        TextBox.ReadOnly = value
      End Set
    End Property

#End Region

#Region "** 表示"

''' <summary>
''' コントロールの背景色を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Color), "Window"), Category("表示"),
     Description("コンポーネントの背景色です。")> _
    Public Shadows Property BackColor() As Color
      Get
        Return TextBox.BackColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           value = _DefaultBackColor
        End If

        TextBox.BackColor = value
      End Set
    End Property

''' <summary>
''' コントロールの前景色を取得または設定します。
''' </summary>

    <DefaultValue(GetType(Color), "WindowText"), Category("表示"),
     Description("テキストを表示するのに使用される、このコンポーネントの前景色です。")> _
    Public Shadows Property ForeColor() As Color
      Get
        Return TextBox.ForeColor
      End Get

      Set(value As Color)
        If value.IsEmpty Then
           value = _DefaultForeColor
        End If

        TextBox.ForeColor = value
      End Set
    End Property

 ''' <summary>
 '''  テキスト ボックスの現在のテキストを取得または設定します。
 ''' </summary>

    <Browsable(True), DefaultValue(""), Category("表示"),
     Editor("System.ComponentModel.Design.MultilineStringEditor, System.Design", GetType(UITypeEditor)),
     Description("コントロールに関連付けされたテキストです。")> _
    Public Shadows Property Text() As String
      Get
        Return _Text
      End Get

      Set(value As String)
        _Text = value
        _TextValue = value

        RemoveHandler TextBox.TextChanged, AddressOf TextBox_TextChanged
          TextBox.Text = _Text

        AddHandler TextBox.TextChanged, AddressOf TextBox_TextChanged

        RaiseEvent TextChanged(Me, New EventArgs)
      End Set
    End Property

''' <summary>
''' コントロールでテキストをどのように配置するかを取得または設定します。
''' </summary>

    <Browsable(True), DefaultValue(HorizontalAlignment.Left), Category("表示"),
     Description("コントロールに対してどのようにテキストを配置するかを示します。")> _
    Public Shadows Property TextAlign As HorizontalAlignment
      Get
        Return TextBox.TextAlign
      End Get

      Set(value As HorizontalAlignment)
        TextBox.TextAlign = value

        If _onCreate Then Me.Invalidate()
      End Set
    End Property

#End Region

#Region "** その他"

''' <summary>
''' テキスト ボックスで選択されている文字数を取得または設定します。
''' </summary>

    <Browsable(False), DefaultValue(0)> _
    Public Property SelectionLength() As Integer
      Get
        Return TextBox.SelectionLength
      End Get

      Set(value As Integer)
        TextBox.SelectionLength = value
      End Set
    End Property

''' <summary>
''' テキスト ボックスで選択されているテキストの開始点を取得または設定します。
''' </summary>

    <Browsable(False), DefaultValue(0)> _
    Public Property SelectionStart() As Integer
      Get
        Return TextBox.SelectionStart
      End Get

      Set(value As Integer)
        TextBox.SelectionStart = value
      End Set
    End Property

''' <summary>
''' コントロールのテキストの長さを取得します。
''' </summary>

    <Browsable(False)> _
    Public ReadOnly Property TextLength() As Integer
      Get
        Return TextBox.TextLength
      End Get
    End Property

#End Region

#End Region

#Region "// Method"

''' <summary>
''' TextBox のすべてのテキストを削除します。
''' </summary>

    Public Sub Clear()
        TextBox.Clear()
    End Sub

''' <summary>
''' TextBox のすべてのテキストを選択します。
''' </summary>

    Public Sub SelectAll()
        TextBox.SelectAll()
    End Sub

#End Region

#Region "// Sub-Routine & Function"

    <DllImport("USER32.DLL", CharSet:=CharSet.Auto)> _
    Private Shared Function HideCaret( _
        ByVal hwnd As IntPtr) As Integer
    End Function

    <DllImport("USER32.DLL", CharSet:=CharSet.Auto)> _
    Private Shared Function ShowCaret( _
        ByVal hwnd As IntPtr) As Integer
    End Function

    <DebuggerStepThrough()> _
    Private Function StringToInt32(ByVal value As String) As Integer

        If String.IsNullOrEmpty(value) Then Return 0

        value = value.Replace(",", "")

        Dim i As Integer
        Dim r As New Regex("^(\s*)?[\+-]?\d*", RegexOptions.None)
        Dim m As Match = r.Match(value)

        If m.Success AndAlso Integer.TryParse(m.Value, i) Then
           Return i
        End If

        Return 0
    End Function

''' <summary>
''' 外枠の描画座標をセットします。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetDrawPoint()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        _FramePoint = {
          New Point(1, 0), New Point(w - 2, 0), New Point(w - 2, 1), New Point(w - 1, 1),
          New Point(w - 1, h - 2), New Point(w - 2, h - 2), New Point(w - 2, h - 1), New Point(1, h - 1),
          New Point(1, h - 2), New Point(0, h - 2), New Point(0, 1), New Point(1, 1)}
    End Sub

''' <summary>
''' Region (Me, Frame) の設定、および、外枠を描画します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub DrawFrame()

        Dim w As Integer = Me.Width
        Dim h As Integer = Me.Height

        If (w < 1) OrElse (h < 1) Then Return

        Dim path As New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(1, 0, w - 2, 1))
        path.AddRectangle(New Rectangle(0, 1, w, h - 2))
        path.AddRectangle(New Rectangle(1, h - 1, w - 2, 1))

        Me.Region = New Region(path)

        Dim b As New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(b)

        g.DrawRectangle(New Pen(SystemColors.ControlLightLight), 1, 1, w - 3, h - 3)
        g.DrawLines(New Pen(SystemColors.ControlDark), _FramePoint)

        Me.Frame.Image = DirectCast(b.Clone, Drawing.Image)

        path = New Drawing2D.GraphicsPath()

        path.AddRectangle(New Rectangle(0, 0, w, h))
        path.AddRectangle(New Rectangle(2, 2, w - 4, h - 4))

        Me.Frame.Region = New Region(path)

        path.Dispose()
        g.Dispose()
        b.Dispose()

        path = Nothing
        g = Nothing
        b = Nothing
    End Sub

''' <summary>
''' OnCreateControl 発生前に、Me.Height を修正します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub SetMeHeight()

        If Not _Multiline Then

           RemoveHandler Me.Resize, AddressOf BeforeOnCreateReSize
             If Application.RenderWithVisualStyles Then
                Me.Height = TextBox.Height
             Else
                Me.Height = TextBox.Font.Height + _BothFrameBorderHeight
             End If
           AddHandler Me.Resize, AddressOf BeforeOnCreateReSize
        End If
    End Sub

''' <summary>
''' コントロールの高さを設定、および、外枠を描画します。
''' </summary>

    <DebuggerStepThrough()>
    Private Sub MeUpdate(ByVal state As Boolean)

        If state Then
           RemoveHandler Me.Resize, AddressOf Me_Resize
           RemoveHandler TextBox.Resize, AddressOf Me_Resize
        End If

        If Not Application.RenderWithVisualStyles Then

           If Not _Multiline Then

           'TextBox の高さ(-1) を固定
                Me.Height = TextBox.Font.Height + _BothFrameBorderHeight
           End If

           If _onCreate Then
              Frame.Size = Me.Size

              SetDrawPoint()
              DrawFrame()
           End If
        Else
           Me.Region = Nothing

           If Not _Multiline Then
              Me.Height = TextBox.Height
           End If
        End If

     'TextBox のサイズを Me に会わせる
        TextBox.Size = Me.Size

        If state Then
           AddHandler Me.Resize, AddressOf Me_Resize
           AddHandler TextBox.Resize, AddressOf Me_Resize
        End If
    End Sub

#End Region

    Public Sub New()

        InitializeComponent()

        Me.Frame.BringToFront()
        Me.Frame.Visible = Not Application.RenderWithVisualStyles
    End Sub

    Protected Overrides Sub OnCreateControl()

        If _onCreate Then
           MyBase.OnCreateControl()
           Return
        End If

        _onCreate = True

     'OnCreateControl 発生前のイベントを削除
        RemoveHandler Me.Resize, AddressOf BeforeOnCreateReSize
        RemoveHandler TextBox.Resize, AddressOf BeforeOnCreateTextBoxResize

        MeUpdate(False)

        AddHandler Me.Resize, AddressOf Me_Resize
        AddHandler TextBox.Resize, AddressOf Me_Resize

        MyBase.OnCreateControl()
    End Sub

    Protected Overrides Sub OnTextChanged(e As EventArgs)

        RemoveHandler TextBox.TextChanged, AddressOf TextBox_TextChanged
          TextBox.Text = _Text

          RaiseEvent TextChanged(Me, e)
        AddHandler TextBox.TextChanged, AddressOf TextBox_TextChanged

        MyBase.OnTextChanged(e)
    End Sub

#Region "// Me"

''' <summary>
''' OnCreateControl 発生前に、TextBox のサイズを取得する為のイベント。
''' </summary>

    Private Sub BeforeOnCreateReSize(sender As Object, e As EventArgs) Handles Me.Resize

        _onResize = True

        SetMeHeight()
    End Sub

''' <summary>
''' OnCreateControl 発生前に、TextBox のサイズを取得する為のイベント。
''' </summary>

    Private Sub BeforeOnCreateTextBoxResize(sender As Object, e As EventArgs) Handles TextBox.Resize

        If Not _onResize Then Return
        If Not _onResizeEventState Then Return

        SetMeHeight()
    End Sub

    Private Sub Me_SystemColorsChanged(sender As Object, e As EventArgs) Handles Me.SystemColorsChanged

        MeUpdate(True)

        Me.Frame.Visible = Not Application.RenderWithVisualStyles
    End Sub

    Private Sub Me_Resize(sender As Object, e As EventArgs)

        If Not _onResizeEventState Then Return

        MeUpdate(True)
    End Sub

    Private Sub Me_Enter(sender As Object, e As EventArgs) Handles Me.Enter
        TextBox.Select()
    End Sub

#End Region

#Region "// TextBox"

' コントロール配置元にイベント通知

#Region "-- Not Defined"

    'Private Sub TextBox_MouseEnter(sender As Object, e As EventArgs) Handles TextBox.MouseEnter
    '    Me.OnMouseEnter(e)
    'End Sub

    'Private Sub TextBox_MouseMove(sender As Object, e As MouseEventArgs) Handles TextBox.MouseMove
    '    Me.OnMouseMove(e)
    'End Sub

    'Private Sub TextBox_MouseHover(sender As Object, e As EventArgs) Handles TextBox.MouseHover
    '    Me.OnMouseHover(e)
    'End Sub

    'Private Sub TextBox_MouseLeave(sender As Object, e As EventArgs) Handles TextBox.MouseLeave
    '    Me.OnMouseLeave(e)
    'End Sub

#End Region

    Private Sub TextBox_TextChanged(sender As Object, e As EventArgs) Handles TextBox.TextChanged
        _Text = TextBox.Text

        RaiseEvent TextChanged(Me, e)
    End Sub

    Private Sub TextBox_MouseDown(sender As Object, e As MouseEventArgs) Handles TextBox.MouseDown
        Me.OnMouseDown(e)
    End Sub

    Private Sub TextBox_Click(sender As Object, e As EventArgs) Handles TextBox.Click
        Me.OnClick(e)
    End Sub

    Private Sub TextBox_KeyDown(sender As Object, e As KeyEventArgs) Handles TextBox.KeyDown
        Me.OnKeyDown(e)
    End Sub

    Private Sub TextBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles TextBox.KeyPress
        Me.OnKeyPress(e)
    End Sub

#End Region

End Class

End Namespace
